#!/bin/bash
###############################################################################
# Revision: $Id: configure.sh 1708 2010-01-26 22:28:01Z alex $
###############################################################################
# Copyright (c) 2007-2009, WOSH - Wide Open Smart Home 
# by Alessandro Polo - OpenSmartHome.com
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     # Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     # Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     # Neither the name of the OpenSmartHome.com WOSH nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Alessandro Polo ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
###############################################################################
## Script used to setup compilation
###############################################################################
#                                                                CONFIGURATION
CONF_CUSTOM_SEL="./custom.srv.pri"
CONF_CUSTOM="./custom.pri"
#                                                                CONFIGURATION
###############################################################################
###############################################################################
#                                                                     FUNCTIONS
check_env() {
	MAKE=`which make 2>&1`
	if [ $? -eq 1 ] ; then
		echo "Error: 'make' not installed!"
		exit 1
	fi
}

check_qt() {
	QMAKE=`which qmake 2>&1`
	if [ $? -eq 1 ] ; then
		QMAKE=`which qmake-qt4`
		if [ $? -eq 1 ] ; then
			echo "Error: Qt4 not installed or configured!"
			exit 1
		fi
	fi
}

clean_built() {

	if [ -f Makefile ] ; then
		make clean &> /dev/null
		rm -f $LIBLIST `find . -iname Makefile` `find -iname Makefile.*` lib/* 
	fi

	RELIST=`find src/ -iname release`
	DELIST=`find src/ -iname debug`
	if [ $? -eq 0 ] ; then
		rm -rf $RELIST $DELIST
	fi

	mkdir -p lib
}
#                                                                     FUNCTIONS
###############################################################################
###############################################################################
echo
echo "WOSH Framework"
echo "-----------------------" 
echo "-configuration  script-"
echo

###############################################################################
echo " => Checking for dependencies..."
check_env
check_qt
echo
###############################################################################
echo " => Cleaning..."
clean_built
echo
###############################################################################
echo " => Select Target..."
echo "   - woshsrv : Static Server [default]"
echo "   - lib : Static Library"
echo
read BUILD_TARGET

case "$BUILD_TARGET" in 
	"lib" )
		CONF_CUSTOM_SEL="custom.lib.pri"
	;;
	* )
	;;
esac

echo "     Copying $CONF_CUSTOM_SEL to $CONF_CUSTOM"
if [ -f $CONF_CUSTOM ] ; then
	rm -f $CONF_CUSTOM
fi

cp $CONF_CUSTOM_SEL $CONF_CUSTOM

###############################################################################

###############################################################################
echo
echo " => All Done!"
echo "     You may run 'qmake' now."
echo
exit 0
