#!/bin/bash
###############################################################################
# Revision: $Id: distr_deb.sh 1708 2010-01-26 22:28:01Z alex $
###############################################################################
# Copyright (c) 2007-2009, WOSH - Wide Open Smart Home 
# by Alessandro Polo - OpenSmartHome.com
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     # Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     # Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     # Neither the name of the OpenSmartHome.com WOSH nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Alessandro Polo ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
###############################################################################
## Script used to create Debian archive
###############################################################################
#                                                                CONFIGURATION

WOSHBIN=woshsrv
CONTROL_FOLDER=./deb-package
TARGET_FOLDER=../distr/deb-package

RELEASE_FILE=../release

CURRUSER=`whoami`

###############################################################################

if [ "$1" = "--help" ];
then
	echo " Options:"
	echo "    'clean'     : Clean all previous binaries [./bin/woshsrv*, $TARGET_FOLDER]"
	echo " OR"
	echo "    '<version>' : Build debian RELEASE-version (else will load from $RELEASE_FILE)"
	echo
	exit 0
fi

if [ "$1" = "clean" ]; then
	echo " Cleaning Package.."
	rm -R -f $TARGET_FOLDER
	rm -f ./bin/woshsrv-*.deb
	echo
	exit 0
fi

RELEASE_VER=$1
if [ ${#RELEASE_VER} = 0 ]; then
	if [ -f $RELEASE_FILE ] ; then
		RELEASE_VER=`cat $RELEASE_FILE`
	else
		echo " ERROR: Failed loading previous release from $RELEASE_FILE"
		echo "        Exiting!"
		exit 1
	fi
fi

WOSH_PKG_FILE="../distr/$WOSHBIN-$RELEASE_VER.deb"

###############################################################################
###############################################################################

exec_if_exists() {
	if [ -f $2 ]; then
		$@
	fi
}

mkdir_if_not_exists() {
	if [ ! -d $1 ]; then
		mkdir -p $1
	fi
}

###############################################################################
###############################################################################
echo
echo "WOSH FRAMEWORK Server"
echo "---------------------" 
echo "-deb-packager script-"
echo
echo " Building Release  : $RELEASE_VER"
echo " Building Target   : $WOSH_PKG_FILE"
echo " Temporary Folder  : $TARGET_FOLDER"

if [ $CURRUSER != "root" ]; then
	echo
	echo " You must be root to install. Please run 'sudo'."
	echo " => Failed. Exiting."
	echo
	exit 1
fi

echo
###############################################################################
echo " => Checking required files.."

if [ ! -e ../bin/woshsrv ]; then
	echo " ERROR: Missing binary ../bin/woshsrv"
	echo "        Exiting."
	echo
	exit 101
fi

###############################################################################
echo " => Cerating Package folder"

rm -R -f $TARGET_FOLDER
rm -f $WOSH_PKG_FILE
mkdir_if_not_exists $TARGET_FOLDER
mkdir_if_not_exists $TARGET_FOLDER/DEBIAN

echo
###############################################################################
###############################################################################
echo " => Copying Package-Control files.."

cp $CONTROL_FOLDER/conffiles $TARGET_FOLDER/DEBIAN/conffiles
cp $CONTROL_FOLDER/control $TARGET_FOLDER/DEBIAN/control
cp $CONTROL_FOLDER/* $TARGET_FOLDER/DEBIAN/

echo
###############################################################################
###############################################################################
echo " => Copying binaries.."

mkdir_if_not_exists $TARGET_FOLDER/usr/bin
cp ../bin/woshsrv $TARGET_FOLDER/usr/bin/

#keep symbols! strip --strip-unneeded $TARGET_FOLDER/usr/bin/*

echo
###############################################################################
echo " => Copying documentation files.."

mkdir_if_not_exists $TARGET_FOLDER/usr/share/doc/woshsrv

exec_if_exists cp ../copyright $TARGET_FOLDER/usr/share/doc/woshsrv/copyright

exec_if_exists cp ../README $TARGET_FOLDER/usr/share/doc/woshsrv/README
exec_if_exists cp ../NEWS $TARGET_FOLDER/usr/share/doc/woshsrv/NEWS
exec_if_exists cp ../ChangeLog.source $TARGET_FOLDER/usr/share/doc/woshsrv/changelog
exec_if_exists cp ../ChangeLog.source $TARGET_FOLDER/usr/share/doc/woshsrv/changelog.Debian

#gzip --best $TARGET_FOLDER/usr/share/doc/woshsrv/copyright
exec_if_exists rm $TARGET_FOLDER/usr/share/doc/woshsrv/changelog.gz
gzip --best $TARGET_FOLDER/usr/share/doc/woshsrv/changelog

exec_if_exists rm $TARGET_FOLDER/usr/share/doc/woshsrv/changelog.Debian.gz
gzip --best $TARGET_FOLDER/usr/share/doc/woshsrv/changelog.Debian

echo
###############################################################################
echo " => Creating man pages.."

mkdir_if_not_exists $TARGET_FOLDER/usr/share/man/man1

exec_if_exists rm $TARGET_FOLDER/usr/share/man/man1/woshsrv.1


nroff -man ../doc/woshsrv.man > $TARGET_FOLDER/usr/share/man/man1/woshsrv.1

exec_if_exists rm $TARGET_FOLDER/usr/share/man/man1/woshsrv.1.gz

gzip --best $TARGET_FOLDER/usr/share/man/man1/woshsrv.1

echo
###############################################################################
echo " => Copying configuration files.."

mkdir_if_not_exists $TARGET_FOLDER/etc/wosh
cp -R ../etc/wosh $TARGET_FOLDER/etc/wosh

echo
###############################################################################
###############################################################################
echo " => Fixing Owner and Permissions.."

find $TARGET_FOLDER -type d | xargs chmod 755
chown -R root:root $TARGET_FOLDER

chmod 644 $TARGET_FOLDER/DEBIAN/*
exec_if_exists "chmod 755" $TARGET_FOLDER/DEBIAN/preinst
exec_if_exists "chmod 755" $TARGET_FOLDER/DEBIAN/prerm
exec_if_exists "chmod 755" $TARGET_FOLDER/DEBIAN/postinst
exec_if_exists "chmod 755" $TARGET_FOLDER/DEBIAN/postrm


chmod 644 $TARGET_FOLDER/etc/wosh*

chmod 644 $TARGET_FOLDER/usr/share/doc/woshsrv/*

echo
###############################################################################
echo " => Creating Package.."

dpkg-deb --build $TARGET_FOLDER
exec_if_exists rm $WOSH_PKG_FILE
mv $TARGET_FOLDER.deb $WOSH_PKG_FILE

echo
###############################################################################

if [ ! -e /usr/bin/lintian ]; then
	echo " => Validating Package.. Skipped [lintian not installed!]"
	echo
else
	echo " => Validating Package.."
	echo
	lintian $WOSH_PKG_FILE
	echo
fi

###############################################################################
echo " => Restoring write permission.."
chmod -R a+w $TARGET_FOLDER
echo
###############################################################################
echo " => Testing Package.."
echo -n "    Do you want to test package? [y/n] "

read DOTEST;
case "$DOTEST" in 
	YES | yes | y | Y)
		echo 
		echo 
		echo "   Running:  dpkg --install $WOSH_PKG_FILE"
		dpkg --install $WOSH_PKG_FILE
		echo -en "\007"
		echo 
		read DOTEST;
		echo -n "    Press something to continue, CTRL-C to quit."
		echo "   Running:  dpkg -L $WOSH_PKG_FILE"
		dpkg -L $WOSH_PKG_FILE
		echo 
		echo "   Running:  dpkg -s $WOSH_PKG_FILE"
		dpkg -s woshsrv
	;;
	*)
		echo
	;;
esac

echo
###############################################################################
echo " => make-deb.sh: All Done!"
echo
echo -en "\007"
exit 0
