/**
 * @file    bundles.h
 * @brief   Documentation: WOSH Framework Bundles
 ****************************************************************************
 * @author  Alessandro Polo
 * @version 0.00 $Id: bundles.h 2193 2010-03-23 06:46:47Z alex $
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
/*! \page page_bundles Bundles
 *
 * This page regards a generic WOSH service, called Bundle.
 *
 * It is suggested to read \ref page_core first, you should give a look to
 * \ref page_framework too.
 *
 *
 * \par
 *  "An agent is anything that can be viewed as perceiving its environment
 *  through sensors and acting upon that environment through effectors."
 *  (Russel & Norving, 1995)
 *
 *
 * \section page_bundles_toc Table of Contents:
 *
 *  - \ref page_bundles_overview
 *
 *  - \ref page_bundles_bundlegeneric
 *
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_bundles_overview Overview
 *
 * wosh::Bundle is the base class of most 'real' component of WOSH Framework.
 *
 * Bundles may be compared to Services on Windows and daemons on POSIX system,
 * but a Bundle may be compared to an application for many aspects.
 *
 * Bundles may (and usually do):
 *  - interact with other Bundles and (system) events
 *  - hold and monitor properties
 *  - perform (and reply to) functionalities
 *  - fork processes (threads)
 *
 * As you should know, WOSH is a network distributed system, but even the interaction
 * with remote bundles (on other hosts, platforms) is transparent to the bundle (user and developer).
 *
 * It can be chosen whether the components are loaded locally (here, the component code is linked to the class which starts the framework),
 * or loaded dynamically* from shared libraries (UNIX) or dynamic link libraries (Windows), for example.
 *
 * Components can be started and stopped at runtime,
 * and the components are notified about the lifecycle of other components, if necessary.
 *
 * wosh::BundleManager is the Manager of bundles and manage their life-cycle (as well their
 * creation/destruction).
 * WOSH URI is .:/Bundles, you may see installed (registered) bundles, their status, the URI
 * executing \c list in a (shell) session.
 * \verbatim
root@wosh28:#
root@wosh28:# cd Bundles
root@wosh28:Bundles# list
  Bundles:
              Name Interfaces   State                      URI
--------------------------------------------------------------
             Alarm          X STARTED     wosh28:Bundles/Alarm
 AutomationManager          X STARTED       wosh28:Automations
   BuildingManager          X STARTED          wosh28:Building
         Discovery          X STARTED wosh28:Bundles/Discovery
     SessionServer          X STARTED          wosh28:Sessions

root@wosh28:Bundles#
\endverbatim
 *
 * See wosh::Bundle for more technical information.
 *
 * Configuration and loading process is described in \ref page_config .
 *
 * Bundles (sources) are located in <a href="../../src/bundles">/src/bundles</a> folder.
 *
 *
 * A wosh::Bundle (implementation) usually implements also a specific interface (or more).
 * Interfaces are defined in <a href="../../src/interfaces">/src/interfaces</a> folder.
 *
 *
 * See also \ref Services, \ref Devices.
 *
 *
 * Refer to file <a href="../../src/bundles/README">/src/bundles/README</a> (prompt here).
 *
 * \include /src/bundles/README
 *
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_bundles_bundlegeneric BundleGeneric
 *
 *
 *
 *
 *
 *
 ****************************************************************************
 *
 */
