>/**
 * \file    running.h
 * \brief   Documentation: How to run WOSH Framework
 ****************************************************************************
 * @author  Alessandro Polo
 * @version $Id: running.h 1675 2010-01-24 15:20:50Z alex $
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

/*! \page page_run Running
 *
 * This page explains how to build WOSH on your system.
 *
 *
 * \section page_run_toc Table of Contents:
 *
 *  - \ref page_run_overview
 *  - \ref page_run_req
 *  - \ref page_config
 *  - \ref page_run_notes
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_run_overview Overview
 * 
 * WOSH is a service-oriented distributed system and you may need some background before running it.
 * You should read \ref page_config before starting the applications.
 *
 * Once the binaries are built (or downloaded) you have to configure WOSH in order to run properly.
 *
 * You probably want to run \ref page_applications_woshsrv "WOSH Server" on POSIX
 * and \ref page_applications_woshshop "WOSH WorkShop", \ref page_applications_woshremote
 * "WOSH Remote" on Windows.
 *
 * Any application based on WOSH Framework follows the same configuration standards,
 * as described in \ref page_config.
 * Read this page and configure your system before running WOSH.
 *
 * If you built the application on your system you can skip next section
 * ( \ref page_run_req ) and jump to \ref page_run_run
 *
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_run_req Requirements
 *
 * This requirements are already fulfilled if you built WOSH on your system.
 * Read only if you downloaded binaries.
 *
 * This application will use Qt 4.1 or later, freely available on http://trolltech.com/
 * Recently QT has been acquired by Nokia group.
 *
 *
 * On Linux Debian-based systems you may install QT executing:
 * \verbatim
alex@linbox:/$ sudo apt-get install qt
\endverbatim
 * 
 * On Windows system, you need to have QT run-time files (DLLs) into program's folder,
 * they are included in both distributions.
 * Required files are:
 *  - QtCore4.dll [QtCored4.dll]
 *  - QtGui4.dll [QtGuid4.dll]
 *  - QtNetwork4.dll [QtNetworkd4.dll]
 *  - QtXml4.dll [QtXmld4.dll]
 *
 * Optional files from QT which may be required (including some custom Bundles) are:
 *  - phonon4.dll [phonond4.dll] (required by wosh::devices::PlayerPhononBundle)
 *
 * \note WOSH is actually compiled as debug-release, so you will need QT libraries
 * with debug information (files flagged with 'd' before '4').
 *
 *
 * WOSH CE Server requires also \c msvcr90d.dll for the specific platform you are working on.
 *
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_run_run Start WOSH Applications
 *
 * The executable file of WOSH applications are located in the <a href="../../bin">/bin</a> folder.
 * The default configuration file is defined as <em> CONFIG_FILE </em> (as '../etc/woshsrv.conf').
 * A different configuration file can be specified as program's argument.
 *
 * Common ways to run the application are:
 * \verbatim
alex@linbox:/home/alex/WOSH/bin$ ./woshsrv
alex@linbox:/home/alex/WOSH/bin$ ./woshsrv ../etc/woshsrv.conf
\endverbatim
 * 
 * On Windows you may just double-click the executable file or pick it
 * from the Start Menu (in 'WOSH' programs group).
 *
 * Check out \ref page_using as introduction to WOSH Framework and applications.
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_run_notes Notes
 *
 * \b Don't run two instances of WOSH on same host (PC) unless you know what you are doing.
 *
 *
 *
 ****************************************************************************
 *
 */ 

