/**
 * @class   JabberGloox10
 * @brief   Service providing JabberGloox features
 *
 * @note
 *  Required OS: \c Linux
 *  Required Library: \c QT, \c libgloox
 *
 * QMake/g++ configuration:
 * @code
 *  INCLUDEPATH += /usr/include
 *  LIBS += -L/usr/lib
 *  LIBS += -lgloox
 * @endcode
 *
 * On Debian-based system, execute as root:
 * @code
 *  $ apt-get install libgloox-dev
 * @endcode
 *
 *
 * @see     http://code.google.com/apis/talk/libjingle 
 *
 ****************************************************************************
 * @version 1.20 $Id: JabberGloox10.h 1873 2010-02-20 13:23:43Z alex $
 * @date    Released Stable on 2010-02-21
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Bundles_Devices_JabberGloox_10_H__
 #define __WOSH_Bundles_Devices_JabberGloox_10_H__

 #include <core/BusConnector.h>

 #include "JabberGlooxImpl.h"


class JabberGloox10 : public JabberGlooxImpl  {

	public:
		/**
		 * @brief   Default constructor. Init vars.
		 */
		JabberGloox10( wosh::BundleGeneric& bundle );
		/**
		 * @brief   Deconstructor. If thread is stll running, it will be stopped.
		 */
		virtual ~JabberGloox10();


	public:
		virtual bool sendMessage( const std::string& user, const std::string& message );

		virtual bool setPresence( bool available );


/** @name gloox::MessageHandler Interface
 * @{
 ******************************************************************************/
	public:
		virtual void handleMessage( const gloox::Message& msg, gloox::MessageSession* session = NULL );
//@}
/** @name gloox::PresenceHandler Interface
 * @{
 ******************************************************************************/
	public:
		virtual void handlePresence( const gloox::Presence& presence );
//@}
/** @name gloox::SubscriptionHandler Interface
 * @{
 ******************************************************************************/
	public:
		virtual void handleSubscription( const gloox::Subscription& subscription );
//@}


}; // class def


#endif //__WOSH_Bundles_Devices_JabberGloox_10_H__
