/** @file    JabberGlooxBundle.cpp
 *  @author  Alessandro Polo
 *  @version $Id: JabberGlooxBundle.cpp 1761 2010-02-02 05:53:36Z alex $
 *  @brief
 * File containing methods for the wosh::services::JabberGlooxBundle class.
 * The header for this class can be found in JabberGlooxBundle.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "JabberGlooxBundle.h"
 
#ifdef libgloox09
# include "JabberGloox09.h"
#else
# include "JabberGloox10.h"
#endif

 #include <core/ObjectAllocator.h>
 #include <core/MethodsDef.h>
 #include <core/MessageFilter.h>
 #include <core/Fact.h>
 #include <core/NEW_List.h>
 #include <core/Table.h>
 #include <core/WoshKernel.h>
 #include <core/DeviceManager.h>
 #include <core/SecurityManager.h>
 #include <core/UserManager.h>
 #include <core/SystemError.h>
 #include <core/BusManager.h>

 #include <iostream>


using namespace std;
namespace wosh {
 namespace devices {

 WOSH_REGISTER(wosh::devices::JabberGlooxBundle, "wosh::interfaces::devices::JabberGloox", _JabberGloox_VERSION, _static_JabberGlooxBundle1 )
 WOSH_REGISTER(wosh::devices::JabberGlooxBundle, "wosh::interfaces::services::CommunicatorService", _JabberGloox_VERSION, _static_JabberGlooxBundle2 )
 WOSH_REGISTER(wosh::devices::JabberGlooxBundle, "wosh::BundleGeneric", _JabberGloox_VERSION, _static_JabberGlooxBundle3 )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

JabberGlooxBundle::JabberGlooxBundle() : BundleGeneric()
 {
	BundleGeneric::setName( _JabberGloox_NAME, false );
	Properties.update( _KEY_Version, _JabberGloox_VERSION );

//Log.setLevel( LOG_VERBOSE );

	Log(LOG_VERBOSE, " Configuring JabberGloox10 worker.." );
#ifdef libgloox09
	this->jabbWorker = new JabberGloox09(*this);
#else
	this->jabbWorker = new JabberGloox10(*this);
#endif
	this->jabbWorker->setServer( "", _JabberGloox_DEFAULT_ServerPort, "" );
	this->jabbWorker->setAccount( "", "" );
	this->jabbWorker->setStatusMessage( _JabberGloox_DEFAULT_StatusMsg );
	this->jabbWorker->setJabberGlooxListener(this);

	Log(LOG_VERBOSE, " Setting default properties and permissions.." );
	Properties.set( _CommunicatorService_KEY_ChannelType,	(int)Notification::TYPE_TEXTUAL, Permission(Permission::Read) );
	Properties.set( _CommunicatorService_KEY_ChannelLink,	(int )Communicator::LINK_SYNCH, Permission(Permission::Read) );
	Properties.set( _CommunicatorService_KEY_Cost,			0.0,Permission( Permission::RW, Permission::Read, Permission::Read) );
	Properties.set( _CommunicatorService_KEY_Priority,		50, Permission( Permission::RW, Permission::RW, Permission::Read) );
	Properties.set( _CommunicatorService_KEY_Reliability,	70, Permission( Permission::RW, Permission::RW, Permission::Read) );
	Properties.set( _CommunicatorService_KEY_Protocol,		(int)Notification::PROTOCOL_SHELL + (int)Notification::PROTOCOL_HUMAN, Permission( Permission::RW, Permission::Read, Permission::Read) );

	Properties.set( _JabberGloox_KEY_ServerAddress,		"", Permission( Permission::RW, Permission::RW, Permission::Read) );
	Properties.set( _JabberGloox_KEY_ServerPort,		_JabberGloox_DEFAULT_ServerPort, Permission( Permission::RW, Permission::RW, Permission::Read) );
	Properties.set( _JabberGloox_KEY_ServerProxy,		"", Permission( Permission::RW, Permission::RW, Permission::Read) );

	Properties.set( _JabberGloox_KEY_UserName,			"",	Permission( Permission::RW, Permission::RW, Permission::Read) );
	Properties.set( _JabberGloox_KEY_UserPassword,		"",	Permission( Permission::RW, Permission::RW, Permission::Read) );
	Properties.set( _JabberGloox_KEY_StatusMessage,		_JabberGloox_DEFAULT_StatusMsg, Permission( Permission::RW, Permission::RW, Permission::Read) );
	Properties.set( _JabberGloox_KEY_UserAccountKey,	_JabberGloox_DEFAULT_UserAccountKey, Permission( Permission::RW, Permission::RW, Permission::Read) );

	Properties.set( _JabberGloox_KEY_AcceptSubscription,	_JabberGloox_DEFAULT_AcceptSubscription, Permission( Permission::RW, Permission::RW, Permission::Read) );
	Properties.set( _JabberGloox_KEY_AcceptUnSubscription,	_JabberGloox_DEFAULT_AcceptUnSubscription, Permission( Permission::RW, Permission::RW, Permission::Read) );

	Properties.set( _JabberGloox_KEY_DefaultProtocol,		(int)Notification::PROTOCOL_SHELL, Permission( Permission::RW, Permission::Read, Permission::Read) );
	Properties.set( _JabberGloox_KEY_DefaultLanguage,		_JabberGloox_DEFAULT_Language, Permission( Permission::RW, Permission::Read, Permission::Read) );
	Properties.set( _JabberGloox_KEY_DefaultTargetWoshUser,	_JabberGloox_DEFAULT_TargetUser, Permission( Permission::RW, Permission::Read, Permission::Read) );
	Properties.monitor(_JabberGloox_KEY_ServerAddress);
	Properties.monitor(_JabberGloox_KEY_ServerPort);
	Properties.monitor(_JabberGloox_KEY_UserName);
	Properties.monitor(_JabberGloox_KEY_UserPassword);


	Log(LOG_VERBOSE, " Registering methods.." );
	MethodMessageResponse* mmSendMessageToUser = Methods.create<MethodMessageResponse>( _CommunicatorService_METHOD_sendmessage, "send message to (wosh) username" );
	mmSendMessageToUser->setMethod( this, (MethodMessageResponsePtr)&JabberGlooxBundle::mmDoSendMessageToUser );
	mmSendMessageToUser->getPermission().setMask( Permission::RX, Permission::RX, Permission::RX );

	MethodRequest* mmListContacts = Methods.create<MethodRequest>( _METHOD_List, "List contacts" );
	mmListContacts->setMethod( this, (MethodRequestPtr)&JabberGlooxBundle::mmDoListContacts );
	mmListContacts->getPermission().setMask( Permission::RX, Permission::RX, Permission::RX );

	MethodRequest* mmContactAdd = Methods.create<MethodRequest>( _JabberGloox_METHOD_ContactAdd, "Add contact" );
	mmContactAdd->setMethod( this, (MethodRequestPtr)&JabberGlooxBundle::mmDoContactAdd );
	mmContactAdd->getPermission().setMask( Permission::RX, Permission::Read, Permission::Read );

	MethodRequest* mmContactRemove = Methods.create<MethodRequest>( _JabberGloox_METHOD_ContactRemove, "Renove contact" );
	mmContactRemove->setMethod( this, (MethodRequestPtr)&JabberGlooxBundle::mmDoContactRemove );
	mmContactRemove->getPermission().setMask( Permission::RX, Permission::Read, Permission::Read );

	MethodRequest* mmContactsSynch = Methods.create<MethodRequest>( _JabberGloox_METHOD_ContactSynch, "Synch contacts against WOSH users" );
	mmContactsSynch->setMethod( this, (MethodRequestPtr)&JabberGlooxBundle::mmDoContactsSynch );
	mmContactsSynch->getPermission().setMask( Permission::RX, Permission::Read, Permission::Read );

	setBundleState(Bundle::STATE_CREATED, false);
 }

JabberGlooxBundle::~JabberGlooxBundle()
 {
	Log(LOG_VERBOSE, " Destroying.." );

	if ( isBundleRunning() ) {
		Log(LOG_WARNING, "~JabberGlooxBundle() : Destroying while RUNNING! Trying to stop.." );
		bundleStop();
	 }

	delete this->jabbWorker; this->jabbWorker = NULL;
	Log(LOG_VERBOSE, ":~JabberGlooxBundle() : Destroyed." );
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL

WRESULT JabberGlooxBundle::bundleStart()
 {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STARTING) )
		return WRET_ERR_WRONG_STATE;


	if ( this->jabbWorker->getServerAddress().empty() ||
		 this->jabbWorker->getServerPort() == 0 ||
		 this->jabbWorker->getAccountName().empty() ) {
		SystemError::raise(this, Log, SystemError::CRITICAL, 0, "bundleStart", "Server/Account was NOT Configured!" );
		return WRET_ERR_PARAM;
	 }

	WRESULT ret = BundleGeneric::start_SynchThread( this->jabbWorker );

	Response* response = new Response( _METHOD_ListProperties, Properties.toTable(false) );
	Message* message = new Message(response);
	message->setSource(this);
	message->getDestination().fromString("*:Communication");
	message->setSecurityToken( SecurityManager::generateToken(this->getPermission().getOwner()) );
	BusCore.postMessage(message);

	if ( WSUCCEEDED(ret) ) {
		Log(LOG_VERBOSE, ":bundleStart() : Registering as Device.." );
		wosh::Device* myself = new wosh::Device(*this);
		myself->getInterfaces() = this->Interfaces;
		WoshKernel::getInstance()->devices().registerDevice( myself );
	 }
	// BUNDLE-STATE (STARTED) will be updated async by WORKER, through call: thread_event()
	return ret;
 }

WRESULT JabberGlooxBundle::bundleStop()
 {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STOPPING) )
		return WRET_ERR_WRONG_STATE;

	WRESULT ret = BundleGeneric::stop_SynchThread( this->jabbWorker );

	if ( WSUCCEEDED(ret) ) {
		Log(LOG_VERBOSE, ":bundleStop() : UnRegistering as Device.." );
		WoshKernel::getInstance()->devices().unRegisterDevice( this->getName() );
	 }
	// BUNDLE-STATE (STOPPED) will be updated async by WORKER, through call: thread_event()
	return ret;
 }

////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////// PROCESS BUS MESSAGES

void JabberGlooxBundle::busMessage( const Message& message, const Bus* source )
 {
	if ( message.isEmpty() ) return;

	if ( message.getContent()->isNotification() ) {
		if ( message.getDestination().isBroadcast() )
			return;
		if ( !MessageFilter::isMessageTarget(message, this) )
			return;
		if ( !isBundleRunning() )
			return;
		const Notification* notif = message.getContent()->asNotification();
		WRESULT sent = notify( notif );
		Message* reply = message.replyTo( new Response( _CommunicatorService_RESPONSE_notif_sent, new Variant(notif->getID()), sent ) );
		reply->setSecurityToken( SecurityManager::generateToken(this->getPermission().getOwner()) );
		BusCore.postMessage(reply);
		if ( WSUCCEEDED(sent) ) {
			long ts = getUserLastSeen( notif->getRecipent_User() );
			if ( ts > 0 && Utilities::std_time() - ts < 10 ) {
				Message* reply2 = message.replyTo( new Response( _CommunicatorService_RESPONSE_notif_read, new Variant(notif->getID()) ) );
				reply2->setSecurityToken( SecurityManager::generateToken(this->getPermission().getOwner()) );
				BusCore.postMessage(reply2);
			 }
		 }
		return;
	 }

	BundleGeneric::busMessage(message, source);
 }

//////////////////////////////////////////////////////////////////////////////////////// PROCESS BUS MESSAGES
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////// GENERATED EVENTS

void JabberGlooxBundle::raiseEvent( MessageContent* content, SecurityToken* security )
 {
	Message* msg_event = new Message();
	msg_event->setSource( this );
	msg_event->setContent( content );
	msg_event->setDestinationBroadcast(true);
	//msg_event->setDestinationBus(_Bus_Devices);
	if ( security != NULL )
		msg_event->setSecurityToken(security);
	else
		msg_event->setSecurityToken( SecurityManager::generateToken(this->getPermission().getOwner()) );
///@todo BUUUUUUUUUUUUUUUUUUUUUUUUUGGGGG must define micro and macro routing strategies!
	//BusDevices.postMessage(msg_event);
	BusCore.postMessage(msg_event);
 }

//////////////////////////////////////////////////////////////////////////////////////////// GENERATED EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENTS

bool JabberGlooxBundle::updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source )
 {
	if ( property_current.getKey() == _JabberGloox_KEY_ServerAddress ||
		 property_current.getKey() == _JabberGloox_KEY_ServerPort ||
		 property_current.getKey() == _JabberGloox_KEY_UserName ||
		 property_current.getKey() == _JabberGloox_KEY_UserPassword )
	 {
		if ( this->jabbWorker->isThreadRunning() )
			return false;
		bool ret = false;

		if ( property_current.getKey() == _JabberGloox_KEY_ServerAddress )
			ret = this->jabbWorker->setServerAddress( value_proposed.toString() );

		else if ( property_current.getKey() == _JabberGloox_KEY_ServerPort )
			ret = this->jabbWorker->setServerPort( value_proposed.toInteger(_JabberGloox_DEFAULT_ServerPort) );

		else if ( property_current.getKey() == _JabberGloox_KEY_ServerPort )
			ret = this->jabbWorker->setServerProxy( value_proposed.toString() );

		else if ( property_current.getKey() == _JabberGloox_KEY_UserName )
			ret = this->jabbWorker->setAccountUserName( value_proposed.toString() );

		else if ( property_current.getKey() == _JabberGloox_KEY_UserPassword ) {
			ret = this->jabbWorker->setAccountUserPassword( value_proposed.toString() );
			property_current.setValue("*********");
			do_update = false;
		 }
		else if ( property_current.getKey() == _JabberGloox_KEY_StatusMessage )
			ret = this->jabbWorker->setStatusMessage( value_proposed.toString() );
		return ret;
	 }
	else // forward processing to base class
		return BundleGeneric::updatingProperty(do_update, value_proposed, property_current, source);
 }

///////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

ContactInfo* JabberGlooxBundle::checkCreateAccount_( const string& accountname )
 {
	// recognize the user
	string username = getUsernameFromAccountName(accountname);
	if ( username == "" ) {
		SystemError::raise(this, Log, SystemError::WARNING, 0, "jabbbot_Message", "Account %s not recognized", accountname.c_str() );
		return NULL;
	 }

	///@todo EVAL USER/SOURCE vs. SECURITY RULES here
	Log(LOG_INFO, ":checkCreateAccount_() : Creating Slot for %s [%s]", accountname.c_str(), username.c_str() );

	ContactInfo* info = new ContactInfo();
	this->contacts.set(accountname, info);

	info->setUser( username );
	info->setUserID( accountname );

	int proto = Properties.getValueOf(_JabberGloox_KEY_DefaultProtocol).toInteger(-1);
	info->setProtocol( (Notification::PROTOCOL)proto );
	Log(LOG_VERBOSE, ":checkCreateAccount_() : Protocol of %s set to %i", info->getUser().c_str(), (int)info->getProtocol() );

	Variant langPref = UserManager::getUserProperty(info->getUser(), "LanguagePreferred");
	if ( !langPref.isStringEmpty() )
		info->setLanguage( langPref.asString() );
	else {
		std::string lang = Properties.getValueOf( _JabberGloox_KEY_DefaultLanguage ).toString();
		if ( !lang.empty() )
			info->setLanguage( lang );
		else
			info->setLanguage( _JabberGloox_DEFAULT_Language );
	 }
	Log(LOG_VERBOSE, ":checkCreateAccount_() : Language of %s set to %s", info->getUser().c_str(), info->getLanguage().c_str() );

	std::string TargetUser = Properties.getValueOf( _JabberGloox_KEY_DefaultTargetWoshUser ).toString();
	info->setLastTargetUser( _JabberGloox_DEFAULT_TargetUser );
	//info->setLastTargetUser( _JabberGloox_DEFAULT_TargetUser );
	Log(LOG_VERBOSE, ":checkCreateAccount_() : TARGET User of %s set to %s", info->getUser().c_str(), info->getLastTargetUser().c_str() );

	return info;
 }

WRESULT JabberGlooxBundle::doContactsSynch( bool add, bool remove )
 {
	// get the list WOSH users who have a valid jabber-account (non-empty)
	std::string userAccountKey = Properties.getValueOf(_JabberGloox_KEY_UserAccountKey).toString();
	std::vector<std::string> users_wosh = UserManager::findUserByProperty( userAccountKey, Variant() );
	Log(LOG_VERBOSE, ":doContactsSynch(%i,%i) : %i users found {%s}", add, remove, users_wosh.size(), wosh::Utilities::joinVector(users_wosh, ";").c_str() );

	// it's better to create two lists (sub, unsub) and release the mutex
	// because of: contacts are updated (write) in callbacks, even if it shouldn't deadlock

	if ( remove ) {
		std::vector<std::string> unsub_accounts;
		// account->user not found ('JabberAccount' was removed/cleaned?)
		// 

		this->contacts.transactionBeginRead();
		std::vector<std::string>::const_iterator it_wu;
		tStrContactInfoMap::ConstIterator it;
		const tStrContactInfoMap::ConstIterator it_end = this->contacts.end();
		for ( it=this->contacts.begin(); it!=it_end; ++it ) {
			std::vector<std::string> users = UserManager::findUserByProperty( userAccountKey, Variant(it->first) );
			if ( users_wosh.empty() ) {
				// no users mapped to this account
				unsub_accounts.push_back(it->first);
				continue;
			 }
			if ( it->second == NULL ) continue;
			// one or more users mapped to this account.. make sure current is one of them
			bool found = false;
			for ( it_wu=users.begin(); it_wu!=users.end(); it_wu++ ) {
				if ( *it_wu == it->second->getUser() ) {
					found = true;
				 }
			 }
			if ( !found )
				unsub_accounts.push_back(it->first);
		 }
		this->contacts.transactionEnd();

		Log(LOG_INFO, ":doContactsSynch(%i,%i) : Adding %i users found {%s}", add, remove, unsub_accounts.size(), wosh::Utilities::joinVector(unsub_accounts, ";").c_str() );
		for ( it_wu=unsub_accounts.begin(); it_wu!=unsub_accounts.end(); it_wu++ ) {
			this->jabbWorker->unSubscribe( *it_wu, "WOSH JabberGloox [Synchonization]" );
		 }
	 }

	if ( add ) {
		std::vector<std::string> sub_accounts;

		this->contacts.transactionBeginRead();
		std::vector<std::string>::const_iterator it;
		for ( it=users_wosh.begin(); it!=users_wosh.end(); it++ ) {
			string userAccount = UserManager::getUserProperty( *it, userAccountKey).toString();
			if ( userAccount.empty() ) {
				Log(LOG_WARNING, ":doContactsSynch(%i,%i) : Empty key for User %s. Skipped.", add, remove, (*it).c_str() );
				continue;
			 }
			ContactInfo* cInfo = this->contacts.find(userAccount);
			if ( cInfo != NULL && cInfo->getUser() == *it )
				continue; // already synched

			sub_accounts.push_back(userAccount);
		 }
		this->contacts.transactionEnd();

		Log(LOG_INFO, ":doContactsSynch(%i,%i) : Adding %i users found {%s}", add, remove, sub_accounts.size(), wosh::Utilities::joinVector(sub_accounts, ";").c_str() );
		for ( it=sub_accounts.begin(); it!=sub_accounts.end(); it++ ) {
			this->jabbWorker->subscribe( *it, "WOSH JabberGloox [Synchonization]" );
		 }
	 }

	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////// INTERNAL COMMANDS

bool JabberGlooxBundle::parseInternalCommand( const string& message, ContactInfo* info )
 {
	if ( message.find("%") != 0 )
		return false;

	if ( message.find("%INFO") == 0 ) {
		Log(LOG_VERBOSE, ":parseInternalCommand() : Internal CMD by %s [%s]", message.c_str(), info->getUser().c_str() );
		string summary = "JabberGloox Informations:\n";
		summary += "ACCOUNT: " + info->getUserID() + "\n";
		summary += "WOSH-USER: " + info->getUser() + "\n";
		summary += "\n";
		summary += "TARGET: " + info->getLastTargetUser() + "\n";
		summary += "LANG: " + info->getLanguage() + "\n";
		summary += "PROTO: " + Utilities::toString<int>(info->getProtocol()) + "[" + Notification::getProtocolAsString(info->getProtocol()) + "]\n";
		this->jabbWorker->sendMessage( info->getUserID(), summary );
		return true;
	 }
	else if ( message.find("%HELP") == 0 ) {
		Log(LOG_VERBOSE, ":parseInternalCommand() : Internal CMD by %s [%s]", message.c_str(), info->getUser().c_str() );
		string summary = "JabberGloox Commands:\n";
		summary += "%INFO : print summary of current session/settings\n";
		summary += "%HELP : show this list\n";
		summary += "%TARGET <USER> : set target user to <USER> [default: wosh]\n";
		summary += "%LANG <LANG> : set language to <LANG> [default: en_GB]\n";
		summary += "%PROTO <INT> : set protocol to <INT> [default 1(SHELL)]\n";
		this->jabbWorker->sendMessage( info->getUserID(), summary );
		return true;
	 }
	else if ( message.find("%TARGET") == 0 ) {
		Log(LOG_VERBOSE, ":parseInternalCommand() : Internal CMD by %s [%s]", message.c_str(), info->getUser().c_str() );
		string new_target = message.substr(7);
		Utilities::trimSpaces(new_target);
		info->setLastTargetUser(new_target);
		if ( !UserManager::isUser(new_target) ) {
			Log(LOG_WARNING, ":parseInternalCommand() : Target Recipent of %s set to '%s' (User Unknown!?)", info->getUser().c_str(), info->getLastTargetUser().c_str() );
			this->jabbWorker->sendMessage( info->getUserID(), "JabberGloox: TargetUser set to '" + info->getLastTargetUser() + "' (but user was not found)" );
				 }
		else {
			Log(LOG_INFO, ":parseInternalCommand() : Target Recipent of %s set to '%s'", info->getUser().c_str(), info->getLastTargetUser().c_str() );
			this->jabbWorker->sendMessage( info->getUserID(), "JabberGloox: TargetUser set to '" + info->getLastTargetUser() + "'" );
		 }
		this->jabbWorker->setStatusMessage(new_target);
		return true;
	 }
	else if ( message.find("%LANG") == 0 ) {
		Log(LOG_VERBOSE, ":parseInternalCommand() : Internal CMD by %s [%s]", message.c_str(), info->getUser().c_str() );
		string new_language = message.substr(5);
		Utilities::trimSpaces(new_language);
		info->setLanguage( new_language );
		Log(LOG_INFO, ":parseInternalCommand() :Language of %s set to %s", info->getUser().c_str(), info->getLanguage().c_str() );
		this->jabbWorker->sendMessage( info->getUserID(), "JabberGloox: Language set to " + info->getLanguage() );
		return true;
	 }
	else if ( message.find("%PROTO") == 0 ) {
		Log(LOG_VERBOSE, ":parseInternalCommand() : Internal CMD by %s [%s]", message.c_str(), info->getUser().c_str() );
		string new_proto = message.substr(6);
		Utilities::trimSpaces(new_proto);
		int inp = Utilities::fromString<int>(new_proto);
		if ( inp == 0 ) {
			Log(LOG_CRITICAL, ":parseInternalCommand() : INVALID value of %PROTO [%s]", message.c_str() );
			return true;
		 }
		info->setProtocol( (Notification::PROTOCOL)inp );
		Log(LOG_INFO, ":parseInternalCommand() :Protocol of %s set to %i", info->getUser().c_str(), (int)info->getProtocol() );
		this->jabbWorker->sendMessage( info->getUserID(), "JabberGloox: Protocol set to " + Notification::getProtocolAsString(info->getProtocol()) );
		return true;
	 }

	return false;
 }

/////////////////////////////////////////////////////////////////////////////////////////// INTERNAL COMMANDS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////// GTALKBOT EVENTS

void JabberGlooxBundle::jabbbot_Message( const string& accountname, const string& message, JabberGlooxImpl* )
 {
	 this->contacts.transactionBeginWrite();
	ContactInfo* info = this->contacts.find(accountname);
	if ( info == NULL ) {
		info = checkCreateAccount_( accountname );
		if ( info == NULL ) {
			this->contacts.transactionEnd();
			return;
		}
	 }
	// update contacts' timestamp 
	info->setLastSeen();

	Log(LOG_VERBOSE, ":jabbbot_Message() : Recognized %s as %s", accountname.c_str(), info->getUser().c_str() );

	if ( message.find("%") == 0 ) {
		bool exit = parseInternalCommand(message, info);
		if ( exit ) {
			this->contacts.transactionEnd();
			return;
		 }
	 }

	TextMessage* txtmsg = new TextMessage();
	txtmsg->setSender_ID( accountname );
	txtmsg->setSender_User( info->getUser() );
	txtmsg->setRecipent_ID( getAccountNameFromUsername(info->getLastTargetUser()) );
	txtmsg->setRecipent_User( info->getLastTargetUser() );
	txtmsg->setProtocol( info->getProtocol() );
	txtmsg->setLanguage( info->getLanguage() );
	txtmsg->setPlain( message );
	this->contacts.transactionEnd();

	// inject TEXT_MESSAGE into WOSH
	raiseEvent( txtmsg, SecurityManager::generateToken(txtmsg->getSender_User()) );
 }

void JabberGlooxBundle::jabbbot_UserStatus( const string& accountname, int status, JabberGlooxImpl* )
 {
	if ( this->jabbWorker == NULL ) return;
	if ( accountname == this->jabbWorker->getAccountName() ) {
		Log(LOG_VERBOSE, ":jabbbot_UserStatus(%s) : I'm online, hopefully i knew already :)", accountname.c_str() );
		return;
	 }

	this->contacts.transactionBeginWrite();
	ContactInfo* info = this->contacts.find(accountname);
	if ( info == NULL ) {
		info = checkCreateAccount_( accountname );
		if ( info == NULL ) {
			this->contacts.transactionEnd();
			return;
		}
	 }

	string username = info->getUser();
	Fact* fact = NULL;

	// update contacts' timestamp 
	if ( status == 0 ) {
		fact = new Fact( _CommunicatorService_EVENT_UserOffline, new Variant(username) );
		info->setLastSeen( -Utilities::std_time() );
		Log(LOG_INFO, ":jabbbot_UserStatus() : User %s is offline (as %s)", username.c_str(), accountname.c_str() );
	 }
	else if ( status == 1 ) {
		fact = new Fact( _CommunicatorService_EVENT_UserOnline, new Variant(username) );
		info->setLastSeen();
		Log(LOG_INFO, ":jabbbot_UserStatus() : User %s is online (as %s)", username.c_str(), accountname.c_str() );
	 }
	else {
		info->setLastSeen( 0 );
		Log(LOG_WARNING, ":jabbbot_UserStatus() : Ignored unhandled status %i of %s [%s]", status, username.c_str(), accountname.c_str() );
	 }
	this->contacts.transactionEnd();

	if ( fact == NULL )
		return;
	// notify user presence
	Message* msg_event = new Message( fact );
	msg_event->setSource(this);
	msg_event->setDestinationBroadcast(true);
	msg_event->setSecurityToken( SecurityManager::generateToken(this->getPermission().getOwner()) );
	WoshKernel::getInstance()->busses().postMessage( msg_event, _Bus_Security );
 }

short JabberGlooxBundle::jabbbot_SubscriptionChanged( const std::string& accountname, int status, JabberGlooxImpl* )
 {
	if ( this->jabbWorker == NULL ) return -1;
	if ( accountname == this->jabbWorker->getAccountName() ) {
		// umh?
		return 0;
	 }

	if ( status == _jabbbot_Subscribe && !Properties.getValueOf(_JabberGloox_KEY_AcceptSubscription).toBoolean(true) ) {
		Log(LOG_INFO, ":jabbbot_SubscriptionChanged(%i) : Ignoring Subscription of User '%s'", status, accountname.c_str() );
		return -1;
	 }
	if ( status == _jabbbot_UnSubscribe && !Properties.getValueOf(_JabberGloox_KEY_AcceptUnSubscription).toBoolean(true) ) {
		Log(LOG_INFO, ":jabbbot_SubscriptionChanged(%i) : Ignoring UnSubscription of User '%s'", status, accountname.c_str() );
		return -1;
	 }

	this->contacts.transactionBeginWrite();
	ContactInfo* info = this->contacts.find(accountname);
	if ( info == NULL ) {
		if ( status == _jabbbot_UnSubscribed )
			return 1; // then don't add it

		info = checkCreateAccount_( accountname );
		if ( info == NULL ) {
			this->contacts.transactionEnd();
			return 0;
		}
	 }
	else {
		if ( status == _jabbbot_UnSubscribed ) { // then remove from contact list
			this->contacts.erase(accountname, true);
			return 1;
		 }
	 }
	Log(LOG_INFO, ":jabbbot_SubscriptionChanged(%i) : Accepting subscription of User '%s' [%s]", status, info->getUser().c_str(), accountname.c_str() );
	this->contacts.transactionEnd();
	return 1;
 }

///////////////////////////////////////////////////////////////////////////////////////////// GTALKBOT EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::string JabberGlooxBundle::getUsernameFromAccountName( const std::string& accountname )
 {
	std::string userAccountKey = Properties.getValueOf(_JabberGloox_KEY_UserAccountKey).toString();
	std::vector<std::string> users = UserManager::findUserByProperty( userAccountKey, Variant(accountname) );
	if ( users.size() == 0 ) {
		Log(LOG_WARNING, ":getUsernameFromAccountName() : User '%s' not recognized! [Property '%s' missing?]", accountname.c_str(), userAccountKey.c_str() );
		return "";
	 }
	else if ( users.size() > 1 ) {
		Log(LOG_WARNING, ":getUsernameFromAccountName() : User '%s' not recognized!", accountname.c_str() );
		return "";
	 }
	return users[0];
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::string JabberGlooxBundle::getAccountNameFromUsername( const std::string& username )
 {
	// retrieve the jabber account mapped to the user, first check if user exists
	bool found = UserManager::isUser(username);
	if ( !found ) {
		Log(LOG_WARNING, ":getAccountNameFromUsername() : wosh-User '%s' not recognized! [not found]", username.c_str() );
		return "";
	 }
	// get the KEY (name) (of Users' properties) where the account name is stored, default is 'JabberAccount'
	std::string userAccountKey = Properties.getValueOf(_JabberGloox_KEY_UserAccountKey).toString();
	// get the 'JabberAccount' property of the target user, it should store his own account name, such as john.doe@gmail.com
	return UserManager::getUserProperty(username, userAccountKey).toString();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////// COMMUNICATOR INTERFACE

bool JabberGlooxBundle::isUserReachable( const std::string& username )
 {
	std::string accountname = getAccountNameFromUsername(username);
	if ( accountname == "" )
		return false;

	MutexLockerRead mL(this->contacts.getMutex());
	ContactInfo* info = this->contacts.find(accountname);
	if ( info == NULL )
		return false;

	// negative means he is offline from abs(.)
	if ( info->isOffline() )
		return false;

	return true;
 }

bool JabberGlooxBundle::isUserRegistered( const string& username )
 {
	std::string accountname = getAccountNameFromUsername(username);
	if ( accountname == "" )
		return false;

	if ( !this->contacts.exists(accountname) )
		return false;
	return true;
 }

long JabberGlooxBundle::getUserLastSeen( const string& username )
 {
	std::string accountname = getAccountNameFromUsername(username);
	if ( accountname == "" )
		return 0;

	MutexLockerRead mL(this->contacts.getMutex());
	ContactInfo* info = this->contacts.find(accountname);
	if ( info == NULL )
		return 0;

	// negative means he is offline from abs(.)
	if ( info->isOffline() )
		return -info->getLastSeen();
	return info->getLastSeen();
 }

WRESULT JabberGlooxBundle::sendMessageToUser( const string& userWosh, const string& message )
 {
	std::string accountname = getAccountNameFromUsername(userWosh);
	if ( accountname == "" ) {
		SystemError::raise(this, Log, SystemError::CRITICAL, 0, "sendMessageToUser", "FAILED mapping user %s", userWosh.c_str() );
		return WRET_ERR_PARAM;
	 }
	return sendMessageTo( accountname, message );
 }

WRESULT JabberGlooxBundle::sendMessageTo( const string& userID, const string& message )
 {
	if ( userID == "" ) return WRET_ERR_PARAM;

	bool sent = this->jabbWorker->sendMessage( userID, message );
	if ( !sent ) {
		SystemError::raise(this, Log, SystemError::CRITICAL, 0, "sendMessageTo", "[%s] failed sending message", userID.c_str() );
		return WRET_ERR_INTERNAL;
	 }
	return WRET_OK;
 }

WRESULT JabberGlooxBundle::notify( const Notification* notification )
 {
	if ( notification == NULL ) return WRET_ERR_PARAM;
	///@todo validate notification (textual, !empty)
	// if has custom account, dont map!
	if ( notification->isForAnyone() || notification->getRecipent_User().size() == 0 ) {
		Log(LOG_WARNING, "notify(%i) to ANYONE", notification->getID(), notification->getRecipent_User().c_str() );
		WRESULT ret = WRET_OK;
		this->contacts.transactionBeginRead();
		tStrContactInfoMap::ConstIterator it;
		const tStrContactInfoMap::ConstIterator it_end = this->contacts.end();
		for ( it=this->contacts.begin(); it!=it_end; ++it ) {
			ret += sendMessageTo( it->first, notification->getPlain() );
		 }
		this->contacts.transactionEnd();
		return ret;
	 }
	Log(LOG_INFO, "notify(%i) to %s", notification->getID(), notification->getRecipent_User().c_str() );
	return sendMessageToUser( notification->getRecipent_User(), notification->getPlain() );
 }

////////////////////////////////////////////////////////////////////////////////////// COMMUNICATOR INTERFACE
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS

Response* JabberGlooxBundle::mmDoContactAdd( const Request* request )
 {
	if ( request == NULL ) return NULL;
	Variant account_name;
	Variant opt_message;
	request->extractArguments(account_name, opt_message );
	if ( this->jabbWorker == NULL )
		return request->replyResponse(WRET_ERR_ILLEGAL_USE, "Bundle is not running!");
	if ( account_name.isStringEmpty() )
		return request->replyResponse(WRET_ERR_ILLEGAL_USE, "invalid contact");
	Log(LOG_VERBOSE, "mmDoContactAdd(%s, %s)", account_name.asString().c_str(), opt_message.toString().c_str() );

	WRESULT ret = this->jabbWorker->subscribe( account_name.asString(), opt_message.toString() );
	if ( WSUCCEEDED(ret) )
		return request->replyResponse(WRET_OK, "Subscription request sent");
	return request->replyResponse(ret, "Subscription request error.");
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* JabberGlooxBundle::mmDoContactRemove( const Request* request )
 {
	if ( request == NULL ) return NULL;
	Variant account_name;
	Variant opt_message;
	request->extractArguments(account_name, opt_message );
	if ( this->jabbWorker == NULL )
		return request->replyResponse(WRET_ERR_ILLEGAL_USE, "Bundle is not running!");
	if ( account_name.isStringEmpty() )
		return request->replyResponse(WRET_ERR_ILLEGAL_USE, "invalid contact");
	Log(LOG_VERBOSE, "mmDoContactRemove(%s, %s)", account_name.asString().c_str(), opt_message.toString().c_str() );
	WRESULT ret = this->jabbWorker->unSubscribe( account_name.asString(), opt_message.toString() );
	if ( WSUCCEEDED(ret) )
		return request->replyResponse(WRET_OK, "UnSubscription request sent");
	return request->replyResponse(ret, "UnSubscription request error");
 }

Response* JabberGlooxBundle::mmDoContactsSynch( const Request* request )
 {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, "mmDoContactsSynch(%s)", option.c_str() );

	bool add = true;
	bool remove = true;
	if ( option == "add" ) remove = false;
	if ( option == "remove" ) add = false;
	WRESULT ret = doContactsSynch( add, remove );
	if ( WSUCCEEDED(ret) )
		return request->replyResponse(WRET_OK, "(Un)Subscription request(s) sent");
	return request->replyResponse(ret, "(Un)Subscription request error");
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* JabberGlooxBundle::mmDoListContacts( const Request* request )
 {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, "mmDoListContacts(%s)", option.c_str() );

	if ( option == "s" || option == "short" ) {
		this->contacts.transactionBeginRead();
		NEW_List* objs = new NEW_List(this->contacts.size());
		tStrContactInfoMap::ConstIterator it;
		const tStrContactInfoMap::ConstIterator it_end = this->contacts.end();
		for ( it=this->contacts.begin(); it!=it_end; ++it ) {
			if ( it->second == NULL ) continue;
			if ( it->second->getUser().empty() ) continue;
			objs->push_back<Variant>( it->second->getUser() );
		 }
		this->contacts.transactionEnd();
		return request->replyResponse(WRET_OK, objs);
	 }
	else {
		this->contacts.transactionBeginRead();
		Table* tbl = new Table( this->contacts.size(), 5, true );
		tbl->setTableName("Contacts");
		tbl->setHeaderHorizontal("Name", 0);
		tbl->setHeaderHorizontal("AccountName", 1);
		tbl->setHeaderHorizontal("TimeStamp", 2);
		tbl->setHeaderHorizontal("Protocol", 3);
		tbl->setHeaderHorizontal("Language", 4);
		unsigned int iRow = 0;
		tStrContactInfoMap::ConstIterator it;
		const tStrContactInfoMap::ConstIterator it_end = this->contacts.end();
		for ( it=this->contacts.begin(); it!=it_end; ++it ) {
			if ( it->second == NULL ) continue;
			tbl->set<Variant>( it->second->getUser(), iRow, 0);
			tbl->set<Variant>( it->first, iRow, 1);
			tbl->set( new Variant(it->second->getLastSeen(), Variant::DATETIME), iRow, 2);
			tbl->set<Variant>( (int)it->second->getProtocol(), iRow, 3);
			tbl->set<Variant>( it->second->getLanguage(), iRow, 4);
			++iRow;
		 }
		this->contacts.transactionEnd();
		return request->replyResponse(WRET_OK, tbl);
	}
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* JabberGlooxBundle::mmDoSendMessageToUser( const Message* requestMessage )
 {
	if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
	if ( !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();
	if ( !request->hasData() ) {
		return request->replyResponse(WRET_ERR_PARAM, "Invalid data");
	 }

	std::string username = "";
	std::string message = "";
	if ( request->getData()->isClass(_NEW_List_TYPE) ) {
		const NEW_List* dl = dynamic_cast<const NEW_List*>(request->getData());
		if ( dl->size() < 2 || dl->itemAt(0) == NULL ) {
			return request->replyResponse(WRET_ERR_PARAM, "Invalid arguments [username, message]");
		 }
		username = dl->itemAt(0)->toString();
		// BUG DEPRECATED BUG BUG
		//dl->join(message, " ", 1);
	 }

	std::string sender = "";
	if ( requestMessage->getSecurityToken() != NULL )
		sender = requestMessage->getSecurityToken()->getUser();

	Log(LOG_INFO, "mmDoSendMessageToUser(%s) : %i chars", username.c_str(), message.size() );

	if ( sender != username && sender != "wosh ")
		message += " [by " + sender + "]";

	WRESULT ret = sendMessageToUser( username, message );
	if ( WSUCCEEDED(ret) )
		return request->replyResponse(ret, "Sent!");
	return request->replyResponse(ret, "Internal Error!");
 }

///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////


 }; // namespace devices
}; // namespace wosh
