/**
 * @class   IPlayerGStreamerListener
 * @brief   Interface for PlayerGStreamer callbacks (events).
 *
 * @see     PlayerGStreamer
 ****************************************************************************
 * @version $Id: IPlayerGStreamerListener.h 1664 2010-01-23 17:57:05Z alex $
 * @author  Alessandro Polo
 * @date    Released on 2009-09-28
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_DEVICES_IPlayerGStreamerListener_H__
 #define __WOSH_DEVICES_IPlayerGStreamerListener_H__

 #include <framework/entertainment/MediaTrack.h>
 
using namespace std;
using namespace wosh;
using namespace wosh::entertainment;

class PlayerGStreamer;

class IPlayerGStreamerListener  {

	public:

		/**
		 * @brief   Raised when play-state has changed.
		 * @param [in] state playing status
		 * @param [in] source the source (instance) of the event
		 */
		virtual void changed_PlayState( int state, PlayerGStreamer* source ) = 0;

		/**
		 * @brief   Raised when play-state has changed.
		 * @param [in] volume volume in range [0:100]
		 * @param [in] muted muted flag
		 * @param [in] source source (instance) of the event
		 */
		virtual void changed_Volume( int volume, bool muted, PlayerGStreamer* source ) = 0;

		/**
		 * @brief   Raised when playlist has changed.
		 * @param [in] list the media track being played (uri) of the media
		 * @param [in] source the source (instance) of the event
		 */
		virtual void changed_PlayList( const wosh::entertainment::MediaList* list, PlayerGStreamer* source ) = 0;

		/**
		 * @brief   Raised when current playing file has changed.
		 * @param [in] track the media track being played (uri) of the media
		 * @param [in] source the source (instance) of the event
		 */
		virtual void changed_PlayItem( const wosh::entertainment::MediaTrack* track, PlayerGStreamer* source ) = 0;


	public:
		virtual ~IPlayerGStreamerListener() { }

}; // class def


#endif //__WOSH_DEVICES_IPlayerGStreamerListener_H__
