/**
 * @class   wosh::devices::TestDeviceBundle
 * @brief   
 *
 * @ingroup Devices
 * @see     wosh::devices::TestDevice
 ****************************************************************************
 * @version $Id: TestDeviceBundle.h 2074 2010-03-10 02:38:16Z alex $
 * @author  Alessandro Polo
 * @date    Released on 2009-06-04
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Bundles_Devices_TestDevice_H__
 #define __WOSH_Bundles_Devices_TestDevice_H__

 #include <core/BundleGeneric.h>
 #include <core/Thread.h>

 #include "ITestDeviceListener.h"
 
 #include <interfaces/Device.h>
 #include <interfaces/devices/TestDevice.h>

class TestDeviceImpl;

using namespace std;
namespace wosh {
 namespace devices {

 #define _TestDevice_NAME						"TestDevice"
 #define _TestDevice_VERSION					0.01


class TestDeviceBundle :	public BundleGeneric,
							public virtual wosh::interfaces::Device,
							public virtual ITestDeviceListener
 {
	WOSH_BUNDLE(wosh::devices::TestDeviceBundle)
	WOSH_BUNDLE_INTERFACE("wosh::devices::Device")

	public:

		/**
		 * @brief   Empty constructor, init bundle and vars, Properties and Methods.
		 */
		TestDeviceBundle();

		/**
		 * @brief   Empty destructor, may (synch) stop bundle and internal threads when still running
		 */
		virtual ~TestDeviceBundle();


/** @name Bundle Interface
 * @{
 ******************************************************************************/
	public:

		/**
		 * @brief   Start the internal thread (used for refreshing stats). Also register bundle as Device
		 * @see    BundleGeneric::bundleStart()
		 */
		virtual WRESULT bundleStart();

		/**
		 * @brief   Stop the internal thread (used for refreshing stats). Also unregister bundle as Device
		 * @see    BundleGeneric::bundleStop()
		 */
		virtual WRESULT bundleStop();

		virtual WRESULT bundleDiagnostic();

		virtual void busMessage( const Message& message, const Bus* source );

//@}
/** @name Device Interface
 * @{
 ******************************************************************************/
	public:

		virtual bool isDeviceLinked() const					{ return (this->devState == wosh::interfaces::Device::LINKED); }
		virtual bool isDeviceUnLinked() const				{ return (this->devState == wosh::interfaces::Device::UNLINKED); }
		virtual bool isDeviceBroken() const					{ return (this->devState == wosh::interfaces::Device::BROKEN); }

		virtual wosh::interfaces::Device::DEVICE_STATE getDeviceState() const
															{ return this->devState; }

//@}
/** @name TestDevice Interface
 * @{
 ******************************************************************************/
	public:

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:

//@}
/** @name Modem Events
 * @{
 ******************************************************************************/
 	public:

//@}
/** @name IPropertiesProviderListener interface
 * @{
 ******************************************************************************/
 	public:
		virtual bool updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source );
//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:
		Response* mmDoForceState( const Request* request );
		Response* mmDoRaiseEvent( const Request* request );

		Response* mmDoWaitMethod( const Message* requestMessage );


		void raiseEvent( MessageContent* content, SecurityToken* security = NULL );

//@}

	protected:

		BusConnector BusDevices;					///< register myself & work as Device too

		TestDeviceImpl* devWorker;					///< real implementation


		wosh::interfaces::Device::DEVICE_STATE devState;		///< current state of device


}; // class def

 }; // namespace devices
}; // namespace wosh

#endif //__WOSH_Bundles_Devices_TestDevice_H__
