/**
 * @class   TestDeviceImpl
 * @brief   
 ****************************************************************************
 * @version $Id: TestDeviceImpl.h 2025 2010-03-03 19:47:55Z alex $
 * @author  Alessandro Polo
 * @date    Released on 2009-06-04
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_DEVICES_TestDevice_H__
 #define __WOSH_DEVICES_TestDevice_H__

 #include <woshDefs.h>
 #include <core/Mutex.h>
 #include <core/BundleGenericWorker.h>

 #include "ITestDeviceListener.h"


class TestDeviceImpl : public wosh::BundleGenericWorker  {

	public:

		/**
		 * @brief   Default constructor. Init vars.
		 */
		TestDeviceImpl( wosh::BundleGeneric& bundle );

		/**
		 * @brief   Deconstructor. If thread is stll running, it will be stopped.
		 */
		virtual ~TestDeviceImpl();


/** @name Setters
 * @{
 ******************************************************************************/
	public:

		/**
		 * @brief   Set the ITestDeviceListener interface for callbacks.
		 * @param listener [in] pointer to object which implements callback-functions
		 * @note    Call this method only once, after creation, it's not reetrant!
		 */
		void setTestDeviceListener( ITestDeviceListener* listener );

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:

		/**
		 * @brief   Get ITestDeviceListener interface class for callbacks.
		 * @return  pointer to object which implements callback-functions
		 */
		ITestDeviceListener* getTestDeviceListener();

//@}
/** @name Modem Interface
 * @{
 ******************************************************************************/
	public:

//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:

		/**
		 * @brief   Perform periodic checkConnected() to update stats.
		 * @see     running
		 * @see     autoRefreshFrequency
		 */
		virtual void runThread();


//@}

	protected:

		ITestDeviceListener* deviceListener;	///< callbacks for stats (such as TestDeviceBundle*)
		wosh::MutexRW configMux;				///< Mutex of configuration (previous vars)


}; // class def

#endif //__WOSH_DEVICES_TestDevice_H__
