/** @file    AimlQBotImpl.cpp
 *  @author  Alessandro Polo
 *  @version $Id: AimlQBotImpl.cpp 1708 2010-01-26 22:28:01Z alex $
 *  @brief
 * File containing methods for the AimlQBotImpl class.
 * The header for this class can be found in AimlQBotImpl.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "AimlQBotImpl.h"

 #include "3rdparty/aimlparser.h"

 #include <QFile>
 #include <QDir>
 #include <QTextStream>


using namespace std;
namespace wosh {
 namespace services {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

AimlQBotImpl::AimlQBotImpl()
 {
	this->parser = new AIMLParser();


 }


AimlQBotImpl::~AimlQBotImpl()
 {
	delete this->parser; this->parser = null;
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

string AimlQBotImpl::evaluate( const string& message )
 {
	log->log( Logger::VERBOSE, " worker:evaluate(%s)", message.c_str() );

	QString result = this->parser->getResponse( QString::fromStdString(message) );
	return result.toStdString();
 }

////////////////////////////////////////////////////////////////////////////////////////////////// RUN THREAD
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT AimlQBotImpl::loadFolderSet( const string& folder_path )
 {
	QString dirname = QString::fromStdString(folder_path);
	QDir dir(dirname);
	QStringList files = dir.entryList( "*.aiml" );

	log->log( Logger::VERBOSE, " worker:loadFolderSet(%s) : %i files found", folder_path.c_str(), files.count() );

	bool loaded = false;
	uint i = 0;
	for ( QStringList::Iterator it = files.begin(); it != files.end(); ++it ) {
		loaded = this->parser->loadAiml(dirname + *it);
		if ( loaded )
			log->log( Logger::INFO, " worker:runThread(): Loaded %s.", (*it).toAscii().data() );
		else
			log->log( Logger::WARNING, " worker:runThread(): FAILED Loading %s.", (*it).toAscii().data() );
		i++;
	}

	this->parser->displayTree();

	return WRET_OK;
 }

WRESULT AimlQBotImpl::loadVars( const string& folder_path )
 {
	log->log( Logger::VERBOSE, " worker:loadVars(%s)", folder_path.c_str() );
	bool loaded = false;
	QString basedir = QString::fromStdString(folder_path);
	loaded = this->parser->loadVars( basedir + "utils/vars.xml", false );
	loaded = this->parser->loadVars( basedir + "utils/bot.xml", true );

	if ( loaded ) return WRET_OK;

	log->log( Logger::CRITICAL, " worker:loadVars(%s) : Failed Loading '%s'", folder_path.c_str(), "utils/vars.xml" );
	return WRET_ERR_INTERNAL;

	return WRET_OK;
 }

WRESULT AimlQBotImpl::saveVars( const string& folder_path )
 {
	log->log( Logger::VERBOSE, " worker:saveVars(%s)", folder_path.c_str() );

	QString basedir = QString::fromStdString(folder_path);
	bool loaded = this->parser->saveVars( basedir + "utils/vars.xml" );

	if ( loaded ) return WRET_OK;

	log->log( Logger::CRITICAL, " worker:saveVars(%s) : Failed Saving '%s'", folder_path.c_str(), "utils/vars.xml" );
	return WRET_ERR_INTERNAL;
 }

    
	

WRESULT AimlQBotImpl::loadSubstitutions( const string& folder_path )
 {
	log->log( Logger::VERBOSE, " worker:loadSubstitutions(%s)", folder_path.c_str() );

	QString basedir = QString::fromStdString(folder_path);
	bool loaded = this->parser->loadSubstitutions( basedir + "utils/substitutions.xml" );

	if ( loaded ) return WRET_OK;

	log->log( Logger::CRITICAL, " worker:loadSubstitutions(%s) : Failed Loading '%s'", folder_path.c_str(), "utils/vars.xml" );
	return WRET_ERR_INTERNAL;

 }



//////////////////////////////////////////////////////////////////////////////////////////////////////////////



 }; // namespace services
}; // namespace wosh
