/**
 * @class   CommunicationManagerParser
 * @brief   
 *
 *
 *
 *
 *
 ****************************************************************************
 * @version $Id: CommunicationManagerParser.h 2057 2010-03-08 21:32:15Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Services_CommunicationManagerParser_H__
 #define __WOSH_Services_CommunicationManagerParser_H__

 #include <core/BundleGenericWorker.h>
 #include <core/Communicator.h>
 #include <core/Interpreter.h>

 #include <interfaces/services/CommunicatorService.h>
 #include <interfaces/services/InterpreterService.h>

 #include "ICommunicationManagerListener.h"


using namespace wosh;


class CommunicationManagerParser : public BundleGenericWorker  {

	public:

		/**
		 * @brief   Default constructor. Init vars.
		 */
		CommunicationManagerParser( BundleGeneric& bundle, tStrCommunicatorMap& communicators, tStrInterpreterMap& interpreters );

		/**
		 * @brief   Deconstructor. If thread is stll running, it will be stopped.
		 */
		virtual ~CommunicationManagerParser();


	public:

		void removeByKernel( const std::string& kernelName );

		WRESULT updateInterpreters();
		WRESULT updateCommunicators();

		WRESULT updateObjects( const std::string& kernel, const std::string& type );


		void evalMessage( const Message& message );


/** @name Setters
 * @{
 ******************************************************************************/
	public:
		void setTimeOut( long seconds )			{ this->timeoutEval = seconds; }


//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:
		long getTimeOut() const					{ return this->timeoutEval; }

//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:

		void removeCommunicator_( Communicator* comm );
		void removeInterpreter_( Interpreter* inter );

		void evalTimeOut();

		/**
		 * @brief   
		 * @see     running
		 * @see     
		 */
		virtual void runThread();

//@}
	protected:

		tStrCommunicatorMap& comms;	///< communicators
		tStrInterpreterMap& transl;	///< interpreters

		long timeoutEval;

}; // class def


#endif //__WOSH_Services_CommunicationManagerParser_H__
