/**
 * @class   wosh::services::DesktopNotifierBundle
 * @brief   
 *
 *
 *
 *
 *
 *
 *
 * @ingroup Services
 ****************************************************************************
 * @version $Id: DesktopNotifierBundle.h 2074 2010-03-10 02:38:16Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Bundles_Services_DesktopNotifier_H__
 #define __WOSH_Bundles_Services_DesktopNotifier_H__

 #include <core/BundleGeneric.h>
 #include <core/Containers.h>

 #include <framework/communication/ContactInfo.h>
 #include <framework/communication/TextMessage.h>

 #include <core/Communicator.h>
 #include <interfaces/Device.h>
 #include <interfaces/services/CommunicatorService.h>
 #include <interfaces/services/DesktopNotifier.h>

 #include "IDesktopNotifierListener.h"

class QSystemTrayIcon;
class DesktopNotifierQObject;

using namespace wosh::communication;
namespace wosh {
 namespace services {

 #define _DesktopNotifier_NAME				"DesktopNotifier"
 #define _DesktopNotifier_VERSION			0.54


class DesktopNotifierBundle :	public BundleGeneric,
								public virtual wosh::interfaces::services::CommunicatorService,
								public virtual wosh::interfaces::services::DesktopNotifier,
								public virtual IDesktopNotifierListener
 {
	WOSH_BUNDLE(wosh::services::DesktopNotifierBundle)
	WOSH_BUNDLE_INTERFACE("wosh::interfaces::devices::DesktopNotifier")


	typedef List22<Notification*> tNotificationList;


	public:

		DesktopNotifierBundle();

		virtual ~DesktopNotifierBundle();


		virtual wosh::interfaces::Device::DEVICE_STATE getDeviceState() const
															{ return this->devState; }

/** @name Bundle Interface
 * @{
 ******************************************************************************/
	public:
		virtual WRESULT bundleStart();
		virtual WRESULT bundleStop();

		virtual void busMessage( const Message& message, const Bus* source );

		virtual bool updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source );

//@}
/** @name IDesktopNotifierListener Interface
 * @{
 ******************************************************************************/
	public:
		virtual void user_MouseNotMoved();
		virtual void user_MouseMoved();
		virtual void user_TrayIconClicked( short mode );
		virtual void user_TrayMessageClicked();

//@}
/** @name DesktopNotifier Interface
 * @{
 ******************************************************************************/
	public:

//virtual WRESULT setParentWidget( QWidget* parent ) { this->parentWidget = parent;  return WRET_OK;}

virtual WRESULT setTrayIcon( QSystemTrayIcon* tray_icon );

WRESULT updateUserLogged( bool online, bool raiseEvent = true );


//@}
/** @name Communicator Interface
 * @{
 ******************************************************************************/
	public:

		virtual bool isUserReachable( const std::string& wosh_username );

		virtual WRESULT sendMessageToUser( const std::string& wosh_username, const std::string& message );

		virtual WRESULT sendMessageTo( const std::string& custom_username, const std::string& message );

		virtual WRESULT notify( const Notification* notification );

//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:
		Response* mmDoForceUser( const Message* requestMessage );
		Response* mmDoSendMessageToUser( const Message* requestMessage );
		Response* mmDoListContacts( const Request* request );
		Response* mmDoListNotifications( const Message* requestMessage );

//@}
	protected:
		std::string getUsernameFromAccountName( const std::string& accountname );

		WRESULT showMessage( const std::string& message );

		void raiseEvent( MessageContent* content, SecurityToken* security );


	protected:

		BusConnector BusDevices;							///< register myself & work as Device too

		ContactInfo account;

		DesktopNotifierQObject* trayObject;

		tNotificationList notifications;
		int notificationsMaxSize;
		int notificationCurrent;

		wosh::interfaces::Device::DEVICE_STATE devState;	///< current state of device


}; // class def

 }; // namespace services
}; // namespace wosh

#endif //__WOSH_Bundles_Services_DesktopNotifier_H__
