/**
 * @class   wosh::services::OrganizerBundle
 * @brief   
 *
 * Manage ToDo, Appointments notes.
 * Mostly designed for Todo List, originally conceived to record bugs and ideas
 *
 *
 * Full Multiplatform
 *
 *
 * @ingroup Services
 ****************************************************************************
 * @version $Id: OrganizerBundle.h 1949 2010-02-25 00:30:00Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Bundles_Services_Organizer_H__
 #define __WOSH_Bundles_Services_Organizer_H__

 #include <core/BundleGeneric.h>
 #include <core/Thread.h>

 #include <interfaces/Service.h>
 #include <interfaces/services/Organizer.h>

 #include <interfaces/services/CommunicatorService.h>

 #include "IOrganizerListener.h"
 #include "OrganizerToDo.h"

class OrganizerImpl;

using namespace std;
namespace wosh {
 namespace services {

 #define _Organizer_VERSION				"0.2"

 #define _Organizer_KEY_DB				"DataBaseFile"
 #define _Organizer_KEY_DB_shadow		"DataBaseShadowFile"
 

class OrganizerBundle :	public BundleGeneric,
						public virtual wosh::interfaces::services::Organizer,
						public virtual IOrganizerListener
 {
	WOSH_BUNDLE(wosh::services::OrganizerBundle)
	WOSH_BUNDLE_INTERFACE("wosh::services::Organizer")

	public:

		/**
		 * @brief   Empty constructor, init bundle and vars, Properties and Methods.
		 */
		OrganizerBundle();

		/**
		 * @brief   Empty destructor, may (synch) stop bundle and internal threads when still running
		 */
		virtual ~OrganizerBundle();


/** @name Bundle Interface
 * @{
 ******************************************************************************/
	public:

		/**
		 * @brief   Start the internal thread (used for refreshing stats). Also register bundle as Device
		 * @see    BundleGeneric::bundleStart()
		 */
		virtual WRESULT bundleStart();

		/**
		 * @brief   Stop the internal thread (used for refreshing stats). Also unregister bundle as Device
		 * @see    BundleGeneric::bundleStop()
		 */
		virtual WRESULT bundleStop();

		/**
		 * @brief   Intercept Notifications
		 * @see    BundleGeneric::busMessage()
		 */
		virtual void busMessage( const Message& message, const Bus* source );

//@}
/** @name Organizer Interface
 * @{
 ******************************************************************************/
	public:

		virtual WRESULT load();
		virtual WRESULT save();

		virtual WRESULT loadToDos();
		virtual WRESULT saveToDos();


		virtual void organizer_raise( OrganizerToDo* todo, OrganizerImpl* source );

//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:

		Response* mmDoListToDos( const Request* request );

		Response* mmDoNewToDo( const Message* requestMessage );

		Response* mmDoNotifyToDo( const Message* requestMessage );

		Message* busMessageTodo( const Message& message, OrganizerToDo* job );//no mutex


//@}
	protected:

		OrganizerImpl* orgWorker;				///< real implementation


}; // class def

 }; // namespace services
}; // namespace wosh

#endif //__WOSH_Bundles_Services_Organizer_H__
