/** @file    OrganizerImpl.cpp
 *  @author  Alessandro Polo
 *  @version $Id: OrganizerImpl.cpp 2082 2010-03-11 06:26:42Z alex $
 *  @brief
 * File containing methods for the OrganizerImpl class.
 * The header for this class can be found in OrganizerImpl.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "OrganizerImpl.h"

 #include <stdlib.h>
 #include <time.h>

using namespace std;
using namespace wosh;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

OrganizerImpl::OrganizerImpl( BundleGeneric& bundle ) : BundleGenericWorker(bundle)
 {
	this->orgListener = NULL;


 }


OrganizerImpl::~OrganizerImpl() {
	Log(LOG_VERBOSE, ":~OrganizerImpl(): Destroying.." );

	if ( isThreadRunning() ) {
		quitThread();
	 }

	if ( this->todos.size() != 0 ) {
		Log(LOG_VERBOSE, ":~CronImpl() : Freeing todos .." );
		this->todos.free();
	 }

 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////// RUN THREAD

void OrganizerImpl::runThread()
 {
	Log(LOG_VERBOSE, " worker:runThread(): Configuring XMPP client.." );

	time_t time_now;
	struct tm *p;
	while( this->running )
	 {
		Thread::sleepForMSec(1000);
		setThreadAlive();

		if ( this->todos.size() == 0 )
			continue;

		time(&time_now);
		p = localtime(&time_now);

		this->todos.mutex().lockForWrite();

		std::vector<OrganizerToDo*>::iterator it;
		std::vector<OrganizerToDo*>::iterator it_end = this->todos.stdEnd();
		for ( it=this->todos.stdBegin(); it!=it_end; it++ ) {
			if ( *it == NULL ) continue;
			OrganizerToDo* todo = (*it);

			this->orgListener->organizer_raise(todo, this);
		 }

		this->todos.mutex().unLock();



	 }

	Log(LOG_VERBOSE, " worker:runThread(): Exited loop.." );
 }

////////////////////////////////////////////////////////////////////////////////////////////////// RUN THREAD
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT OrganizerImpl::registerToDo( OrganizerToDo* todo )
 {
	if ( todo == NULL ) return WRET_ERR_PARAM;
	MutexLockerWrite mL(&this->todos.mutex());
	//todo->id = Utilities::randomLong();
	this->todos.add_(todo);
	return WRET_OK;
 }

WRESULT OrganizerImpl::unRegisterToDo( OrganizerToDo* todo )
 {
	if ( todo == NULL ) return WRET_ERR_PARAM;
	MutexLockerWrite mL(&this->todos.mutex());
	this->todos.remove_(todo);
	return WRET_OK;
 }


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
