/**
 * @class   wosh::services::Organizer
 * @brief   Service providing Organizer features
 *

 *
 ****************************************************************************
 * @version $Id: OrganizerImpl.h 1708 2010-01-26 22:28:01Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_SERVICES_Organizer_H__
 #define __WOSH_SERVICES_Organizer_H__

 #include <core/BundleGenericWorker.h>

 #include "IOrganizerListener.h"
 #include "OrganizerToDo.h"

using namespace std;
using namespace wosh;

class OrganizerImpl : public BundleGenericWorker  {

	public:

		/**
		 * @brief   Default constructor. Init vars.
		 */
		OrganizerImpl( BundleGeneric& bundle );

		/**
		 * @brief   Deconstructor. If thread is stll running, it will be stopped.
		 */
		virtual ~OrganizerImpl();


	public:


		WRESULT registerToDo( OrganizerToDo* todo );
		WRESULT unRegisterToDo( OrganizerToDo* todo );


//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:


		/**
		 * @brief   Set the IOrganizerListener interface for callbacks.
		 * @param listener [in] pointer to object which implements callback-functions
		 * @note    Call this method only once, after creation, it's not reetrant!
		 */
		void setOrganizerListener( IOrganizerListener* listener ) {
			this->orgListener = listener;
		 }

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:

		List<OrganizerToDo*>& getToDos()								{ return this->todos; }


		/**
		 * @brief   Get getOrganizerListener interface class for callbacks.
		 * @return  pointer to object which implements callback-functions
		 */
		inline IOrganizerListener* getOrganizerListener()			{ return this->orgListener; }

//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:

		/**
		 * @brief   Listen/Write loop, managed by gloox::Client 
		 * @see     running
		 * @see     xmppJclient
		 */
		virtual void runThread();

//@}
	protected:

		IOrganizerListener* orgListener;	///< callbacks for stats (such as OrganizerBundle*)

		List<OrganizerToDo*> todos;
	//	List<OrganizerSchedule*> schedules;


}; // class def

#endif //__WOSH_SERVICES_Organizer_H__
