/** @file    OrganizerPublisher.cpp
 *  @author  Alessandro Polo
 *  @version $Id: OrganizerPublisher.cpp 1950 2010-02-25 00:40:58Z alex $
 *  @brief
 * File containing methods for the OrganizerPublisher class.
 * The header for this class can be found in OrganizerPublisher.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include "OrganizerPublisher.h"

 #include <core/Message.h>
 #include <core/Utilities.h>
 #include <string.h>


using namespace std;
namespace wosh {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

OrganizerPublisher::OrganizerPublisher( ) : PropertiesHandler(), Object()
 {
	clear();
 }

OrganizerPublisher::OrganizerPublisher( const OrganizerPublisher& m  ) : PropertiesHandler(), Object()
 {
	clear();
	*this = m;
 }

OrganizerPublisher::~OrganizerPublisher()
 {

 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////// CONFIGURATION

void OrganizerPublisher::clear()
 {
	this->id = 0;
	this->username = "";

	this->subject = "";
	this->message = "";
	this->priority = 0;

	this->createdTimestamp = 0;
	this->readenOnTimestamp = 0;
 }

void OrganizerPublisher::setCreatedTimestamp() {
	 this->createdTimestamp = Utilities::std_time();
 }

void OrganizerPublisher::setReadenOnTimestamp() {
	 this->readenOnTimestamp = Utilities::std_time();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

OrganizerPublisher& OrganizerPublisher::operator=(const OrganizerPublisher& m)
 {
	if (this == &m) return *this; // same object?

	this->id = m.id;
	this->username = m.username;

	this->subject = m.subject;
	this->message = m.message;
	this->priority = m.priority;

	this->createdTimestamp = m.createdTimestamp;
	this->readenOnTimestamp = m.readenOnTimestamp;

	return *this;
 }
 
 
bool OrganizerPublisher::operator==(const OrganizerPublisher &other) const
 {
	if ( this->id != other.id )
		return false;

	return true;
 }

//////////////////////////////////////////////////////////////////////////////////////////////// DEBUG & INFO
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////// SERIALIZATION

std::string OrganizerPublisher::toString() const
 {
	return "";
 }

/////////////////////////////////////////////////////////////////////////////////////////////// SERIALIZATION
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

int OrganizerPublisher::getKeys( std::vector<std::string>& keys ) const
 {
	keys.push_back( _KEY_name );
	keys.push_back( _KEY_type );

//	keys.push_back( _OrganizerPublisher_KEY_id );

	keys.push_back( _OrganizerPublisher_KEY_Subject );
	keys.push_back( _OrganizerPublisher_KEY_Message );
	keys.push_back( _OrganizerPublisher_KEY_CreatedOn );
	keys.push_back( _OrganizerPublisher_KEY_ReadenOn );


	return keys.size();
 }

Data OrganizerPublisher::getProperty(const std::string& key) const
 {
	if ( key == _OrganizerPublisher_KEY_id )					return Data( (int)this->id );
	else if ( key == _KEY_name )						return Data( this->getName() );
	else if ( key == _KEY_type )						return Data( this->getClassName() );

	else if ( key == _OrganizerPublisher_KEY_Subject )		return Data( this->subject );
	else if ( key == _OrganizerPublisher_KEY_Message )		return Data( this->message );

	else if ( key == _OrganizerPublisher_KEY_CreatedOn )		return Data( (int)this->createdTimestamp );
	else if ( key == _OrganizerPublisher_KEY_ReadenOn )		return Data( (int)this->readenOnTimestamp );

	return Data();
 }
 

WRESULT OrganizerPublisher::setProperty(const std::string& key, const Data& value)
 {
	if ( key == _OrganizerPublisher_KEY_id ) {
		return WRET_ERR_ILLEGAL_USE;
	 }
	else if ( key == _KEY_name ) {
		return WRET_ERR_ILLEGAL_USE;
	 }


	else if ( key == _OrganizerPublisher_KEY_CreatedOn ) {
//		if ( !value.isInteger() ) return WRET_ERR_PARAM;
		value.get( this->createdTimestamp );
		return WRET_OK;
	 }
	else if ( key == _OrganizerPublisher_KEY_ReadenOn ) {
//		if ( !value.isInteger() ) return WRET_ERR_PARAM;
		value.get( this->readenOnTimestamp );
		return WRET_OK;
	 }

	else if ( key == _OrganizerPublisher_KEY_Subject ) {
//		if ( !value.isInteger() ) return WRET_ERR_PARAM;
		this->subject = value.getData();
		return WRET_OK;
	 }
	else if ( key == _OrganizerPublisher_KEY_Message ) {
//		if ( !value.isInteger() ) return WRET_ERR_PARAM;
		this->message = value.getData();
		return WRET_OK;
	 }

	return WRET_ERR_UNKNOWN;
 }
 



};
