/**
 * @class   wosh::OrganizerSchedule
 * @brief   
 *

 *
 *
 * @version	$Id: OrganizerSchedule.h 2037 2010-03-05 15:04:52Z alex $
 * @author	Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_OrganizerSchedule_H__
 #define __WOSH_OrganizerSchedule_H__

 #include <core/Object.h>
 #include <core/Serializable.h>
 #include <core/PropertiesHandler.h>

 #include <core/Variant.h>
 #include <core/MessageQueue.h>

 #include <vector>
 

using namespace std;
namespace wosh {

 #define _OrganizerSchedule_TYPE				"wosh::OrganizerSchedule"

// #define _Object_KEY_name				"name"
// #define _Object_KEY_type				"type"


class OrganizerSchedule_S_XML;


class OrganizerSchedule : public Object, public virtual PropertiesHandler, public virtual ISerializable  {

	public:

		OrganizerSchedule();

		OrganizerSchedule( const OrganizerSchedule& m );

		virtual ~OrganizerSchedule();

		virtual OrganizerSchedule* clone() const				{ return new OrganizerSchedule(*this); }



		OrganizerSchedule& operator=(const OrganizerSchedule& m);

		bool operator==(const OrganizerSchedule &other) const;
		
//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:
		void setDate( int year, int month, int day );
		void setTime( int hour, int min );

		void setYear( int value )					{ this->year = value; }
		void setMonth( int value )					{ this->month = value; }
		void setDay( int value )					{ this->day = value; }
		void setHour( int value )					{ this->hour = value; }
		void setMin( int value )					{ this->min = value; }

		void setEnabled( bool value )				{ this->enabled = value; }

		void setLastExecutedTime();

		void setID( long uid )						{ this->id = uid; }


		// mapping with real fields
		virtual WRESULT setProperty( const std::string& key, const Data& value );


//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:

		std::string getDate() const;
		std::string getTime() const;

		virtual long getID() const					{ return this->id; }

		bool isEnabled() const						{ return this->enabled; }

		long getLastExecutedTime() const			{ return this->lastExecutedTime; }

		int getYear() const							{ return this->year; }
		int getMonth() const						{ return this->month; }
		int getDay() const							{ return this->day; }
		int getHour() const							{ return this->hour; }
		int getMin() const							{ return this->min; }

		bool isYearly() const						{ return (this->year == -1); }
		bool isMonthly() const						{ return (this->month == -1); }
		bool isDaily() const						{ return (this->day == -1); }
		bool isHourly() const						{ return (this->hour == -1); }
		bool isMinutely() const						{ return (this->min == -1); }


		const std::vector<Message*>& getEffects() 	{ return this->effects; }

		std::string getBrief() const;
		
		// mapping with real fields
		virtual Data getProperty( const std::string& key ) const;

		virtual int getKeys( std::vector<std::string>& keys ) const;

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string getClassName() const	{ return _OrganizerSchedule_TYPE; }

		virtual std::string toString() const;

//@}


	protected:

		long id;					///< unique ID of 

		bool enabled;

//		MessageQueue effects;
		std::vector<Message*> effects;

		int year; 					///< 2009->
		int month;					///< 0->11
		int day; 					///< 1-31
		int hour; 					///< 0-23
		int min; 					///< 0-59
		int sec; 					///< 0-59

		long lastExecutedTime;

		friend class OrganizerSchedule_S_XML;

		virtual void clear();

}; // class def

}; // namespace wosh

#endif //__WOSH_OrganizerSchedule_H__
