/**
 * @class   wosh::OrganizerToDo
 * @brief   
 *

 *
 *
 * @version	$Id: OrganizerToDo.h 2037 2010-03-05 15:04:52Z alex $
 * @author	Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_OrganizerToDo_H__
 #define __WOSH_OrganizerToDo_H__

 #include <core/Object.h>
 #include <core/Serialization.h>


using namespace std;
namespace wosh {

 #define _OrganizerToDo_TYPE				"wosh::OrganizerToDo"

 #define _OrganizerToDo_KEY_id				"id"

 #define _OrganizerToDo_KEY_Subject			"Subject"
 #define _OrganizerToDo_KEY_Message			"Message"
 #define _OrganizerToDo_KEY_CreatedOn		"CreatedOn"
 #define _OrganizerToDo_KEY_ReadenOn		"ReadenOn"
 #define _OrganizerToDo_KEY_				""

 
class OrganizerToDo_S_XML;


class OrganizerToDo : public Object
 {
	WOSH_SERIALIZABLE(wosh::OrganizerToDo)
	WOSH_COMPARE_RTTI(wosh::OrganizerToDo)


	public:

		OrganizerToDo();
		OrganizerToDo( const OrganizerToDo& m );

		virtual ~OrganizerToDo();


	public:

		OrganizerToDo& operator=(const OrganizerToDo& m);

		bool operator==(const OrganizerToDo &other) const;

//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:

		void setUsername( const std::string& value )	{ this->username = value; }

		void setSubject( const std::string& value )		{ this->subject = value; }

		void setMessage( const std::string& value )		{ this->message = value; }

		void setCreatedTimestamp();
		void setCreatedTimestamp( long epoch )			{ this->createdTimestamp = epoch; }

		void setReadenOnTimestamp();
		void setReadenOnTimestamp( long epoch )			{ this->readenOnTimestamp = epoch; }


//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:

		const std::string& getUsername() const			{ return this->username; }

		const std::string& getSubject() const			{ return this->subject; }

		const std::string& getMessage() const			{ return this->message; }

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const;
		virtual WRESULT store( IEncoderWriter* serializer ) const;
		virtual WRESULT retrieve( IEncoderReader* deserializer );

//@}
	protected:

		virtual void clear();

	protected:

		long id;					///< unique ID of 

		std::string username;

		std::string subject;
		std::string message;

		long priority;

		long createdTimestamp;

		long readenOnTimestamp;

	//	std::string sourceInfo;


		friend class OrganizerToDo_S_XML;

}; // class def

}; // namespace wosh

#endif //__WOSH_OrganizerToDo_H__
