/**
 * @class   wosh::services::SessionServerBundle
 * @brief   Service providing SessionServer features
 *
 *
 *
 *
 *
 *
 *
 *
 *
 * @ingroup Services
 ****************************************************************************
 * @version $Id: SessionServerBundle.h 2129 2010-03-14 18:15:59Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Bundles_Services_SessionServer_H__
 #define __WOSH_Bundles_Services_SessionServer_H__

 #include <core/BundleGeneric.h>

 #include <interfaces/Service.h>
 #include <interfaces/services/SessionServer.h>
 #include <interfaces/services/CommunicatorService.h>

 #include <framework/communication/Session.h>
 #include <framework/communication/SessionShell.h>

class SessionServerImpl;

using namespace wosh::communication;
namespace wosh {
 namespace services {

 #define _SessionServer_VERSION						0.21

 #define _SessionServer_KEY_AutoAllocSessionIM		"AutoAllocSessionIM"


class SessionServerBundle : public BundleGeneric,
							public virtual wosh::interfaces::services::SessionServer,
							public virtual wosh::interfaces::services::CommunicatorService
 {
	WOSH_BUNDLE(wosh::services::SessionServerBundle)
	WOSH_BUNDLE_INTERFACE("wosh::interfaces::services::SessionServer")

	public:

		SessionServerBundle();

		virtual ~SessionServerBundle();

//		virtual Response* processRequest( const Request* action );


	public:

		virtual WRESULT bundleStart();
		virtual WRESULT bundleStop();

		virtual bool updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source );

		virtual void busMessage( const Message& message, const Bus* source );

/** @name SessionServer Interface
 * @{
 ******************************************************************************/
	public:

		long createLocalShellSession( const std::string& username, bool skipLogin = true, bool autoStart = true );

		// deprecated
		static WRESULT registerSession( Session* session, bool start_session = true );
		// deprecated
		static WRESULT unRegisterSession( Session* session );


	public:
		virtual WRESULT registerProtocol( SessionProtocol* session_proto );
		virtual WRESULT unRegisterProtocol( SessionProtocol* session_proto );

	protected:
		virtual bool isUserLogged( const std::string& username ) const;

		virtual long getSessionIDByUser( const std::string& username ) const;

//@}
/** @name SessionServer Interface for Session callbacks
 * @{
 ******************************************************************************/
	public:

		virtual bool session_is_auth( wosh::communication::Session* session, const std::string& user, const std::string& pwd );

		virtual bool session_post_message( wosh::communication::Session* session, Message* message );

		virtual WRESULT changeSessionProtocol( wosh::communication::Session* session, const std::string& protocol_name );

//@}
/** @name Communicator Interface
 * @{
 ******************************************************************************/
	public:
		virtual bool isUserReachable( const std::string& username );

		virtual bool isUserRegistered( const std::string& username );

		virtual long getUserLastSeen( const std::string& username );

		virtual WRESULT sendMessageToUser( const std::string& userWosh, const std::string& message );

		virtual WRESULT sendMessageTo( const std::string& userID, const std::string& message );

		virtual WRESULT notify( const Notification* notification );

//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:
		Response* mmDoGetSessions( const Message* requestMessage );
//@}

		virtual WRESULT notify( const Notification* notification, const Message& message );

	public:

		///@bug to be removed
		static SessionServerBundle* getInstance();

	protected:

		SessionServerImpl* ssrvWorker;

		static SessionServerBundle* ms_SessionServer_instance;

		bool autoAllocSessionIM;


}; // class def

 }; // namespace services
}; // namespace wosh

#endif //__WOSH_Bundles_Services_SessionServer_H__
