/**
 * @class   SessionServerImpl
 * @brief   Service providing SessionServer features
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 ****************************************************************************
 * @version $Id: SessionServerImpl.h 2067 2010-03-09 23:40:30Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Services_SessionServer_Impl_H__
 #define __WOSH_Services_SessionServer_Impl_H__

 #include <core/Object.h>

 #include <core/BusConnector.h>
 #include <core/LogContext.h>

 #include <framework/communication/Session.h>
 #include <framework/communication/SessionProtocol.h>
 #include <framework/communication/ShellCommand.h>
 #include <interfaces/services/SessionServer.h>


using namespace wosh;
using namespace wosh::communication;

typedef List22<wosh::communication::Session*> tSessionList;

class SessionServerImpl :  public virtual wosh::interfaces::services::SessionServer
 {

	typedef List22<wosh::communication::SessionProtocol*> tSessionProtocolList;


	public:

		SessionServerImpl( BusConnector& connector, LogContext& logger, Object* owner );

		virtual ~SessionServerImpl();

	public:


		const Object* getOwner() const					{ return this->ownerObj; }


	public:

		virtual Session* getSessionByID_( long id );
		virtual Session* getSessionByUser_( const std::string& username );

		virtual long getSessionIDByUser( const std::string& username ) const;

		virtual WRESULT registerSession_( Session* session );

		virtual WRESULT unRegisterSession_( Session* session );


		const tSessionList& getSessions() const 		{ return this->sessions; }
		tSessionList& getSessions()						{ return this->sessions; }


		virtual bool isUserLogged( const std::string& username ) const { return (getSessionIDByUser(username) != 0); }




	public:


		void forwardMessage_( const Message& message, long id );

		virtual WRESULT registerProtocol( SessionProtocol* session_proto );
		virtual WRESULT unRegisterProtocol( SessionProtocol* session_proto );

	public:

		bool session_is_auth( Session* session, const std::string& user, const std::string& pwd );

		bool session_post_message( Session* session, Message* message );

		virtual WRESULT changeSessionProtocol( Session* session, const std::string& protocol_name );

		friend class Session;

	protected:
		tSessionList sessions;
		tSessionProtocolList protocols;

		SessionProtocol* protocolDefault;

		BusConnector& BusCore;
		LogContext& Log;
		const Object* ownerObj;


}; // class def

#endif //__WOSH_Services_SessionServer_Impl_H__
