/** @file    SystemHost_SerializationXML.cpp
 *  @author  Alessandro Polo
 *  @version $Id: SystemHost_SerializationXML.cpp 1968 2010-02-27 05:52:38Z alex $
 *  @brief
 * File containing methods for Serialization.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "SystemHost_SerializationXML.h"

 #include <framework/database/CoderXML_Core.h>


using namespace std;
namespace wosh {
 namespace database {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT EncoderXML_template__SystemHost::serializeTo( const ISerializable* object_s, QDomDocument& doc, QDomElement& contNode, bool compact )
 {
	if ( object_s == NULL ) return WRET_ERR_PARAM;
	const wosh::SystemHost* object = dynamic_cast<const wosh::SystemHost*>(object_s);
	WRESULT ret = WRET_OK;

	if ( compact ) {
		ret += EncoderXML_template__Object().serializeTo( object, doc, contNode, true );
	 }
	else {
		QDomElement objectNode = doc.createElement("object");
		contNode.appendChild(objectNode);
		ret += EncoderXML_template__Object().serializeTo( object, doc, objectNode, false );
	 }


	if ( compact ) {
		contNode.setAttribute("mac", QString(object->getMAC().c_str()) );
		contNode.setAttribute("ip", QString(object->getIP().c_str()) );
		contNode.setAttribute("hostname", QString(object->getHostName().c_str()) );

	 }
	else {
		QDomElement Node_mac = doc.createElement("mac");
		Node_mac.appendChild( doc.createTextNode( QString(object->getMAC().c_str()) ) );
		contNode.appendChild(Node_mac);

		QDomElement Node_ip = doc.createElement("ip");
		Node_ip.appendChild( doc.createTextNode( QString(object->getIP().c_str()) ) );
		contNode.appendChild(Node_ip);

		QDomElement Node_hostname = doc.createElement("hostname");
		Node_hostname.appendChild( doc.createTextNode( QString(object->getHostName().c_str()) ) );
		contNode.appendChild(Node_hostname);

	 }

	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT EncoderXML_template__SystemHost::deserializeFrom( ISerializable* object_s, const QDomElement& contNode )
 {
 	if ( object_s == NULL ) return WRET_ERR_PARAM;
	wosh::SystemHost* object = dynamic_cast<wosh::SystemHost*>(object_s);
	WRESULT ret = WRET_OK;

	QDomElement objectNode = contNode.namedItem("object").toElement();
	if ( !objectNode.isNull() )
		ret += EncoderXML_template__Object().deserializeFrom( object, objectNode );
	else
		ret += EncoderXML_template__Object().deserializeFrom( object, contNode );

	QDomElement contNode_mac = contNode.namedItem("mac").toElement();
	if ( contNode_mac.isNull() )
		object->setMAC( string( contNode.attribute("mac").toLatin1().data() ) );
	else
		object->setMAC( string( contNode_mac.text().toLatin1().data())  );

	QDomElement contNode_ip = contNode.namedItem("ip").toElement();
	if ( contNode_ip.isNull() )
		object->setIP( string( contNode.attribute("ip").toLatin1().data() ) );
	else
		object->setIP( string( contNode_ip.text().toLatin1().data())  );

	QDomElement contNode_hostname = contNode.namedItem("hostname").toElement();
	if ( contNode_hostname.isNull() )
		object->setHostName( string( contNode.attribute("hostname").toLatin1().data() ) );
	else
		object->setHostName( string( contNode_hostname.text().toLatin1().data())  );


	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////





 }; // namespace database
}; // namespace wosh
