/** @file    wakeonlan.cpp
 *  @author  Alessandro Polo
 *  @version $Id: wakeonlan.cpp 2076 2010-03-10 04:19:17Z alex $
 *  @brief
 * File containing methods for the wosh::services::SystemServicesBundle class.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#include <woshDefs.h>
#include <framework/network/SocketUdp.h>
#include <core/ObjectFactory.h>

#include <core/Utilities.h>

using namespace std;
using namespace wosh;
using namespace wosh::network;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT do_wakeonlan( const string& address_port, const string& mac_address );

char* generateMagicPacketFor( const char* MAC );

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

const int MagicPacketSize = 102;

const char* MagicHeader = "\xFF\xFF\xFF\xFF\xFF\xFF"; //6 bytes
const short MagicHeaderSize = 6; //bytes

const short MagicMacRepeat = 16; // times (16*6bytes=96bytes)
const short MagicMacSize = 6; //bytes

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT do_wakeonlan( const string& address_port, const string& mac_address )
 {
	int size = 0;
	char* MAC = Utilities::string2hex(size, mac_address, 2, 1);
	char* data = generateMagicPacketFor(MAC);

	SocketUdp* socket = ObjectFactory::createTypeOf<wosh::network::SocketUdp>();
	if ( socket == NULL )
		return WRET_ERR_INTERNAL;
	int sent = socket->sendDatagram(address_port, data, MagicPacketSize);

	delete [] data;
	delete [] MAC;

	if ( sent == MagicPacketSize )
		return WRET_OK;

	return WRET_ERR_INTERNAL;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

char* generateMagicPacketFor( const char* MAC )
 {
	char* data = new char[MagicPacketSize];
	memset(data, 0, MagicPacketSize);

	memcpy(data, MagicHeader, MagicHeaderSize );

	if ( MAC == NULL )
		return data;

	for(short i=0; i<MagicMacRepeat; i++)
		memcpy(data+MagicHeaderSize+(i*MagicMacSize), MAC, MagicMacSize );

	return data;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

