/**
 * @class   wosh::services::TaskManagerBundle
 * @brief   
 *
 *
 *
 *
 * @ingroup Services
 ****************************************************************************
 * @version $Id: TaskManagerBundle.h 2051 2010-03-07 08:27:48Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Bundles_Services_TaskManager_H__
 #define __WOSH_Bundles_Services_TaskManager_H__

 #include <core/BundleGeneric.h>
 #include <core/Thread.h>

 #include <framework/automations/Automation.h>

 #include <interfaces/Service.h>
 #include <interfaces/services/TaskManager.h>
 
class TaskManagerImpl;

using namespace wosh::automations;
namespace wosh {
 namespace services {

 #define _TaskManager_VERSION				0.10


class TaskManagerBundle :	public BundleGeneric,
							public virtual wosh::interfaces::services::TaskManager
 {
	WOSH_BUNDLE(wosh::services::TaskManagerBundle)
	WOSH_BUNDLE_INTERFACE("wosh::interfaces::services::TaskManager")

	public:
		TaskManagerBundle();

		virtual ~TaskManagerBundle();

	public:
		virtual WRESULT bundleStart();
		virtual WRESULT bundleStop();

		virtual void busMessage( const Message& message, const Bus* source );

		virtual bool updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source );

	public:
		virtual WRESULT addAutomation( Automation* automation );

		virtual WRESULT loadAutomations();
		virtual WRESULT saveAutomations();

//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:
		Response* mmDoListTasks( const Request* request );
		Response* mmDoExecTask( const Request* request );

//@}
	protected:
		TaskManagerImpl* taskWorker;	///< real implementation


}; // class def

 }; // namespace services
}; // namespace wosh

#endif //__WOSH_Bundles_Services_TaskManager_H__
