/**
 * @class   wosh::services::TelnetServerBundle
 * @brief   Service providing TelnetServer features
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 * @ingroup Services
 ****************************************************************************
 * @version $Id: TelnetServerBundle.h 1947 2010-02-25 00:05:20Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Bundles_Services_TelnetServer_H__
 #define __WOSH_Bundles_Services_TelnetServer_H__

 #include <core/BundleGeneric.h>

 #include <interfaces/Service.h>
 #include <interfaces/services/TelnetServer.h>
 #include <interfaces/services/SessionServer.h>

 #include <framework/communication/Session.h>


class TelnetServerImpl;

using namespace std;
using namespace wosh::communication;
namespace wosh {
 namespace services {

 #define _TelnetServer_VERSION					"0.2"


class TelnetServerBundle :	public BundleGeneric,
							public virtual wosh::interfaces::services::SessionServer,
							public virtual wosh::interfaces::services::TelnetServer
 {
	WOSH_BUNDLE(wosh::services::TelnetServerBundle)
	WOSH_BUNDLE_INTERFACE("wosh::services::TelnetServer")

	public:

		TelnetServerBundle();

		virtual ~TelnetServerBundle();

//		virtual Response* processRequest( const Request* action );


		virtual WRESULT bundleStart();
		virtual WRESULT bundleStop();


	public:

		virtual void busMessage( const Message& message, const Bus* source );

/** @name TelnetServer Interface
 * @{
 ******************************************************************************/
	public:

//		virtual Session* newSession( const string& username );

	//	virtual Session* getSession( long id );


	//	virtual WRESULT registerSession( Session* session );
	//	virtual WRESULT unRegisterSession( Session* session );

		virtual bool isUserLogged( const std::string& username ) const;

		virtual long getSessionIDByUser( const std::string& username ) const;

		//deprecated
		virtual WRESULT startSession( Session* session );
		virtual WRESULT stopSession( Session* session );


		virtual WRESULT registerProtocol( SessionProtocol* session_proto );
		virtual WRESULT unRegisterProtocol( SessionProtocol* session_proto );

//@}
/** @name TelnetServer Interface for Session callbacks
 * @{
 ******************************************************************************/
	public:

		virtual bool session_is_auth( Session* session, const string& user, const string& pwd );

		virtual bool session_post_message( Session* session, Message* message );

		virtual WRESULT changeSessionProtocol( Session* session, const string& protocol_name );

//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:

		Response* mmDoListSessions( const Message* requestMessage );
		
//@}


	public:

		static TelnetServerBundle* getInstance();

	protected:

		TelnetServerImpl* ssrvWorker;

		static TelnetServerBundle* ms_TelnetServer_instance;


}; // class def

 }; // namespace services
}; // namespace wosh

#endif //__WOSH_Bundles_Services_TelnetServer_H__
