/** @file    TelnetServerImpl.cpp
 *  @author  Alessandro Polo
 *  @version $Id: TelnetServerImpl.cpp 2082 2010-03-11 06:26:42Z alex $
 *  @brief
 * File containing methods for the TelnetServerImpl class.
 * The header for this class can be found in TelnetServerImpl.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "TelnetServerImpl.h"

 #include <framework/communication/SessionProtocolShell.h>
 #include <core/Message.h>
 #include <core/Request.h>
 #include <core/UserManager.h>
 #include <core/SecurityManager.h>


using namespace std;
using namespace wosh;
//using namespace wosh::services;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

TelnetServerImpl::TelnetServerImpl( BundleGeneric& bundle ) : BundleGenericWorker(bundle)
 {
	this->protocolDefault = new SessionProtocolShell();
	this->protocolDefault->setSessionServer(this);
	this->protocols.push_back( this->protocolDefault );

	this->tcpServer = NULL;
 }


TelnetServerImpl::~TelnetServerImpl() {
	if ( Thread::isThreadRunning() ) {
		Thread::quitThread(10000);
	 }

	this->sessions.mutex().lockForWrite();

	std::vector<TelnetSession*>::const_iterator it;
	std::vector<TelnetSession*>::const_iterator it_end = this->sessions.stdEnd();
	for ( it=this->sessions.stdBegin(); it!=it_end; it++ ) {
		if ( *it == NULL ) continue;
		(*it)->sessionStop( true ); //synch
	 }

	this->sessions.mutex().unLock();

	if ( this->sessions.size() != 0 ) {
		Log(LOG_VERBOSE, ":~TelnetServerImpl() : Freeing Sessions.." );
		this->sessions.free();
	 }

	Log(LOG_VERBOSE, ":~TelnetServerImpl() : Freeing protocolDefault.." );

	delete this->protocolDefault; this->protocolDefault = NULL;

//	log->log( LOG_VERBOSE, ":~TelnetServerImpl() : Freeing Protocols.." );
//	std::vector<SessionProtocol*>::iterator it_pr;
//	for ( it_pr=this->protocols.begin(); it_pr!=this->protocols.end(); it_pr++ ) {
//		if ( *it_pr == NULL ) continue;
//		delete *it_pr;
//	 }

 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////// RUN THREAD

void TelnetServerImpl::runThread()
 {
	Log(LOG_VERBOSE, " worker:runThread(): Starting loop.." );

	this->tcpServer = new ServerTcp();
	this->tcpServer->setListener(this);

	bool set = this->tcpServer->setAddressPort( this->bindAddressPort );
	if ( !set ) {
		Log(LOG_CRITICAL, ":listen() FAILED setting bind address '%s'!", this->tcpServer->getAddressPort().c_str() );
		return;
	 }

	Log(LOG_VERBOSE, ":listen() Binding TCP server to '%s'..", this->tcpServer->getAddressPort().c_str() );

	WRESULT ret = this->tcpServer->bindSocket();
	if ( WFAILED(ret) ) {
		delete this->tcpServer; this->tcpServer = NULL;
		Log(LOG_CRITICAL, ":listen() FAILED setting bind address '%s'!", this->tcpServer->getAddressPort().c_str() );
		return;
	 }

	this->ready = true;
	exec();

	ret = this->tcpServer->unbindSocket();
	if ( WFAILED(ret) ) {
		this->ready = false;
		Log(LOG_CRITICAL, ":stopListening() FAILED Binding socket to %s [%i]", this->tcpServer->getAddressPort().c_str(), this->tcpServer->isListening() );
		return;
	 }

	this->tcpServer->setListener(NULL);
	delete this->tcpServer; this->tcpServer = NULL;

	if ( !this->ready ) {
		Log(LOG_CRITICAL, ":disconnect() Not Connected!" );
		return;
	 }

 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void TelnetServerImpl::serverTcp_incomingConnection( int socket_id, ServerTcp* )
 {
	(void)socket_id;
/*
	if ( connection == NULL ) {
		log->log( LOG_WARNING, ":serverTcp_incomingConnection() NULL incoming Socket!" );
		return;
	 }

	this->sessions.mutex().lockForWrite();

	TelnetSession* session = new TelnetSession((wosh::interfaces::services::SessionServer*)this->getOwner());
	session->setParentObject( this->getOwner() );
	session->getURI().registerGlobal();
	session->setSessionProtocol( this->protocolDefault );

	session->socket = connection;
	session->socket->setListener(session);

	this->sessions.add_(session);
	this->sessions.mutex().unLock();

	session->sessionStart();
*/
 }


////////////////////////////////////////////////////////////////////////////////////////////////// RUN THREAD


WRESULT TelnetServerImpl::registerSession( Session* session )
 {
	if ( session == NULL ) return WRET_ERR_PARAM;

	Log(LOG_INFO, ":*registerSession(%s)", session->getName().c_str() );
/*
	session->setSessionProtocol( this->protocolDefault );

	session->setParentObject( this->getOwner() );
	session->getURI().registerGlobal();

	this->sessions.mutex().lockForWrite();

	this->sessions.add_(session);

	this->sessions.mutex().unLock();
*/
	return WRET_OK;
 }

WRESULT TelnetServerImpl::unRegisterSession( Session* session )
 {
	if ( session == NULL ) return WRET_ERR_PARAM;
/*
	Log(LOG_INFO, ":*unRegisterSession(%s)", session->getName().c_str() );

	session->setSessionProtocol(NULL);

	this->sessions.mutex().lockForWrite();

	this->sessions.remove_(session);

	this->sessions.mutex().unLock();
*/
	return WRET_OK;
 }

Session* TelnetServerImpl::getSession_( long id )
 {
	std::vector<TelnetSession*>::const_iterator it;
	std::vector<TelnetSession*>::const_iterator it_end = this->sessions.stdEnd();
	for ( it=this->sessions.stdBegin(); it!=it_end; it++ ) {
		if ( *it == NULL ) continue;
		Session* currObj = *it;
		if ( currObj->getID() == id )
			return currObj;
	 }
	return NULL;
 }

long TelnetServerImpl::getSessionIDByUser( const std::string& username ) const
 {
	MutexLockerRead mL(&this->sessions.mutex());
	std::vector<TelnetSession*>::const_iterator it;
	std::vector<TelnetSession*>::const_iterator it_end = this->sessions.stdEnd();
	for ( it=this->sessions.stdBegin(); it!=it_end; it++ ) {
		if ( *it == NULL ) continue;
		if ( (*it)->getUsername() != username ) continue;
		return (*it)->getID();
	 }
	return 0;
 }

WRESULT TelnetServerImpl::changeSessionProtocol( Session* session, const string& protocol_name )
 {
	if ( session == NULL ) return WRET_ERR_PARAM;
	SessionProtocol* newProtocol = NULL;

	std::vector<SessionProtocol*>::iterator it;
	for ( it=this->protocols.begin(); it!=this->protocols.end(); it++ ) {
		if ( *it == NULL ) continue;
		if ( (*it)->getProtocolName() != protocol_name )
			continue;
		newProtocol = *it;
		break;
	 }

	if ( newProtocol == NULL ) {
		Log(LOG_WARNING, ":*changeSessionProtocol(): Protocol Not found '%s'", protocol_name.c_str() );
		return WRET_ERR_PARAM;
	 }
	Log(LOG_INFO, ":*changeSessionProtocol(): #%i Switching to Protocol '%s'", session->getID(), protocol_name.c_str() );

	session->setSessionProtocol( newProtocol );
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT TelnetServerImpl::registerProtocol( SessionProtocol* session_proto )
 {
	if ( session_proto == NULL ) return WRET_ERR_PARAM;

	std::vector<SessionProtocol*>::iterator it;
	for ( it=this->protocols.begin(); it!=this->protocols.end(); it++ ) {
		if ( *it == NULL ) continue;
		if ( *it == session_proto )
			return WRET_ERR_PARAM;
	 }
	Log(LOG_INFO, ":*registerProtocol(%s)", session_proto->getProtocolName().c_str() );
	this->protocols.push_back(session_proto);
	session_proto->setSessionServer(this);
	return WRET_OK;
 }

WRESULT TelnetServerImpl::unRegisterProtocol( SessionProtocol* session_proto )
 {
	if ( session_proto == NULL ) return WRET_ERR_PARAM;

	std::vector<SessionProtocol*>::iterator it;
	for ( it=this->protocols.begin(); it!=this->protocols.end(); it++ ) {
		if ( *it == NULL ) continue;
		if ( *it == session_proto ) {
			this->protocols.erase(it);
			Log(LOG_INFO, ":*unRegisterProtocol(%s)", session_proto->getProtocolName().c_str() );
			return WRET_OK;
		}
	 }
	return WRET_ERR_PARAM;
 }


//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool TelnetServerImpl::session_is_auth( Session* session, const string& user, const string& pwd )
 {
	if ( session == NULL ) return false;

	bool auth = UserManager::isAuthorized(user, pwd);

	return auth;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void TelnetServerImpl::forwardMessage_( const Message& message, long id )
 {
	std::vector<TelnetSession*>::const_iterator it;
	std::vector<TelnetSession*>::const_iterator it_end = this->sessions.stdEnd();
	for ( it=this->sessions.stdBegin(); it!=it_end; it++ ) {
		if ( *it == NULL ) continue;
		if ( id != 0 && id != (*it)->getID() ) continue;
		(*it)->appendMessage( message.clone() );
	 }
 }

bool TelnetServerImpl::session_post_message( Session* session, Message* message )
 {
	if ( session == NULL )
		return false;

	SecurityToken* msec = SecurityManager::generateToken( session->getUsername() );
	message->setSecurityToken(msec);

	BusCore.postMessage( message );
	return true;
 }


//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT TelnetServerImpl::setAddressPort( const string& address_port )
 {
	if ( Thread::isThreadRunning() ) {
		return WRET_ERR_ILLEGAL_USE;
	 }

	this->bindAddressPort = address_port;
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT TelnetServerImpl::getSessionsName( std::vector<std::string>& sessions_names ) const
 {
	MutexLockerRead mL(&this->sessions.mutex());
	std::vector<TelnetSession*>::const_iterator it;
	std::vector<TelnetSession*>::const_iterator it_end = this->sessions.stdEnd();
	for ( it=this->sessions.stdBegin(); it!=it_end; it++ ) {
		if ( *it == NULL ) continue;
		sessions_names.push_back( (*it)->getName() );
	 }
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT TelnetServerImpl::getSessionsUnixStyle( std::vector<std::string>& sessions_UnixStyle ) const
 {
	MutexLockerRead mL(&this->sessions.mutex());
	std::vector<TelnetSession*>::const_iterator it;
	std::vector<TelnetSession*>::const_iterator it_end = this->sessions.stdEnd();
	for ( it=this->sessions.stdBegin(); it!=it_end; it++ ) {
		if ( *it == NULL ) continue;
		const Session* session = *it;
		sessions_UnixStyle.push_back( session->getPermission().toUnixStyle() + "\t" + session->getUsername() );
	 }
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
