////////////////////////////////////////////////////////////////////////////////////////////////////////
// A simple C++ program calculating the sunrise and sunset for
// the current date and a set location (latitude,longitude)
// e-mail: jjlammi@yahoo.com
// C++ program calculating the sunrise and sunset for
// the current date and a constant location(latitude,longitude)
// Jarmo Lammi 1999 - 2004
// Last update
// Jan 3rd, 2004 - change to compile with gcc-3.3.2 by Stephan Wynhoff
// http://wynhoff.home.cern.ch/wynhoff/weather/gex.html
// It compiles well also with old gcc version 2.95.4 20011002 (Debian prerelease)
// Try compiling in Linux with command g++ -o rscalc rscalc.cc
// Also the time formats are tidy now.


 #include "sunstuff.h"

void dHour2TM(double dhr, struct tm& time_out)
 {
	int hr = (int)dhr;
	int mn = (int) ((dhr - (double) hr)*60);
	memset (&time_out,'\0', sizeof(struct tm) );
	time_out.tm_hour = hr;
	time_out.tm_min = mn;
 }

string TM2Time(const struct tm& tim)
 {
	char buffer [8];
	strftime (buffer,8,"%M:%S",&tim);
	return string(buffer);
 }


//   Get the days to J2000
//   h is UT in decimal hours
//   FNday only works between 1901 to 2099 - see Meeus chapter 7
double FNday (int y, int m, int d, float h)
 {
	long int luku = - 7 * (y + (m + 9)/12)/4 + 275*m/9 + d;
	// type casting necessary on PC DOS and TClite to avoid overflow
	luku+= (long int)y*367;
	return (double)luku - 730531.5 + h/24.0;
 }

//   the function below returns an angle in the range 0 to 2*pi
double FNrange (double x)
 {
    double b = x / (M_PI*2.0);
    double a = (M_PI*2.0) * (b - (long)(b));
    if (a < 0) a = (M_PI*2.0) + a;
    return a;
 }

// Calculating the hourangle
double f0(double lat, double declin)
 {
	double fo,dfo;
	// Correction: different sign at S HS
	dfo = M_RADS*(0.5*SUN_RADIUS_D + AIR_REFR); if (lat < 0.0) dfo = -dfo;
	fo = tan(declin + dfo) * tan(lat*M_RADS);
	if (fo>0.99999) fo=1.0; // to avoid overflow //
	fo = asin(fo) + M_PI/2.0;
	return fo;
 }

// Calculating the hourangle for twilight times
double f1(double lat, double declin)
 {
	double fi,df1;
	// Correction: different sign at S HS
	df1 = M_RADS * 6.0;
	if (lat < 0.0) df1 = -df1;
	fi = tan(declin + df1) * tan(lat*M_RADS);
	if (fi>0.99999) fi=1.0; // to avoid overflow //
	fi = asin(fi) + M_PI/2.0;
	return fi;
 }

//   Find the ecliptic longitude of the Sun
double FNsun (double d, double& L, double& g)
 {
	//   mean longitude of the Sun
	L = FNrange(280.461 * M_RADS + .9856474 * M_RADS * d);
	//   mean anomaly of the Sun
	g = FNrange(357.528 * M_RADS + .9856003 * M_RADS * d);
	//   Ecliptic longitude of the Sun
	return FNrange(L + 1.915 * M_RADS * sin(g) + .02 * M_RADS * sin(2 * g));
 }


void calculateSunTimes(
							const double& latitude, const double& longitude, const double& timezone,
							double& riset,
							double& settm,
							double& twam,
							double& twpm,
							double& daylen,
							double& noont,
							double& altmax,
							double& declin
							)
 {
	time_t sekunnit;
	struct tm *p;

	//  get the date and time from the user read system date and extract the year
	// First get time
	time(&sekunnit);
	// Next get localtime
	p=localtime(&sekunnit);

	double year = p->tm_year + 1900; // this is Y2K compliant method
	double month = p->tm_mon + 1;
	double day = p->tm_mday;
	double hour = 12;

	double L,g;

	double d = FNday((int)year, (int)month, (int)day, hour); //  Get the days to J2000

	double lambda = FNsun(d,L,g);	// Use FNsun to find the ecliptic longitude of the Sun
	double obliq = 23.439 * M_RADS - .0000004 * M_RADS * d;	// Obliquity of the ecliptic

	// Find the RA and DEC of the Sun
	double alpha = atan2(cos(obliq) * sin(lambda), cos(lambda));
	double delta = asin(sin(obliq) * sin(lambda));

	// Find the Equation of Time in minutes Correction suggested by David Smith
	double LL = L - alpha;
	if (L < M_PI)
		LL += M_PI*2.0;

	double equation = 1440.0 * (1.0 - LL / (M_PI*2.0));
	double ha = f0(latitude,delta);
	double hb = f1(latitude,delta);
	double twx = hb - ha; // length of twilight in radians
	twx = 12.0*twx/M_PI; // length of twilight in hours
	//cout << "ha=" << ha << "  hb=" << hb << endl;
	
	// Conversion of angle to hours and minutes
	daylen = M_DEGS*ha/7.5;
	if (daylen<0.0001)
		daylen = 0.0;// arctic winter

	riset = 12.0 - 12.0 * ha/M_PI + timezone - longitude/15.0 + equation/60.0;
	settm = 12.0 + 12.0 * ha/M_PI + timezone - longitude/15.0 + equation/60.0;
	noont = riset + 12.0 * ha/M_PI;
	altmax = 90.0 + delta * M_DEGS - latitude; 

	// Correction for S HS suggested by David Smith to express altitude as degrees from the N horizon
	if (latitude < delta * M_DEGS)
		altmax = 180.0 - altmax;

	twam = riset - twx; // morning twilight begin
	twpm = settm + twx; // evening twilight end

	if (riset > 24.0) riset-= 24.0;
	if (settm > 24.0) settm-= 24.0;

	declin = delta * (180.0/M_PI);
 }
