/**
 * @class   wosh::Map
 * @brief   
 *          
 *          
 ****************************************************************************
 * @version $Id: Map.h 1963 2010-02-26 09:25:16Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_Containers_H__
 #define __WOSH_Core_Containers_H__

 #include <woshDefs.h>
 #include <core/Reflection.h>
 #include <core/Mutex.h>

 #include <map>
 #include <vector>

 
namespace wosh {



template <typename T>
class List22
 {
	WOSH_CLASSNAME(wosh::List22)
	WOSH_CLASS_ABSTRACT(false)

	public:
		typedef typename std::vector<T>::const_iterator ConstIterator;
		typedef typename std::vector<T>::iterator Iterator;

	public:
		/**
		 * @brief  Default constructor.
		 */
		List22() { }

		List22( const List22<T>& m ) {
		 }

		virtual ~List22() {
		 }


/** @name reentrant methods
 * @{
 ******************************************************************************/
	public:
		inline int count() const {
			MutexLockerRead mL(&this->mux);
			return this->list.size();
		 }

		inline bool isEmpty() const {
			MutexLockerRead mL(&this->mux);
			return this->list.empty();
		 }

		virtual inline void clear() {
			MutexLockerWrite mL(&this->mux);
			free();
		 }

		inline void append( const T& item ) {
			MutexLockerRead mL(&this->mux);
			this->list.push_back(item);
		 }

//@}
/** @name non-reentrant (proxy for STL methods)
 * @{
 ******************************************************************************/
	public:
		inline bool exists( const T& item ) const {
			return ( indexOf(item) != -1 );
		 }

		inline int indexOf( const T& item ) const {
			int index = 0;
			ConstIterator it;
			const ConstIterator it_end = this->list.end();
			for ( it=this->list.begin(); it!=it_end; ++it, ++index ) {
				if ( *it == item ) return index;
			 }
			return -1;
		 }

		inline const T& itemAt( int index ) const {
			int iLoc = 0;
			ConstIterator it;
			const ConstIterator it_end = this->list.end();
			for ( it=this->list.begin(); it!=it_end; ++it, ++iLoc ) {
				if ( index == iLoc ) return *it;
			 }
			return NULL;
		 }
		inline T& itemAt( int index ) {
			int iLoc = 0;
			Iterator it;
			const ConstIterator it_end = this->list.end();
			for ( it=this->list.begin(); it!=it_end; ++it, ++iLoc ) {
				if ( index == iLoc ) return *it;
			 }
			return NULL;
		 }
		inline Iterator find( int index ) {
			int iLoc = 0;
			Iterator it;
			const ConstIterator it_end = this->list.end();
			for ( it=this->list.begin(); it!=it_end; ++it, ++iLoc ) {
				if ( iLoc != index ) continue;
				return it;
			 }
			return this->list.end();
		 }

		virtual void free() {
			this->list.clear();
		 }

		inline void push_back( const T& item, bool check_existence = false ) {
			if ( check_existence && exists(item) )
				return;
			this->list.push_back(item);
		 }

		 // BUGGY
		inline void set( int index, const T& item ) {
			if ( index < 0 ) return;
			this->list[index] = item;
		 }


//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:
		inline ConstIterator begin() const					{ return this->list.begin(); }
		inline Iterator begin()								{ return this->list.begin(); }

		inline ConstIterator end() const					{ return this->list.end(); }
		inline Iterator end()								{ return this->list.end(); }

		inline const T& front() const						{ return this->list.front(); }
		inline T& front()									{ return this->list.front(); }

		inline const T& back() const						{ return this->list.back(); }
		inline T& back()									{ return this->list.back(); }

		inline int size() const								{ return this->list.size(); }

		inline const std::vector<T*>& getVector() const		{ return this->list; }
		inline std::vector<T*>& getVector()					{ return this->list; }

		inline void transactionBeginRead() const			{ this->mux.lockForRead(); }
		inline void transactionBeginWrite() const			{ this->mux.lockForWrite(); }
		inline void transactionEnd() const					{ this->mux.unLock(); }

		inline wosh::MutexRW* getMutex() const				{ return &this->mux; }

//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:



//@}
	protected:
		mutable std::vector<T> list;
		mutable MutexRW mux;


}; // class def



template <typename T>  
class List22<T*>
 {
	WOSH_CLASSNAME(wosh::List22)
	WOSH_CLASS_ABSTRACT(false)

	public:
		typedef typename std::vector<T*>::const_iterator ConstIterator;
		typedef typename std::vector<T*>::iterator Iterator;

	public:
		/**
		 * @brief  Default constructor.
		 */
		List22() { }

		List22( const List22<T*>& m ) {
			*this = m;
		 }

		virtual ~List22() { }// do not free elements!


/** @name reentrant methods
 * @{
 ******************************************************************************/
	public:
		inline int count() const {
			MutexLockerRead mL(&this->mux);
			return this->list.size();
		 }

		inline bool isEmpty() const {
			MutexLockerRead mL(&this->mux);
			return this->list.empty();
		 }

		virtual inline void clear() {
			MutexLockerWrite mL(&this->mux);
			free();
		 }

		inline void append( T* item ) {
			MutexLockerRead mL(&this->mux);
			this->list.push_back(item);
		 }

//@}
/** @name non-reentrant (proxy for STL methods)
 * @{
 ******************************************************************************/
	public:
		inline bool exists( const T* item ) const {
			return ( indexOf(item) != -1 );
		 }

		inline int indexOf( const T* item ) const {
			int index = 0;
			ConstIterator it;
			const ConstIterator it_end = this->list.end();
			for ( it=this->list.begin(); it!=it_end; ++it, ++index ) {
				if ( *it == item ) return index;
			 }
			return -1;
		 }
		inline Iterator find( int index ) {
			int iLoc = 0;
			Iterator it;
			const ConstIterator it_end = this->list.end();
			for ( it=this->list.begin(); it!=it_end; ++it, ++iLoc ) {
				if ( iLoc != index ) continue;
				return it;
			 }
			return this->list.end();
		 }

		inline const T* itemAt( int index ) const {
			int iLoc = 0;
			ConstIterator it;
			const ConstIterator it_end = this->list.end();
			for ( it=this->list.begin(); it!=it_end; ++it, ++iLoc ) {
				if ( index == iLoc ) return *it;
			 }
			return NULL;
		 }
		inline T* itemAt( int index ) {
			int iLoc = 0;
			Iterator it;
			const ConstIterator it_end = this->list.end();
			for ( it=this->list.begin(); it!=it_end; ++it, ++iLoc ) {
				if ( index == iLoc ) return *it;
			 }
			return NULL;
		 }

		inline bool pop( T* obj ) {
			typename std::vector<T*>::iterator it;
			for ( it=this->list.begin(); it!=this->list.end(); ++it ) {
				if ( *it != obj ) continue;
				this->list.erase(it);
				return true;
			 }
			return false;
		 }

		inline T* pop( int index ) {
			if ( index < 0 || index >= (int)this->list.size() ) return NULL;
			int i=0;
			typename std::vector<T*>::iterator it;
			for ( it=this->list.begin(); it!=this->list.end(); ++it, ++i ) {
				if ( i != index ) continue;
				T* obj = *it;
				this->list.erase(it);
				return obj;
			 }
			return NULL;
		 }

		virtual void free() {
			Iterator it;
			const ConstIterator it_end = this->list.end();
			for( it=this->list.begin(); it!=it_end; ++it ) {
				if ( *it == NULL ) continue;
				delete *it;
			 }
			this->list.clear();
		 }

		inline void push_back( T* item, bool check_existence = false ) {
			if ( check_existence && exists(item) )
				return;
			this->list.push_back(item);
		 }

		 // BUGGY
		inline void set( int index, T* item ) {
			if ( index < 0 ) return;
			this->list[index] = item;
		 }

		inline void reserve( int n ) {
			this->list.reserve(n);
			//this->list.assign( n, (T*)NULL );
		 }

		WRESULT erase( Iterator it, bool deleteContent = true ) {
			if ( it == this->list.end() ) return WRET_ERR_PARAM;
			if ( deleteContent && *it != NULL )
				delete *it;
			this->list.erase(it);
			return WRET_OK;
		 }

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:
		inline ConstIterator begin() const					{ return this->list.begin(); }
		inline Iterator begin()								{ return this->list.begin(); }

		inline ConstIterator end() const					{ return this->list.end(); }
		inline Iterator end()								{ return this->list.end(); }

		inline const T* front() const {
			if ( this->list.empty() ) return NULL;
			return this->list.front();
		 }
		inline T* front() {
			if ( this->list.empty() ) return NULL;
			return this->list.front();
		 }

		inline const T* back() const {
			if ( this->list.empty() ) return NULL;
			return this->list.back();
		 }
		inline T* back() {
			if ( this->list.empty() ) return NULL;
			return this->list.back();
		 }

		inline int size() const								{ return this->list.size(); }

		inline const std::vector<T*>& getVector() const		{ return this->list; }
		inline std::vector<T*>& getVector()					{ return this->list; }

		inline void transactionBeginRead() const			{ this->mux.lockForRead(); }
		inline void transactionBeginWrite() const			{ this->mux.lockForWrite(); }
		inline void transactionEnd() const					{ this->mux.unLock(); }

		inline wosh::MutexRW* getMutex() const				{ return &this->mux; }

//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:

		virtual List22<T*>& operator=( const List22<T*>& m ) {
			if (this == &m) return *this; // same object?
			m.transactionBeginRead();
			this->transactionBeginWrite();
			free();
			this->list.reserve( m.size() );
			ConstIterator it;
			for ( it=m.list.begin(); it!=m.list.end(); ++it ) {
				if ( *it == NULL ) continue;
				this->list.push_back( *it );
			 }
			transactionEnd();
			m.transactionEnd();
			return *this;
		 }

		virtual bool operator==( const List22<T*>& other ) const {
			this->transactionBeginRead();
			other.transactionBeginRead();
			if ( this->list.size() != other.list.size() ) {
				other.transactionEnd();
				this->transactionEnd();
				return false;
			 }
			bool result = std::equal( this->list.begin(), this->list.end(), other.list.begin() );
			other.transactionEnd();
			this->transactionEnd();
			return result;
		 }

		virtual bool operator!=( const List22<T*>& other ) const	{ return !(*this == other);  }

//@}
	protected:
		mutable std::vector<T*> list;
		mutable MutexRW mux;


}; // class def


////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////


template <typename Key, typename Value>  
class Map22
 {
	WOSH_CLASSNAME(wosh::Map22)
	WOSH_CLASS_ABSTRACT(false)

	public:
		typedef typename std::map<Key,Value>::const_iterator ConstIterator;
		typedef typename std::map<Key,Value>::iterator Iterator;

	public:
		/**
		 * @brief  Default constructor.
		 */
		Map22() { }

		Map22( const Map22<Key,Value>& m ) {
		//	*this = m;
		 }

		virtual ~Map22() {
		 }


/** @name reentrant methods
 * @{
 ******************************************************************************/
	public:

		inline int count() const {
			MutexLockerRead mL(&this->mux);
			return this->map.size();
		 }

		inline bool isEmpty() const {
			MutexLockerRead mL(&this->mux);
			return this->map.empty();
		 }

		inline void clear() {
			MutexLockerWrite mL(&this->mux);
			this->map.clear();
		 }

		WRESULT remove( const Key& key ) {
			MutexLockerWrite mL(&this->mux);
			Iterator it = this->map.find(key);
			if ( it == this->map.end() )
				return WRET_ERR_PARAM;
			this->map.erase(it);
			return WRET_OK;
		 }


		void set( const Key& key, const Value& value ) {
			this->map[key] = value;
		 }

		//BUGGY
		Value& find( const Key& key ) {
			return this->map[key];
		 }


		WRESULT erase( const Key& key ) {
			Iterator it = this->map.find(key);
			if ( it == this->map.end() )
				return WRET_ERR_PARAM;
			this->map.erase(it);
			return WRET_OK;
		 }
		WRESULT erase( Iterator it ) {
			if ( it == this->map.end() ) return WRET_ERR_PARAM;
			this->map.erase(it);
			return WRET_OK;
		 }

		inline bool exists( const Key& key ) const {
			ConstIterator it = this->map.find(key);
			return ( it != this->map.end() );
		 }


		const Value& valueAt( int index ) const {
			ConstIterator it; int ci = 0;
			const ConstIterator it_end = this->map.end();
			for( it=this->map.begin(); it!=it_end; ++it, ++ci ) {
				if ( index == ci ) return it->second;
			 }
			return Value();
		 }
		Value& valueAt( int index ) {
			Iterator it; int ci = 0;
			const ConstIterator it_end = this->map.end();
			for( it=this->map.begin(); it!=it_end; ++it, ++ci ) {
				if ( index == ci ) return it->second;
			 }
			return Value();
		 }

		const Key& keyAt( int index ) const {
			ConstIterator it; int ci = 0;
			const ConstIterator it_end = this->map.end();
			for( it=this->map.begin(); it!=it_end; ++it, ++ci ) {
				if ( index == ci ) return it->first;
			 }
			return Key();
		 }

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:

		inline ConstIterator begin() const					{ return this->map.begin(); }
		inline Iterator begin()								{ return this->map.begin(); }

		inline ConstIterator end() const					{ return this->map.end(); }
		inline Iterator end()								{ return this->map.end(); }

		inline int size() const								{ return this->map.size(); }

		inline const std::map<Key,Value>& getMap() const	{ return this->map; }
		inline std::map<Key,Value>& getMap()				{ return this->map; }

		inline void transactionBeginRead() const			{ this->mux.lockForRead(); }
		inline void transactionBeginWrite() const			{ this->mux.lockForWrite(); }
		inline void transactionEnd() const					{ this->mux.unLock(); }

		inline wosh::MutexRW* getMutex() const				{ return &this->mux; }

//@}

	protected:
		mutable std::map<Key,Value> map;
		mutable MutexRW mux;

};

////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////

template <typename Key, typename Value>  
class Map22<Key, Value*>
 {
	WOSH_CLASSNAME(wosh::Map22)
	WOSH_CLASS_ABSTRACT(false)

	typedef typename std::map<Key,Value*>::const_iterator ConstIterator;
	typedef typename std::map<Key,Value*>::iterator Iterator;


	public:
		/**
		 * @brief  Default constructor.
		 */
		Map22() { }

		Map22( const Map22<Key,Value*>& ) { }

		virtual ~Map22() { }// do not free elements!


/** @name reentrant methods
 * @{
 ******************************************************************************/
	public:

		inline int count() const {
			MutexLockerRead mL(&this->mux);
			return this->map.size();
		 }

		inline bool isEmpty() const {
			MutexLockerRead mL(&this->mux);
			return this->map.empty();
		 }

		inline void clear() {
			MutexLockerWrite mL(&this->mux);
			free();
		 }

		inline WRESULT remove( const Key& key, bool delete_content = true ) {
			MutexLockerWrite mL(&this->mux);
			return erase(key, delete_content);
		 }

		inline void copyTo( Map22<Key,Value*>& targetMap, bool deletePrevious = true ) const {
			if (this == &targetMap) return; // same object?
			this->transactionBeginRead();
			targetMap.transactionBeginWrite();
			targetMap.free();
			ConstIterator it;
			const ConstIterator it_end = this->map.end();
			for( it=this->map.begin(); it!=it_end; ++it ) {
				if ( deletePrevious && targetMap.find(it->first) != NULL )
					delete targetMap.find(it->first);
				if ( it->second == NULL )
					targetMap.map[it->first] = NULL;
				else
					targetMap.map[it->first] = new Value(*it->second);
			 }
			targetMap.transactionEnd();
			this->transactionEnd();
		 }

/** @name try
 * @{
 ******************************************************************************/
	public:
		void set( const Key& key, Value* value ) {
			this->map[key] = value;
		 }

		WRESULT erase( const Key& key, bool delete_content = true ) {
			Iterator it = this->map.find(key);
			if ( it == this->map.end() ) return WRET_ERR_PARAM;
			if ( it->second == NULL ) {
				this->map.erase(it);
				return WRET_ERR_PARAM;
			 }
			if ( delete_content ) delete it->second;
			this->map.erase(it);
			return WRET_OK;
		 }
		WRESULT erase( Iterator it, bool delete_content = true ) {
			if ( it == this->map.end() ) return WRET_ERR_PARAM;
			if ( delete_content && it->second != NULL )
				delete it->second;
			this->map.erase(it);
			return WRET_OK;
		 }

		WRESULT eraseByValue( Value* item, bool delete_content = true ) {
			Iterator it;
			const ConstIterator it_end = this->map.end();
			for( it=this->map.begin(); it!=it_end; ++it ) {
				if ( it->second != item ) continue;
				if ( delete_content )
					delete it->second;
				this->map.erase(it);
				return WRET_OK;
			 }
			return WRET_ERR_PARAM;
		 }

		inline bool exists( const Key& key ) const {
			ConstIterator it = this->map.find(key);
			return ( it != this->map.end() );
		 }

		inline bool existsValue( const Value* item ) const {
			ConstIterator it;
			const ConstIterator it_end = this->map.end();
			for( it=this->map.begin(); it!=it_end; ++it ) {
				if ( it->second == item ) return true;
			 }
			return false;
		 }

		const Value* find( const Key& key ) const {
			const ConstIterator it = this->map.find(key);
			if ( it == this->map.end() ) return NULL;
			if ( it->second == NULL ) return NULL;
			return it->second;
		 }

		Value* find( const Key& key ) {
			Iterator it = this->map.find(key);
			if ( it == this->map.end() ) return NULL;
			if ( it->second == NULL ) return NULL;
			return it->second;
		 }

		const Value* valueAt( int index ) const {
			ConstIterator it; int ci = 0;
			const ConstIterator it_end = this->map.end();
			for( it=this->map.begin(); it!=it_end; ++it, ++ci ) {
				if ( index == ci ) return it->second;
			 }
			return NULL;
		 }
		Value* valueAt( int index ) {
			Iterator it; int ci = 0;
			const ConstIterator it_end = this->map.end();
			for( it=this->map.begin(); it!=it_end; ++it, ++ci ) {
				if ( index == ci ) return it->second;
			 }
			return NULL;
		 }

		Key keyAt( int index ) const { // it's a copy
			ConstIterator it; int ci = 0;
			const ConstIterator it_end = this->map.end();
			for( it=this->map.begin(); it!=it_end; ++it, ++ci ) {
				if ( index == ci ) return it->first;
			 }
			return Key();
		 }


		void free() {
			Iterator it;
			const ConstIterator it_end = this->map.end();
			for( it=this->map.begin(); it!=it_end; ++it ) {
				if ( it->second == NULL ) continue;
				delete it->second;
			 }
			this->map.clear();
		 }

		int getKeys( std::vector<Key>& keys ) {
			ConstIterator it;
			const ConstIterator it_end = this->map.end();
			for( it=this->map.begin(); it!=it_end; ++it ) {
				keys.push_back(it->first);
			 }
			return keys.size();
		 }

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:
		inline ConstIterator begin() const					{ return this->map.begin(); }
		inline Iterator begin()								{ return this->map.begin(); }

		inline ConstIterator back() const					{ return this->list.back(); }
		inline Iterator back()								{ return this->list.back(); }

		inline ConstIterator end() const					{ return this->map.end(); }
		inline Iterator end()								{ return this->map.end(); }

		inline int size() const								{ return this->map.size(); }

		inline const std::map<Key,Value*>& getMap() const	{ return this->map; }
		inline std::map<Key,Value*>& getMap()				{ return this->map; }

		inline void transactionBeginRead() const			{ this->mux.lockForRead(); }
		inline void transactionBeginWrite() const			{ this->mux.lockForWrite(); }
		inline void transactionEnd() const					{ this->mux.unLock(); }

		inline wosh::MutexRW* getMutex() const				{ return &this->mux; }

//@}
		virtual Map22<Key,Value*>& operator=(const Map22<Key,Value*>& ) {
			return *this;
		 }

		virtual bool operator==( const Map22<Key,Value*>& other ) const {
			this->transactionBeginRead();
			other.transactionBeginRead();
			if ( this->map.size() != other.map.size() ) {
				other.transactionEnd();
				this->transactionEnd();
				return false;
			 }
			bool result = std::equal( this->map.begin(), this->map.end(), other.map.begin() );
			other.transactionEnd();
			this->transactionEnd();
			return result;
		 }

		virtual bool operator!=( const Map22<Key,Value*> &other ) const	{ return !(*this == other);  }


	protected:
		mutable std::map<Key,Value*> map;
		mutable MutexRW mux;

}; // class def

////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////

//tree here



}; // namespace wosh

#endif //__WOSH_Core_Containers_H__
