/**
 * @file    DataMining.h
 * @brief   Under PLANNING.
 *
 *          Data Mining is one requirement of applied Artificial Intelligence in WOSH.
 *          See also \ref page_ai.
 *
 * Related Topics:
 *  - prediction, relationship, description
 *  - natural language
 *
 *
 * Given a generic DataMiningProvider object, we know:
 *   - implemented interfaces (\c IsA relation)
 *   - offered (common) fields (\c HasA relation)
 *
 * a class names, hierarchical until core (and well-known interfaces) types.
 *
 * Example:
 *  wosh::building::Garden \b "is a" {wosh::building::Garden, wosh::building::Floor, wosh::building::BuildingObject, wosh::Object}
 *  wosh::building::Garden \b "has a" {wosh::PropertiesProvider, wosh::MethodsProvider}
 *
 *
 *
 *
 *data warehousing
 *
 *
 *
 ****************************************************************************
 * @version $Id: DataMining.h 2036 2010-03-04 16:09:59Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_DataMining_H__
 #define __WOSH_Core_DataMining_H__

 #include <woshDefs.h>
 #include <core/Serializable.h>
 #include <vector>


namespace wosh {


class IDataMiner : public virtual IReflection
 {
// many stuff provided by IReflection

/** @name Data-Mining Generalization interface (IsA)
 * @{
 ******************************************************************************/
	public:
		// implemented using ObjectFactory class graph
		virtual inline bool isA( const char* classname ) const {
			if ( classname == NULL ) return false;
			return (ObjectFactory::getDistanceOf(classname) > 0);
		 }

//@}
/** @name Data-Mining Composition interface (HasA)
 * @{
 ******************************************************************************/
	public:
		// example: has(Properties) ?
		virtual inline bool hasA( const char* classname ) const {
			return false;
		 }
//@}



	public:
		virtual ~DataMiningProvider() { }


}; // class def

}; // namespace wosh

#endif //__WOSH_Core_DataMining_H__
