/**
 * @class   wosh::LogContext
 * @brief   
 *          
 * http://www.gamedev.net/community/forums/topic.asp?topic_id=426065&whichpage=1
 *
 ****************************************************************************
 * @version $Id: LogContext.h 1938 2010-02-24 20:18:36Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_LogContext_H__
 #define __WOSH_Core_LogContext_H__

 #include <woshDefs.h>
 #include <core/Object.h>
 #include <core/Containers.h>
 #include <stdarg.h>

 #include <stdio.h> //plaintext handler


namespace wosh {

 class SystemLogger;

#define _LogContext_CONTEXT_MAX_SIZE	31

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

enum LogLevel {
	LOG_UNKNOWN		= 0x00,
	LOG_NONE		= 0x01,
	LOG_CRITICAL	= 0x02,
	LOG_WARNING		= 0x03,
	LOG_INFO		= 0x04,
	LOG_VERBOSE		= 0x05,
	LOG_DEBUG		= 0x06,
	LOG_ALL			= 0x07,
};

void log( LogLevel level, const char* context, const char* format, ... );

class LogHandler;

class LogContext : public Object  {
	public:
		LogContext();
		virtual ~LogContext();

	public:
		void operator() ( LogLevel level, const char* format, ... );

		void verbose( const char* format, ... );
	//	void info( const char *format , ... );
	//	void warn( const char *format , ... );
	//	void critical( const char *format , ... );

	public:
		void setContext( const std::string& name, bool printContext = true );
		void setIndent( short spaces );
		void setLevel( LogLevel min_level );
		void setContextPrint( bool enabled );

		void setHandler( LogHandler* handler );

	public:
		inline std::string getContext() const {
			MutexLockerRead mL(&this->configMux);
			return this->contextShadow;
		 }

		inline bool hasHandler() const {
			MutexLockerRead mL(&this->configMux);
			return (this->handler != NULL);
		 }

		inline short getIndent() const {
			MutexLockerRead mL(&this->configMux);
			return this->indent;
		 }

		inline LogLevel getLevel() const {
			MutexLockerRead mL(&this->configMux);
			return this->minLevel;
		 }

		inline bool isContextPrintEnabled() const {
			MutexLockerRead mL(&this->configMux);
			return this->printContext;
		 }

	public:
		static const char* getLogLevelAsString( LogLevel level );
		static LogLevel getLogLevelFromString( const char* level );

	protected:
		void rebuiltContext();

	protected:
		char context[_LogContext_CONTEXT_MAX_SIZE];
		std::string contextShadow;
		short indent;
		LogLevel minLevel;
		bool printContext;
		mutable MutexRW configMux;

		LogHandler* handler;

		static List22<LogContext*>& getInstances();

		friend class wosh::SystemLogger;

	private:
		LogContext(const LogContext&) : Object() { }
		LogContext& operator=(const LogContext&) { return *this; }

 };


}; // namespace wosh

#endif //__WOSH_Core_LogContext_H__
