/**
 * @class   wosh::MessageFilter
 * @brief   The class extends wosh::Message providing comparison methods.
 *          
 *
 *
 ****************************************************************************
 * @version $Id: MessageFilter.h 2081 2010-03-11 01:51:44Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_MessageFilter_H__
 #define __WOSH_Core_MessageFilter_H__

 #include <woshDefs.h>
 #include <core/Message.h>
 #include <core/Object.h>

namespace wosh {

class Bundle;
class PropertiesProvider;
class InterfacesProvider;
class BusConnector;

class MessageFilter : public Message  {

	public:

		MessageFilter();

		virtual ~MessageFilter();

	public:

		virtual int compare( const Message& message ) const;


	public:

		static bool isMessageTarget( const Message& message, const Object* object );

		static bool isMessageTarget( const Message& message, const URI& itemURI, const PropertiesProvider* properties = NULL, const InterfacesProvider* interfaces = NULL );

		static bool isMessageTarget( const Message& message, const Bundle* bundle );

		static short filterReplica( const Message& message, const BusConnector* mainBus );


	public:
		static bool validateExpression( const UriExpression& expression, const PropertiesProvider* properties, const InterfacesProvider* interfaces, const SecurityToken* security );

		///@deprecated to be removed
		// static bool validateExpressions( const string& propExpressions, const PropertiesProvider* properties, const SecurityToken* security );
		// static bool validateExpression( const string& propExpression, const PropertiesProvider* properties, const SecurityToken* security );
		// static bool popExpression( string& input, string& propExpression );


}; // class def

}; // namespace wosh

#endif //__WOSH_Core_MessageFilter_H__
