/** @file    MessageQueue.cpp
 *  @author  Alessandro Polo
 *  @version $Id: MessageQueue.cpp 2164 2010-03-18 03:10:58Z alex $
 *  @brief
 * File containing methods for the wosh::MessageQueue class.
 * The header for this class can be found in MessageQueue.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <core/MessageQueue.h>

 #include <core/ObjectAllocator.h>


using namespace std;
namespace wosh {

 WOSH_REGISTER(wosh::MessageQueue, "", 1.01, _static_MessageQueue)

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void MessageQueue::clear()
 {
	MutexLocker mL(&queueMux);

	while ( !this->queueMsg.empty() )
	 {
		Message* message = this->queueMsg.top();
		this->queueMsg.pop();

		if ( message != NULL )
			delete message;
	}
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Message* MessageQueue::pop()
 {
	MutexLocker mL(&queueMux);

	if ( this->queueMsg.empty() )
		return NULL;

	// priority ordered FIFO
	Message* message = this->queueMsg.top();
	// remove it from queue
	this->queueMsg.pop();

	return message;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void MessageQueue::push( Message* message )
 {
	if ( message == NULL )
		return;

	this->queueMux.lock();
	this->queueMsg.push( message );
	this->queueMux.unLock();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

}; // namespace wosh
