/**
 * @class   wosh::MethodArguments
 * @brief   Store the arguments' definition of a Method
 *
 *
 *


 - expose arguments of a method (its interface)
 - basic input validation
 - ???????????

 - map Message (Request) to a (real) membeer function


 - MethodGeneric  template con ret/args comuni
 - una specializzazione con WRESULT => mappa ret#code a response string error













 *
 *
 ****************************************************************************
 * @version $Id: MethodArguments.h 2126 2010-03-13 19:23:10Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR MethodS;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_MethodArguments_H__
 #define __WOSH_Core_MethodArguments_H__

 #include <woshDefs.h>
 #include <core/Serialization.h>
 #include <core/Variant.h>

 #include <vector>


namespace wosh {

 #define _MethodArgument_TYPE			"wosh::MethodArgument"


class MethodArgument : public virtual ISerializable
 {
	WOSH_SERIALIZABLE(wosh::MethodArgument)
	WOSH_COMPARE_RTTI(wosh::MethodArgument)

	public:

		MethodArgument() {
			this->typeName = "";
			this->dataType = Variant::EMPTY;
			this->optional = false;
			this->ordered = -1;
		 }

		MethodArgument( Variant::DATA_TYPE type, bool is_optional = false, int index = -1 ) {
			set(type, is_optional, index);
		 }

		MethodArgument( const char* classname, bool is_optional = false, int index = -1 ) {
			set(classname, is_optional, index);
		 }

		MethodArgument( const MethodArgument& m ) {
			*this = m;
		 }

		virtual ~MethodArgument() { }


	public:

		void set( Variant::DATA_TYPE type, bool is_optional = false, int index = -1 ) {
			this->typeName = _Variant_TYPE;
			this->dataType = type;
			this->optional = is_optional;
			this->ordered = index;
		 }

		void set( const char* classname, bool is_optional = false, int index = -1 ) {
			this->typeName.assign(classname);
			this->dataType = Variant::EMPTY;
			this->optional = is_optional;
			this->ordered = index;
		 }

/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:

		MethodArgument& operator=(const MethodArgument& m) {
			this->typeName = m.typeName;
			this->dataType = m.dataType;
			this->optional = m.optional;
			this->ordered = m.ordered;
			return *this;
		 }

		bool operator==(const MethodArgument &other) const {
			if ( this->typeName != other.typeName )
				return false;
			if ( this->dataType != other.dataType )
				return false;
			if ( this->optional != other.optional )
				return false;
			if ( this->ordered != other.ordered )
				return false;
			return true;
		 }

		bool operator!=(const MethodArgument &other) const	{ return !(*this == other);  }

		bool operator>(const MethodArgument& other) const {
			if ( this->ordered > other.ordered )
				return true;
			if ( this->typeName > other.typeName )
				return true;
			if ( this->dataType > other.dataType )
				return true;
			if ( this->optional > other.optional )
				return true;
			if ( this->ordered < other.ordered )
				return false;
			if ( this->dataType < other.dataType )
				return false;
			if ( this->optional < other.optional )
				return false;
			return true;
		 }

		bool operator<(const MethodArgument& other) const {
			if ( this->ordered < other.ordered )
				return true;
			if ( this->typeName < other.typeName )
				return true;
			if ( this->dataType < other.dataType )
				return true;
			if ( this->optional < other.optional )
				return true;
			return false;
		 }

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const {
			std::string res = "";
			if ( this->typeName.empty() )
				res = "[]";
			else if ( this->typeName == _Variant_TYPE )
				res = Utilities::format("[Variant::%s]", Variant::getTypeAsString(this->dataType, true));
			else
				res = "[" + this->typeName + "]";
			if ( optional )
				res = "(o)";
			return res;
		 }

		virtual WRESULT store( IEncoderWriter* serializer ) const {
			if ( serializer == NULL ) return WRET_ERR_PARAM;
			WRESULT ret = WRET_OK;
			return ret;
		 }
		virtual WRESULT retrieve( IEncoderReader* deserializer ) {
			if ( deserializer == NULL ) return WRET_ERR_PARAM;
			WRESULT ret = WRET_OK;
			return ret;
		 }

//@}
	public:
		std::string typeName;
		wosh::Variant::DATA_TYPE dataType;
		bool optional;
		int ordered;

}; // class def



 #define _MethodArguments_TYPE			"wosh::MethodArguments"

class MethodArguments :	protected std::vector<wosh::MethodArgument>,
						public virtual ISerializable
 {
	WOSH_SERIALIZABLE(wosh::MethodArguments)
	WOSH_COMPARE_RTTI(wosh::MethodArguments)

	public:

		MethodArguments() : std::vector<MethodArgument>() { }

		MethodArguments( const MethodArguments& m ) : std::vector<MethodArgument>() {
			*this = m;
		 }

		virtual ~MethodArguments() { }


	public:


		using std::vector<MethodArgument>::clear;

//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:

		void set( unsigned int i, const MethodArgument& arg ) {
			if ( this->size() <= i ) {
				this->resize(i+1);
			 }
			this->at(i) = arg;
		 }

		void set( unsigned int i, Variant::DATA_TYPE type, bool optional = false ) {
			set( i,  MethodArgument(type,optional,i) );
		 }

		void set( unsigned int i, const char* classname, bool optional = false ) {
			set( i,  MethodArgument(classname,optional,i) );
		 }

		void setCount( unsigned int i ) {
			this->resize(i);
		 }

		MethodArgument& get( unsigned int i ) {
			if ( this->size() <= i ) {
				this->resize(i+1);
			 }
			return this->at(i);
		 }

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:


//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:

		MethodArguments& operator=(const MethodArguments& m) {
			this->clear();
			std::vector<MethodArgument>::const_iterator it;
			for(it=m.begin(); it!=m.end(); ++it) {
				push_back(*it);
			 }
			return *this;
		 }

		bool operator==(const MethodArguments &other) const {
			if ( this->size() != other.size() )
				return false;

			return true;
		 }

		bool operator!=(const MethodArguments &other) const	{ return !(*this == other);  }

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const {
			std::string ret = "";
			std::vector<wosh::MethodArgument>::const_iterator it;
			for(it=this->begin(); it!=this->end(); ++it) {
				ret += (*it).toString() + "; ";
			 }
			return ret;
		 }

		virtual WRESULT store( IEncoderWriter* serializer ) const {
			if ( serializer == NULL ) return WRET_ERR_PARAM;
			WRESULT ret = WRET_OK;
			return ret;
		 }
		virtual WRESULT retrieve( IEncoderReader* deserializer ) {
			if ( deserializer == NULL ) return WRET_ERR_PARAM;
			WRESULT ret = WRET_OK;
			return ret;
		 }

//@}

	protected:


}; // class def




}; // namespace wosh

#endif //__WOSH_Core_Method_H__
