/** @file    NetworkProtocol.cpp
 *  @author  Alessandro Polo
 *  @version $Id: NetworkProtocol.cpp 2084 2010-03-11 10:49:02Z alex $
 *  @brief
 * File containing methods for the wosh::NetworkProtocol class.
 * The header for this class can be found in NetworkProtocol.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <core/NetworkProtocol.h>

 #include <core/NetworkManager.h>
 #include <core/ObjectFactory.h>
 #include <core/BusManager.h>
 #include <core/MessageProcessingCommon.h>
 #include <core/MethodsDef.h>
 #include <core/Utilities.h>


using namespace std;
namespace wosh {

 WOSH_REGISTER_INTERFACE(wosh::NetworkProtocol, "wosh::Object", 0.66, _static_NetworkProtocol)

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

NetworkProtocol::NetworkProtocol() : Object( NetworkManager::getInstance() )
 {
	Object::setName("NetworkProtocol", Utilities::randomLong() );
	this->listener = NULL;

	Log.setIndent( 3 );
	Log.setContext( "NetworkProtocol" );
	Log.setLevel( LOG_INFO );

	Properties.set( _NetworkProtocol_KEY_ProtocolPrefix,"",	Permission(Permission::Read) );
	Properties.set( _NetworkProtocol_KEY_Priority,		0,	Permission::RW_RW_R__() );
	Properties.set( _NetworkProtocol_KEY_ServersCount,	-1, Permission(Permission::Read) );
	Properties.set( _NetworkProtocol_KEY_ClientsCount,	-1, Permission(Permission::Read) );

	Properties.updateObjectKeys(this);
	Properties.setPropertiesProviderListener(this);


	MethodRequest* mmConnect = Methods.create<MethodRequest>( _NetworkProtocol_METHOD_connect, "connect", Permission::R_XR_XR__() );
	mmConnect->setMethod( this, (MethodRequestPtr)&NetworkProtocol::mmDoConnect );

	MethodRequest* mmListen = Methods.create<MethodRequest>( _NetworkProtocol_METHOD_listen, "listen", Permission::R_XR_XR__() );
	mmListen->setMethod( this, (MethodRequestPtr)&NetworkProtocol::mmDoListen );

 }

NetworkProtocol::~NetworkProtocol() {
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////// 

WRESULT NetworkProtocol::setListener( INetworkProtocolListener* ch_listener ) {
	 this->listener = ch_listener;
	 return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENTS

bool NetworkProtocol::updatingProperty( bool& , const Variant& , Property& , const PropertiesProvider* ) {
	return true;
 }

///////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* NetworkProtocol::mmDoConnect( const Request* request ) {
	if ( request == NULL ) return NULL;
	Variant proto_string;
	request->extractArguments(proto_string);
	if ( proto_string.isStringEmpty() )
		return request->replyResponse(WRET_ERR_PARAM, "invalid protocol string");
	Log(LOG_VERBOSE, ":mmDoConnect(%s) : Calling connectChannel(asynch)..", proto_string.asString().c_str() );
	WRESULT ret = WRET_ERR_PARAM;
	ret = connectChannel( proto_string.asString(), 0 );
// ret = connectChannel( const std::vector<std::string>& protocolsAvailable, 0 ) = 0;
	return request->replyResponse(ret);
 }

Response* NetworkProtocol::mmDoListen( const Request* request ) {
	if ( request == NULL ) return NULL;
	Variant proto_string;
	request->extractArguments(proto_string);
	if ( proto_string.isStringEmpty() )
		return request->replyResponse(WRET_ERR_PARAM, "invalid protocol string");
	Log(LOG_VERBOSE, ":mmDoConnect(%s) : Calling connectChannel(asynch)..", proto_string.asString().c_str() );
	WRESULT ret = WRET_ERR_PARAM;
	ret = listenChannel( proto_string.asString(), 0 );
	return request->replyResponse(ret);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////


};
