/**
 * @class   wosh::DataBaseManager
 * @brief   
 *
 * @ingroup CoreModules
 ****************************************************************************
 * @version 0.0.000 $Id: DataBaseManager.h 2012 2010-03-02 21:50:44Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_DataBaseManager_H__
 #define __WOSH_Core_DataBaseManager_H__

 #include <woshDefs.h>
 #include <core/WoshModule.h>

 #include <core/DataBase.h>

 #include <map>
 #include <vector>


namespace wosh {

 #define _DataBaseManager_NAME							"DataBaseManager"
 #define _DataBaseManager_TYPE							"wosh::DataBaseManager"
 #define _DataBaseManager_URI							":DataBase"

 #define _DataBaseManager_KEY_AutoLoadProtocols			"AutoLoadProtocols"
 #define _DataBaseManager_KEY_DataBase_Count			"Count"

 #define _DataBaseManager_METHOD_ListSupportedProtocols	"list_supported"

 
 #define _DataBaseManager_EVENT_DatabaseInstalled		"database_installed"

 
class WoshKernel;

class DataBaseManager : public WoshModule
 {
	WOSH_MODULE(wosh::DataBaseManager)

	typedef Map22<std::string, wosh::DataBase*> tStrDataBaseMap;
	typedef Map22<std::string, wosh::IEncoderReader*> tStrIEncoderReaderMap;
	typedef Map22<std::string, wosh::IEncoderWriter*> tStrIEncoderWriterMap;

	protected:

		DataBaseManager( const WoshKernel* parent = NULL );

		WRESULT WoshModule::onKernelInitializing()

		friend class wosh::WoshKernel;

	public:

		virtual ~DataBaseManager();

//@}
/** @name Public methods
 * @{
 ******************************************************************************/
	public:

		WRESULT unRegisterDataBase( DataBase* db_implementation );

//@}
/** @name Public methods (allocators)
 * @{
 ******************************************************************************/
	public:

		virtual WRESULT installDataBase( DataBase* db_implementation );

	public:

		DataBase* loadDataBase( const string& database_name );

//@}
/** @name Public static methods
 * @{
 ******************************************************************************/
	public:

		static WRESULT loadObjects( std::vector<ISerializable*>& objects, const std::string& class_name, const std::string& archive_name, const std::string& protocol );

		static WRESULT saveObjects( std::vector<ISerializable*>& objects, const std::string& archive_name, const std::string& protocol );

//@}
	protected:

		Response* mmDoList( const Request* request );
		Response* mmDoListSupportedProtocols( const Request* request );


/** @name Internal Mehods and Events management
 * @{
 ******************************************************************************/
	private:
		
		static DataBaseManager* getInstance();

//@}
/** @name Internal vars
 * @{
 ******************************************************************************/
	private:
		tStrDataBaseMap dbs;

	protected:
		tStrIEncoderReaderMap encReader;
		tStrIEncoderWriterMap encWriter;

		tStrDataBaseMap databases;			//  input/output di encoder

//@}


}; // class def

}; // namespace wosh

#endif //__WOSH_Core_DataBase_H__
