/** @file    Response.cpp
 *  @author  Alessandro Polo
 *  @version $Id: Response.cpp 2167 2010-03-18 06:41:05Z alex $
 *  @brief
 * File containing methods for the wosh::Response class.
 * The header for this class can be found in Response.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <core/Response.h>

 #include <core/ObjectAllocator.h>


using namespace std;
namespace wosh {

 WOSH_REGISTER(wosh::Response, "wosh::MessageContentGeneric", 1.01, _static_Response)

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

Response::Response()
		: MessageContentGeneric(), returnValue(-1) { }

Response::Response( const std::string& method, wosh::ISerializable* data, WRESULT ReturnValue )
		: MessageContentGeneric(method, data), returnValue( ReturnValue ) { }

Response::Response( const wosh::Response& m )
		: MessageContentGeneric( (const wosh::MessageContentGeneric&)m ) { 
	this->returnValue = m.returnValue;
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Response::clear()
 {
	MessageContentGeneric::clear();
	this->returnValue = 0;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////// OPERATORS

wosh::Response& Response::operator=(const wosh::Response& m)
 {
	if (this == &m) return *this; // same object?

	(wosh::MessageContentGeneric&)*this = (const wosh::MessageContentGeneric&)m;
	this->returnValue = m.returnValue;

	return *this;
 }

bool Response::operator==(const wosh::Response &other) const
 {
	if ( (const wosh::MessageContentGeneric&)*this != (const wosh::MessageContentGeneric&)other )
		return false;
	return ( this->returnValue == other.returnValue );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

}; // namespace wosh
