/**
 * @class   wosh::SecurityToken
 * @brief   Represent a security token, used by messages to store the creator
 *          information (object and user).
 *












 *
 ****************************************************************************
 * @version $Id: SecurityToken.h 2164 2010-03-18 03:10:58Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_SecurityToken_H__
 #define __WOSH_Core_SecurityToken_H__

 #include <woshDefs.h>
 #include <core/Serialization.h>

 #include <string>

namespace wosh {

 #define _SecurityToken_TYPE			"wosh::SecurityToken"

 namespace database {
  class EncoderXML_template__Message;
  class EncoderXML_template__SecurityToken;
 };
 class SecurityManager;


class SecurityToken : public virtual ISerializable
 {
	WOSH_SERIALIZABLE(wosh::SecurityToken)
	WOSH_COMPARE_RTTI(wosh::SecurityToken)

	public:

		SecurityToken( const SecurityToken& m )
			: TTL(0) {
			*this = m;
		 }

		virtual ~SecurityToken() { }


		// bool isValid() const;

/** @name Getters
 * @{
 ******************************************************************************/
	public:
		/**
		 * @brief  Get the time-to-live of message
		 * @return TTL in seconds format (0 means disabled/non-applicable)
		 */
		inline unsigned long getTTL() const					{ return this->TTL; }


		const std::string& getUser()	const				{ return this->user; }
		const std::string& getCreatorUUID()	const			{ return this->creator; }

//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:

		virtual SecurityToken& operator=(const SecurityToken& m) {
			if (this == &m) return *this; // Same object?
			this->user = m.user;
			this->creator = m.creator;
			return *this;
		 }

		virtual bool operator==(const SecurityToken &other) const {
			if ( this->user != other.user )
				return false;
			if ( this->creator != other.creator )
				return false;
			return true;
		 }
 
		virtual bool operator!=(const SecurityToken &other) const
															{ return !(*this == other);  }

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const				{ return ""; }

		virtual WRESULT store( IEncoderWriter* serializer ) const {
			if ( serializer == NULL ) return WRET_ERR_PARAM;
			WRESULT ret = WRET_OK;
			return ret;
		 }
		virtual WRESULT retrieve( IEncoderReader* deserializer ) {
			if ( deserializer == NULL ) return WRET_ERR_PARAM;
			WRESULT ret = WRET_OK;
			return ret;
		 }
//@}

	protected:

		SecurityToken() : TTL(0) { }

		SecurityToken( const std::string& username, const std::string& creator_uuid ) {
			this->user = username;
			this->creator = creator_uuid;
		 }

		friend class wosh::SecurityManager;
		friend class wosh::database::EncoderXML_template__Message;
		friend class wosh::database::EncoderXML_template__SecurityToken;

	protected:
		unsigned long TTL;					///< time-to-live in seconds, 0=not applicable


		std::string user;		///< user
		std::string creator;	///< creator of the token


}; // class def

}; // namespace wosh

#endif //__WOSH_Core_SecurityToken_H__
