/**
 * @class   wosh::ISerializable
 * @brief   
 *
 *
 *
 *
KNOWN PROBLEM
regards virtual inheritance and Covariant Return Types
on MSC (9.0 and previous)

// virtual ISerializable ==>> MSC CRASH on clone() cos of wrong member pointer!!
// QUESTION: is it a MSC BUG??? (it looks like, G++ works fine as espected)

class MessageContent : public ISerializable


 *
 *
 ****************************************************************************
 * @version $Id: Serializable.h 1652 2010-01-23 07:47:30Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_Serialization_H__
 #define __WOSH_Core_Serialization_H__

 #include <woshDefs.h>
 #include <core/Reflection.h>


namespace wosh {




// so you can call Super::do_superclass_implementation (in generic way)
# undef WOSH_SUPER
# define WOSH_SUPER(CLASSNAME)											\
		public: typedef CLASSNAME Super;







// suppose Covariant Return Types
// else: public: virtual ISerializable* clone() const { return new CLASSNAME(*this); }

#ifdef _USE_COVARIANT_RETURN_TYPE
# undef WOSH_CLONE_CONSTRUCTOR
# define WOSH_CLONE_CONSTRUCTOR(CLASSNAME)											\
		public: virtual CLASSNAME* clone() const { return new CLASSNAME(*this); }

# undef WOSH_CLONE_ABSTRACT
# define WOSH_CLONE_ABSTRACT(CLASSNAME)												\
		public: virtual CLASSNAME* clone() const = 0;
#endif


#undef WOSH_COPY_FORBIDDEN
#define WOSH_COPY_FORBIDDEN(CLASSNAME)												\
		private: CLASSNAME(const CLASSNAME&) { }									\
		private: CLASSNAME& operator=(const CLASSNAME&) { return *this; }			\
		public: virtual CLASSNAME* clone() const { return NULL; }


#ifdef _USE_RTTI
# undef WOSH_COMPARE_RTTI
# define WOSH_COMPARE_RTTI(CLASSNAME)												\
		public: virtual short compare(const wosh::ISerializable* other) const {		\
			const CLASSNAME* other_c = dynamic_cast<const CLASSNAME*>(other);		\
			if ( other_c == NULL ) return -255;										\
			if ( *this == *other_c ) return 0;										\
			return 1;																\
		 }
# undef WOSH_COMPAREALL_RTTI
# define WOSH_COMPAREALL_RTTI(CLASSNAME)											\
		public: virtual short compare(const wosh::ISerializable* other) const {		\
			const CLASSNAME* other_c = dynamic_cast<const CLASSNAME*>(other);		\
			if ( other_c == NULL ) return -255;										\
			if ( *this == *other_c ) return 0;										\
			if ( *this < *other_c ) return -1;										\
			return 1;																\
		 }
#endif //_USE_RTTI



// implements:
// ALL from IReflection; IsAbstract() = false
// clone() based on copy-constructor

#undef WOSH_SERIALIZABLE
#define WOSH_SERIALIZABLE(CLASSNAME)			\
		WOSH_CLASSNAME(CLASSNAME)				\
		WOSH_CLASS_ABSTRACT(false)				\
												\
		WOSH_CLONE_CONSTRUCTOR(CLASSNAME)

#undef WOSH_SERIALIZABLE_ABSTRACT
#define WOSH_SERIALIZABLE_ABSTRACT(CLASSNAME)	\
		WOSH_CLASSNAME(CLASSNAME)				\
		WOSH_CLASS_ABSTRACT(true)				\
												\
		WOSH_CLONE_ABSTRACT(CLASSNAME)




enum SerializeStrategy {
	SERIALIZE_DEFAULT	= 0x0000,
	SERIALIZE_FIELD		= 0x0001,
	SERIALIZE_NODE		= 0x0010
};

class IEncoderWriter;
class IEncoderReader;


/*
class IComparable {

};
*/

class ISerializable : public virtual IReflection  {

/** @name Serialization Interface
 * @{
 ******************************************************************************/
	public:
		/**
		 * @brief  Create a cloned object.
		 * @return pointer to the new object, caller must care about de-allocation
		 * @see    WOSH_CLONE
		 */
		virtual ISerializable* clone() const = 0;   // The Virtual (Copy) Constructor

		template <typename T>
		inline T* clone() const {
			return dynamic_cast<T*>( this->clone() );
		 }

		/**
		 * @brief  Compare object against another of same type.
		 * @param other [in] the object to be compared
		 * @return when object are identical return 0; return -1 when object are of different types
		 * @see    WOSH_COMPARE_RTTI
		 */
		virtual short compare( const ISerializable* other ) const = 0;

		/**
		 * @brief  Get  object minimal (not full!) representation.
		 * @return usually object's name or value
		 */
		virtual std::string toString() const = 0;


		virtual WRESULT store( IEncoderWriter* serializer ) const = 0;
		virtual WRESULT retrieve( IEncoderReader* deserializer ) = 0;


//@}
	public:
		virtual ~ISerializable() { }

}; // class def


}; // namespace wosh

#endif //__WOSH_Core_Serialization_H__
