/**
 * @class   wosh::SystemLogger
 * @brief   
 *
 *
 * - list info about any (instance of) LogContext
 *
 *
 * - manage log handlers
 * - change settings of a context: change handler (ex: enable stdout)
 *
 *
send log message to someone.. as Message
interazione con systemmonitor
 *
 * @ingroup CoreModules
 ****************************************************************************
 * @version 0.8.013 $Id: SystemLogger.h 2198 2010-03-23 12:03:07Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/*! \page page_core
 * \htmlonly <hr/> \endhtmlonly
 * \section page_core_systemlogger System Logger
 *
 *
 *
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR ThreadS;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_SystemLogger_H__
 #define __WOSH_Core_SystemLogger_H__

 #include <woshDefs.h>
 #include <core/WoshModule.h>
 #include <core/Containers.h>
 #include <core/ObjectAllocator.h>


namespace wosh {

 #define _SystemLogger_NAME				"SystemLogger"
 #define _SystemLogger_TYPE				"wosh::SystemLogger"
 #define _SystemLogger_URI				":Logger"

 #define _SystemLogger_METHOD_handlers	"handlers"


class WoshKernel;

class SystemLogger : public WoshModule
 {
	WOSH_MODULE(wosh::SystemLogger)

	//typedef List22<wosh::ObjectAllocator*> tObjectAllocatorList;

//static StaticAllocator BuiltInAllocator;


	protected:
		SystemLogger( const WoshKernel* parent = NULL );

		WRESULT onKernelInitializing();

		friend class wosh::WoshKernel;

	public:
		virtual ~SystemLogger();



	public:

		static SystemLogger& getInstance();

/** @name Public methods (mapped from Kernel)
 * @{
 ******************************************************************************/
	private:
		Response* mmDoGetContexts( const Request* request );
		Response* mmDoGetHandlers( const Request* request );

		WRESULT mmDoDump();

//@}
/** @name Internal vars
 * @{
 ******************************************************************************/
	private:
	//	static tObjectAllocatorList allocators;


//@}


}; // class def

}; // namespace wosh

#endif //__WOSH_Core_SystemLogger_H__
