/**
 * @class   wosh::ThreadMethod
 * @brief   
 *
 *
 *  un thread
 * gli si associa una funzione (pointer)
 * lui la esegue in RUN()
 * da usare in MessageBus POOL
 *
 * sarebbe fiCO che accettasse anche METODI
 *
 *
 ****************************************************************************
 * @version $Id: ThreadMethod.h 2147 2010-03-16 20:56:25Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR ThreadMethodS;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_ThreadMethod_H__
 #define __WOSH_Core_ThreadMethod_H__

 #include <woshDefs.h>
 #include <core/Thread.h>
 #include <core/Mutex.h>


namespace wosh {


//typedef void (MethodsProvider::*MethodPtr)(const Message * message);


class ThreadMethod : public Thread {

	public:

		/**
		 * @brief   Initialize ThreadMethod and register ThreadMethod to global list
		 * @see     wosh::ThreadMethodCollector
		 */
		ThreadMethod(const Object* owner, const std::string& name )
			: Thread(owner, name), processing(false) { }

		/**
		 * @brief   unregister ThreadMethod from global list
		 * @see     wosh::ThreadMethodCollector
		 */
		virtual ~ThreadMethod() {
			if ( isThreadRunning() ) {
				quitThread();
			 }
		 }

	protected:
		virtual void work() = 0;


	public:
		virtual bool isIdle() const {
			MutexLockerRead mL(&this->processingLock);
			return !this->processing;
		 }

	protected:
		virtual void runThread()
		 {
			while( this->running) {
				setThreadAlive();
				work();
				setThreadAlive();
			 }
		 }

	protected:
		bool processing;			///< actually processing flag
		MutexRW processingLock;		///< mutex for processing flag

}; // class def


}; // namespace wosh

#endif //__WOSH_Core_ThreadMethod_H__
