/**
 * @class   wosh::Timer
 * @brief   
 *       
 *
 * @par Credits
 *  Song Ho Ahn (song.ahn@gmail.com)
 *  http://www.songho.ca/misc/timer/timer.html
 *
 *
 *
 ****************************************************************************
 * @version $Id: Timer.h 1708 2010-01-26 22:28:01Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_Timer_H__
 #define __WOSH_Core_Timer_H__

#include <woshDefs.h>


namespace wosh {

struct TimeMs {
	long tv_sec;	///< seconds
	long tv_usec;	///< microseconds
};

class Timer  {

	public:

		Timer();

		/**
		 * @brief  Empty destructor.
		 */
		virtual ~Timer();

	public:

		virtual void start();

		virtual void stop();


	public:
		void clear();
		void clearStart();
		void clearStop();

	public:

		bool isStarted() const						{ return ( this->start_clock != 0 ); }
		bool isStopped() const						{ return ( this->stop_clock != 0 ); }

		double getTimeSpanMicroSec() const;

		inline double getTimeSpanMilliSec() const	{ return this->getTimeSpanMicroSec() * 0.001; }
		inline double getTimeSpanSec() const		{ return this->getTimeSpanMicroSec() * 0.000001; }


		double getTimeSpanClocksMs() const;

		static long now_clocks();
		static long now_epoch();
		static TimeMs now_time();

//@}
/** @name Internal vars
 * @{
 ******************************************************************************/
	protected:

//__int64 start_clock;
//__int64 start_time;

		int64 start_clock;	// ok std
		TimeMs start_time;	// ok posix

		int64 stop_clock;
		TimeMs stop_time;


}; // class def

}; // namespace wosh

#endif //__WOSH_Core_Timer_H__

