/**
 * @class   wosh::Tree
 * @brief   
 *
 ****************************************************************************
 * @version $Id: Tree.h 1937 2010-02-24 20:04:23Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_Tree_H__
 #define __WOSH_Core_Tree_H__

 #include <woshDefs.h>
 #include <core/Serialization.h>
 #include <core/Containers.h>


namespace wosh {

// ----------------------------------------------------------------- temp tree

template <typename T>  
class TreeNode22
 {
	public:

	typedef List22<wosh::TreeNode22<T>*> tTreeNode22List;
	typedef typename tTreeNode22List::Iterator ChildIterator;
	typedef typename tTreeNode22List::ConstIterator ChildConstIterator;


	public:
		TreeNode22( const T& data, TreeNode22<T>* node_parent = NULL )
			: parent(node_parent), content(data) { 
			if ( this->parent != NULL ) {
				this->parent->getChildren().push_back(this, false);
			 }
		}

		TreeNode22( TreeNode22<T>* node_parent )
			: parent(node_parent) {
			if ( this->parent != NULL ) {
				this->parent->getChildren().push_back(this, false);
			 }
		}

		TreeNode22( const TreeNode22<T>& ) {
		 }

		virtual ~TreeNode22() { }


		void free( bool nested, bool ) {
			if ( !nested ) return;
			ChildIterator it;
			const ChildConstIterator it_end = this->children.end();
			for( it=this->children.begin(); it!=it_end; ++it) {
				(*it)->free(true, true);
			 }
			this->children.free();
		 }


		inline ChildConstIterator begin() const		{ return this->children.begin(); }
		inline ChildIterator begin()				{ return this->children.begin(); }

		inline ChildConstIterator end() const		{ return this->children.end(); }
		inline ChildIterator end()					{ return this->children.end(); }


		const TreeNode22<T>* getParent() const		{ return this->parent; }
		TreeNode22<T>* getParent()					{ return this->parent; }

		const tTreeNode22List& getChildren() const		{ return this->children; }
		tTreeNode22List& getChildren()					{ return this->children; }


		TreeNode22<T>* addChild( const T& data ) {
			TreeNode22<T>* child = new TreeNode22<T>(data, this);
			return child;
		 }

		void setParent( wosh::TreeNode22<T>* node, bool fix_references = false ) {
			if ( this->parent == node || this == node ) return;
			if ( fix_references && this->parent != NULL ) {
				if ( this->parent != NULL ) {
					this->parent->getChildren().pop(this);
				 }
			 }
			this->parent = node;
			if ( fix_references && node != NULL ) {
				node->getChildren().push_back(this, false); // add if not present
			 }
		 }


		void setContent( const T& object ) {
			this->content = object;
		 }

		const T& getContent() const					{ return this->content; }
		T& getContent()								{ return this->content; }


		void dumpTree( int indend ) {
			ChildConstIterator it;
			const ChildConstIterator it_end = this->children.end();
			for( it=this->children.begin(); it!=it_end; ++it ) {
				int i=0; while(++i<=indend) { printf(" "); };

				if ( (*it)->getChildren().isEmpty() )
					printf( "-" );
				else
					printf( "+" );
				printf( " %s\n", (*it)->getContent().toString().c_str() );
				(*it)->dumpTree(indend+2);
			 }
		}

		inline void transactionBeginRead() const			{ this->mux.lockForRead(); }
		inline void transactionBeginWrite() const			{ this->mux.lockForWrite(); }
		inline void transactionEnd() const					{ this->mux.unLock(); }

		inline wosh::MutexRW* getMutex() const				{ return &this->mux; }


	protected:
		TreeNode22* parent;
		tTreeNode22List children;
		T content;

		mutable MutexRW mux;

}; // class def


////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////


template <typename T>  
class TreeNode22<T*>
 {
	public:

	typedef List22<wosh::TreeNode22<T*>*> tTreeNode22List;
	typedef typename tTreeNode22List::Iterator ChildIterator;
	typedef typename tTreeNode22List::ConstIterator ChildConstIterator;


	public:
		TreeNode22( TreeNode22<T*>* node_parent, T* data = NULL )
			: parent(node_parent), content(data) { 
			if ( this->parent != NULL ) {
				this->parent->getChildren().push_back(this, false);
			 }
		}

		TreeNode22( const TreeNode22<T*>& )
			: parent(NULL), content(NULL) { }

		virtual ~TreeNode22() {
			
		 }

		void free( bool nested, bool delete_content ) {
			if ( delete_content )
				setContent( NULL, true );
			if ( !nested ) return;
			ChildIterator it;
			const ChildConstIterator it_end = this->children.end();
			for( it=this->children.begin(); it!=it_end; ++it) {
				(*it)->free(true, delete_content);
			 }
			this->children.free();
		 }


		inline ChildConstIterator begin() const		{ return this->children.begin(); }
		inline ChildIterator begin()				{ return this->children.begin(); }

		inline ChildConstIterator end() const		{ return this->children.end(); }
		inline ChildIterator end()					{ return this->children.end(); }


		const TreeNode22<T*>* getParent() const		{ return this->parent; }
		TreeNode22<T*>* getParent()					{ return this->parent; }

		const tTreeNode22List& getChildren() const		{ return this->children; }
		tTreeNode22List& getChildren()					{ return this->children; }


		TreeNode22<T*>* addChild( T* data ) {
			TreeNode22<T*>* child = new TreeNode22<T*>(this, data);
			return child;
		 }

		void setParent( wosh::TreeNode22<T*>* node, bool fix_references = false ) {
			if ( this->parent == node || this == node ) return;
			if ( fix_references && this->parent != NULL ) {
				if ( this->parent != NULL ) {
					this->parent->getChildren().pop(this);
				 }
			 }
			this->parent = node;
			if ( fix_references && node != NULL ) {
				node->getChildren().push_back(this, false); // add if not present
			 }
		 }


		void setContent( T* object, bool delete_previous ) {
			if ( delete_previous == true && this->content != NULL && this->content != object )
				delete this->content;
			this->content = object;
		 }

		const T* getContent() const					{ return this->content; }
		T* getContent()								{ return this->content; }


		inline void transactionBeginRead() const			{ this->mux.lockForRead(); }
		inline void transactionBeginWrite() const			{ this->mux.lockForWrite(); }
		inline void transactionEnd() const					{ this->mux.unLock(); }

		inline wosh::MutexRW* getMutex() const				{ return &this->mux; }


	protected:
		TreeNode22* parent;
		T* content;
		tTreeNode22List children;

		mutable MutexRW mux;

}; // class def

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <typename TREENODE_TYPE>
class Tree22
 {

	public:
		Tree22()
			: rootNode(NULL), tree_name("Tree") { }
		Tree22( const wosh::Tree22<TREENODE_TYPE>& m )
			: rootNode(NULL), tree_name(m.tree_name) {
			
		 }
		virtual ~Tree22() { }


		void free( bool nested, bool delete_content ) {
			if ( this->rootNode == NULL )
				return;
			this->rootNode->free(nested, delete_content);
			delete this->rootNode; this->rootNode = NULL;
		 }

	public:
		inline const TREENODE_TYPE* getRoot() const		{ return this->rootNode; }
		inline TREENODE_TYPE* getRoot()					{ return this->rootNode; }

		inline bool hasRoot() const							{ return (this->rootNode != NULL); }

		void setRoot( TREENODE_TYPE* root_node, bool deletePrevious = true ) {
			if ( deletePrevious && this->rootNode != NULL ) delete this->rootNode; this->rootNode = NULL;
			this->rootNode = root_node;
		 }

		inline bool isEmpty() const {
			if ( this->rootNode == NULL )
				return true;
			return this->rootNode->isEmpty();
		 }

		inline const std::string& getTreeName() const		{ return this->tree_name; }
		inline std::string& getTreeName()					{ return this->tree_name; }

		void setTreeName( const std::string& name ) 		{ this->tree_name = name; }

	protected:
		mutable TREENODE_TYPE* rootNode;
		std::string tree_name;							///< name of the tree

}; // class def

// ----------------------------------------------------------------- temp tree







 #define _Tree_TYPE			"wosh::Tree"


class TreeNode : public TreeNode22<ISerializable*>, public ISerializable
 {
	WOSH_SERIALIZABLE(wosh::TreeNode)
	WOSH_COMPARE_RTTI(wosh::TreeNode)

	//typedef List22<TreeNode*> tTreeNodeList;
	//typedef tTreeNodeList::Iterator ChildIterator;
	//typedef tTreeNodeList::ConstIterator ChildConstIterator;

	public:
		/**
		 * @brief  Empty constructor.
		 */
		TreeNode( TreeNode* node_parent, ISerializable* data = NULL )
			: TreeNode22<ISerializable*>(node_parent, data) { }

		TreeNode( const TreeNode& m )
			: TreeNode22<ISerializable*>(NULL, NULL) {
			*this = m;
		 }

		virtual ~TreeNode() {
			free(true, true);
		 }

	public:

/*		using TreeNode22<ISerializable*>::addChild;

		inline ChildConstIterator begin() const		{ return this->children.begin(); }
		inline ChildIterator begin()				{ return this->children.begin(); }

		inline ChildConstIterator end() const		{ return this->children.end(); }
		inline ChildIterator end()					{ return this->children.end(); }
*/

/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		virtual TreeNode& operator=( const TreeNode& m );

		virtual bool operator==( const TreeNode& ) const {

			return false;
		 }

		virtual bool operator!=( const TreeNode& other ) const	{ return !(*this == other);  }

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const {
			std::string temp;
			return temp;
		 }

		WRESULT store( IEncoderWriter* serializer ) const {
			if ( serializer == NULL ) return WRET_ERR_PARAM;
			return WRET_OK;
		 }
		WRESULT retrieve( IEncoderReader* deserializer ) {
			if ( deserializer == NULL ) return WRET_ERR_PARAM;
			return WRET_OK;
		 }
//@}




}; // class def



class Tree : public Tree22<TreeNode>, public ISerializable
 {
	WOSH_SERIALIZABLE(wosh::Tree)
	WOSH_COMPARE_RTTI(wosh::Tree)


	public:
		/**
		 * @brief  Empty constructor.
		 */
		Tree() : Tree22<TreeNode>() { }
		/**
		 * @brief  Copy constructor.
		 */
		Tree( const Tree& m ) : Tree22<TreeNode>(m) {
			*this = m;
		 }
		virtual ~Tree() {
			free(true, true);
		 }

	public:

		using Tree22<TreeNode>::getRoot;

/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		virtual Tree& operator=( const Tree& m ) {
			if (this == &m) return *this; // same object?
			// not optimized
			if ( m.getRoot() == NULL )
				setRoot( NULL, true );
			else
				setRoot( m.getRoot()->clone(), true );
			return *this;
		 }

		virtual bool operator==( const Tree& other ) const {
			if ( other.rootNode == NULL && this->rootNode == NULL )
				return true;
			if ( other.rootNode != NULL && this->rootNode != NULL )
				return (*other.rootNode == *this->rootNode);
			return false;
		 }

		virtual bool operator!=( const Tree& other ) const	{ return !(*this == other);  }

//@}
	public:
/*
		void dumpTree( int indend ) {
			ChildConstIterator it;
			const ChildConstIterator it_end = this->children.end();
			for( it=this->children.begin(); it!=it_end; ++it ) {
				int i=0; while(++i<=indend) { printf(" "); };

				if ( (*it)->getChildren().isEmpty() )
					printf( "-" );
				else
					printf( "+" );
				printf( " %p =>", *it );
				if ( (*it)->getContent() != NULL )
					printf( " %s\n", (*it)->getContent()->toString().c_str() );
				(*it)->dumpTree(indend+2);
			 }
		}
*/

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const {
			std::string temp;
			return temp;
		 }

		WRESULT store( IEncoderWriter* serializer ) const {
			if ( serializer == NULL ) return WRET_ERR_PARAM;
			return WRET_OK;
		 }
		WRESULT retrieve( IEncoderReader* deserializer ) {
			if ( deserializer == NULL ) return WRET_ERR_PARAM;
			return WRET_OK;
		 }
//@}

}; // class def



}; // namespace wosh

#endif //__WOSH_Core_Tree_H__
