/**
 * @class   wosh::automations::AutomationDeviceLink
 * @brief   Automation mapping devices events with method calls.
 *          Class supports multiple mapping, triggers are wosh::Fact
 *          Generated messages are posted on wosh.Bus.Devices,
 *
 ****************************************************************************
 * @version $Id: AutomationDeviceLink.h 2057 2010-03-08 21:32:15Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Automations_AutomationDeviceLink_H__
 #define __WOSH_Automations_AutomationDeviceLink_H__

 #include <framework/automations/Automation.h>

 #include <core/Message.h>
 #include <core/MessageFilter.h>


namespace wosh {
 namespace automations {

 #define _AutomationDeviceLink_TYPE				"wosh::automations::AutomationDeviceLink"
 #define _AutomationDeviceLink_VERSION			0.02


class DeviceTrigger {
	public:
		DeviceTrigger() { }
		virtual ~DeviceTrigger() { }

	public:
		virtual bool compare( const URI& event_source, const Fact* fact ) const;

	public:
		std::string source;
		std::string event_name;
		Variant event_data;

};

class DeviceEffect {
	public:
		DeviceEffect() { }
		virtual ~DeviceEffect() { }
	public:
		virtual Message* generateMessage() const;

	public:
		std::string target;
		std::string method_name;
		Variant method_arg;

};


class AutomationDeviceLink : public Automation
 {
	WOSH_SERIALIZABLE(wosh::automations::AutomationDeviceLink)
	WOSH_COMPARE_RTTI(wosh::automations::AutomationDeviceLink)

	public:

		AutomationDeviceLink();

		AutomationDeviceLink( const AutomationDeviceLink& m );

		virtual ~AutomationDeviceLink();

	public:

		virtual WRESULT addMapping( DeviceTrigger* fact_data, DeviceEffect* request_data );

//@}
/** @name Automation Interface - 
 * @{
 ******************************************************************************/
	public:
		virtual WRESULT execute( const Message* message = NULL, int64 time = 0 );

		virtual bool isTrigger( const Message* message = NULL, int64 time = 0 ) const;

/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:

		virtual AutomationDeviceLink& operator=(const AutomationDeviceLink& m);

		virtual bool operator==(const AutomationDeviceLink &other) const;

/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const;
		virtual WRESULT store( IEncoderWriter* serializer ) const;
		virtual WRESULT retrieve( IEncoderReader* deserializer );
//@}
	protected:

		DeviceEffect* getEffect_( const Message* message ) const;

	protected:

		std::map<DeviceTrigger*, DeviceEffect*> mappings;


}; // class def

 }; // namespace automations
}; // namespace wosh

#endif //__WOSH_Automations_AutomationDeviceLink_H__
