/** @file    BuildingGeometry.cpp
 *  @author  Alessandro Polo
 *  @version $Id: BuildingGeometry.cpp 1760 2010-02-02 05:45:11Z alex $
 *  @brief
 * File containing methods for the wosh::building::BuildingGeometry class.
 * The header for this class can be found in BuildingGeometry.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <framework/building/BuildingGeometry.h>

 #include <core/Table.h>


using namespace std;
namespace wosh {
 namespace building {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

BuildingGeometry::BuildingGeometry( )
 {
	clear();
 }

BuildingGeometry::~BuildingGeometry()
 {

 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void BuildingGeometry::clear()
 {
	this->position.x = this->position.y = this->position.z = 0;

	this->boundingBox.x = this->boundingBox.y = this->boundingBox.z = 0;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void BuildingGeometry::setPosition( const Point3D& pos ) {
	this->position = pos;
 }

void BuildingGeometry::setPosition( float x, float y, float z ) {
	this->boundingBox.x = x;
	this->boundingBox.y = y;
	this->boundingBox.z = z;
 }

void BuildingGeometry::setBox( float size_x, float size_y, float size_z )
 {
	this->boundingBox.x = size_x;
	this->boundingBox.y = size_y;
	this->boundingBox.z = size_z;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

BuildingGeometry& BuildingGeometry::operator=( const BuildingGeometry& m )
 {
	if (this == &m) return *this; // same object?

	this->position = m.position;
	this->boundingBox = m.boundingBox;

	return *this;
 }

bool BuildingGeometry::operator==( const BuildingGeometry &other ) const
 {
	if ( this->position.x != other.position.x ) return false;
	if ( this->position.y != other.position.y ) return false;
	if ( this->position.z != other.position.z ) return false;
	if ( this->boundingBox.x != other.boundingBox.x ) return false;
	if ( this->boundingBox.y != other.boundingBox.y ) return false;
	if ( this->boundingBox.z != other.boundingBox.z ) return false;

	return true;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::string  BuildingGeometry::toString() const
 {
	return "";
 }

WRESULT BuildingGeometry::store( IEncoderWriter* serializer ) const {
	if ( serializer == NULL ) return WRET_ERR_PARAM;
	WRESULT ret = WRET_OK;

	return ret;
 }
WRESULT BuildingGeometry::retrieve( IEncoderReader* deserializer ) {
	if ( deserializer == NULL ) return WRET_ERR_PARAM;
	WRESULT ret = WRET_OK;

	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace building
}; // namespace wosh
