/**
 * @class   wosh::building::BuildingGeometry
 * @brief   
 *
 * @see     wosh::building::BuildingObject
 ****************************************************************************
 * @version $Id: BuildingGeometry.h 1708 2010-01-26 22:28:01Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Building_BuildingGeometry_H__
 #define __WOSH_Building_BuildingGeometry_H__

 #include <woshDefs.h>
 #include <core/Serialization.h>



namespace wosh {
 namespace building {

 #define _BuildingGeometry_TYPE					"wosh::building::BuildingGeometry"

struct Point3D {
	float x;
	float y;
	float z;
 };

class BuildingGeometry : public virtual ISerializable
 {
	WOSH_SERIALIZABLE(wosh::building::BuildingGeometry)
	WOSH_COMPARE_RTTI(wosh::building::BuildingGeometry)

	public:
		/**
		 * @brief  Empty constructor.
		 */
		BuildingGeometry();

		/**
		 * @brief  Copy constructor.
		 */
		BuildingGeometry( const BuildingGeometry& m ) {
			*this = m;
		 }

		/**
		 * @brief  Empty destructor.
		 */
		virtual ~BuildingGeometry();


/** @name Getters
 * @{
 ******************************************************************************/
	public:
		const Point3D& getPosition() const		{ return this->position; }
		Point3D& getPosition()					{ return this->position; }

//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:
		void setPosition( const Point3D& position );
		void setPosition( float x, float y, float z = 0.0 );

		void setBox( float size_x, float size_y, float size_z );



//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		virtual BuildingGeometry& operator=(const BuildingGeometry& m);

		virtual bool operator==(const BuildingGeometry &other) const;
		virtual bool operator!=(const BuildingGeometry &other) const { return !(*this == other);  }

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const;
		virtual WRESULT store( IEncoderWriter* serializer ) const;
		virtual WRESULT retrieve( IEncoderReader* deserializer );
//@}
/** @name Internal mehods
 * @{
 ******************************************************************************/
	protected:
		virtual void clear();

//@}
/** @name Internal vars
 * @{
 ******************************************************************************/
	protected:

		Point3D position;		///< position relative to parent

		Point3D boundingBox;	///< size of bounding box, relative to (0,0,0)

//@}


}; // class def

 }; // namespace building
}; // namespace wosh

#endif //__WOSH_Building_BuildingGeometry_H__
