/** @file    Floor.cpp
 *  @author  Alessandro Polo
 *  @version $Id: Floor.cpp 2167 2010-03-18 06:41:05Z alex $
 *  @brief
 * File containing methods for the wosh::building::Floor class.
 * The header for this class can be found in Floor.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include <framework/building/Floor.h>

 #include <core/ObjectAllocator.h>
 #include <framework/building/Room.h>
 #include <framework/building/Appliance.h>

 #include <core/MethodsDef.h>


namespace wosh {
 namespace building {

 WOSH_REGISTER(wosh::building::Floor, "wosh::building::BuildingObject", _Floor_VERSION, _static_Floor )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

Floor::Floor() : BuildingObject()
 {
	BuildingObject::setName( Utilities::format("wosh.building.Floor#%ld", Utilities::randomLong()) );
 }

void Floor::init()
 {
	Log(LOG_VERBOSE, " Setting default properties and permissions.." );

	Log(LOG_VERBOSE, " Registering methods.." );
	MethodWRESULTwSecurityToken* mmLightsTurnOn = Methods.create<MethodWRESULTwSecurityToken>( _Room_METHOD_lights_on, "turn lights on" );
	mmLightsTurnOn->setMethod( this, (MethodWRESULTwSecurityTokenPtr)&Floor::mmDoLightsTurnOn );
	mmLightsTurnOn->getPermission().setMask( Permission::RX, Permission::RX, Permission::Read );

	MethodWRESULTwSecurityToken* mmLightsTurnOff = Methods.create<MethodWRESULTwSecurityToken>( _Room_METHOD_lights_off, "turn lights off" );
	mmLightsTurnOff->setMethod( this, (MethodWRESULTwSecurityTokenPtr)&Floor::mmDoLightsTurnOff );
	mmLightsTurnOff->getPermission().setMask( Permission::RX, Permission::RX, Permission::Read );

	BuildingObject::init();
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////// PROCESS CHILD MESSAGES

void Floor::busMessageChild( const Message& childMessage )
 {
	if ( childMessage.isEmpty() ) return;


 }

////////////////////////////////////////////////////////////////////////////////////// PROCESS CHILD MESSAGES
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////// GENERATED EVENTS

WRESULT Floor::mmDoLightsTurnOn( const SecurityToken* security )
 {
	Log(LOG_INFO, ":mmDoLightsTurnOn()" );
	// state update is asynch. raised by Child-Device (as Fact)
	Request requestApp( _Appliance_METHOD_on );
	requestApp.setOmitResponse(true);
	BuildingObject::controlChildren( _Appliance_TYPE, requestApp, security );

	Request requestRoom( _Room_METHOD_lights_on );
	requestRoom.setOmitResponse(true);
	BuildingObject::controlChildren( _Room_TYPE, requestRoom, security );

	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT Floor::mmDoLightsTurnOff( const SecurityToken* security )
 {
	Log(LOG_INFO, ":mmDoLightsTurnOff()" );
	// state update is asynch. raised by Child-Device (as Fact)
	Request requestApp( _Appliance_METHOD_off );
	requestApp.setOmitResponse(true);
	BuildingObject::controlChildren( _Appliance_TYPE, requestApp, security );

	Request requestRoom( _Room_METHOD_lights_off );
	requestRoom.setOmitResponse(true);
	BuildingObject::controlChildren( _Room_TYPE, requestRoom, security );

	return WRET_OK;
 }


//////////////////////////////////////////////////////////////////////////////////////////// GENERATED EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// DEVICE METHODS


////////////////////////////////////////////////////////////////////////////////////////////// DEVICE METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace building
}; // namespace wosh
