/**
 * @class   wosh::building::Floor
 * @brief   BuildingObject representing a generic (real) floor.
 *          wosh::building::Floor is usually a child of wosh::building::House
 *
 ****************************************************************************
 * @version $Id: Floor.h 2210 2010-03-26 14:27:24Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Building_Floor_H__
 #define __WOSH_Building_Floor_H__

 #include <framework/building/BuildingObject.h>


namespace wosh {
 namespace building {

 #define _Floor_TYPE					"wosh::building::Floor"
 #define _Floor_VERSION					0.02

 
class Floor : public BuildingObject
 {
	WOSH_BUILDING(wosh::building::Floor)

	public:
		/**
		 * @brief  Empty constructor.
		 */
		Floor();
		/**
		 * @brief  Copy constructor.
		 */
		Floor( const Floor& m ) : BuildingObject(m) { }
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~Floor() { }


	public:
		virtual void busMessageChild( const Message& message );

//@}
/** @name Floor Interface - 
 * @{
 ******************************************************************************/
	public:
		virtual WRESULT lightsTurnOn()				{ return mmDoLightsTurnOn(NULL); }
		virtual WRESULT lightsTurnOff()				{ return mmDoLightsTurnOff(NULL); }
//@}
	private:
		WRESULT mmDoLightsTurnOn( const SecurityToken* security );
		WRESULT mmDoLightsTurnOff( const SecurityToken* security );

	protected:
		virtual void init();

}; // class def

 }; // namespace building
}; // namespace wosh

#endif //__WOSH_Building_Floor_H__
