/**
 * @class   wosh::building::Garden
 * @brief   BuildingObject representing a generic (real) garden.
 *          wosh::building::Garden inherits wosh::building::Floor
 *          wosh::building::Garden is usually a child of wosh::building::House
 *
 *
 ****************************************************************************
 * @version $Id: Garden.h 1968 2010-02-27 05:52:38Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Building_Garden_H__
 #define __WOSH_Building_Garden_H__

 #include <framework/building/Floor.h>


namespace wosh {
 namespace building {

 #define _Garden_TYPE					"wosh::building::Garden"
 #define _Garden_VERSION				0.02

class Garden : public Floor
 {
	WOSH_BUILDING(wosh::building::Garden)

	public:
		/**
		 * @brief  Empty constructor.
		 */
		Garden() : Floor()
		 {
			BuildingObject::setName( Utilities::format("wosh.building.Garden#%ld", Utilities::randomLong()) );
			Interfaces.add( _Garden_TYPE );
		 }
		/**
		 * @brief  Copy constructor.
		 */
		Garden( const Garden& m ) : Floor(m) { }
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~Garden() { }


}; // class def

 }; // namespace building
}; // namespace wosh

#endif //__WOSH_Building_Garden_H__
