/** @file    Location.cpp
 *  @author  Alessandro Polo
 *  @version $Id: Location.cpp 2080 2010-03-10 05:38:54Z alex $
 *  @brief
 * File containing methods for the wosh::building::Location class.
 * The header for this class can be found in Location.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include <framework/building/Location.h>


namespace wosh {
 namespace building {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

Location::Location() : Object()
 {
	Object::setName( Utilities::format("wosh.building.Location#%ld", Utilities::randomLong()) );

	this->locationType = TYPE_UNKNOWN;

 }

Location::Location( const Location& m ) : Object(m)
 {
	*this = m;
 }


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Location::setParent( Location* location, bool update_parent )
 {
	if ( location == NULL ) return;
	this->parentName = location->getName();
	if ( update_parent )
		location->addChild(this, false);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Location::addChild( Location* location, bool update_parent )
 {
	if ( location == NULL ) return;
	this->children.push_back( location->getName() );
	if ( update_parent )
		location->setParent(this, false);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Location::addAccessTo( Location* location, bool update_other, double probability )
 {
	if ( location == NULL ) return;
	if ( update_other )
		location->addAccessTo(this, false, probability);
	if ( probability == -1.0 ) {
		probability = 1.0 / (this->accessTo.size() + 1);
	 }
	this->accessTo[location->getName()] = probability;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool Location::isUserInLocation( const std::string& username ) const
 {
	if ( this->users.find(username) == this->users.end() )
		return false;
	if ( this->users[username].timestamp <= 0 )
		return false;
	if ( this->users[username].probability == 0.0 )
		return false;
	return true;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::vector<std::string> Location::getDevicesURI() const
 {
	std::vector<std::string> tmp;
	std::map<URI, std::string>::const_iterator it;
	std::map<URI, std::string>::const_iterator it_end = this->devices.end();
	for ( it=this->devices.begin(); it!=it_end; ++it ) {
		tmp.push_back(it->first.toString());
	 }
	return tmp;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::vector<std::string> Location::getUsersName() const
 {
	std::vector<std::string> tmp;
	std::map<std::string, UserInfoLocationData>::const_iterator it;
	std::map<std::string, UserInfoLocationData>::const_iterator it_end = this->users.end();
	for ( it=this->users.begin(); it!=it_end; ++it ) {
		tmp.push_back(it->first);
	 }
	return tmp;
 }

std::string Location::getUser( unsigned int index ) const
 {
	unsigned int i = 0;
	std::map<std::string, UserInfoLocationData>::const_iterator it;
	std::map<std::string, UserInfoLocationData>::const_iterator it_end = this->users.end();
	for ( it=this->users.begin(); it!=it_end; ++it, ++i ) {
		if ( i == index )
			return it->first;
	 }
	return "";
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace building
}; // namespace wosh
