/**
 * @file    framework/communication.h
 * @brief   Include all components of the communication framework
 * @version 0.60 $Id: communication.h 1690 2010-01-25 05:19:57Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/*! @namespace wosh::communication
 * @brief     Contains classes regarding communication. They represent common
 *            messages (textual, audio) and provide user sessions (required
 *            by Session server and remote control through instant messaging.
 *            Refer to wosh::Notification interface and wosh::services::CommunicationManagerBundle.
 *
 ****************************************************************************/
/*! \page page_framework
 * \htmlonly <hr/> \endhtmlonly
 * \section page_framework_communication Communication
 *
 * [..]
 * Sessions, Messaging!
 *
 * wosh::Notification is the base object (interface) ...
 *  - wosh::communication::AudioMessage
 *  - wosh::communication::TextMessage
 *
 * Interpreter services (implementing  interface) translates notifications..
 *
 * wosh::communication::NotificationConversion
 * 
 *
 * Communicator services should use ..
 *  - wosh::communication::ContactInfo
 *
 * wosh::communication::NotificationFlow holds the whole flow of a notif..
 *
 * wosh::Communicator wosh::Interpreter
 *
 * wosh::services::CommunicationManagerBundle
 *
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_FrameWork_Communication_H__
 #define __WOSH_FrameWork_Communication_H__

 #include <woshDefs.h>

namespace wosh {
 namespace communication { };
};

 #include <framework/communication/AudioMessage.h>
 #include <framework/communication/ContactInfo.h>
 #include <framework/communication/ImageMessage.h>
 
 #include <framework/communication/NotificationConversion.h>
 #include <framework/communication/NotificationFlow.h>
 
 #include <framework/communication/Session.h>
 #include <framework/communication/SessionAsynch.h>
 #include <framework/communication/SessionIM.h>
 #include <framework/communication/SessionProtocol.h>
 #include <framework/communication/SessionProtocolShell.h>
 #include <framework/communication/SessionShell.h>
 #include <framework/communication/ShellCommand.h>

 #include <framework/communication/TextMessage.h>
 


#endif //__WOSH_FrameWork_Communication_H__
