/** @file    ImageMessage.cpp
 *  @author  Alessandro Polo
 *  @version $Id: ImageMessage.cpp 1708 2010-01-26 22:28:01Z alex $
 *  @brief
 * File containing methods for the wosh::communication::ImageMessage class.
 * The header for this class can be found in ImageMessage.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include <framework/communication/ImageMessage.h>

 #include <core/ObjectAllocator.h>
 #include <core/Utilities.h>


using namespace std;
namespace wosh {
 namespace communication {

 WOSH_REGISTER(wosh::communication::ImageMessage, "wosh::Notification", 1.0, _static_ImageMessage )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

ImageMessage::ImageMessage() : Notification()
 {
	clear();
 }

ImageMessage::ImageMessage( const std::string& filename ) : Notification()
 {
	clear();
	this->file_path = filename;
 }

ImageMessage::ImageMessage( const ImageMessage& m ) : Notification(m)
 {
	clear();
	*this = m;
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////// CONFIGURATION

void ImageMessage::clear()
 {
	Notification::clear();

	this->protocol = Notification::PROTOCOL_HUMAN;

	this->file_path = "";
	//this->file_uri.clear();
 }


ImageMessage& ImageMessage::operator=( const ImageMessage& m )
 {
	if (this == &m) return *this; // same object?

	(Notification&)*this = (const Notification&)m;

	//this->file_uri = m.file_uri;
	this->file_path = m.file_path;
	this->optPlain = m.optPlain;

	return *this;
 }

bool ImageMessage::operator==( const ImageMessage &other ) const
 {
	if ( (Notification&)*this != (const Notification&)other )
		return false;

	if ( this->file_path != other.file_path )
		return false;
//	if ( this->file_uri != other.file_uri )
//		return false;

	return true;
 }

//////////////////////////////////////////////////////////////////////////////////////////////// DEBUG & INFO
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool ImageMessage::fromNotification( const Notification* notif )
 {
	if ( notif == NULL ) return false;

	setFlowID( notif->getFlowID() );

	this->sender_id = notif->getSender_ID();
	this->sender_user = notif->getSender_User();
	this->recipent_id = notif->getRecipent_ID();
	this->recipent_user = notif->getRecipent_User();
	this->recipent_group = notif->getRecipent_Group();

	// still keep text
	this->optPlain = notif->getPlain();

	return true;
 }

/////////////////////////////////////////////////////////////////////////////////////////////// SERIALIZATION

std::string ImageMessage::toString() const
 {
	return "";
 }

WRESULT ImageMessage::store( IEncoderWriter* serializer ) const {
	if ( serializer == NULL ) return WRET_ERR_PARAM;

	return WRET_OK;
 }
WRESULT ImageMessage::retrieve( IEncoderReader* deserializer ) {
	if ( deserializer == NULL ) return WRET_ERR_PARAM;

	return WRET_OK;
 }

/////////////////////////////////////////////////////////////////////////////////////////////// SERIALIZATION
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace communication
}; // namespace wosh
