/** @file    SessionIM.cpp
 *  @author  Alessandro Polo
 *  @version $Id: SessionIM.cpp 2167 2010-03-18 06:41:05Z alex $
 *  @brief
 * File containing methods for the wosh::communication::SessionIM class.
 * The header for this class can be found in SessionIM.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <framework/communication/SessionIM.h>

 #include <core/ObjectAllocator.h>
 #include <core/Utilities.h>
 #include <core/WoshKernel.h>
 #include <core/Request.h>
 #include <core/SecurityManager.h>

 #include <framework/communication/TextMessage.h>

 #define _ENDLINE	"\r\n"

 
using namespace std;
using namespace wosh::communication;
namespace wosh {
 namespace communication {

 WOSH_REGISTER(wosh::communication::SessionIM, "wosh::communication::Session", 1.0, _static_SessionIM )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

SessionIM::SessionIM() : Session(), Thread(NULL, "wosh.SessionIM.Thread")
 {
	// override some base class fields
	this->skipWelcome = true;
 }

SessionIM::~SessionIM() {
	if ( isThreadRunning() )
		sessionStop();
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT SessionIM::sessionStart( unsigned long timeout )
 {
	if ( this->parentSServer == NULL )
		return WRET_ERR_ILLEGAL_USE;
	if ( this->protocol == NULL )
		return WRET_ERR_ILLEGAL_USE;
	return startThread(timeout);
 }

WRESULT SessionIM::sessionStop( unsigned long timeout ) {
	return quitThread(timeout);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool SessionIM::initializingThread()
 {
	setSessionState(Session::CONNECTED);

	if ( !this->skipWelcome ) {
		sendMessageToCommunicator( this->msgWelcome );
	 }

	setSessionState(Session::OPENED);
	return true;
 }

void SessionIM::runThread()
 {
	this->running = true;
	while( this->running )
	 {
		Thread::sleepForMSec(100);
		setThreadAlive();

		if ( this->incomingMsg.isEmpty() ) {
			Thread::sleepForMSec(100);
			continue;
		 }

		Message* message = this->incomingMsg.pop();
		this->protocol->evalMessage( *this, message );

		delete message;
	 }
 }

void SessionIM::exitingThread()
 {
	setSessionState(Session::CLOSED);

	sendMessageToCommunicator( "Logged Out." );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SessionIM::sendMessageToCommunicator( const string& message )
 {
	TextMessage* txtmsg = new TextMessage();
	txtmsg->setSender_User( "wosh" );
	txtmsg->setRecipent_User( getUsername() );
	txtmsg->setProtocol( Notification::PROTOCOL_SHELL );
	txtmsg->setLanguage( "en_GB" );
	txtmsg->setPlain( message );
	txtmsg->setPreferredCommunicator(this->communicator_uri);

	Message* msg_notif = new Message(txtmsg);
	msg_notif->setSource(this);
	msg_notif->getDestination().fromString("*:Communication");
	msg_notif->setSecurityToken( SecurityManager::generateToken(getUsername()) );

	bool sent = this->parentSServer->session_post_message( this, msg_notif );
	(void)sent;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SessionIM::evalNotificationInput( const Notification* notif, const Message& )
 {
	if ( notif == NULL )
		return;

	parseMessage( notif->getPlain() );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SessionIM::setCommunicator( const URI& comm_uri )
 {
	this->communicator_uri = comm_uri;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace communication
}; // namespace wosh
