/**
 * @class   wosh::communication::SessionIM
 * @brief   The class represent and manage life of an IM (instant messaging)
 *          session.
 *
 * @see     wosh::services::SessionServerBundle
 ****************************************************************************
 * @version $Id: SessionIM.h 2121 2010-03-13 18:02:40Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Communication_SessionIM_H__
 #define __WOSH_Communication_SessionIM_H__

 #include <framework/communication/Session.h>
 #include <framework/communication/ShellCommand.h>
 #include <framework/communication/SessionProtocol.h>
 #include <core/Thread.h>



namespace wosh {
 namespace communication {

 #define _SessionIM_TYPE			"wosh::communication::SessionIM"


class SessionIM :	public Session,
					protected Thread
 {
	WOSH_SESSION(wosh::communication::SessionIM)


	public:

		SessionIM();

		virtual ~SessionIM();



		virtual void evalNotificationInput( const Notification* notif, const Message& message );


/** @name Session Interface
 * @{
 ******************************************************************************/
	public:

		virtual WRESULT sessionStart( unsigned long timeout = LIMIT_ULONG_MAX );
		virtual WRESULT sessionStop( unsigned long timeout = LIMIT_ULONG_MAX );

		virtual void printMessage( const std::string& msg ) {
			sendMessageToCommunicator(msg);
		 }


//@}
/** @name Getters
 * @{
 ******************************************************************************/
 	public:


//@}
/** name Setters
 * @{
 ******************************************************************************/
 	public:
		void setCommunicator( const URI& comm_uri );

//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:

		void sendMessageToCommunicator( const std::string& message );

		virtual bool initializingThread();
		virtual void runThread();
		virtual void exitingThread();

	protected:


		URI communicator_uri;


}; // class def

 }; // namespace communication
}; // namespace wosh

#endif //__WOSH_Communication_SessionIM_H__
