/**
 * @class   wosh::communication::SessionProtocol
 * @brief   The class provides processing of input commands (by the user) and
 *          responses (by the system).
 *
 ****************************************************************************
 * @version $Id: SessionProtocol.h 2214 2010-03-26 16:03:31Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Communication_SessionProtocol_H__
 #define __WOSH_Communication_SessionProtocol_H__

 #include <framework/communication/Session.h>

namespace wosh {
 namespace communication {


class SessionProtocol  {

	public:
		SessionProtocol()
			: parent(NULL) { }

		virtual ~SessionProtocol() { }

	public:

		virtual std::string getProtocolName() const = 0;

		virtual double getProtocolVersion() const = 0;


		virtual void printPrompt( Session& session ) const = 0;

		virtual void parse_command( Session& session, const std::string& dataline ) = 0;

		virtual void evalMessage( Session& session, const Message* message ) = 0;


		void setSessionServer( wosh::interfaces::services::SessionServer* server )		{ this->parent = server; }

	protected:

		wosh::interfaces::services::SessionServer* parent;	///< dad Shell Server
		friend class wosh::interfaces::services::SessionServer;

}; // class def

 }; // namespace communication
}; // namespace wosh

#endif //__WOSH_Communication_SessionProtocol_H__
