/**
 * @class   wosh::communication::SessionProtocolShell
 * @brief   SessionProtocol using the shell syntax, very similar to POSIX.
 *
 * @see     wosh::services::SessionServerBundle
 ****************************************************************************
 * @version $Id: SessionProtocolShell.h 2050 2010-03-07 01:58:01Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Communication_SessionProtocolShell_H__
 #define __WOSH_Communication_SessionProtocolShell_H__

 #include <framework/communication/SessionProtocol.h>
 #include <framework/communication/ShellCommand.h>

 #include <map>

namespace wosh {
 class NEW_List;
 class Table;
 namespace communication {


class SessionProtocolShell : public SessionProtocol  {

	public:
		SessionProtocolShell();
		virtual ~SessionProtocolShell();

	public:

		virtual std::string getProtocolName() const			{ return "shell"; }

		virtual double getProtocolVersion() const 			{ return 0.1; }


		// build prompt 
		virtual void printPrompt( Session& session ) const;

		virtual void parse_command( Session& session, const std::string& dataline );

		virtual void evalMessage( Session& session, const Message* message );


	protected:
		bool internal_command( Session& session, ShellCommand* command );

		void renderList( Session& session, const wosh::NEW_List* list, const std::string& title );
		void renderTable( Session& session, const wosh::Table* table, const std::string& title );

		void renderMan( Session& session, const Response* response, const Message* message );

	protected:
		std::map<std::string, ShellCommand*> aliases;
		ShellCommand lastCommand;

		friend class wosh::interfaces::services::SessionServer;

}; // class def

 }; // namespace communication
}; // namespace wosh

#endif //__WOSH_Communication_SessionProtocolShell_H__
