/**
 * @class   wosh::communication::SessionShell
 * @brief   The class represent and manage life of an local shell session.
 *          
 *
 * @see     woshsrv
 * @see     wosh::services::SessionServerBundle
 ****************************************************************************
 * @version $Id: SessionShell.h 2082 2010-03-11 06:26:42Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Communication_SessionShell_H__
 #define __WOSH_Communication_SessionShell_H__

 #include <framework/communication/Session.h>
 #include <framework/communication/ShellCommand.h>
 #include <framework/communication/SessionProtocol.h>
 #include <core/Thread.h>

 #include <map>


namespace wosh {
 namespace communication {

 #define _SessionShell_TYPE			"wosh::communication::SessionShell"


class SessionShell :	public Session,
						protected Thread
 {
	WOSH_SESSION(wosh::communication::SessionShell)


	public:

		SessionShell();

		virtual ~SessionShell();


/** @name Session Interface
 * @{
 ******************************************************************************/
	public:

		virtual WRESULT sessionStart( unsigned long timeout = LIMIT_ULONG_MAX );
		virtual WRESULT sessionStop( unsigned long timeout = LIMIT_ULONG_MAX );

		virtual inline std::istream& istream()				{ return *this->streamInput; }
		virtual inline std::ostream& ostream()				{ return *this->streamOutput; }

		virtual void printMessage( const std::string& msg ) {
			ostream() << msg << std::flush;
		 }

//@}
/** @name Getters
 * @{
 ******************************************************************************/
 	public:

		inline std::iostream& getStreamOwn()				{ return *this->streamOwn; }

//@}
/** name Setters
 * @{
 ******************************************************************************/
 	public:

		WRESULT setStreamInput( std::istream* input_stream );
		WRESULT setStreamOutput( std::ostream* output_stream );

		WRESULT setStreamStdIO();
		WRESULT setStreamOwn( std::iostream* ios = NULL );

//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:

		virtual bool initializingThread();
		virtual void runThread();
		virtual void exitingThread();

//@}
	protected:

		std::istream* streamInput;		///< pointer to current INPUT stream
		std::ostream* streamOutput;		///< pointer to current OUTPUT stream

		std::iostream* streamOwn;		///< I/O stream owned by me, using stringbuf


}; // class def

 }; // namespace communication
}; // namespace wosh

#endif //__WOSH_Communication_SessionShell_H__
