/** @file    TextMessage.cpp
 *  @author  Alessandro Polo
 *  @version $Id: TextMessage.cpp 2167 2010-03-18 06:41:05Z alex $
 *  @brief
 * File containing methods for the wosh::communication::TextMessage class.
 * The header for this class can be found in TextMessage.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include <framework/communication/TextMessage.h>

 #include <core/ObjectAllocator.h>
 #include <core/Utilities.h>


using namespace std;
namespace wosh {
 namespace communication {

 WOSH_REGISTER(wosh::communication::TextMessage, "wosh::Notification", 1.0, _static_TextMessage )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

TextMessage::TextMessage() : Notification()
 {
	clear();
 }

TextMessage::TextMessage( const std::string& Plain ) : Notification()
 {
	clear();
	this->subject = "";
	this->plain = Plain;
 }

TextMessage::TextMessage( const std::string& Subject, const std::string& Plain ) : Notification()
 {
	clear();
	this->subject = Subject;
	this->plain = Plain;
 }

TextMessage::TextMessage( const TextMessage& m ) : Notification(m)
 {
	clear();
	*this = m;
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////// CONFIGURATION

void TextMessage::clear()
 {
	Notification::clear();

	this->messageType = TextMessage::MESSAGE_GENERIC;

	this->subject = "";
	this->plain = "";
 }


TextMessage& TextMessage::operator=( const TextMessage& m )
 {
	if (this == &m) return *this; // same object?

	(Notification&)*this = (const Notification&)m;

	this->subject = m.subject;
	this->plain = m.plain;
	this->messageType = m.messageType;

	return *this;
 }

bool TextMessage::operator==( const TextMessage &other ) const
 {
	if ( (Notification&)*this != (const Notification&)other )
		return false;

	if ( this->subject != other.subject )
		return false;

	if ( this->plain != other.plain )
		return false;

	if ( this->messageType != other.messageType )
		return false;

	return true;
 }

//////////////////////////////////////////////////////////////////////////////////////////////// DEBUG & INFO
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::string TextMessage::getSummary( int maxsize ) const
 {
	 if ( this->plain.size() == 0 ) {
		 if ( maxsize < 0 || this->subject.size() < (unsigned int)maxsize )
			 return this->subject;
		 else
			 return this->subject.substr(0, maxsize);
	  }
	 if ( this->subject.size() == 0 ) {
		 if ( maxsize < 0 || this->plain.size() < (unsigned int)maxsize )
			 return this->plain;
		 else
			 return this->plain.substr(0, maxsize);
	  }

	 // to do better...
	if ( maxsize < 0 || this->subject.size() < (unsigned int)maxsize )
		return this->subject;
	else
		return this->subject.substr(0, maxsize);
 }

/////////////////////////////////////////////////////////////////////////////////////////////// SERIALIZATION

std::string TextMessage::toString() const
 {
	return "";
 }

WRESULT TextMessage::store( IEncoderWriter* serializer ) const {
	if ( serializer == NULL ) return WRET_ERR_PARAM;

	return WRET_OK;
 }
WRESULT TextMessage::retrieve( IEncoderReader* deserializer ) {
	if ( deserializer == NULL ) return WRET_ERR_PARAM;

	return WRET_OK;
 }

/////////////////////////////////////////////////////////////////////////////////////////////// SERIALIZATION
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool TextMessage::fromNotification( const Notification* notif )
 {
	if ( notif == NULL ) return false;

	setFlowID( notif->getFlowID() );

	this->sender_id = notif->getSender_ID();
	this->sender_user = notif->getSender_User();
	this->recipent_id = notif->getRecipent_ID();
	this->recipent_user = notif->getRecipent_User();
	this->recipent_group = notif->getRecipent_Group();

	this->plain = notif->getPlain();

	return true;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace communication
}; // namespace wosh
