/**
 * @file   CoderXML_Automations.h
 * @brief   
 *
 ****************************************************************************
 * @version $Id: CoderXML_Automations.h 2057 2010-03-08 21:32:15Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_DataBase_CoderXML_Automations_H__
 #define __WOSH_DataBase_CoderXML_Automations_H__

 #include <framework/database/SerializableXML.h>

 #include <framework/automations/Automation.h>
 #include <framework/automations/AutomationMessage.h>
 #include <framework/automations/AutomationDeviceLink.h>
 #include <framework/automations/CronJob.h>


using namespace wosh;
namespace wosh {
 namespace database {

void EncoderXML_register_Automations();

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__Automation : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return NULL; }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__Automation() { }
		virtual ~EncoderXML_template__Automation() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__AutomationMessage : public EncoderXML_template__Automation  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::automations::AutomationMessage(); }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__AutomationMessage() : EncoderXML_template__Automation() { }
		virtual ~EncoderXML_template__AutomationMessage() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__AutomationDeviceLink : public EncoderXML_template__Automation  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::automations::AutomationDeviceLink(); }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__AutomationDeviceLink() : EncoderXML_template__Automation() { }
		virtual ~EncoderXML_template__AutomationDeviceLink() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__CronJob : public EncoderXML_template__AutomationMessage  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::automations::CronJob(); }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__CronJob() : EncoderXML_template__AutomationMessage() { }
		virtual ~EncoderXML_template__CronJob() { }

}; // class def


//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace database
}; // namespace wosh

#endif //__WOSH_DataBase_CoderXML_Automations_H__
