/**
 * @file   CoderXML_Communication.h
 * @brief   
 *
 ****************************************************************************
 * @version $Id: CoderXML_Communication.h 1968 2010-02-27 05:52:38Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_DataBase_CoderXML_Communication_H__
 #define __WOSH_DataBase_CoderXML_Communication_H__

 #include <framework/database/SerializableXML.h>

 #include <framework/communication/AudioMessage.h>
 #include <framework/communication/NotificationConversion.h>
 #include <framework/communication/TextMessage.h>
 #include <framework/communication/VoiceCall.h>

using namespace wosh;
namespace wosh {
 namespace database {

void EncoderXML_register_Communication();

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__AudioMessage : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::communication::AudioMessage(); }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__AudioMessage() { }
		virtual ~EncoderXML_template__AudioMessage() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__NotificationConversion : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::communication::NotificationConversion(); }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__NotificationConversion() { }
		virtual ~EncoderXML_template__NotificationConversion() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__TextMessage : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::communication::TextMessage(); }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__TextMessage() { }
		virtual ~EncoderXML_template__TextMessage() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__VoiceCall : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::communication::VoiceCall(); }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__VoiceCall() { }
		virtual ~EncoderXML_template__VoiceCall() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace database
}; // namespace wosh

#endif //__WOSH_DataBase_CoderXML_Communication_H__
