/**
 * @file   CoderXML_Core.h
 * @brief   
 *
 ****************************************************************************
 * @version $Id: CoderXML_Core.h 2143 2010-03-16 14:00:54Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_DataBase_CoderXML_Core_H__
 #define __WOSH_DataBase_CoderXML_Core_H__

 #include <framework/database/SerializableXML.h>

 #include <core/Variant.h>
 #include <core/NEW_List.h>
 #include <core/Table.h>
 #include <core/Permission.h>
 #include <core/Property.h>
 #include <core/PropertiesProvider.h>
 #include <core/Object.h>

 #include <core/Message.h>
 #include <core/MessageFilter.h>
 #include <core/SecurityToken.h>
 #include <core/MessageNetworkForwarding.h>
 #include <core/Request.h>
 #include <core/Response.h>
 #include <core/Fact.h>
 #include <core/Notification.h>
 #include <core/SystemError.h>

 #include <core/Communicator.h>
 
 #include <core/User.h>
 #include <core/WoshHost.h>


using namespace wosh;
namespace wosh {
 namespace database {

void EncoderXML_register_Core();

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__Communicator : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::Communicator(); }
	
		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__Communicator() { }
		virtual ~EncoderXML_template__Communicator() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__Variant : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::Variant(); }
	
		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__Variant() { }
		virtual ~EncoderXML_template__Variant() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__NEW_List : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::NEW_List(); }
	
		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__NEW_List() { }
		virtual ~EncoderXML_template__NEW_List() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__Table : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::Table(); }
	
		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__Table() { }
		virtual ~EncoderXML_template__Table() { }

}; // class def


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__Permission : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::Permission(); }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__Permission() { }
		virtual ~EncoderXML_template__Permission() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__URI : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::URI(); }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__URI() { }
		virtual ~EncoderXML_template__URI() { }

}; // class def


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__Object : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return NULL; }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__Object() { }
		virtual ~EncoderXML_template__Object() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__Property : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::Property(); }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__Property() { }
		virtual ~EncoderXML_template__Property() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__PropertiesProvider : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::PropertiesProvider(); }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__PropertiesProvider() { }
		virtual ~EncoderXML_template__PropertiesProvider() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__Message : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::Message(); }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__Message() { }
		virtual ~EncoderXML_template__Message() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__MessageFilter : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::MessageFilter(); }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__MessageFilter() { }
		virtual ~EncoderXML_template__MessageFilter() { }

}; // class def


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__SecurityToken : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return NULL; }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__SecurityToken() { }
		virtual ~EncoderXML_template__SecurityToken() { }

}; // class def


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__MessageNetworkForwarding : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::MessageNetworkForwarding(); }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__MessageNetworkForwarding() { }
		virtual ~EncoderXML_template__MessageNetworkForwarding() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__Request : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::Request(); }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__Request() { }
		virtual ~EncoderXML_template__Request() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__Response : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::Response(); }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__Response() { }
		virtual ~EncoderXML_template__Response() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__Fact : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::Fact(); }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__Fact() { }
		virtual ~EncoderXML_template__Fact() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__Notification : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return NULL; }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__Notification() { }
		virtual ~EncoderXML_template__Notification() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__SystemError : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::SystemError(); }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__SystemError() { }
		virtual ~EncoderXML_template__SystemError() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__User : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::User(); }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__User() { }
		virtual ~EncoderXML_template__User() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class EncoderXML_template__WoshHost : public EncoderXML_base  {

	public:
		virtual ISerializable* allocObject()		{ return new wosh::WoshHost(); }

		virtual WRESULT serializeTo( const ISerializable* object, QDomDocument& doc, QDomElement& contNode, bool compact );

		virtual WRESULT deserializeFrom( ISerializable* object, const QDomElement& contNode );

	public:
		EncoderXML_template__WoshHost() { }
		virtual ~EncoderXML_template__WoshHost() { }

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace database
}; // namespace wosh

#endif //__WOSH_DataBase_CoderXML_Core_H__
