/**
 * @class   wosh::database::EncoderSQL
 * @brief   
 *
 *
 * apt-get install libqt4-sql libqt4-sql-mysql
 ****************************************************************************
 * @version $Id: EncoderSQL.h 2067 2010-03-09 23:40:30Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_EncoderSQL_H__
 #define __WOSH_EncoderSQL_H__

 #include <woshDefs.h>
 #include <core/DataBase.h>
 #include <core/Variant.h>
 #include <core/Message.h>
 #include <core/Mutex.h>
 #include <core/Serialization.h>
 #include <core/LogContext.h>

 #include <framework/database/SerializableSQL.h>
 #include <framework/database/EncoderSQLCommon.h>

 #include <map>

 #include <QtSql>
 #include <QByteArray>


namespace wosh {
 namespace database {

 #define _DataBaseSQL_NAME					"EncoderSQL"
 #define _DataBaseSQL_TYPE					"wosh::EncoderSQL"

 #define _DataBaseSQL_PROTOCOL				"sql"
 #define _DataBaseSQL_SqlRootNodeName		"WOSH"


 #define SQL_QUERY_SECURITY	"INSERT INTO `security` ( `id` , `sw_id` , `hw_id` , `event` , `timeref` , `source` , `level` , `notes` ) VALUES ( NULL , '%1', '%2', '%3', NOW( ) , '%4', '%5', '%6' ); "
 #define SQL_QUERY_DEVICES		"SELECT * FROM `devices`;"
 
 
class EncoderSQL : public DataBase
 {
	WOSH_DATABASE(wosh::database::EncoderSQL)

	public:

		EncoderSQL();

		virtual ~EncoderSQL();

	public:

		virtual std::string getProtocol() const			{ return _DataBaseSQL_PROTOCOL; }

		virtual WRESULT loadObjects( std::vector<ISerializable*>& objects, const std::string& class_name, const std::string& table_name );

		virtual WRESULT saveObjects( std::vector<ISerializable*>& objects, const std::string& table_name );

	public:

		static WRESULT registerDataType( const std::string& class_name, EncoderSQL_base* object );

		static EncoderSQL_base* findEncoderFor( const std::string& tagname );


	protected:

		template <class T>
		static WRESULT serializeTo( const T* object, QSqlQuery& query )
		 {
			if ( EncoderSQL::objectRegs.find( object->getClassName() ) == EncoderSQL::objectRegs.end() ) {
				return WRET_ERR_PARAM;
			 }
			EncoderSQL_base* encoder = EncoderSQL::objectRegs[ string(object->getClassName()) ];

			return encoder->serializeTo( object, query );
		 }

		template <class T>
		static WRESULT deserializeFrom( T* object, const QSqlQuery& query )
		 {

			if ( EncoderSQL::objectRegs.find( object->getClassName() ) == EncoderSQL::objectRegs.end() ) {
				return WRET_ERR_PARAM;
			 }
			EncoderSQL_base* encoder = EncoderSQL::objectRegs[ string(object->getClassName()) ];

			return encoder->deserializeFrom( object, query );
		 }
/*
		static EncoderSQL_base* getCustomEncoder( std::string class_name )
		 {
			if ( EncoderSQL::objectRegs.find( class_name ) == EncoderSQL::objectRegs.end() ) {
				return NULL;
			 }
			return EncoderSQL::objectRegs[ class_name ];
		 }
*/

	protected:

		void freeObjecsRegs_();

		WRESULT checkConnect();

	protected:

		LogContext Log;

		static std::map<std::string, EncoderSQL_base*> objectRegs;
		static Mutex objRegsMux;

		static QSqlDatabase dbSQL;



}; // class def

 }; // namespace database
}; // namespace wosh

#endif //__WOSH_EncoderSQL_H__
