/**
 * @class   wosh::entertainment::MediaTrack
 * @brief   
 *
 * used inside Palyers devices too!?
 * used inside PlayList (MediaList)
 *
 *
 ****************************************************************************
 * @version $Id: MediaTrack.h 2142 2010-03-16 12:45:50Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Entertainment_MediaTrack_H__
 #define __WOSH_Entertainment_MediaTrack_H__

 #include <woshDefs.h>
 #include <core/ResourcePath.h>
 #include <core/PropertiesProvider.h>


namespace wosh {
 namespace entertainment {

 #define _MediaTrack_TYPE				"wosh::entertainment::MediaTrack"

 #define _MediaTrack_KEY_Length			"Length"

 #define _MediaTrack_KEY_Artist			"Artist"
 #define _MediaTrack_KEY_Album			"Album"
 #define _MediaTrack_KEY_Title			"Title"
 #define _MediaTrack_KEY_Genre			"Genre"

 #define _MediaTrack_KEY_TrackNumber	"TrackNumber"
 #define _MediaTrack_KEY_Description	"Description"


class MediaTrack : public virtual ISerializable
 {
	WOSH_SERIALIZABLE(wosh::entertainment::MediaTrack)
	WOSH_COMPARE_RTTI(wosh::entertainment::MediaTrack)

	public:

		/**
		 * @brief   Default constructor. Init vars.
		 */
		MediaTrack() { }

		MediaTrack( const ResourcePath& fullpath ) {
			setTrack(fullpath);
		 }
		/**
		 * @brief   Copy constructor.
		 */
		MediaTrack( const MediaTrack& m ) {
			*this = m;
		 }
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~MediaTrack() { }


/** @name Setters
 * @{
 ******************************************************************************/
	public:
		void setTrack( const wosh::ResourcePath& value )			{ this->track = value; }

		void setArtist( const std::string& artist ) {
			Properties.update( _MediaTrack_KEY_Artist, artist );
		 }
		void setAlbum( const std::string& album ) {
			Properties.update( _MediaTrack_KEY_Album, album );
		 }
		void setTitle( const std::string& title ) {
			Properties.update( _MediaTrack_KEY_Title, title );
		 }
		void setGenre( const std::string& gerne ) {
			Properties.update( _MediaTrack_KEY_Genre, gerne );
		 }

		void setLength( long seconds ) {
			Properties.update( _MediaTrack_KEY_Length, Variant(seconds, Variant::TIMESPAN) );
		 }

		void clear() {
			this->track.clear();
			Properties.clear();
		 }
//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:

	//	long getID() const								{ return this->id; }
	//	long getTimeStamp() const						{ return this->timestamp; }

		const wosh::ResourcePath& getTrack() const				{ return this->track; }
		wosh::ResourcePath& getTrack()							{ return this->track; }

		const wosh::PropertiesProvider& getProperties() const	{ return this->Properties; }
		wosh::PropertiesProvider& getProperties()				{ return this->Properties; }

//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:

		virtual MediaTrack& operator=(const MediaTrack& m) {
			if (this == &m) return *this; // same object?

			this->track = m.track;
			this->Properties = m.Properties;
			return *this;
		 }

		virtual bool operator==(const MediaTrack &other) const {
			if ( this->track.getPath() != other.track.getPath() )
				return false;
			return true;
		 }

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const			{ return ""; }

		virtual WRESULT store( IEncoderWriter* serializer ) const {
			if ( serializer == NULL ) return WRET_ERR_PARAM;
			WRESULT ret = WRET_OK;
			return ret;
		 }
		virtual WRESULT retrieve( IEncoderReader* deserializer ) {
			if ( deserializer == NULL ) return WRET_ERR_PARAM;
			WRESULT ret = WRET_OK;
			return ret;
		 }
//@}
/** @name Internal vars
 * @{
 ******************************************************************************/
	protected:

		wosh::ResourcePath track;
		wosh::PropertiesProvider Properties;

//@}


}; // class def

 }; // namespace entertainment
}; // namespace wosh


#endif //__WOSH_Entertainment_MediaTrack_H__
