/**
 * @class   wosh::gui::BusConnectorQt
 * @brief   Connect a IBusListener to a Bus.
 *          
 *
 ****************************************************************************
 * @version $Id: BusConnectorQt.h 2184 2010-03-20 13:19:23Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_BusConnectorQt_H__
 #define __WOSH_Gui_BusConnectorQt_H__

 #include <core/BusConnector.h>

 #include <QtGui>

namespace wosh {
 namespace gui {


class BusConnectorQt : protected QObject, public BusConnector  {

	Q_OBJECT

	public:

		/**
		 * @brief  Empty constructor.
		 */
		BusConnectorQt();

		/**
		 * @brief  Empty destructor. Disconnect if Bus is still connected.
		 */
		virtual ~BusConnectorQt();

		using BusConnector::connect;
		using BusConnector::disconnect;


/** @name IBusListener Interface
 * @{
 ******************************************************************************/
	public:
		virtual void busMessage( const Message& message, const Bus* source );

		virtual void busEvent( Bus::BUS_STATE state, const Bus* source );

//@}


	protected slots:

		void slot_new_message( Message* message );
		void slot_event( Bus::BUS_STATE bus_state );

	signals:

		void signal_new_message( Message* message );
		void signal_event( Bus::BUS_STATE bus_state );



}; // class def

 }; // namespace gui
}; // namespace wosh


#endif //__WOSH_Gui_BusConnectorQt_H__
