/**
 * @class   wosh::gui::FormBundle
 * @brief   Graphic interface of wosh::Bundle interface.
 *
 ****************************************************************************
 * @version $Id: FormBundle.h 2184 2010-03-20 13:19:23Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_FormBundle_H__
 #define __WOSH_Gui_FormBundle_H__

 #include <framework/gui/IWidget.h>
 #include <framework/gui/WoshTableModel.h>
 #include <framework/gui/WidgetProperties.h>

 #include <core/Bundle.h>


using namespace wosh;
namespace Ui {
 class FormBundle;
};
namespace wosh {
 namespace gui {


class FormBundle :  public IWidgetGeneric  {

	Q_OBJECT

	public:

		FormBundle( QWidget *parent = NULL );
		virtual ~FormBundle();



	public:

		virtual void refreshAll();

	public:
		virtual void busMessage( const Message& message, const Bus* source );

		virtual void setTargetURI( const URI& target_uri );


	protected slots:
		void cmbDebugLevel_activated(const QString& text);

		void refreshInformation();
		void refreshInterfaces();
		void refreshRequirements();

	protected:
		void changeEvent(QEvent *e);

	protected:
		PropertiesMapper* propMapper;

	private:
		Ui::FormBundle *m_ui;

}; // class def

 }; // namespace gui
}; // namespace wosh


#endif //__WOSH_Gui_FormBundle_H__
