/**
 * @class   wosh::gui::FormConsole
 * @brief   
 *          
 *          
 *
 ****************************************************************************
 * @version $Id: FormConsole.h 2184 2010-03-20 13:19:23Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
#ifndef __WOSH_Gui_FormConsole_H__
 #define __WOSH_Gui_FormConsole_H__

 #include <framework/gui/IWidget.h>
 #include <framework/communication/SessionAsynch.h>
 #include <interfaces/services/CommunicatorService.h>


namespace Ui {
    class FormConsole;
}
using namespace wosh::communication;
namespace wosh {
 namespace gui {

#define _FormConsole_TYPE		"wosh::gui::FormConsole"

class FormConsole : public IWidgetGeneric,
					public virtual ISessionAsynchListener  {

	Q_OBJECT

	public:
		FormConsole(QWidget *parent = 0);
		virtual ~FormConsole();


	public slots:
		virtual void refreshAll() { }


	protected:
		WRESULT notify( const Notification* notification );


	public:
		virtual void session_StateChanged( wosh::communication::Session::STATE state, SessionAsynch* source );
		virtual void session_UriChanged( const URI& location, SessionAsynch* source );
		virtual void session_UsernameChanged( const std::string& username, SessionAsynch* source );

		virtual void session_PostWelcome( const std::string& message, SessionAsynch* source );
		virtual void session_PrintMessage( const std::string& message, SessionAsynch* );

		virtual void session_MessageQueued( Message* message, SessionAsynch* source );
		virtual void session_MessageParsing( Message* message, SessionAsynch* source );


	protected slots:
		void shellSend();

		void e_sentMessage( QString message );
		void e_sessionInfoChanged();

		void txtCurrentUser_editingFinished();
		void txtCurrentBus_editingFinished();
		void txtCurrentKernel_editingFinished();
		void chkScrollAuto_clicked();
		void cmbSessionDebug_activated(int index);
		void cmbWordWrap_activated(int index);
		void btnFontChange_clicked();
		void btnFontColorChange_clicked();
		void btnBackgroundChange_clicked();

	protected:
		void changeEvent(QEvent *e);
		void closeEvent(QCloseEvent* e);

	protected:
		virtual void busMessage( const Message& message, const Bus* source );

	signals:
		void s_sentMessage( QString message );
		void s_sessionInfoChanged();


	protected:
		wosh::communication::SessionAsynch* session;
		bool sessionDebug;
		bool scrollAuto;

	private:
		Ui::FormConsole *m_ui;

		
}; // class def

 }; // namespace gui
}; // namespace wosh


#endif //__WOSH_Gui_FormConsole_H__
