/** @file    GuiDialogs.cpp
 *  @author  Alessandro Polo
 *  @version $Id: GuiDialogs.cpp 2165 2010-03-18 04:13:32Z alex $
 *  @brief
 * File containing methods for the dialog classes.
 * The header for this class can be found in GuiDialogs.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "GuiDialogs.h"
 #include "GuiUtilities.h"

 #include <QtGui>

 #include "ui_DialogGuiMapping.h"
 


using namespace std;
namespace wosh {
 namespace gui {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

int GuiMappingTableModel::rowCount( const QModelIndex &parent ) const {
	(void)parent;
	return GuiMappings::getMappings().count();
 }


QVariant GuiMappingTableModel::data(const QModelIndex &index, int role) const {
	if (  !index.isValid() ) return QVariant();
	if ( role == Qt::DisplayRole || role == Qt::EditRole ) {
		MutexLockerRead mL(GuiMappings::getMappings().getMutex());
		const std::string& guiType = GuiMappings::getMappings().keyAt(index.row()); // lame way
		const ObjectTypeInfo& info = GuiMappings::getMappings().valueAt(index.row());
		switch(index.column()) {
			case 0: return QVariant( guiType.c_str() );
			case 1: return QVariant( info.name.c_str() );
			case 2: return QVariant( info.version );
			default: return QVariant();
		 }
	 }
	else if ( role == Qt::BackgroundColorRole ) { }
	else if ( role == Qt::ToolTipRole ) { }
	return QVariant();
 }

QVariant GuiMappingTableModel::headerData(int section, Qt::Orientation orientation, int role) const {
	if (role != Qt::DisplayRole) return QVariant();
	 if (orientation == Qt::Horizontal ) {
		switch(section) {
			case 0: return QVariant("ClassType");
			case 1: return QVariant("GUI ClassType");
			case 2: return QVariant("Version");
			default: return QVariant();
		 }
	 }
	return QVariant();
 }

DialogGuiMapping::DialogGuiMapping( QWidget *parent ) : QDialog(parent) {
	this->m_ui = new Ui::DialogGuiMapping();
	this->m_ui->setupUi(this);

	this->tableModel = new GuiMappingTableModel(this);
	this->m_ui->tableView->setModel(this->tableModel);

	QObject::connect(this->m_ui->btnClose, SIGNAL(clicked()), this, SLOT(accept()));
	QObject::connect(this->m_ui->btnRefresh, SIGNAL(clicked()), this, SLOT(refreshData()));

	refreshData();
 }

DialogGuiMapping::~DialogGuiMapping() {
	delete this->m_ui;
 }

void DialogGuiMapping::changeEvent(QEvent *e) {
	QWidget::changeEvent(e);
	if ( e->type() == QEvent::LanguageChange )
		this->m_ui->retranslateUi(this);
 }

void DialogGuiMapping::refreshData() {
	this->tableModel->reset();
	this->m_ui->tableView->resizeColumnsToContents();
	this->m_ui->tableView->horizontalHeader()->setResizeMode(0,QHeaderView::Stretch);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////





























//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace gui
}; // namespace wosh
