/**
 * @class   wosh::IWidgetSerializable
 * @brief   Graphical interface of wosh::ISerializable
 *          
 *          
 *
 ****************************************************************************
 * @version $Id: IWidgetSerializable.h 2184 2010-03-20 13:19:23Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_IWidgetSerializable_H__
 #define __WOSH_Gui_IWidgetSerializable_H__

 #include <woshDefs.h>
 #include <core/Serialization.h>

class QWidget;

namespace wosh {
 namespace gui {


class IWidgetSerializable {

	public:
		virtual QWidget* asQWidget() = 0;

	public:
		virtual WRESULT loadObject( const ISerializable* object ) = 0;
		virtual WRESULT saveObject( ISerializable* object ) const = 0;

		virtual ISerializable* newObject() const = 0;

		virtual const char* getSerializableType() const = 0;

	public:
		virtual ISerializable* saveObject() const {
			ISerializable* obj = newObject();
			saveObject(obj);
			return obj;
		 }

		virtual inline bool isSerializableType( const char* classname ) const {
			if ( classname == NULL ) return false;
			return ( strcmp(getSerializableType(), classname ) == 0 );
		 }

	public:
		virtual ~IWidgetSerializable() { }


}; // class def

 }; // namespace gui
}; // namespace wosh


#endif //__WOSH_Gui_IWidgetSerializable_H__
