/**
 * @class   wosh::gui::WidgetWoshBrowser
 * @brief   Shows Properties (PropertiesProvider) and allow user
 *          to edit then (asynch, remote through bus messages).
 *
 ****************************************************************************
 * @version $Id: WidgetWoshBrowser.h 2184 2010-03-20 13:19:23Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_WidgetWoshBrowser_H__
 #define __WOSH_Gui_WidgetWoshBrowser_H__

#include <framework/gui/IWidget.h>

#include <framework/gui/WoshObjectTreeModel.h>


#include <framework/gui/WoshTableModel.h>

#include <core/PropertiesProvider.h>
#include <core/NEW_List.h>
#include <core/Message.h>


using namespace wosh;

namespace Ui {
    class WidgetWoshBrowser;
}
namespace wosh {
 namespace gui {


class WidgetWoshBrowser :  public QWidget, public IWidget  {

	Q_OBJECT

	public:

		WidgetWoshBrowser(QWidget *parent = 0);
		virtual ~WidgetWoshBrowser();

	public:

		void requestHosts();

	public:

		virtual QWidget* asQWidget()		{ return this; }

		virtual void busMessage( const Message& message, const Bus* source );

 
	protected:
		void changeEvent(QEvent *e);

		WoshObject* getSelectedItem();
		void refresh_inspector( const WoshObject* item, bool force_refresh );

	public slots:

		virtual void refreshAll();

	protected slots:
		void treeModel_requestChildrenOf( WoshObject* item );

		void clicked( const QModelIndex & index );
		void doubleClicked( const QModelIndex & index );

		void btnOptions_clicked();
		void tBtnRefreshCurrent_clicked();

	signals:
		void request_create_GUI( const std::string&, const std::string&, const URI&, const QStringList& = QStringList() );

	protected:
		WoshObjectTreeModel* treeModel;
		WoshTableModel* wTableModel;

	private:
		Ui::WidgetWoshBrowser *m_ui;

}; // class def

 }; // namespace gui
}; // namespace wosh


#endif //__WOSH_Gui_WidgetWoshBrowser_H__
