/**
 * @class   wosh::gui::WoshObjectTreeModel
 * @brief   
 *
 ****************************************************************************
 * @version $Id: WoshObjectTreeModel.h 2184 2010-03-20 13:19:23Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_WoshObjectTreeModel_H__
 #define __WOSH_Gui_WoshObjectTreeModel_H__

 #include <framework/gui/WoshObject.h>
 #include <QAbstractTableModel>
 #include <vector>

namespace wosh {
 class Table;

 namespace gui {


int updateChildrenOf( const wosh::Table* table, WoshObject* objParent );
int updateChildren( const wosh::Table* table, const wosh::URI& parent_uri, WoshObject* root );
WoshObject* findChildOf( const wosh::URI& uri, WoshObject* parent );

int updateObjectsFromTable( const wosh::Table* table, WoshObject* parent, WoshObject* newTemplate = NULL );
int updateObjectsFromTable( const wosh::Table* table, WoshObject* parent, std::vector<WoshObject*>& children, WoshObject* newTemplate = NULL );

class WoshObjectTreeModel :  public QAbstractItemModel  {

	Q_OBJECT

	public:

		WoshObjectTreeModel( WoshObject* root_item, QObject *parent = 0);

		virtual ~WoshObjectTreeModel() { }

	public:

		void updated();

		void updateChildrenOf( const wosh::Table* table, WoshObject* objParent );

		void updateChildren( const wosh::Table* table, const wosh::URI& parent_uri );

	public:

		const WoshObject* getRootItem() const 		{ return this->rootItem; }
		WoshObject* getRootItem()					{ return this->rootItem; }



	public:
		void setFetchMore( bool value )				{ this->fetchMoreEnabled = value; }

		QModelIndex getIndexOf( const wosh::URI& uri, const QModelIndex &parent = QModelIndex() );

	public:

		int rowCount( const QModelIndex &parent = QModelIndex() ) const;

		int columnCount( const QModelIndex &parent = QModelIndex() ) const;

		virtual QModelIndex index( int row, int column, const QModelIndex &parent = QModelIndex() ) const;

		virtual QModelIndex parent( const QModelIndex &child ) const;

		QVariant data( const QModelIndex &index, int role ) const;

		QVariant headerData( int section, Qt::Orientation orientation, int role = Qt::DisplayRole ) const;


		Qt::ItemFlags flags( const QModelIndex &index ) const;

	public:

		virtual bool canFetchMore( const QModelIndex & parent ) const;

		virtual void fetchMore( const QModelIndex & parent );

	signals:

		void s_requestChildrenOf( WoshObject* item );

	protected:

		mutable WoshObject* rootItem;
		bool fetchMoreEnabled;

}; // class def


 }; // namespace gui
}; // namespace wosh


#endif //__WOSH_Gui_WoshObjectTreeModel_H__
