/**
 * @class   wosh::gui::WoshTableModel
 * @brief   
 *          
 *          
 *
 ****************************************************************************
 * @version $Id: WoshTableModel.h 2222 2010-03-26 20:49:35Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_WoshTableModel_H__
 #define __WOSH_Gui_WoshTableModel_H__

 #include <core/Variant.h>
 #include <core/Table.h>
 #include <core/MethodsDef.h>
 #include <core/Permission.h>
 #include <core/SecurityManager.h>
 #include <framework/gui/GuiUtilities.h>

 #include <framework/gui/IWidgetSerializable.h>

 #include <QtGui/QWidget>
 #include <QAbstractTableModel>
 #include <framework/gui/GuiCache.h>

 #include <QTableView>
 #include <QHeaderView>

 #include <QItemDelegate>
 #include <QtGui/QSpinBox>
 #include <QtGui/QDoubleSpinBox>
 #include <QtGui/QDateTimeEdit>
 #include <QtGui/QListWidget>


namespace wosh {
 namespace gui {


class GenericItemDelegate : public QItemDelegate
 {
	Q_OBJECT

	public:
		GenericItemDelegate(QObject *parent = 0) : QItemDelegate(parent) { }

		virtual ~GenericItemDelegate() { }

	public:
		QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const {
			if ( index.data().type() == QVariant::Int ) {
				QSpinBox *editor = new QSpinBox(parent);
				return editor;
			 }
			else if ( index.data().type() == QVariant::Double ) {
				QDoubleSpinBox *editor = new QDoubleSpinBox(parent);
				return editor;
			 }
			else if ( index.data().type() == QVariant::DateTime ) {
				QDateTimeEdit *editor = new QDateTimeEdit(parent);
				return editor;
			 }
/*
			else if ( index.data().canConvert<wosh::NEW_List>() ) {
				QListWidget *editor = new QListWidget(parent);
				return editor;
			 }
*/
			return QItemDelegate::createEditor(parent, option, index);
		 }

		void setEditorData(QWidget *editor, const QModelIndex &index) const {
			if ( index.data().type() == QVariant::Int ) {
				QSpinBox *spinBox = static_cast<QSpinBox*>(editor);
				int value = index.model()->data(index, Qt::EditRole).toInt();
				spinBox->setValue(value);
			 }
			else if ( index.data().type() == QVariant::Double ) {
				QDoubleSpinBox *spinBox = static_cast<QDoubleSpinBox*>(editor);
				double value = index.model()->data(index, Qt::EditRole).toDouble();
				spinBox->setValue(value);
			 }
			else if ( index.data().type() == QVariant::DateTime ) {
				QDateTimeEdit *dateEditBox = static_cast<QDateTimeEdit*>(editor);
				QDateTime value = index.model()->data(index, Qt::EditRole).toDateTime();
				dateEditBox->setDateTime(value);
			 }
/*			else if ( index.data().canConvert<wosh::NEW_List>() ) {
				QListWidget *editor = static_cast<QListWidget*>(editor);
				wosh::NEW_List c;
				c = index.data().value<wosh::NEW_List>();
				editor->addItem("asd");
			 }
*/
			else
				QItemDelegate::setEditorData(editor, index);
		 }

		void setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const {
			if ( index.data().type() == QVariant::Int ) {
				QSpinBox *spinBox = static_cast<QSpinBox*>(editor);
				spinBox->interpretText();
				int value = spinBox->value();
				model->setData(index, value, Qt::EditRole);
			 }
			else if ( index.data().type() == QVariant::Double ) {
				QDoubleSpinBox *spinBox = static_cast<QDoubleSpinBox*>(editor);
				spinBox->interpretText();
				double value = spinBox->value();
				model->setData(index, value, Qt::EditRole);
			 }
			else if ( index.data().type() == QVariant::DateTime ) {
				QDateTimeEdit *dateEditBox = static_cast<QDateTimeEdit*>(editor);
				dateEditBox->interpretText();
				QDateTime value = dateEditBox->dateTime();
				model->setData(index, value, Qt::EditRole);
			 }
			else
				QItemDelegate::setModelData(editor, model, index);
		 }


		void updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const {
			(void)index;
			editor->setGeometry(option.rect);
		 }

}; // class def




class WoshTableModel : public QAbstractTableModel
 {
	Q_OBJECT

	public:
		WoshTableModel(const wosh::Table* table, QObject *parent = 0) : QAbstractTableModel(parent) {
			this->dataTable = NULL;
			this->editingUserSecurity = NULL;
			this->forceKeyCol = -2;
			this->permissionCol = -2;
			this->permissionType = Permission::Write;
			loadTable( table );
		 }

		virtual ~WoshTableModel() {
			if ( this->dataTable != NULL )
				delete this->dataTable; this->dataTable = NULL;
			if ( this->editingUserSecurity != NULL )
				delete this->editingUserSecurity; this->editingUserSecurity = NULL;
		 }

	public:
		void setPermissionCol(int mode)				{ this->permissionCol = mode; }

		void setPermissionType( Permission::PERMISSION_TYPE type )
													{ this->permissionType = type; }

		wosh::Table* getDataTable()					{ return this->dataTable; }
		const wosh::Table* getDataTable() const 	{ return this->dataTable; }

		void loadTable( const wosh::Table* table ) {
			if ( this->dataTable != NULL )
				delete this->dataTable; this->dataTable = NULL;
			if ( table == NULL )
				return;
			this->dataTable = table->clone();
			analyzeTable();
			reset();
		 }

		void acquireTable( wosh::Table* table) {
			if ( this->dataTable != NULL )
				delete this->dataTable; this->dataTable = NULL;
			this->dataTable = table;
			analyzeTable();
			reset();
		 }

		void analyzeTable() {
			if ( this->dataTable == NULL )
				return;
			if ( this->permissionCol == -2 ) {
				for(unsigned int col=0; col<this->dataTable->getCols(); col++) {
					if ( this->dataTable->getClassNameOfCol(col) != "wosh::Permission" )
						continue;
					this->permissionCol = col;
					break;
				 }
			 }
			if ( this->forceKeyCol == -2 ) {
				if ( this->dataTable->hasHeaderHorizontal() ) {
					for(unsigned int col=0; col<this->dataTable->getCols(); col++) {
						const ISerializable* object = this->dataTable->getHeaderHorizontal(col);
						if ( object == NULL || !object->isClass(_Variant_TYPE) )
							continue;
						const wosh::Variant* dataObject = dynamic_cast<const wosh::Variant*>(object);
						if ( dataObject== NULL || *dataObject != "Name" )
							continue;
						this->forceKeyCol = col;
						break;
					 }
				 }
			 }
		 }

		void updateTable( const wosh::Table* table ) {
			if ( this->dataTable == NULL ) {
				loadTable(table);
				return;
			 }
			if ( table == NULL ) {
				delete this->dataTable; this->dataTable = NULL;
				reset();
				return;
			 }
			if ( table->getCols() != this->dataTable->getCols() ) {
				// different colums count => refresh ALL
				loadTable(table);
				return;
			 }

			if ( forceKeyCol > -1 && forceKeyCol < (int)this->dataTable->getCols() ) {
				// optimization when updating a single row:
				unsigned int rowsNew = this->dataTable->getRows();
				unsigned int rows = this->dataTable->getRows();
				unsigned int cols = this->dataTable->getCols();
				for(unsigned int rowNew=0; rowNew<rowsNew; rowNew++) {
					const ISerializable* objectKey = table->get(0, forceKeyCol);
					if ( objectKey == NULL ) continue;
					// iterate each cached item
					for(unsigned int row=0; row<rows; row++) {
						ISerializable* objectOld = this->dataTable->get(row, forceKeyCol);
						if ( objectOld == NULL ) continue;
						if ( objectKey->compare(objectOld) == 0 ) {
							// update row (but key)
							for(unsigned int col=0; col<cols; col++) {
								if ( (int)col == forceKeyCol ) continue;
								const ISerializable* objectCurr = table->get(rowNew, col);
								if ( objectCurr != NULL )
									this->dataTable->set(objectCurr->clone(), row, col, true);
								else
									this->dataTable->set(NULL, row, col, true);
							 }
						 }
					 }
				 }
				reset();
				return;
			 }
			///@todo may be optimized more!!
			loadTable(table);
			return;
		 }

	public:
		void setSecurityToken(SecurityToken* msec) {
			 if ( this->editingUserSecurity  != NULL )
				delete this->editingUserSecurity;
			 this->editingUserSecurity = msec;
		 }

		SecurityToken* getSecurityToken()				{ return this->editingUserSecurity; }
		const SecurityToken* getSecurityToken() const	{ return this->editingUserSecurity; }

	public:
		virtual int rowCount(const QModelIndex &parent = QModelIndex()) const {
			(void)parent;
			if ( this->dataTable == NULL ) return 0;
			return this->dataTable->getRows();
		 }

		virtual int columnCount(const QModelIndex &parent = QModelIndex()) const {
			(void)parent;
			if ( this->dataTable == NULL ) return 0;
			return this->dataTable->getCols();
		 }

		virtual QVariant data(const QModelIndex &index, int role) const {
			if ( this->dataTable == NULL || !index.isValid() ) return QVariant();
			if ( role == Qt::DisplayRole || role == Qt::EditRole ) {
				const ISerializable* object = this->dataTable->get(index.row(), index.column() );
				if ( object == NULL )
					return QVariant();
				return GuiUtilities::convertSerializable2QVariant(object);
			 }
			else if ( role == Qt::BackgroundColorRole ) {
				if ( dataTable->getClassNameOfCol(index.column()) != "wosh::Permission" )
					return QVariant();
				const ISerializable* object = this->dataTable->get(index.row(), index.column() );
				if ( object == NULL || !object->isClass(_Permission_TYPE) )
					return QVariant();
				const wosh::Permission* dataPerm = dynamic_cast<const wosh::Permission*>(object);
				bool allowInteract = SecurityManager::evalAuthorization(this->editingUserSecurity, dataPerm, this->permissionType );
				if ( allowInteract )
					return QColor(229,255,229);
				else
					return QColor(255,198,179);
			 }
			else if ( role == Qt::ToolTipRole ) {
				const ISerializable* object = this->dataTable->get(index.row(), index.column() );
				if ( object == NULL )
					return QVariant("NULL Object");
				if ( object->isClass(_Variant_TYPE) ){
					const wosh::Variant* dataObject = dynamic_cast<const wosh::Variant*>(object);
					return QString("wosh::Variant [%1]").arg(dataObject->getTypeAsString());
				 }
				return QVariant(object->getClassName());
			 }
			return QVariant();
		 }

		virtual QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const {
			if ( this->dataTable == NULL ) return QVariant();
			if (role != Qt::DisplayRole) return QVariant();
			 if (orientation == Qt::Horizontal ) {
				 if ( this->dataTable->hasHeaderHorizontal() ) {
					ISerializable* object = this->dataTable->getHeaderHorizontal(section);
					if ( object == NULL )
						return QString("Col%1").arg(section);
					if ( object->isClass(_Variant_TYPE) ) {
						wosh::Variant* dataObject = dynamic_cast<wosh::Variant*>(object);
						return QVariant(dataObject->toString().c_str());
					 }
					else
						return QVariant(object->toString().c_str());
				 }
				//return QString("Col%1").arg(section);
			 }
			else if (orientation == Qt::Vertical ) {
				if ( this->dataTable->hasHeaderVertical() ) {
					ISerializable* object = this->dataTable->getHeaderVertical(section);
					if ( object == NULL )
						return QString("Row%1").arg(section);
					if ( object->isClass(_Variant_TYPE) ) {
						wosh::Variant* dataObject = dynamic_cast<wosh::Variant*>(object);
						return QVariant(dataObject->toString().c_str());
					 }
					else
						return QVariant(object->toString().c_str());
				 }
				//return QString("Row%1").arg(section);
			 }
			return QVariant();
		 }

	public: // sortable
		void sort ( int column, Qt::SortOrder  order = Qt::AscendingOrder ) {
			if ( this->dataTable == NULL ) return;
			if ( column < 0 ) return;
			this->dataTable->sortByColumn((unsigned int)column, (short)order);
			reset();
		 }

	public: // editable
		 Qt::ItemFlags flags(const QModelIndex &index) const {
			if ( this->dataTable == NULL || !index.isValid() )
				 return QAbstractItemModel::flags(index) | Qt::ItemIsEnabled;
			if ( this->permissionCol == -2 )
				 return QAbstractItemModel::flags(index) | Qt::ItemIsEnabled;
			if ( this->permissionCol == -3 )
				 return QAbstractItemModel::flags(index) | Qt::ItemIsEditable;

			if ( this->forceKeyCol == index.column() )
				 return QAbstractItemModel::flags(index) | Qt::ItemIsEnabled;

			if ( this->permissionCol == -1 ) {
				bool allowWrite = SecurityManager::evalAuthorization(this->editingUserSecurity, &this->tablePermission, Permission::Write );
				if ( !allowWrite )
					return QAbstractItemModel::flags(index) | Qt::ItemIsEnabled;
			 }

			ISerializable* object = this->dataTable->get(index.row(), this->permissionCol );
			if ( object == NULL )
				 return QAbstractItemModel::flags(index) | Qt::ItemIsEnabled;
			if ( !object->isClass(_Permission_TYPE) )
				 return QAbstractItemModel::flags(index) | Qt::ItemIsEnabled;
			wosh::Permission* dataPerm = dynamic_cast<wosh::Permission*>(object);

			bool allowWrite = SecurityManager::evalAuthorization(this->editingUserSecurity, dataPerm, Permission::Write );
			if ( !allowWrite )
				return QAbstractItemModel::flags(index) | Qt::ItemIsEnabled;

			return QAbstractItemModel::flags(index) | Qt::ItemIsEditable;
		  }

		bool setData(const QModelIndex &index, const QVariant &value, int role) {
			if ( this->dataTable == NULL ) return false;
			if ( !index.isValid() || role != Qt::EditRole) return false;

			ISerializable* object = this->dataTable->get(index.row(), index.column() );
			if ( object == NULL )
				return false;

			if ( object->isClass(_Variant_TYPE) ) {
				wosh::Variant* dataObject = dynamic_cast<wosh::Variant*>(object);
				GuiUtilities::convertQVariant2Variant( dataObject, value );
				emit dataChanged(index, index);
				emit tableCell_changed(index.row(), index.column(), object );
				return true;
			 }
			 return false;
		}

/*
tablePermission
		 bool insertRows(int position, int rows, const QModelIndex &index = QModelIndex()) {
			 beginInsertRows(QModelIndex(), position, position+rows-1);
			 for (int row = 0; row < rows; ++row) {
				 stringList.insert(position, "");
			 }
			 endInsertRows();
			 return true;
		 }

		 bool removeRows(int position, int rows, const QModelIndex &index = QModelIndex()) {
			 beginRemoveRows(QModelIndex(), position, position+rows-1);
			 for (int row = 0; row < rows; ++row) {
				 stringList.removeAt(position);
			 }
			 endRemoveRows();
			 return true;
		 }
*/

	signals:
		void tableCell_changed( unsigned int row, unsigned int col, ISerializable* newValue );

	protected:
		wosh::Table* dataTable;
		int permissionCol;
		Permission tablePermission;
		SecurityToken* editingUserSecurity;
		int forceKeyCol;
		Permission::PERMISSION_TYPE permissionType;

}; // class def



class WidgetTable :		public QTableView,
						public virtual IWidgetSerializable
 {
	Q_OBJECT

	public:
		WidgetTable( QWidget* parent = NULL ) : QTableView(parent) {
			this->wTableModel = new WoshTableModel(NULL, this);
			this->wTableModel->setSecurityToken(SecurityManager::generateToken(GuiCache::CurrentUser));
			QTableView::setModel(this->wTableModel);
			this->request = NULL;
			this->resizeMode = 3;
			this->resizeStretch = -1;
			setDataRequest(_METHOD_List); // default
			setSortingEnabled(true);
//			connect(this->horizontalHeader(), SIGNAL(sortIndicatorChanged(int,Qt::SortOrder)), this, SLOT(sortByColumn(int,Qt::SortOrder)));
		 }

		virtual ~WidgetTable() {
			if ( this->wTableModel != NULL ) delete this->wTableModel;
			if ( this->request != NULL ) delete this->request;
		 }

	public:
		inline const WoshTableModel* getTableModel() const		{ return this->wTableModel; }
		inline WoshTableModel* getTableModel()					{ return this->wTableModel; }

		inline const Request* getDataRequest() const			{ return this->request; }
		inline Request* getDataRequest()						{ return this->request; }

		void setDataRequest( const std::string& method_name ){ setDataRequest(new Request(method_name)); }
		void setDataRequest( Request* data_request ) {
			if ( this->request != NULL ) delete this->request;
			this->request = data_request;
		 }

		void setResizeStretch( int value )				{ this->resizeStretch = value; }
		void setResizeMode( int value )					{ this->resizeMode = value; }


	public:
		virtual QWidget* asQWidget()					{ return this; }
		virtual const char* getSerializableType() const	{ return _Table_TYPE; }
		virtual ISerializable* newObject() const		{ return new Table(); }

		virtual WRESULT loadObject( const ISerializable* object ) {
			if ( object == NULL ) {
				this->wTableModel->loadTable(NULL);
				return WRET_OK;
			 }
			if ( !object->isClass(_Table_TYPE) )
				return WRET_ERR_PARAM;

			const Table* table = dynamic_cast<const Table*>(object);
			this->wTableModel->updateTable(table);
			this->horizontalHeader()->setSortIndicator(-1, Qt::AscendingOrder);
			return WRET_OK;
		 }

		virtual WRESULT saveObject( ISerializable* object ) const {
			if ( object == NULL ) return WRET_ERR_PARAM;

			return WRET_OK;
		 }

	public:
		inline const ISerializable* getCurrentItem() const {
			return getItem( this->currentIndex().row(),this->currentIndex().column() );
		 }

		const ISerializable* getItem( int row, int column ) const {
			if ( this->wTableModel == NULL ) return NULL;
			if ( this->wTableModel->getDataTable() == NULL ) return NULL;
			if ( row < 0 )
				return this->wTableModel->getDataTable()->getHeaderHorizontal(column);
			if ( column < 0 )
				return this->wTableModel->getDataTable()->getHeaderVertical(row);
			return this->wTableModel->getDataTable()->get(row, column);
		 }

		template<class DYNAMIC_CAST>
		const DYNAMIC_CAST* getItem( int row, int column ) const {
			return dynamic_cast<const DYNAMIC_CAST*>( getItem(row, column) );
		 }

		inline Variant getCurrentItemAsVariantAtCol( int column ) const {
			return getItemAsVariant( this->currentIndex().row(), column );
		 }

		Variant getItemAsVariant( int row, int column ) const {
			const ISerializable* obj = getItem( row,column );
			if ( obj == NULL ) return Variant();
			if ( !obj ->isClass(_Variant_TYPE) ) return Variant();
			const Variant* data = dynamic_cast<const Variant*>(obj);
			if ( data == NULL ) return Variant();
			return *data;
		 }

	public:
		void evalResponse( const Response* response) {
			if ( this->request == NULL ) return;
			if ( response->getMethod() != this->request->getMethod() ) return;

			if ( !response->hasData() ) return;
			if ( !response->getData()->isClass(_Table_TYPE) ) return;

			const wosh::Table* dataTable = dynamic_cast<const wosh::Table*>(response->getData());
			this->wTableModel->loadTable(dataTable);

			this->horizontalHeader()->setSortIndicator(-1, Qt::AscendingOrder);

			if ( this->resizeMode > 0 ) {
				this->horizontalHeader()->setVisible(dataTable->hasHeaderHorizontal());
				this->resizeColumnsToContents();
			 }
			if ( this->resizeMode > 2 ) {
				this->verticalHeader()->setVisible(dataTable->hasHeaderVertical());
			 }

			if ( this->resizeStretch >= 0 && this->resizeStretch < this->wTableModel->columnCount() )
				this->horizontalHeader()->setResizeMode(this->resizeStretch,QHeaderView::Stretch);

			emit updated();
		 }

	signals:
		void updated();

	protected:
		WoshTableModel* wTableModel;

		Request* request;
		int resizeMode;
		int resizeStretch;


}; // class def



 }; // namespace gui
}; // namespace wosh

#endif //__WOSH_Gui_WoshTableModel_H__
