/**
 * @class   wosh::gui::FormPlayerAudio
 * @brief   Graphic interface of wosh::interfaces::devices::PlayerAudio bundle.
 *
 ****************************************************************************
 * @version $Id: FormPlayerAudio.h 1716 2010-01-28 02:50:00Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_FormPlayerAudio_H__
 #define __WOSH_Gui_FormPlayerAudio_H__

 #include <framework/gui/IWidget.h>
 #include <framework/gui/WidgetProperties.h>
 #include <framework/gui/WidgetQMenu.h>


namespace Ui {
  class FormPlayerAudio;
};
using namespace std;
namespace wosh {
 namespace gui {


class FormPlayerAudio : public IWidgetGeneric
 {
	Q_OBJECT

	WOSH_CLASSNAME(wosh::gui::FormPlayerAudio)
	WOSH_CLASS_ABSTRACT(false)


	public:

		FormPlayerAudio(QWidget *parent = 0);
		virtual ~FormPlayerAudio();

	public:
		virtual void busMessage( const Message& message, const Bus* source );

	public slots:
		virtual void refreshAll();
		void refreshPlayList();
		void refreshProperties();

	protected slots:
		void btnPlayFile_clicked();
		void btnEnqueue_clicked();
		void btnRemove_clicked();


		void chkMuted_clicked();
		void sliderVolume_sliderReleased();

		void tableView_doubleClicked( const QModelIndex& index );
		void menuSynch_triggeredItem( WidgetQMenuItem* item );

	protected:
		void changeEvent(QEvent *e);
		void dragEnterEvent(QDragEnterEvent *e);
		void dropEvent(QDropEvent *e);


	protected:
		PropertiesMapper* propMapper;

	private:
		Ui::FormPlayerAudio *m_ui;

		QMenu* menuSynch;
		WidgetQMenu* menu_Players;

}; // class def

 }; // namespace gui
}; // namespace wosh


#endif // __WOSH_Gui_FormPlayerAudio_H__
