/** @file    GadgetModem.cpp
 *  @author  Alessandro Polo
 *  @version $Id: GadgetModem.cpp 226 2009-04-28 05:47:50Z alex $
 *  @brief
 * File containing methods for the wosh::application::GadgetModem class.
 * The header for this class can be found in GadgetModem.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include "GadgetModem.h"
 #include "ui_GadgetModem.h"

 #include <framework/gui/WidgetProperties.h>
 #include <interfaces/devices/Modem.h>
 #include <framework/gui/GuiUtilities.h>


namespace wosh {
  namespace gui {

 WOSH_REGISTER(wosh::gui::GadgetModem, "wosh::gui::KioskGadgetGeneric", 1.0, _static_GadgetModem )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

GadgetModem::GadgetModem() : KioskGadgetGeneric()
 {
	this->m_ui = new Ui::GadgetModem();
	this->m_ui->setupUi(this);

//	Object::setName("GadgetModem");
//	Object::setType(_GadgetModem_TYPE);
	KioskGadgetGeneric::setTitle(tr("DSL Connection"));

	this->getWidgetURI().setName( Utilities::format("GadgetModem#%10lu", Utilities::randomLong()) );
	this->getTargetURI().fromString( "*:Bundles/ModemDsl*" );

	this->propMapper = new PropertiesMapper(this);
	this->propMapper->getMap().insert(_Modem_KEY_ConnectionStatus, this->m_ui->lblConnectionStatus);
	this->propMapper->getMap().insert(_Modem_KEY_IpAddress, this->m_ui->lblIpAddress);
	this->propMapper->getMap().insert(_Modem_KEY_ConnectedTime, this->m_ui->lblConnectionTime);
	this->propMapper->getMap().insert(_Modem_KEY_statsSpeed, this->m_ui->lblConnectionSpeed);

	QObject::connect(this->m_ui->pushButton, SIGNAL(clicked()), this, SLOT(refreshAll()));

//	KioskGadgetGeneric::setRefreshAuto( 10 );
 }

GadgetModem::~GadgetModem()
 {
	delete this->propMapper;
	delete this->m_ui;
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void GadgetModem::setTargetURI( const URI& uri ) {
	IWidget::setTargetURI(uri);
	if ( uri.isEmpty() || !uri.isValid() )
		IWidget::setTargetURI( URI("*:Bundles/ModemDsl*") );
 }

void GadgetModem::refreshAll() {
	NEW_List* dlProps = this->propMapper->createKeysNEW_List();
	Request* request_method = new Request( _METHOD_GetProperties, dlProps );
	postMessageForTarget( new Message(request_method) );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void GadgetModem::busMessage( const Message& message, const Bus* )
 {
	if ( message.isEmpty() ) return;
	if ( !message.getContent()->isResponse() ) return;
	if ( !isMessageForMeOrFromTarget(message) )
		return;
	const wosh::Response* response = message.getContent()->asResponse();

	this->propMapper->evalResponse(response);
 }
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

  }; // namespace gui
}; // namespace wosh
