/**
 * @class   wosh::gui::FormBuildingManager
 * @brief   Graphic interface of wosh::interfaces::service::BuildingManager bundle.
 *          Official implementation is wosh::service::BuildingManagerBundle
 *
 ****************************************************************************
 * @version $Id: FormBuildingManager.h 1716 2010-01-28 02:50:00Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_FormBuildingManager_H__
 #define __WOSH_Gui_FormBuildingManager_H__

 #include <framework/gui/IWidget.h>
 
 #include <framework/building/BuildingObject.h>
 #include <core/Table.h>
 #include <core/Request.h>
 #include <core/Response.h>
 #include <core/MethodsDef.h>

 #include <interfaces/services/BuildingManager.h>
 #include <framework/gui/BusConnectorQt.h>
 #include <framework/gui/BuildingObjectGraphicsItem.h>

 #include <framework/gui/WoshTableModel.h>
 #include <framework/gui/WoshObjectTreeModel.h>

// #include <framework/gui/FormWoshBrowser.h>
 

namespace Ui {
    class FormBuildingManager;
}
using namespace std;
namespace wosh {
 namespace gui {


class BuildingObjectCache : public QObject  {

	Q_OBJECT

	public:

		BuildingObjectCache() : QObject()
		 {
			this->building = new WoshObject();
			this->building->setRemoteName("Building");

//			this->wTableModel = new WoshTableModel(NULL, this);
//			this->wTableModel->setSecurityToken(SecurityManager::generateToken("root"));

		 }

		virtual ~BuildingObjectCache() {
			delete this->building;
		 }

	public:

		QList<BuildingObjectGraphicsItem*> getItemsByType( const string& itemType )
		 {
			QList<BuildingObjectGraphicsItem*> res_items;
			std::map<string,BuildingObjectGraphicsItem*>::const_iterator it;
			std::map<string,BuildingObjectGraphicsItem*>::const_iterator it_end = this->items.end();
			for ( it=this->items.begin(); it!=it_end; it++ ) {
				if ( it->second == NULL ) continue;
				if ( it->second->getRemoteType() == itemType )
				res_items.append(it->second);
			 }
			return res_items;
		 }
		QStringList getItemsNameByType( const string& itemType )
		 {
			QStringList res_items;
			std::map<string,BuildingObjectGraphicsItem*>::const_iterator it;
			std::map<string,BuildingObjectGraphicsItem*>::const_iterator it_end = this->items.end();
			for ( it=this->items.begin(); it!=it_end; it++ ) {
				if ( it->second == NULL ) continue;
				if ( it->second->getRemoteType() == itemType )
				res_items.append(it->first.c_str());
			 }
			return res_items;
		 }

		BuildingObjectGraphicsItem* getCreateItem( const string& itemName, const string& itemType, const URI& uri )
		 {
			BuildingObjectGraphicsItem* item = this->items.find(itemName);
			if ( item == NULL ) {
				item = BuildingObjectGraphicsItem::createItemOf(itemType, this);

				item->setRemoteName(itemName);
				if ( itemType.empty() )
					item->setRemoteType("wosh::building::BuildingObject");
				else
					item->setRemoteType(itemType);
				item->setRemoteURI(uri);
				this->items.set(itemName, item);

				emit objectAdded( item );
			 }
			if ( !itemType.empty() && item->getRemoteType().empty() )
				item->setRemoteType(itemType);
			if ( !uri.isEmpty() && item->getRemoteURI().isEmpty() )
				item->setRemoteURI(uri);

			return item;
		 }



		void parseObjects( const wosh::Table* bobjTable, const Message& sourceMessage )
		 {
			string itemType = "";
			BuildingObjectGraphicsItem* item = NULL;
			unsigned int rows = bobjTable->getRows();
			for (unsigned int row=0; row<rows; row++) {
				const wosh::ISerializable* objName = bobjTable->get(row, 0);
				if ( objName == NULL ) continue;
				if ( !objName->isClass(_Variant_TYPE) ) continue;
				const wosh::Variant* itemName = dynamic_cast<const wosh::Variant*>(objName);

				const wosh::ISerializable* objInts = bobjTable->get(row, 1);
				if ( objInts != NULL && objInts->isClass(_Variant_TYPE) ) {
					const wosh::Variant* itemData = dynamic_cast<const wosh::Variant*>(objInts);
					itemType = itemData->toString();
				 }

				URI uri = sourceMessage.getSource(); uri.clearPathName();
				uri.getPath().push_back("Building");
				uri.setName(itemName->toString());
				item = getCreateItem( itemName->toString(), itemType, uri );

				const wosh::ISerializable* objParent = bobjTable->get(row, 2);
				if ( objParent != NULL && objParent->isClass(_Variant_TYPE) ) {
					const wosh::Variant* parentData = dynamic_cast<const wosh::Variant*>(objParent);
					item->bObjParent = parentData->toString();
				 }

				emit objectUpdated( item );
			}

			std::map<string,BuildingObjectGraphicsItem*>::const_iterator it;
			std::map<string,BuildingObjectGraphicsItem*>::const_iterator it_end = this->items.end();
			for ( it=this->items.begin(); it!=it_end; it++ ) {
				if ( it->second == NULL ) continue;
				if ( it->second->bObjParent.empty() ) {
					it->second->setParent(this->building, true);
					it->second->setZValue( 2 );
				 }
				else {
					BuildingObjectGraphicsItem* itemParent = this->items.find(it->second->bObjParent);
					if ( itemParent == NULL || it->second == itemParent ) continue;
					it->second->setParentItem(itemParent);
					it->second->setParent(itemParent, true);
					it->second->setZValue( (itemParent->zValue()+1)*5 );
				 }
			 }
		 }

/*
		void parseProperties( const wosh::Table* bobjTable, const Message& message )
		 {
			string itemName = message.getSource().getName();
			BuildingObjectGraphicsItem* item = getCreateItem( itemName, "", message.getSource() );
			item->parseMessage(message);
		 }
*/

		void parseMessage( const Message& message )
		 {
			BuildingObjectGraphicsItem* item = this->items.find(message.getSource().getName());
			if ( item != NULL ) {
				item->parseMessage(message);
			 }
		 }

	public:

		Map22<string, BuildingObjectGraphicsItem*>& getItems()	{ return this->items; }

		WoshObject* getRootItem()								{ return this->building; }


	protected:
		Map22<string, BuildingObjectGraphicsItem*> items;
		WoshObject* building;

		void item_event( BuildingObjectGraphicsItem* item, Message* message )
		 {
			message->setDestination( item->getRemoteURI() );
			emit objectEvent(item, message);
		 }

		friend class BuildingObjectGraphicsItem;


	signals:

		void objectAdded( BuildingObjectGraphicsItem* item );
		void objectUpdated( BuildingObjectGraphicsItem* item );

		void objectEvent( BuildingObjectGraphicsItem* item, Message* message );


}; // class def









class FormBuildingManager : public IWidgetGeneric
 {
	Q_OBJECT

	WOSH_CLASSNAME(wosh::gui::FormBuildingManager)
	WOSH_CLASS_ABSTRACT(false)


	public:

		FormBuildingManager(QWidget *parent = 0);
		virtual ~FormBuildingManager();

	public:

		virtual void busMessage( const Message& message, const Bus* source );

	public slots:

		virtual void refreshAll();
		void reDrawAll();

		virtual WRESULT methodReload(); 

	protected slots:

		void render_item( const QString& text );

		void gArea_zoomIn();
		void gArea_zoomOut();
		void gArea_rotateLeft();
		void gArea_rotateRight();
		void gArea_save();
		void gArea_setupMatrix();

		void gArea_reset();

		void gArea_option_triggered( QAction* action );


	protected slots:

		void objectAdded( BuildingObjectGraphicsItem* item );
		void objectUpdated( BuildingObjectGraphicsItem* item );
		void objectEvent( BuildingObjectGraphicsItem* item, Message* message );

	protected:
		void changeEvent(QEvent *e);

	protected:
		WoshTableModel* wTableModel;
		WoshObjectTreeModel* treeModel;

		BuildingObjectCache bObjCache;

		BusConnectorQt BusBuilding;

		bool ignoreSourceKernel;

	private:
		Ui::FormBuildingManager *m_ui;

		QAction* actItemsMoveable;
		QAction* actItemsVisible;

}; // class def

 }; // namespace gui
}; // namespace wosh


#endif // __WOSH_Gui_FormBuildingManager_H__
