/** @file    FormDesktopNotifier.cpp
 *  @author  Alessandro Polo
 *  @version $Id: FormDesktopNotifier.cpp 1708 2010-01-26 22:28:01Z alex $
 *  @brief
 * File containing methods for the wosh::gui::FormDesktopNotifier class.
 * The header for this class can be found in FormDesktopNotifier.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "FormDesktopNotifier.h"
 #include "ui_FormDesktopNotifier.h"

 #include <core/BusManager.h>
 #include <core/WoshKernel.h>
 #include <core/MethodsDef.h>
 #include <core/Request.h>
 #include <core/Response.h>
 #include <core/NEW_List.h>

 #include <QtGui>
 #include <framework/gui/GuiUtilities.h>


using namespace std;
namespace wosh {
 namespace gui {

 WOSH_REGISTER(wosh::gui::FormDesktopNotifier, "wosh::gui::IWidgetGeneric", 1.0, _static_FormDesktopNotifier )
 WOSH_GUI_MAP(wosh::gui::FormDesktopNotifier, "wosh::interfaces::services::DesktopNotifier", 0.0, _static_Map_FormDesktopNotifier )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

FormDesktopNotifier::FormDesktopNotifier(QWidget *parent) : IWidgetGeneric(parent)
 {
	this->m_ui = new Ui::FormDesktopNotifier();
	this->m_ui->setupUi(this);

	this->wTableModel = new WoshTableModel(NULL, this);
	this->m_ui->tableView->setModel(this->wTableModel);
	this->setWindowTitle("DesktopNotifier");
	this->getWidgetURI().setName( Utilities::format("FormDesktopNotifier#%10lu", Utilities::randomLong()) );
	this->getTargetURI().clearPathName();
	this->getTargetURI().setName("Bundles");

//	connect(this->m_ui->btnMonitorBus, SIGNAL(clicked()), this, SLOT(btnMonitorBus_clicked()));
	connect(this->m_ui->wUpBarNotifications, SIGNAL(updateClicked()), this, SLOT(refreshAll()));
	connect(this->m_ui->wUpBarSettings, SIGNAL(updateClicked()), this, SLOT(refreshSettings()));

	connect(this->m_ui->btnApply, SIGNAL(clicked()), this, SLOT(btnApply_clicked()));



 }

FormDesktopNotifier::~FormDesktopNotifier()
 {
	delete this->m_ui;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormDesktopNotifier::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormDesktopNotifier::refreshSettings()
 {
	NEW_List* dl = new NEW_List();
	dl->push_back<Variant>(_DesktopNotifier_KEY_DoNotDisturb);
	dl->push_back<Variant>(_DesktopNotifier_KEY_CacheSize);
	dl->push_back<Variant>(_DesktopNotifier_KEY_MouseMoveAccurancy);
	executeRemoteMethod( _METHOD_GetProperties, dl );
	this->m_ui->wUpBarSettings->sayRetrieving();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormDesktopNotifier::refreshAll() {
	this->setWindowTitle( QString("DesktopNotifier [%1]").arg(this->getTargetURI().getKernelName().c_str()) );

	executeRemoteMethod( _DesktopNotifier_METHOD_listnotif );
	this->m_ui->wUpBarNotifications->sayRetrieving();

	refreshSettings();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormDesktopNotifier::btnApply_clicked()
 {
	this->m_ui->wUpBarSettings->showMessage( tr("Applying Settings..") );
	Table* tbl = new Table( 3, 2 );
	tbl->set<Variant>( _DesktopNotifier_KEY_DoNotDisturb, 0,0);
	tbl->set( new Variant(this->m_ui->chkDoNotDistrurb->isChecked()), 0, 1);
	tbl->set<Variant>(_DesktopNotifier_KEY_CacheSize, 1,0);
	tbl->set<Variant>(this->m_ui->spinNotificationCacheSize->value(), 1, 1);
	tbl->set<Variant>(_DesktopNotifier_KEY_MouseMoveAccurancy, 2,0);
	tbl->set<Variant>(this->m_ui->spinMouseDetectionAccurency->value(), 2, 1);

	executeRemoteMethod( _METHOD_SetProperties, tbl );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormDesktopNotifier::busMessage( const Message& message, const Bus* )
 {
	if ( !message.getContent()->isResponse() ) return;
	if ( !isMessageForMeOrFromTarget(message) )
		return;

	const wosh::Response* response = message.getContent()->asResponse();
	if ( response->getMethod() == _DesktopNotifier_METHOD_listnotif ) {
		if ( !response->hasData() || !response->getData()->isClass(_Table_TYPE) ) return;
		const wosh::Table* notifTable = dynamic_cast<const wosh::Table*>(response->getData());
		this->wTableModel->updateTable(notifTable);
		this->m_ui->tableView->clearSelection();
		this->m_ui->tableView->resizeColumnsToContents();
		this->m_ui->tableView->horizontalHeader()->setStretchLastSection(true);
		this->m_ui->wUpBarNotifications->sayReady();
	 }
	else if ( response->getMethod() == _METHOD_GetProperties ) {
		if ( !response->hasData() || !response->getData()->isClass(_Table_TYPE) ) return;
		const wosh::Table* table = dynamic_cast<const wosh::Table*>(response->getData());
		for(unsigned int row=0; row<table->getRows(); row++) {
			const ISerializable* objectKey = table->get(row, 0);			
			if ( objectKey == NULL ) continue;
			if ( objectKey->isClass(_Property_TYPE) ) {
				const wosh::Property* prop = dynamic_cast<const wosh::Property*>(objectKey);
				if ( prop->getKey() == _DesktopNotifier_KEY_DoNotDisturb ) 
					this->m_ui->chkDoNotDistrurb->setChecked( prop->getValue().toBoolean(false) );
				else if ( prop->getKey() == _DesktopNotifier_KEY_CacheSize )
					this->m_ui->spinNotificationCacheSize->setValue( prop->getValue().toInteger(0) );
				else if ( prop->getKey() == _DesktopNotifier_KEY_MouseMoveAccurancy ) 
					this->m_ui->spinMouseDetectionAccurency->setValue( prop->getValue().toInteger(0) );
				continue;
			 }
			if ( !objectKey->isClass(_Variant_TYPE) ) continue;
			const ISerializable* objectValue = table->get(row, 1);			
			if ( objectValue == NULL || !objectValue->isClass(_Variant_TYPE) ) continue;
			const wosh::Variant* dataKey = dynamic_cast<const wosh::Variant*>(objectKey);
			const wosh::Variant* dataValue = dynamic_cast<const wosh::Variant*>(objectValue);
			if ( dataKey == NULL || dataValue == NULL ) continue;
			if ( *dataKey == _DesktopNotifier_KEY_DoNotDisturb ) 
				this->m_ui->chkDoNotDistrurb->setChecked( dataValue->toBoolean(false) );
			else if ( *dataKey == _DesktopNotifier_KEY_CacheSize )
				this->m_ui->spinNotificationCacheSize->setValue( dataValue->toInteger(0) );
			else if ( *dataKey == _DesktopNotifier_KEY_MouseMoveAccurancy ) 
				this->m_ui->spinMouseDetectionAccurency->setValue( dataValue->toInteger(0) );
		 }
		this->m_ui->wUpBarSettings->sayReady();
	 }
	else if ( response->getMethod() == _METHOD_SetProperties ) {
		if ( WFAILED(response->getReturnValue()) )
			this->m_ui->wUpBarSettings->showMessage( tr("FAILED Appling Settings!") );
		else
			this->m_ui->wUpBarSettings->showMessage( tr("Settings Applied.") );
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace gui
}; // namespace wosh
