/**
 * @class   wosh::gui::FormMediaDirector
 * @brief   Graphic interface of wosh::interfaces::service::CommunicationManager bundle.
 *          Official implementation is wosh::service::CommunicationManagerBundle
 *
 ****************************************************************************
 * @version $Id: FormMediaDirector.h 1716 2010-01-28 02:50:00Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_FormMediaDirector_H__
 #define __WOSH_Gui_FormMediaDirector_H__

 #include <framework/gui/IWidget.h>
 #include <framework/gui/WoshTableModel.h>

 #include <interfaces/services/MediaDirector.h>

 #include <QMenu>


namespace Ui {
    class FormMediaDirector;
}
using namespace std;
namespace wosh {
 namespace gui {



class FormMediaDirector :  public IWidgetGeneric
 {
	Q_OBJECT

	WOSH_CLASSNAME(wosh::gui::FormMediaDirector)
	WOSH_CLASS_ABSTRACT(false)


	public:
		FormMediaDirector(QWidget *parent = 0);
		virtual ~FormMediaDirector();

	public:
		virtual void busMessage( const Message& message, const Bus* source );

	public slots:
		virtual void refreshAll();

	protected slots:
		virtual void refreshZones();
		virtual void refreshMatrix();
		virtual void refreshPlayLists();

		void play_( const QModelIndex& index );
		void stop_(const QModelIndex& index );

		void btnMatrixPlay_clicked();
		void tableViewMatrix_doubleClicked( const QModelIndex& index );

		void tableViewZones_doubleClicked( const QModelIndex& index );
		void tableViewPlaylists_doubleClicked( const QModelIndex& );

		void tableViewMatrix_ContextMenu( const QPoint& pos );

	protected:

		void getMatrixInfo( string& zone, string& playlist, const QModelIndex& index );

	protected:
		void buildMenus();
		QMenu* menuMatrix;
		QAction* actionMatrixTitle;
		QAction* actionMatrixPlay;
		QAction* actionMatrixMove;
		QAction* actionMatrixStop;
		QAction* actionMatrixStopOther;

	protected:
		void changeEvent(QEvent *e);

	protected:
		WoshTableModel* wTableModelZones;
		WoshTableModel* wTableModelPlaylists;
		//WoshTableModel* wMediaMatrix;
		//MediaMatrixModel* wMediaMatrix;

	private:
		Ui::FormMediaDirector *m_ui;

}; // class def




 }; // namespace gui
}; // namespace wosh


#endif //__WOSH_Gui_FormMediaDirector_H__
