/** @file    GadgetImageViewer.cpp
 *  @author  Alessandro Polo
 *  @version $Id: GadgetImageViewer.cpp 226 2009-04-28 05:47:50Z alex $
 *  @brief
 * File containing methods for the wosh::gui::GadgetImageViewer class.
 * The header for this class can be found in GadgetImageViewer.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include "GadgetImageViewer.h"
 #include "ui_GadgetImageViewer.h"


namespace wosh {
  namespace gui {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

GadgetImageViewer::GadgetImageViewer() : KioskGadgetGeneric()
 {
	this->m_ui = new Ui::GadgetImageViewer();
	this->m_ui->setupUi(this);

//	Object::setName("GadgetImageViewer");
//	Object::setType(_GadgetImageViewer_TYPE);
//	KioskGadgetGeneric::setTitle(tr("Pictures"));

this->m_ui->lblOutput->setBackgroundRole(QPalette::Base);
this->m_ui->lblOutput->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
this->m_ui->lblOutput->setScaledContents(true);



	connect(this->m_ui->btnPrevious, SIGNAL(clicked()), this, SLOT(previous()));
	connect(this->m_ui->btnNext, SIGNAL(clicked()), this, SLOT(next()));
	connect(this->m_ui->btnMore, SIGNAL(clicked()), this, SLOT(loadFiles()));

 }

GadgetImageViewer::~GadgetImageViewer()
 {
	delete this->m_ui;
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void GadgetImageViewer::refreshAll() {

 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void GadgetImageViewer::loadFiles()
 {
	QDir dir("Y:\\PICTURES\\pranzo_2010-01-10\\");

	dir.setFilter(QDir::Files | QDir::Hidden | QDir::NoSymLinks);
	dir.setSorting(QDir::Size | QDir::Reversed);

	this->picList = dir.entryInfoList();
	for (int i = 0; i < picList.size(); ++i) {
		;
	}
	this->cPicIndex = 0 ;

 }

void GadgetImageViewer::display( int index )
 {
	if ( index < 0 )
		index = this->picList.size();
	if ( index >= this->picList.size() )
		index = 0;

	if ( this->picList.size() == 0 ) {
		return;
	 }

	QImage image(this->picList.at(index).filePath());
	this->m_ui->lblOutput->setPixmap(QPixmap::fromImage(image));
	this->m_ui->lblOutput->adjustSize();
 }


void GadgetImageViewer::previous()
 {
	display( --this->cPicIndex );
 }

void GadgetImageViewer::next()
 {
	display( ++this->cPicIndex );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void GadgetImageViewer::busMessage( const Message& message, const Bus* )
 {
	if ( message.isEmpty() ) return;
	if ( !message.getContent()->isResponse() ) return;
	if ( !isMessageForMeOrFromTarget(message) )
		return;
//	const wosh::Response* response = message.getContent()->asResponse();

 }
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

  }; // namespace gui
}; // namespace wosh
