/**
 * @class   wosh::gui::KioskSheet
 * @brief   
 *
 *
 ****************************************************************************
 * @version $Id: KioskSheet.h 1690 2010-01-25 05:19:57Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_KioskSheet_H__
 #define __WOSH_Gui_KioskSheet_H__


 #include <core/Object.h>
 #include <core/Utilities.h>

 #include <framework/gui/IWidget.h>

 #include <QtGui/QWidget>
 #include <QtGui/QGraphicsWidget>
 #include <QtGui/QGraphicsProxyWidget>


using namespace std;
namespace wosh {
 namespace gui {



class NavigatorItem {
	public:
		NavigatorItem() { }
		virtual ~NavigatorItem() { }
	public:
		string context;
		string sheet_name;

};

class INavigatorListener {
	public:
		virtual void itemSelected( NavigatorItem* item ) = 0;
	public:
		virtual ~INavigatorListener() { }
};


class KioskSheet : public IWidget  {

	public:
		KioskSheet() : IWidget() {
			this->proxyItem = NULL;
		}
		virtual ~KioskSheet() { }

	public:
		virtual QGraphicsWidget* asQGraphicsWidget() {
			if ( this->proxyItem != NULL )
				return this->proxyItem;
			this->proxyItem = new QGraphicsProxyWidget();
			this->proxyItem->setWidget( this->asQWidget() );
			return this->proxyItem;
		 }

	public:
		virtual void setName( const string& name ) {
			this->sheetName = name;
		 }
		const string& getName()				{ return this->sheetName; }

		virtual void setContext( const string& context ) {
			this->sheetContext = context;
		 }
		const string& getContext()			{ return this->sheetContext; }

	protected:
		QGraphicsProxyWidget* proxyItem;

		string sheetName;
		string sheetContext;


}; // class def



class KioskSheetProxy : public KioskSheet  {
	public:
		KioskSheetProxy( QWidget* Item ) : KioskSheet() { 
			this->itemWidget = Item;
			this->itemIWidget = NULL;
		}
		KioskSheetProxy( IWidget* Iitem, int ) : KioskSheet() { 
			this->itemWidget = NULL;
			this->itemIWidget = Iitem;
			if ( this->itemIWidget != NULL )
				this->itemWidget = Iitem->asQWidget();
		}
		virtual ~KioskSheetProxy() { }

		virtual QWidget* asQWidget() { return this->itemWidget; }

		virtual void refreshAll() {
			if ( this->itemIWidget != NULL )
				this->itemIWidget->refreshAll();
		 }

		virtual void busMessage( const Message&, const Bus* ) { }

	protected:
		QWidget* itemWidget;
		IWidget* itemIWidget;

}; // class def



class KioskSheetGeneric : public QWidget, public KioskSheet  {


	public:
		KioskSheetGeneric() : QWidget(), KioskSheet() { 
			
		}
		virtual ~KioskSheetGeneric() { }

	public:
		virtual QWidget* asQWidget() { return this; }


	protected:


}; // class def


  }; // namespace gui
}; // namespace wosh

#endif //__WOSH_Gui_KioskSheet_H__
