/**
 * @class   wosh::gui::FormBusManager
 * @brief   Graphic interface of wosh::BusManager core-module.
 *
 * \image html extras/screenshots/WOSH.GUI.FormBusManager.png "FormBusManager"
 *
 * @ingroup Gui
 ****************************************************************************
 * @version 0.8.499 $Id: FormBusManager.h 1716 2010-01-28 02:50:00Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_FormBusManager_H__
 #define __WOSH_Gui_FormBusManager_H__

 #include <framework/gui/IWidget.h>
 #include <QtGui/QMenu>


namespace Ui {
 class FormBusManager;
};
using namespace std;
namespace wosh {
 namespace gui {


class FormBusManager :  public IWidgetGeneric
 {
	Q_OBJECT

	WOSH_CLASSNAME(wosh::gui::FormBusManager)
	WOSH_CLASS_ABSTRACT(false)


	public:
		FormBusManager(QWidget *parent = 0);
		virtual ~FormBusManager();

	public:
		virtual void busMessage( const Message& message, const Bus* source );

	public slots:
		virtual void refreshAll();

	protected slots:
		void btnMonitorBus_clicked();
		void btnCreateBus_clicked();
		void btnStartBus_clicked();
		void btnStopBus_clicked();
		void btnFlushQueue_clicked();
		void tableView_ContextMenu( const QPoint& pos );

	protected:
		void changeEvent(QEvent *e);
		void controlBus( const Variant& busname, const Variant& action );

	protected:
		void buildMenus();
		QMenu* menuTable;
		QAction* menuTitle;
		QAction* menuSharedLocal;
		QAction* menuSharedNetwork;
		QAction* menuFlush;
		QAction* menuSendMessage;

	private:
		Ui::FormBusManager *m_ui;


}; // class def

 }; // namespace gui
}; // namespace wosh


#endif //__WOSH_Gui_FormBusManager_H__
