/** @file    FormDataBaseManager.cpp
 *  @author  Alessandro Polo
 *  @version $Id: FormDataBaseManager.cpp 1708 2010-01-26 22:28:01Z alex $
 *  @brief
 * File containing methods for the wosh::gui::FormDataBaseManager class.
 * The header for this class can be found in FormDataBaseManager.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "FormDataBaseManager.h"
 #include "ui_FormDataBaseManager.h"

 #include <core/DataBaseManager.h>
 #include <core/WoshKernel.h>
 #include <core/MethodsDef.h>
 #include <core/Request.h>
 #include <core/Response.h>
 #include <QtGui>
 #include <framework/gui/GuiUtilities.h>


using namespace std;
namespace wosh {
 namespace gui {

 WOSH_REGISTER(wosh::gui::FormDataBaseManager, "wosh::gui::IWidgetGeneric", 1.0, _static_FormDataBaseManager )
 WOSH_GUI_MAP(wosh::gui::FormDataBaseManager, "wosh::DataBaseManager", 0.0, _static_Map_FormDataBaseManager )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

FormDataBaseManager::FormDataBaseManager(QWidget *parent) : IWidgetGeneric(parent)
 {
	this->m_ui = new Ui::FormDataBaseManager();
	this->m_ui->setupUi(this);

	setupFor(_DataBaseManager_URI);

	connect(this->m_ui->tableView, SIGNAL(updated()), this->m_ui->wUpBarDevices, SLOT(sayReady()));
	connect(this->m_ui->wUpBarDevices, SIGNAL(updateClicked()), this, SLOT(refreshDevices()));

 }

FormDataBaseManager::~FormDataBaseManager() {
	delete this->m_ui;
 }

void FormDataBaseManager::changeEvent(QEvent *e) {
	QWidget::changeEvent(e);
	if ( e->type() == QEvent::LanguageChange )
		this->m_ui->retranslateUi(this);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormDataBaseManager::refreshAll() {
	this->setWindowTitle( QString("DataBase Manager [%1]").arg(this->getTargetURI().getKernelName().c_str()) );
	refreshDevices();
 }

void FormDataBaseManager::refreshDevices() {
	executeRemoteMethod( _METHOD_List );
	this->m_ui->wUpBarDevices->sayRetrieving();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormDataBaseManager::busMessage( const Message& message, const Bus* ) {
	if ( message.isEmpty() || !message.getContent()->isResponse() ) return;
	if ( !isMessageForMeOrFromTarget(message) ) return;
	const wosh::Response* response = message.getContent()->asResponse();

	this->m_ui->tableView->evalResponse(response);
	promptMethodResponse(message);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace gui
}; // namespace wosh
