/**
 * @class   wosh::gui::FormDeviceManager
 * @brief   Graphic interface of wosh::DeviceManager core-module.
 *
 * \image html extras/screenshots/WOSH.GUI.FormDeviceManager.1.png "FormDeviceManager::Tab 1"
 * \image html extras/screenshots/WOSH.GUI.FormDeviceManager.2.png "FormDeviceManager::Tab 2"
 * \image html extras/screenshots/WOSH.GUI.FormDeviceManager.3.png "FormDeviceManager::Tab 3"
 *
 * @ingroup Gui
 ****************************************************************************
 * @version 0.8.499 $Id: FormDeviceManager.h 1716 2010-01-28 02:50:00Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_FormDeviceManager_H__
 #define __WOSH_Gui_FormDeviceManager_H__

 #include <framework/gui/IWidget.h>
 #include <framework/gui/WidgetProperties.h>
 #include <QtGui/QMenu>


namespace Ui {
 class FormDeviceManager;
};
using namespace std;
namespace wosh {
 namespace gui {


class FormDeviceManager :  public IWidgetGeneric
 {
	Q_OBJECT

	WOSH_CLASSNAME(wosh::gui::FormDeviceManager)
	WOSH_CLASS_ABSTRACT(false)


	public:
		FormDeviceManager(QWidget *parent = 0);
		virtual ~FormDeviceManager();

	public:
		virtual void busMessage( const Message& message, const Bus* source );

	public slots:
		virtual void refreshAll();

	protected slots:
		void refreshDevices();
		void refreshEventsLog();
		void refreshConfiguration();

		void btnEmulateEvent_clicked();
		void btnRegisterNew_clicked();
		void btnUnRegister_clicked();
		void tableViewDevices_doubleClicked( const QModelIndex& index );
		void tableViewDevices_ContextMenu( const QPoint& pos );
		void tableViewEventsLog_ContextMenu( const QPoint& pos );
		void chkMonitorDeviceBus_clicked();

	signals:
		void request_create_GUI(const string&,const string&,const URI&,const QStringList& = QStringList() );

	protected:
		void changeEvent(QEvent *e);

	protected:
		PropertiesMapper* propMapperCfg;

	protected:
		void buildMenus();
		QMenu* menuDevice;
		QAction* menuDeviceTitle;
		QAction* menuDeviceForceUpdate;

		QMenu* menuEventLog;
		QAction* menuEventLogTitle;
		QAction* menuEventLogRePlay;
		QAction* menuEventLogFillSheet;

		BusConnector BusDevices;

	private:
		Ui::FormDeviceManager *m_ui;

}; // class def

 }; // namespace gui
}; // namespace wosh


#endif //__WOSH_Gui_FormDeviceManager_H__
