/** @file    FormFileSystem.cpp
 *  @author  Alessandro Polo
 *  @version $Id: FormFileSystem.cpp 1710 2010-01-26 23:24:55Z alex $
 *  @brief
 * File containing methods for the wosh::gui::FormFileSystem class.
 * The header for this class can be found in FormFileSystem.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "FormFileSystem.h"
 #include "ui_FormFileSystem.h"
 #include <core/FileSystem.h>


using namespace std;
namespace wosh {
 namespace gui {

 WOSH_REGISTER(wosh::gui::FormFileSystem, "wosh::gui::IWidgetGeneric", 1.0, _static_FormFileSystem )
 WOSH_GUI_MAP(wosh::gui::FormFileSystem, "wosh::FileSystem", 0.0, _static_Map_FormFileSystem )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

FormFileSystem::FormFileSystem(QWidget *parent) : IWidgetGeneric(parent)
 {
	this->m_ui = new Ui::FormFileSystem();
	this->m_ui->setupUi(this);

	setupFor(_FileSystem_URI);

	// tab FileSystem [0]

	// tab Aliases [1]
	this->m_ui->tableViewAliases->setDataRequest(_METHOD_List);
	this->m_ui->tableViewAliases->setResizeStretch(0);
	connect(this->m_ui->tableViewAliases, SIGNAL(updated()), this->m_ui->wUpBarAliases, SLOT(sayReady()));
	connect(this->m_ui->wUpBarAliases, SIGNAL(updateClicked()), this, SLOT(refreshAliases()));

//	connect(this->wTableModelAliases, SIGNAL(tableCell_changed(unsigned int,unsigned int, ISerializable*)), this, SLOT(cell_valueChanged(unsigned int,unsigned int, ISerializable*)));
//	connect(this->m_ui->tableViewAliases, SIGNAL(doubleClicked(const QModelIndex&)), this, SLOT(tableViewAliases_doubleClicked(const QModelIndex&)));
 }

FormFileSystem::~FormFileSystem() {
	delete this->m_ui;
 }

void FormFileSystem::changeEvent(QEvent *e) {
	QWidget::changeEvent(e);
	if ( e->type() == QEvent::LanguageChange )
		this->m_ui->retranslateUi(this);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormFileSystem::refreshAll() {
	this->setWindowTitle( QString("FileSystem [%1]").arg(this->getTargetURI().getKernelName().c_str()) );
	refreshAliases();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormFileSystem::refreshAliases() {
	executeRemoteMethod( _METHOD_List );
	this->m_ui->wUpBarAliases->sayRetrieving();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormFileSystem::cell_valueChanged( unsigned int row, unsigned int, ISerializable* newValue ) {
/*	if ( newValue == NULL || !newValue->isClass(_Variant_TYPE) ) return;
	ISerializable* objName = this->wTableModelAliases->getDataTable()->get(row, 0 );
	if ( objName == NULL || !objName->isClass(_Variant_TYPE) ) {
		return;
	 }
	const wosh::Variant* aliasName = dynamic_cast<const wosh::Variant*>(objName);
	wosh::Variant* aliasValue = dynamic_cast<wosh::Variant*>(newValue);

	wosh::NEW_List* dList = new wosh::NEW_List();
	dList->push_back(aliasName->clone());
	dList->push_back(aliasValue->clone());
	executeRemoteMethod( _FileSystem_METHOD_set_alias, dList );

	this->m_ui->wUpBarAliases->showMessage( QString(tr("Applying changes of '%1'..")).arg(aliasName->toString().c_str()) );
*/
 }

void FormFileSystem::tableViewAliases_doubleClicked( const QModelIndex& index ) {
/*	if ( this->wTableModelAliases->getDataTable() == NULL ) return;
	const ISerializable* objName = this->wTableModelAliases->getDataTable()->get(index.row(), 0);
	if ( objName == NULL ) return;
	if ( !objName->isClass(_Variant_TYPE) ) return;
	const Variant* dataName = dynamic_cast<const Variant*>(objName);
	if ( dataName == NULL ) return;

	WidgetFileSystemBrowser* wFSb = new WidgetFileSystemBrowser(this);

	wFSb->show();
*/
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormFileSystem::busMessage( const Message& message, const Bus* ) {
	if ( message.isEmpty() || !message.getContent()->isResponse() ) return;
	if ( !isMessageForMeOrFromTarget(message) ) return;
	const wosh::Response* response = message.getContent()->asResponse();

	this->m_ui->tableViewAliases->evalResponse(response);

	if ( response->getMethod() == _FileSystem_METHOD_set_alias ) {
		this->m_ui->wUpBarAliases->showError( tr("Applied Alias!") );
		return;
	 }

 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace gui
}; // namespace wosh
