/**
 * @class   wosh::gui::FormFileSystem
 * @brief   Graphic interface of wosh::FileSystem core-module.
 *
 * \image html extras/screenshots/WOSH.GUI.FormFileSystem.1.png "FormFileSystem::Tab 1"
 * \image html extras/screenshots/WOSH.GUI.FormFileSystem.2.png "FormFileSystem::Tab 2"
 *
 * @ingroup Gui
 ****************************************************************************
 * @version $Id: FormFileSystem.h 1716 2010-01-28 02:50:00Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_FormFileSystem_H__
 #define __WOSH_Gui_FormFileSystem_H__

 #include <framework/gui/IWidget.h>
 #include <QModelIndex>


namespace Ui {
 class FormFileSystem;
};
using namespace std;
namespace wosh {
 namespace gui {


class FormFileSystem :  public IWidgetGeneric
 {
	Q_OBJECT

	WOSH_CLASSNAME(wosh::gui::FormFileSystem)
	WOSH_CLASS_ABSTRACT(false)


	public:
		FormFileSystem(QWidget *parent = 0);
		virtual ~FormFileSystem();

	public:
		virtual void busMessage( const Message& message, const Bus* source );

	public slots:
		virtual void refreshAll();

	protected slots:
		virtual void refreshAliases();

		void cell_valueChanged( unsigned int row, unsigned int col, ISerializable* newValue );
		void tableViewAliases_doubleClicked( const QModelIndex& index );

	protected:
		void changeEvent(QEvent *e);

	private:
		Ui::FormFileSystem *m_ui;

}; // class def

 }; // namespace gui
}; // namespace wosh


#endif //__WOSH_Gui_FormFileSystem_H__
