/** @file    FormNetworkManager.cpp
 *  @author  Alessandro Polo
 *  @version $Id: FormNetworkManager.cpp 1710 2010-01-26 23:24:55Z alex $
 *  @brief
 * File containing methods for the wosh::gui::FormNetworkManager class.
 * The header for this class can be found in FormNetworkManager.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "FormNetworkManager.h"
 #include "ui_FormNetworkManager.h"
 #include <core/NetworkManager.h>
 #include <interfaces/services/DiscoveryUdp.h>

 #include <QtGui/QInputDialog>


using namespace std;
namespace wosh {
 namespace gui {

 WOSH_REGISTER(wosh::gui::FormNetworkManager, "wosh::gui::IWidgetGeneric", 1.0, _static_FormNetworkManager )
 WOSH_GUI_MAP(wosh::gui::FormNetworkManager, "wosh::NetworkManager", 0.0, _static_Map_FormNetworkManager )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

FormNetworkManager::FormNetworkManager(QWidget *parent) : IWidgetGeneric(parent)
 {
	this->m_ui = new Ui::FormNetworkManager();
	this->m_ui->setupUi(this);

	setupFor(_NetworkManager_URI);

	// tab Information [0]
	this->m_ui->tableViewNeighbours->setDataRequest(_NetworkManager_METHOD_Neighbours_list);
	connect(this->m_ui->tableViewNeighbours, SIGNAL(updated()), this->m_ui->wUpBarNeighbours, SLOT(sayReady()));
	connect(this->m_ui->wUpBarNeighbours, SIGNAL(updateClicked()), this, SLOT(refreshNeighbours()));

	connect(this->m_ui->wUpBarNetworkTree, SIGNAL(updateClicked()), this, SLOT(refreshNetworkTree()));

	// tab Management [1]

	// tab Hosts [2]
	this->m_ui->tableViewHosts->setDataRequest(_NetworkManager_METHOD_Hosts_list);
	connect(this->m_ui->tableViewHosts, SIGNAL(updated()), this->m_ui->wUpBarHosts, SLOT(sayReady()));
	connect(this->m_ui->wUpBarHosts, SIGNAL(updateClicked()), this, SLOT(refreshHosts()));

	// tab Channels [3]
	this->m_ui->tableViewChannels->setDataRequest(_NetworkManager_METHOD_Channels_list);
	connect(this->m_ui->tableViewChannels, SIGNAL(updated()), this->m_ui->wUpBarChannels, SLOT(sayReady()));
	connect(this->m_ui->wUpBarChannels, SIGNAL(updateClicked()), this, SLOT(refreshChannels()));

	// tab Protocols [4]
	this->m_ui->tableViewProtocols->setDataRequest(_NetworkManager_METHOD_Protocols_list);
	connect(this->m_ui->tableViewProtocols, SIGNAL(updated()), this->m_ui->wUpBarProtocols, SLOT(sayReady()));
	connect(this->m_ui->wUpBarProtocols, SIGNAL(updateClicked()), this, SLOT(refreshProtocols()));

	// tab Routing [5]
	this->m_ui->tableViewRoutesStatic->setDataRequest(_NetworkManager_METHOD_RoutesStatic_list);
	connect(this->m_ui->tableViewRoutesStatic, SIGNAL(updated()), this->m_ui->wUpBarRoutesStatic, SLOT(sayReady()));
	connect(this->m_ui->wUpBarRoutesStatic, SIGNAL(updateClicked()), this, SLOT(refreshRoutesStatic()));
	this->m_ui->tableViewRoutesDynamic->setDataRequest(_NetworkManager_METHOD_RoutesDynamic_list);
	connect(this->m_ui->tableViewRoutesDynamic, SIGNAL(updated()), this->m_ui->wUpBarRoutesDynamic, SLOT(sayReady()));
	connect(this->m_ui->wUpBarRoutesDynamic, SIGNAL(updateClicked()), this, SLOT(refreshRoutesDynamic()));

 }

FormNetworkManager::~FormNetworkManager() {
	delete this->m_ui;
 }

void FormNetworkManager::changeEvent(QEvent *e) {
	QWidget::changeEvent(e);
	if ( e->type() == QEvent::LanguageChange )
		this->m_ui->retranslateUi(this);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormNetworkManager::refreshAll() {
	this->setWindowTitle( QString("Network Manager [%1]").arg(this->getTargetURI().getKernelName().c_str()) );
	refreshHosts();
	refreshChannels();
	refreshProtocols();
	refreshNeighbours();
	refreshNetworkTree();
	refreshRoutesStatic();
	refreshRoutesDynamic();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormNetworkManager::refreshHosts() {
	executeRemoteMethod( _NetworkManager_METHOD_Hosts_list );
	this->m_ui->wUpBarHosts->sayRetrieving();
 }

void FormNetworkManager::refreshChannels() {
	executeRemoteMethod( _NetworkManager_METHOD_Channels_list );
	this->m_ui->wUpBarChannels->sayRetrieving();
 }

void FormNetworkManager::refreshProtocols() {
	executeRemoteMethod( _NetworkManager_METHOD_Protocols_list );
	this->m_ui->wUpBarProtocols->sayRetrieving();
 }

void FormNetworkManager::refreshNeighbours() {
	executeRemoteMethod( _NetworkManager_METHOD_Neighbours_list );
	this->m_ui->wUpBarNeighbours->sayRetrieving();
 }

void FormNetworkManager::refreshNetworkTree() {
	executeRemoteMethod( _NetworkManager_METHOD_NetworkTree_list );
	this->m_ui->wUpBarNetworkTree->sayRetrieving();
 }

void FormNetworkManager::refreshRoutesStatic() {
	executeRemoteMethod( _NetworkManager_METHOD_RoutesStatic_list );
	this->m_ui->wUpBarRoutesStatic->sayRetrieving();
 }

void FormNetworkManager::refreshRoutesDynamic() {
	executeRemoteMethod( _NetworkManager_METHOD_RoutesDynamic_list );
	this->m_ui->wUpBarRoutesDynamic->sayRetrieving();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormNetworkManager::btnForceDiscovery_clicked() {
	Message* message = new Message( new Request( _Discovery_METHOD_discover ) );
	message->setSource( this->myURI );
	message->getDestination().fromString( ":Bundles/Discovery" );
	message->setDestinationBus( _Bus_Core );
	message->setSecurityToken( SecurityManager::generateToken( GuiCache::CurrentUser ) );
	BusGui.postMessage(message);
 }

void FormNetworkManager::btnChannelCreate_clicked() {
	bool ok;
	QString text = QInputDialog::getText(this, this->windowTitle(), tr("Connection String: (PROTOCOL://IP_ADDRESS:PORT)\n(such as TCP://192.168.0.1:9797, usually the IP refers to localhost)"), QLineEdit::Normal, "", &ok);
	if ( !ok || text.isEmpty())
		return;
	string proto = text.toLatin1().data();
	executeRemoteMethod( _NetworkManager_METHOD_CreateChannel, proto );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormNetworkManager::busMessage( const Message& message, const Bus* ) {
	if ( message.isEmpty() || !message.getContent()->isResponse() ) return;
	if ( !isMessageForMeOrFromTarget(message) ) return;
	const wosh::Response* response = message.getContent()->asResponse();

	this->m_ui->tableViewHosts->evalResponse(response);
	this->m_ui->tableViewChannels->evalResponse(response);
	this->m_ui->tableViewProtocols->evalResponse(response);
	this->m_ui->tableViewNeighbours->evalResponse(response);
	this->m_ui->tableViewRoutesStatic->evalResponse(response);
	this->m_ui->tableViewRoutesDynamic->evalResponse(response);

	if ( response->getMethod() == _NetworkManager_METHOD_CreateChannel ) {
		if ( response->hasData() )
			refreshChannels();
	 }
	promptMethodResponse(message);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace gui
}; // namespace wosh
