/**
 * @class   wosh::gui::FormNetworkManager
 * @brief   Graphic interface of wosh::NetworkManager core-module.
 *
 * \image html extras/screenshots/WOSH.GUI.FormNetworkManager.1.png "FormNetworkManager::Tab 1"
 * \image html extras/screenshots/WOSH.GUI.FormNetworkManager.2.png "FormNetworkManager::Tab 2"
 * \image html extras/screenshots/WOSH.GUI.FormNetworkManager.3.png "FormNetworkManager::Tab 3"
 * \image html extras/screenshots/WOSH.GUI.FormNetworkManager.4.png "FormNetworkManager::Tab 4"
 * \image html extras/screenshots/WOSH.GUI.FormNetworkManager.5.png "FormNetworkManager::Tab 5"
 * \image html extras/screenshots/WOSH.GUI.FormNetworkManager.6.png "FormNetworkManager::Tab 6"
 *
 * @ingroup Gui
 ****************************************************************************
 * @version $Id: FormNetworkManager.h 1716 2010-01-28 02:50:00Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_FormNetworkManager_H__
 #define __WOSH_Gui_FormNetworkManager_H__

 #include <framework/gui/IWidget.h>
 #include <framework/gui/WoshTableModel.h>


namespace Ui {
 class FormNetworkManager;
};
using namespace std;
namespace wosh {
 namespace gui {


class FormNetworkManager :  public IWidgetGeneric
 {
	Q_OBJECT

	WOSH_CLASSNAME(wosh::gui::FormNetworkManager)
	WOSH_CLASS_ABSTRACT(false)


	public:
		FormNetworkManager(QWidget *parent = 0);
		virtual ~FormNetworkManager();

	public:
		virtual void busMessage( const Message& message, const Bus* source );

	public slots:
		virtual void refreshAll();

	protected slots:
		void refreshHosts();
		void refreshChannels();
		void refreshProtocols();
		void refreshNeighbours();
		void refreshNetworkTree();
		void refreshRoutesDynamic();
		void refreshRoutesStatic();

		void btnForceDiscovery_clicked();
		void btnChannelCreate_clicked();

	protected:
		void changeEvent(QEvent *e);

	protected:

	private:
		Ui::FormNetworkManager *m_ui;

}; // class def


 }; // namespace gui
}; // namespace wosh


#endif //__WOSH_Gui_FormNetworkManager_H__
