/** @file    FormSystemLogger.cpp
 *  @author  Alessandro Polo
 *  @version $Id: FormSystemLogger.cpp 1708 2010-01-26 22:28:01Z alex $
 *  @brief
 * File containing methods for the wosh::gui::FormSystemLogger class.
 * The header for this class can be found in FormSystemLogger.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "FormSystemLogger.h"
 #include "ui_FormSystemLogger.h"
 #include <core/SystemLogger.h>

 #include <QtGui/QInputDialog>


using namespace std;
namespace wosh {
 namespace gui {

 WOSH_REGISTER(wosh::gui::FormSystemLogger, "wosh::gui::IWidgetGeneric", 1.0, _static_FormSystemLogger )
 WOSH_GUI_MAP(wosh::gui::FormSystemLogger, "wosh::SystemLogger", 0.0, _static_Map_FormSystemLogger )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

FormSystemLogger::FormSystemLogger(QWidget *parent) : IWidgetGeneric(parent)
 {
	this->m_ui = new Ui::FormSystemLogger();
	this->m_ui->setupUi(this);

	setupFor(_SystemLogger_URI);

	// tab LogContexts [0]
	this->m_ui->tableViewContexts->setDataRequest(_METHOD_List);
	this->m_ui->tableViewContexts->setResizeStretch(0);
	connect(this->m_ui->tableViewContexts, SIGNAL(updated()), this->m_ui->wUpBarContexts, SLOT(sayReady()));
	connect(this->m_ui->wUpBarContexts, SIGNAL(updateClicked()), this, SLOT(refreshContexts()));

	// tab LogHandlers [1]
	this->m_ui->tableViewHandlers->setDataRequest(_SystemLogger_METHOD_handlers);
	this->m_ui->tableViewHandlers->setResizeStretch(0);
	connect(this->m_ui->tableViewHandlers, SIGNAL(updated()), this->m_ui->wUpBarHandlers, SLOT(sayReady()));
	connect(this->m_ui->wUpBarHandlers, SIGNAL(updateClicked()), this, SLOT(refreshHandlers()));
	connect(this->m_ui->btnHandlerAdd, SIGNAL(clicked()), this, SLOT(btnHandlerAdd_clicked()));

 }

FormSystemLogger::~FormSystemLogger() {
	delete this->m_ui;
 }

void FormSystemLogger::changeEvent(QEvent *e) {
	QWidget::changeEvent(e);
	if ( e->type() == QEvent::LanguageChange )
		this->m_ui->retranslateUi(this);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormSystemLogger::refreshAll() {
	this->setWindowTitle( QString("System Logger [%1]").arg(this->getTargetURI().getKernelName().c_str()) );
	refreshContexts();
	refreshHandlers();
 }

void FormSystemLogger::refreshContexts() {
	executeRemoteMethod( _METHOD_List );
	this->m_ui->wUpBarContexts->sayRetrieving();
 }

void FormSystemLogger::refreshHandlers() {
	executeRemoteMethod( _SystemLogger_METHOD_handlers );
	this->m_ui->wUpBarHandlers->sayRetrieving();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormSystemLogger::btnHandlerAdd_clicked() {
	bool ok;
	QString htype = QInputDialog::getText(this, tr("Add New Handler.."), tr("Handler Type:"), QLineEdit::Normal, "wosh::", &ok);
	if ( !ok || htype.isEmpty() )
		return;

 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormSystemLogger::busMessage( const Message& message, const Bus* ) {
	if ( message.isEmpty() || !message.getContent()->isResponse() ) return;
	if ( !isMessageForMeOrFromTarget(message) ) return;
	const wosh::Response* response = message.getContent()->asResponse();

	this->m_ui->tableViewContexts->evalResponse(response);
	this->m_ui->tableViewHandlers->evalResponse(response);

	promptMethodResponse(message);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace gui
}; // namespace wosh
