 /**
 * @class   wosh::gui::FormUserManager
 * @brief   Graphic interface of wosh::UserManager core-module.
 *
 * \image html extras/screenshots/WOSH.GUI.FormUserManager.1.png "FormUserManager::Tab 1"
 * \image html extras/screenshots/WOSH.GUI.FormUserManager.2.png "FormUserManager::Tab 2"
 * \image html extras/screenshots/WOSH.GUI.FormUserManager.3.png "FormUserManager::Tab 3"
 *
 * @ingroup Gui
 ****************************************************************************
 * @version 0.7.000 $Id: FormUserManager.h 1716 2010-01-28 02:50:00Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_FormUserManager_H__
 #define __WOSH_Gui_FormUserManager_H__

 #include <framework/gui/IWidget.h>
 #include <QModelIndex>
 #include <QtGui/QMenu>


namespace Ui {
 class FormUserManager;
};
using namespace std;
namespace wosh {
 namespace gui {


class FormUserManager :  public IWidgetGeneric
 {
	Q_OBJECT

	WOSH_CLASSNAME(wosh::gui::FormUserManager)
	WOSH_CLASS_ABSTRACT(false)


	public:
		FormUserManager(QWidget *parent = 0);
		virtual ~FormUserManager();

	public:
		virtual void busMessage( const Message& message, const Bus* source );

	protected slots:
		virtual void refreshAll();
		virtual void refreshUsers();
		virtual void refreshGroups();

		void tableViewUsers_doubleClicked( const QModelIndex& index );
		void tableViewUsers_ContextMenu( const QPoint& pos );
		void tableViewGroups_doubleClicked( const QModelIndex& index );
		void tableViewGroups_ContextMenu( const QPoint& pos );

		void btnLookupByProperty_clicked();
		void clearLookupResults();


	protected:
		void changeEvent(QEvent *e);

	protected:
		void buildMenus();
		QMenu* menuUser;
		QAction* menuUserTitle;
		QAction* menuUserSendMessage;

		QMenu* menuGroup;
		QAction* menuGroupTitle;
		QAction* menuGroupInquire;

	private:
		Ui::FormUserManager *m_ui;

}; // class def

 }; // namespace gui
}; // namespace wosh


#endif //__WOSH_Gui_FormUserManager_H__
