/** @file    NetworkProtocolUdp.cpp
 *  @author  Alessandro Polo
 *  @version $Id: NetworkUtilities.cpp 1873 2010-02-20 13:23:43Z alex $
 *  @brief
 * File containing methods for the wosh::network::NetworkProtocolUdp class.
 * The header for this class can be found in NetworkProtocolUdp.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <framework/network/NetworkUtilities.h>

 #include <core/Utilities.h>

 #include <QNetworkInterface> // for listAvailableInterfaces


using namespace std;
namespace wosh {
 namespace network {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////



//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT NetworkUtilities::listAvailableInterfaces( std::vector<std::string>& interfaces )
 {
	QList<QHostAddress> hosts = QNetworkInterface::allAddresses ();
	for (int i = 0; i < hosts.count(); ++i) {
		interfaces.push_back( string(hosts[i].toString().toLatin1().data()) );
	 }

#ifdef _OS_POSIX
/*
QProcess *proc=new QProcess();
proc->start("ifconfig eth0 | grep inet  | awk {print $2} | sed s/addr://");
proc->waitForFinished(-1);
QString out=proc->readLine();
*/
#else

#endif
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool NetworkUtilities::isAddressPortValid( const std::string& address )
 {
	if ( address.size() < 9 ) // x.y.z.j:k
		return false;

	if ( getPortFromDestination(address) <= 0 )
		return false;

	// ipv4
	std::string ip_ = wosh::network::NetworkUtilities::getAddressFromDestination(address);
	std::vector<std::string> ip_oct;
	wosh::Utilities::splitString(ip_, ".", ip_oct, false, true);
	if ( ip_oct.size() != 4 )
		return false;

	return true;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

short NetworkUtilities::compareSubNet( const std::string& ip, const std::string& mask )
 {
	std::string ip_ = wosh::network::NetworkUtilities::getAddressFromDestination(ip);
	std::string mask_ = wosh::network::NetworkUtilities::getAddressFromDestination(mask);

	std::vector<std::string> ip_oct;
	wosh::Utilities::splitString(ip_, ".", ip_oct, false, true);
	std::vector<std::string> mask_oct;
	wosh::Utilities::splitString(mask_, ".", mask_oct, false, true);

	int min = _GET_Min(ip_oct.size(), mask_oct.size());
	int ind = -1;
	int sub = 0;
	while( ++ind < min ) {
		if ( ip_oct.at(ind) == mask_oct.at(ind) )
			++sub;
	 }

	return sub;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::string NetworkUtilities::getAddressFromDestination( const std::string& address )
 {
	std::size_t found = address.find(":");
	if ( found == std::string::npos )
		return address;

	return address.substr(0,found);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

int NetworkUtilities::getPortFromDestination( const std::string& address )
 {
	int port = 0;
	std::size_t found = address.find(":");
	if ( found != std::string::npos )
		port = std::atoi( address.substr(found+1,6).c_str()) ;

	return port;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::string NetworkUtilities::getBroadcastAddress( const std::string& address, short subnet )
 {
	std::vector<std::string> ip_oct;
	wosh::Utilities::splitString(address, ".", ip_oct, false, true);
	if ( subnet < 0 || (unsigned int)subnet > ip_oct.size() )
		return address;
	ip_oct.at(subnet) = "255";
	return wosh::Utilities::joinVector( ip_oct, "." );
 }
 

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace network
}; // namespace wosh
