/**
 * @class   wosh::network::qt::ServerTcpQt
 * @brief   The class provides a TCP-based server, it handles
 *          incoming TCP connections.
 *
 ****************************************************************************
 * @version $Id: ServerTcpQt.h 1917 2010-02-22 12:05:14Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Network_Qt_ServerTcpQt_H__
 #define __WOSH_Network_Qt_ServerTcpQt_H__

 #include <framework/network/ServerTcp.h>
 #include <vector>
 #include <framework/network/Qt/SocketTcpQt.h>

 #include <QTcpServer>


namespace wosh {
 namespace network {
  namespace qt {

// #define _ServerTcpQt_DEBUG_ENABLED


class ServerTcpQt :		private QTcpServer,
						public wosh::network::ServerTcp
 {
	Q_OBJECT
	WOSH_CLASSNAME(wosh::network::qt::ServerTcpQt)
	WOSH_CLASS_ABSTRACT(false)


	public:

		ServerTcpQt();

		virtual ~ServerTcpQt();

	public:

		virtual WRESULT bindSocket( unsigned long timeout = LIMIT_ULONG_MAX );

		virtual WRESULT unbindSocket( unsigned long timeout = LIMIT_ULONG_MAX );


		virtual bool isListening() const		{ return QTcpServer::isListening(); }


	protected:
		virtual void incomingConnection( int socketId );

	protected:

#ifdef _ServerTcpQt_DEBUG_ENABLED
		mutable wosh::LogContext Log;
#endif

		friend class wosh::network::qt::SocketTcpQt;


}; // class def

  }; // namespace qt
 }; // namespace network
}; // namespace wosh

#endif //__WOSH_Network_Qt_ServerTcpQt_H__
