/**
 * @class   wosh::network::ServerTcp
 * @brief   The class provides a TCP-based server, it handles
 *          incoming TCP connections.
 *
 ****************************************************************************
 * @version $Id: ServerTcp.h 1917 2010-02-22 12:05:14Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Network_libSockets_ServerTcp_H__
 #define __WOSH_Network_libSockets_ServerTcp_H__

 #define _ServerTcp_DEBUG_ENABLED

 #include <core/Thread.h>
 #include <framework/network/ServerTcp.h>
 #include <framework/network/libSockets/SocketTcpLS.h>


class ServerTcpWrapper;
namespace Sockets {
	class SocketHandler;
};
//struct sockaddr;

namespace wosh {
 namespace network {
  namespace libsockets {


class ServerTcpLS :		private wosh::Thread,
						public wosh::network::ServerTcp
 {
	WOSH_CLASSNAME(wosh::network::libsockets::ServerTcpLS)
	WOSH_CLASS_ABSTRACT(false)


	public:

		ServerTcpLS();

		virtual ~ServerTcpLS();

	public:

		virtual WRESULT bindSocket( unsigned long timeout = LIMIT_ULONG_MAX );

		virtual WRESULT unbindSocket( unsigned long timeout = LIMIT_ULONG_MAX );


		virtual bool isListening() const;


	protected:
		bool initializingThread();
		void runThread();
		void exitingThread();

		Sockets::SocketHandler* sHandler;
		ServerTcpWrapper* serverWapper;

		friend class ServerTcpWrapper;
		friend class wosh::network::SocketTcp;


	public:
	//	virtual void incomingConnection( int socketId );

void OnConnect();
void OnDisconnect();

}; // class def

  }; // namespace libsockets
 }; // namespace network
}; // namespace wosh

#endif //__WOSH_Network_libSockets_ServerTcp_H__
