/**
 * @class   wosh::EncoderWriterXml
 * @brief   
 *
 *
 *
 *
___________________________________________ OK
List ok users
	- SAVE: foreach(user) push(user, NODE);
	- LOAD: while (hasNext) pop(user);
___________________________________________________________
group
	- SAVE: 
	 - push(name, FIELD)
	 - push(description, FIELD)
	 - foreach(user) push(user, NODE);
	- LOAD
	 - pop(name)
	 - pop(description)
	 - while (hasNext) pop(user);

___________________________________________________________
===========================================================
3 SERIALIZATION CLASSES:
	- STREAM
	  -push/pop (char*, size), MARSHALLING, DELIMITERS, EOF
	- TABLE
	  - count(row, col)
	  - get(row, col)
	- TREE
	  - getRoot()
	  - TREE_NODE::getChildren(), TREE_NODE::getChild(index),
	  - TREE_NODE::get()
	  - TREE_NODE::getAttributes::(), ATTRIBUTE TREE_NODE::getChild(index),
	  - TREE_NODE::ATTRIBUTE::get()
	- (+interleaved) using inheritance & composition


	- MAP
	  - ISerializable => ISerializable

___________________________________________________________
===========================================================

 - ROOT
  - NODE[]
   - ATTRIBUTE[] { key<string>:value<string> }
   - NODE[]


START_MODE {FIELD, NODE}
DO [..]
STOP MODE
[..]

________________
XML:
 - read/parse(validate) XML DOC
 - iterator: select(ROOT)

  - recognize class-name:
    - HOW allocate?  => YOU NEED mapping<string, CLASSNAME>

	Tree* tree;
	return tree->retrieve(this);

 *
 *
 ****************************************************************************
 * @version $Id: EncoderWriterXml.h 1920 2010-02-22 13:42:51Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Persistence_Qt_EncoderWriterXml_H__
 #define __WOSH_Persistence_Qt_EncoderWriterXml_H__

 #include <woshDefs.h>
 #include <core/EncoderWriter.h>

 #include <core/LogContext.h>

class QDomDocument;
class QDomElement;
 #include <QtXml/QDomDocument>
 #include <QtXml/QDomElement>



namespace wosh {
 namespace persistence {


 #define _EncoderWriterXml_TYPE					"wosh::persistence::EncoderWriterXml"

class EncoderWriterXml : public IEncoderWriter
 {

	public:

		EncoderWriterXml();

		virtual ~EncoderWriterXml() {
			close();
		 }

		virtual int getType() const		{ return 0; } // { UNKNOWN, BINARY, TEXT, XML, SQL }

	public:
		virtual WRESULT open( const std::string& filename);
		virtual WRESULT close();

	public:
		virtual WRESULT push( const std::string& obj, const std::string& field_name = "", SerializeStrategy strategy = SERIALIZE_DEFAULT );
		virtual WRESULT push( const long obj, const std::string& field_name = "", SerializeStrategy strategy = SERIALIZE_DEFAULT );


		virtual WRESULT push( const ISerializable* object, const std::string& field_name, SerializeStrategy strategy = SERIALIZE_NODE );


	public:
		virtual void setIndentLevel( short offset, bool reset = false ) {
			(void)offset; (void)reset;
		 }

	public:

		template <typename T>
		WRESULT createNode( const std::string& tagName, const std::string& classType, const T& content )
		 {
			if ( this->doc == NULL || this->currentNode == NULL )
				return WRET_ERR_INTERNAL;
			
			QDomElement newNode = this->doc->createElement( tagName.c_str() );

			if ( this->explicitTypes )
				newNode.setAttribute( "type", classType.c_str() );

			newNode.appendChild( this->doc->createTextNode( QString("%1").arg(content) ) );
			this->currentNode->appendChild(newNode);

			return WRET_OK;
		 }

	protected:
		wosh::LogContext Log;

		QDomDocument* doc;
		std::string docFilename;

		bool explicitTypes;

	protected:
		QDomElement* currentNode;


	private:
		EncoderWriterXml( const EncoderWriterXml& ) : IEncoderWriter() { }
		EncoderWriterXml& operator=( const EncoderWriterXml& ) { return *this; }

}; // class def



 }; // namespace persistence
}; // namespace wosh

#endif //__WOSH_Persistence_Qt_EncoderWriterXml_H__
