/**
 * @file    AlarmListeners.h
 * @brief   Defines listeners for callbacks.
 *
 *
 *
 ****************************************************************************
 * @version $Id: AlarmListeners.h 2057 2010-03-08 21:32:15Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Services_AlarmListeners_H__
 #define __WOSH_Services_AlarmListeners_H__

 #include <core/Message.h>


namespace wosh {
 namespace security {


class AlarmEvent;
class AlarmState;

class IAlarmStateListener  {

	public:
		virtual void changed_AlarmState( int state, AlarmState* source ) = 0;

		virtual void raised_AlarmEvent( const AlarmEvent* alarm_event, AlarmState* source ) = 0;

	public:
		IAlarmStateListener() { }
		virtual ~IAlarmStateListener() { }

}; // class def



class AlarmBrain;

class IAlarmBrainListener  {

	public:
		virtual void raised_AlarmBrain_Error( int error, const std::string& description, AlarmBrain* source ) = 0;

	public:
		IAlarmBrainListener() { }
		virtual ~IAlarmBrainListener() { }

}; // class def



class AlarmEngine;

class IAlarmEngineListener : public virtual IAlarmStateListener, public virtual IAlarmBrainListener  {

	public:
		virtual void alarm_armed( AlarmEngine* source ) = 0;
		virtual void alarm_disarmed( AlarmEngine* source ) = 0;

		virtual void postMessage( Message* message, AlarmEngine* source ) = 0;

	public:
		IAlarmEngineListener() { }
		virtual ~IAlarmEngineListener() { }



}; // class def

 }; // namespace security
}; // namespace wosh

#endif //__WOSH_Services_AlarmListeners_H__
