/**
 * @class   wosh::security::AlarmTriggerGeneric
 * @brief   
 *
 *
 *
 *
 *
 ****************************************************************************
 * @version $Id: AlarmTriggerGeneric.h 2057 2010-03-08 21:32:15Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Security_AlarmTriggerGeneric_H__
 #define __WOSH_Security_AlarmTriggerGeneric_H__

 #include <core/MessageFilter.h>
 #include <framework/security/AlarmTrigger.h>

 #include <framework/building/Door.h>

namespace wosh {
 namespace security {

 #define _AlarmTriggerGeneric_TYPE			"wosh::security::AlarmTriggerGeneric"


class AlarmTriggerGeneric : public AlarmTrigger
 {
	WOSH_SERIALIZABLE(wosh::security::AlarmTriggerGeneric)
	WOSH_COMPARE_RTTI(wosh::security::AlarmTriggerGeneric)

	typedef List22<wosh::MessageFilter*> tMessageFilterList;


	public:
		AlarmTriggerGeneric() : AlarmTrigger() {
		 }
		/**
		 * @brief  Copy constructor.
		 */
		AlarmTriggerGeneric( const AlarmTriggerGeneric& m ) : AlarmTrigger(m) {
			*this = m;
		 }

		virtual ~AlarmTriggerGeneric() { }

	public:

		virtual long evaluateMessage( const Message* message, AlarmState* currentState )
		 {
			if ( message == NULL || message->isEmpty() || currentState == NULL ) return 0;

			if ( isTrigger(message) ) {
				return raiseEvent(currentState, Utilities::format(this->description.c_str(), message->getSource().getName().c_str() ) );
			 }

			return 1;
		 }


		virtual bool isTrigger( const Message* message )
		 {
			MutexLockerRead mL(this->triggers.getMutex());

			int trig = 0;
			tMessageFilterList::ConstIterator it;
			const tMessageFilterList::ConstIterator it_end = this->triggers.end();
			for ( it=this->triggers.begin(); it!=it_end; ++it ) {
				if ( *it == NULL ) continue;
				trig = (*it)->compare(*message);
				if ( trig == 0 ) {
					return true;
				 }
			 }

			return false;
		 }

		const tMessageFilterList getTriggers() const	{ return this->triggers; }
		tMessageFilterList getTriggers()				{ return this->triggers; }


	protected:
		tMessageFilterList triggers;




}; // class def

 }; // namespace security
}; // namespace wosh

#endif //__WOSH_Security_AlarmTriggerGeneric_H__
