/**
 * @class   wosh::security::AlarmTriggerPerimetral
 * @brief   
 *
 *
 *
 *
 *
 ****************************************************************************
 * @version $Id: AlarmTriggerPerimetral.h 2057 2010-03-08 21:32:15Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Security_AlarmTriggerPerimetral_H__
 #define __WOSH_Security_AlarmTriggerPerimetral_H__

 #include <framework/security/AlarmTrigger.h>

 #include <framework/building/Door.h>


namespace wosh {
 namespace security {

 #define _AlarmTriggerPerimetral_TYPE			"wosh::security::AlarmTriggerPerimetral"


class AlarmTriggerPerimetral : public AlarmTrigger
 {
	WOSH_SERIALIZABLE(wosh::security::AlarmTriggerPerimetral)
	WOSH_COMPARE_RTTI(wosh::security::AlarmTriggerPerimetral)

	public:

		AlarmTriggerPerimetral() : AlarmTrigger() {
		 }
		/**
		 * @brief  Copy constructor.
		 */
		AlarmTriggerPerimetral( const AlarmTriggerPerimetral& m ) : AlarmTrigger(m) {
			*this = m;
		 }

		virtual ~AlarmTriggerPerimetral() { }

	public:

		virtual long evaluateMessage( const Message* message, AlarmState* currentState )
		 {
			if ( message == NULL || message->isEmpty() || currentState == NULL ) return 0;

			// is a Building Object event?
			if ( message->getSourceBus() != _Bus_Building )
				return 0;
			// it's an event/fact ?
			if ( !message->getContent()->isFact() )
				return 0;
			const Fact* fact_event = message->getContent()->asFact();

			// is a (external) door ?!
			///@bug SYSTEM PROBLEM! how to detect this?!?????

			// is (right) event?
			if ( fact_event->getEvent() != _Door_EVENT_DoorStateChanged )
				return 0;
			const ISerializable* obj = fact_event->getData();
			
			AlarmEvent* alarm_event = new AlarmEvent();
			alarm_event->setAlarmLevel( getAlarmLevel() );
			if ( obj->isClass(_Variant_TYPE) == 0 ) {
				const wosh::Variant* obj_data = dynamic_cast<const wosh::Variant*>(obj);
				if ( obj_data->toInteger(-1) == 100 )
					alarm_event->setDescription("Door !! has been opened!");
				else
					alarm_event->setDescription("Door !! has been closed!");
			 }

			currentState->raiseEvent(alarm_event);

			return 1;
		 }

//@}
	protected:




}; // class def

 }; // namespace security
}; // namespace wosh


#endif //__WOSH_Security_AlarmTriggerPerimetral_H__
