/**
 * @class   wosh::xtended::WeatherInfo
 * @brief   
 *
 *
 *
 ****************************************************************************
 * @version $Id: WeatherInfo.h 1708 2010-01-26 22:28:01Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Xtended_WeatherInfo_H__
 #define __WOSH_Xtended_WeatherInfo_H__

 #include <woshDefs.h>
 #include <core/Serialization.h>
 

namespace wosh {
 namespace xtended {

 #define _WeatherInfo_TYPE					"wosh::xtended::WeatherInfo"
 #define _WeatherInfo_VERSION				0.12

 #define _WeatherInfo_TemperatureUndefined	-10000


class WeatherInfo : public virtual ISerializable
 {
	WOSH_SERIALIZABLE(wosh::xtended::WeatherInfo)
	WOSH_COMPARE_RTTI(wosh::xtended::WeatherInfo)

	public:

		WeatherInfo() {
			clear();
		 }

		WeatherInfo( const WeatherInfo& m ) {
			*this = m;
		 }

		virtual ~WeatherInfo() { }


//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:
		void setTimestamp( long ts = -1 ) {
			this->timestamp = ts;
		 }

		void setPeriodName( const std::string& value )		{ this->period_name = value; }

		void setConditionKey( const std::string& value )		{ this->condition_key = value; }
		void setConditionSummary( const std::string& value )	{ this->condition_summary = value; }

		void setTemperatureHighest( int value )			{ this->temperature_highest = value; }
		void setTemperatureLowest( int value )			{ this->temperature_lowest = value; }


//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:
		long getTimestamp() const					{ return this->timestamp; }

		const std::string& getPeriodName() const			{ return this->period_name; }
		long getPeriodStart() const					{ return this->period_start; }
		long getPeriodEnd() const					{ return this->period_end; }

		const std::string& getConditionKey() const		{ return this->condition_key; }
		const std::string& getConditionSummary() const	{ return this->condition_summary; }

		int getTemperatureHighest() const			{ return this->temperature_highest; }
		int getTemperatureLowest() const			{ return this->temperature_lowest; }


//		const string& getConditionIcon() const		{ return this->condition_icon; }

//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:

		virtual WeatherInfo& operator=(const WeatherInfo& m) {
			if (this == &m) return *this; // same object?

			this->timestamp = m.timestamp;

			this->period_name = m.period_name;
			this->period_start = m.period_start;
			this->period_end = m.period_end;

			this->condition_key = m.condition_key;
			this->condition_summary = m.condition_summary;

			this->temperature_highest = m.temperature_highest;
			this->temperature_lowest = m.temperature_lowest;

			return *this;
		 }
		virtual bool operator==(const WeatherInfo &other) const {
			if ( this->timestamp != other.timestamp )
				return false;
			return true;
		 }
		bool operator!=(const WeatherInfo &other) const	{ return !(*this == other);  }

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const		{ return ""; }

		virtual WRESULT store( IEncoderWriter* serializer ) const {
			if ( serializer == NULL ) return WRET_ERR_PARAM;

			return WRET_OK;
		 }
		virtual WRESULT retrieve( IEncoderReader* deserializer ) {
			if ( deserializer == NULL ) return WRET_ERR_PARAM;

			return WRET_OK;
		 }

//@}
	protected:
		void clear() {
			this->timestamp = 0;

			this->period_name = "";
			this->period_start = 0;
			this->period_end = 0;

			this->condition_key = "";
			this->condition_summary = "";

			this->temperature_highest = _WeatherInfo_TemperatureUndefined;
			this->temperature_lowest = _WeatherInfo_TemperatureUndefined;
		 }

	protected:
		long timestamp;				///< (epoch) timestamp of the information

		std::string period_name;			///< human_friendly name of the period (such as 'thus')
		long period_start;			///< (epoch) beginning of period
		long period_end;			///< (epoch) end of period 

		std::string condition_key;		///< identifier of the condition (such as 'rain')
		std::string condition_summary;	///< textual summary of the condition

		int temperature_highest;	///< highest temperature in degrees celsius
		int temperature_lowest;		///< lowest temperature in degrees celsius

		// PRECIPITATION
		// WIND


}; // class def

 }; // namespace xtended
}; // namespace wosh

#endif //__WOSH_Xtended_WeatherInfo_H__
