/**
 * @class   wosh::interfaces::Device
 * @brief   Standard interface of generic device.
 *
 * @ingroup Interfaces
 ****************************************************************************
 * @version 0.10 $Id: Device.h 1947 2010-02-25 00:05:20Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Interfaces_Device_H__
 #define __WOSH_Interfaces_Device_H__

 #include <woshDefs.h>
 #include <interfaces/IBundle.h>


namespace wosh {
 namespace interfaces {
  namespace devices { };

 #define _Device_TYPE					"wosh::Device"

 #define _Device_TYPE_Unknown			"wosh::Device::UNKNOWN"


class Device : public virtual IBundle
 {
	WOSH_BUNDLE_INTERFACE("wosh::Device")

	public:
		static const int AllowedOnSameKernel = -1;
		static const int AllowedOnSameNetwork = -1;


	public:
	
		/**
		 * @brief   states of a device
		 */
		enum DEVICE_STATE {
			UNKNOWN = 0,
			LINKED = 1,
			UNLINKED = 2,
			BROKEN = 4
		};


	public:

		static std::string getDeviceStateAsString( wosh::interfaces::Device::DEVICE_STATE state )
		 {
			switch(state) {
				case wosh::interfaces::Device::LINKED:		return "LINKED";
				case wosh::interfaces::Device::UNLINKED:	return "UNLINKED";
				case wosh::interfaces::Device::BROKEN:		return "BROKEN";
				case wosh::interfaces::Device::UNKNOWN:
				default:
													return "UNKNOWN";
			 }
		 }

		static wosh::interfaces::Device::DEVICE_STATE getDeviceStateFromString( std::string state )
		 {
			if ( state == "LINKED" )				return wosh::interfaces::Device::LINKED;
			else if ( state == "UNLINKED" )			return wosh::interfaces::Device::UNLINKED;
			else if ( state == "BROKEN" )			return wosh::interfaces::Device::BROKEN;
			return wosh::interfaces::Device::UNKNOWN;
		 }


/** @name Device Interface
 * @{
 ******************************************************************************/
	public:

		inline bool isDeviceLinked() const		{ return (getDeviceState() == wosh::interfaces::Device::LINKED); }
		inline bool isDeviceUnLinked() const	{ return (getDeviceState() == wosh::interfaces::Device::UNLINKED); }
		inline bool isDeviceBroken() const		{ return (getDeviceState() == wosh::interfaces::Device::BROKEN); }

		virtual wosh::interfaces::Device::DEVICE_STATE getDeviceState() const = 0;

//@}

	public:
		virtual ~Device() { }

}; // class def

 }; // namespace interfaces
}; // namespace wosh

#endif //__WOSH_Interfaces_Device_H__
