/**
 * @class   wosh::interfaces::devices::MailBox
 * @brief   Standard interface (representation) of mail-box.
 *
 * @see     wosh::devices::MailBoxBundle
 * @ingroup Interfaces
 ****************************************************************************
 * @version 0.00 $Id: MailBox.h 2074 2010-03-10 02:38:16Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Interfaces_Devices_MailBox_H__
 #define __WOSH_Interfaces_Devices_MailBox_H__

 #include <interfaces/Device.h>


namespace wosh {
 namespace interfaces {
  namespace devices {

 #define _MailBox_TYPE					"wosh::devices::MailBox"

#define _MailBox_KEY_AutoCheckFrequency	"AutoCheckFrequency"

#define _MailBox_KEY_SmtpHostName		"SmtpHostName"
#define _MailBox_KEY_SmtpUserName		"SmtpUserName"
#define _MailBox_KEY_SmtpUserPassword	"SmtpUserPassword"

#define _MailBox_KEY_Pop3HostName		"Pop3HostName"
#define _MailBox_KEY_Pop3UserName		"Pop3UserName"
#define _MailBox_KEY_Pop3UserPassword	"Pop3UserPassword"

#define _MailBox_METHOD_sendmail		"sendmail"



class MailBox : public virtual Device
 {
	WOSH_BUNDLE_INTERFACE("wosh::interfaces::devices::MailBox")


	public:
		static const int AllowedOnSameKernel = -1;	///< There can be multiple (any) instances within a kernel
		static const int AllowedOnSameNetwork = -1;	///< There can be multiple (any) instances on the network

	public:
		/**
		 * @brief   states of a connection
		 */
		enum CONNECTION_STATE {
			UNKNOWN,
			CONNECTED,
			DISCONNECTED,
			STALLED
		};



	public:
		virtual ~MailBox() { }

}; // class def

  }; // namespace devices
 }; // namespace interfaces
}; // namespace wosh

#endif //__WOSH_Interfaces_Devices_MailBox_H__
