/**
 * @file    device_registration.cpp
 * @brief   Register interfaces of Devices on dynamic-initialization
 *          (see ObjectAllocator and MACROS for details).
 *
 ****************************************************************************
 * @version 0.8.499 $Id: device_registration.cpp 1946 2010-02-24 23:34:37Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <interfaces/Device.h>
 #include <core/ObjectAllocator.h>


 #include <interfaces/devices/DeviceX10.h>
 #include <interfaces/devices/Dimmer.h>
 #include <interfaces/devices/JabberGloox.h>
 #include <interfaces/devices/MailBox.h>
 #include <interfaces/devices/Modem.h>
 #include <interfaces/devices/ModemDsl.h>
 #include <interfaces/devices/Phone.h>
 #include <interfaces/devices/PlayerAudio.h>
 #include <interfaces/devices/SensorLight.h>
 #include <interfaces/devices/SensorMotion.h>
 #include <interfaces/devices/Switch.h>
 #include <interfaces/devices/TestDevice.h>
 #include <interfaces/devices/VGettyAM.h>


namespace wosh {
 namespace interfaces {
  namespace devices {

  WOSH_REGISTER_INTERFACE(wosh::interfaces::Device, "wosh::interfaces::IBundle", 1.00, _static_Device )


  // DeviceX10
  WOSH_REGISTER_INTERFACE(wosh::interfaces::devices::DeviceX10, "wosh::interfaces::Device", 1.00, _static_DeviceX10 )
  // Dimmer
  WOSH_REGISTER_INTERFACE(wosh::interfaces::devices::Dimmer, "wosh::interfaces::Device", 1.00, _static_Dimmer )
  // Jabber(Gloox)
  WOSH_REGISTER_INTERFACE(wosh::interfaces::devices::JabberGloox, "wosh::interfaces::Device", 1.00, _static_JabberGloox )
  // MailBox
  WOSH_REGISTER_INTERFACE(wosh::interfaces::devices::MailBox, "wosh::interfaces::Device", 1.00, _static_MailBox )
  // Modem and ModemDsl
  WOSH_REGISTER_INTERFACE(wosh::interfaces::devices::Modem, "wosh::interfaces::Device", 1.00, _static_Modem )
  WOSH_REGISTER_INTERFACE(wosh::interfaces::devices::ModemDsl, "wosh::interfaces::devices::Modem", 1.00, _static_ModemDsl )
  // Phone
  WOSH_REGISTER_INTERFACE(wosh::interfaces::devices::Phone, "wosh::interfaces::Device", 1.00, _static_Phone )
  // Player Audio
  WOSH_REGISTER_INTERFACE(wosh::interfaces::devices::PlayerAudio, "wosh::interfaces::Device", 1.00, _static_PlayerAudio )
  // SensorLight
  WOSH_REGISTER_INTERFACE(wosh::interfaces::devices::SensorLight, "wosh::interfaces::Device", 1.00, _static_SensorLight )
  // SensorMotion
  WOSH_REGISTER_INTERFACE(wosh::interfaces::devices::SensorMotion, "wosh::interfaces::Device", 1.00, _static_SensorMotion )
  // Switch
  WOSH_REGISTER_INTERFACE(wosh::interfaces::devices::Switch, "wosh::interfaces::Device", 1.00, _static_Switch )
  // TestDevice
  WOSH_REGISTER_INTERFACE(wosh::interfaces::devices::TestDevice, "wosh::interfaces::Device", 1.00, _static_TestDevice )
  // VGetty Answering Machine
  WOSH_REGISTER_INTERFACE(wosh::interfaces::devices::VGettyAM, "wosh::interfaces::devices::Phone", 1.00, _static_VGettyAM )


  }; // namespace devices
 }; // namespace interfaces
}; // namespace wosh
