/**
 * @class   wosh::interfaces::services::Discovery
 * @brief   Standard interface of Discovery (service).
 *
 * @ingroup Interfaces
 ****************************************************************************
 * @version 1.00 $Id: Discovery.h 2016 2010-03-03 00:19:15Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Interfaces_Services_Discovery_H__
 #define __WOSH_Interfaces_Services_Discovery_H__

 #include <interfaces/Service.h>


namespace wosh {
 namespace interfaces {
  namespace services {

 #define _Discovery_TYPE						"wosh::services::Discovery"

/**
 * @def    _Discovery_KEY_NotifyFrequency
 * @brief  Property: the interval between each notification in seconds. Type: INT
 */
 #define _Discovery_KEY_NotifyFrequency			"NotifyFrequency"

/**
 * @def    _Discovery_METHOD_discover
 * @brief  Method: force a discovery of remote hosts (annunce myself on the network)
 */
 #define _Discovery_METHOD_discover				"discover"


class Discovery : public virtual Service
 {
	WOSH_BUNDLE_INTERFACE("wosh::interfaces::services::Discovery")

	public:
		static const int AllowedOnSameKernel = 1;	///< Only one instance is allowed on same kernel
		static const int AllowedOnSameNetwork = -1;	///< There can be multiple (any) instances on the network


	public:
		virtual ~Discovery() { }

}; // class def

  }; // namespace services
 }; // namespace interfaces
}; // namespace wosh

#endif //__WOSH_Interfaces_Services_Discovery_H__
