/** @file    ServiceStandaloneBundle.cpp
 *  @author  Alessandro Polo
 *  @version $Id: ServiceStandaloneBundle.cpp 2142 2010-03-16 12:45:50Z alex $
 *  @brief
 * File containing methods for the wosh::services::ServiceStandaloneBundle class.
 * The header for this class can be found in ServiceStandaloneBundle.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "ServiceStandaloneBundle.h"

 #include <core/MethodsDef.h>
 #include <core/Fact.h>
 #include <core/Table.h>
 #include <core/MessageProcessingCommon.h>
 #include <core/MessageFilter.h>
 #include <core/DataBaseManager.h>
 #include <core/SecurityManager.h>
 #include <core/BusManager.h>
 #include <core/DeviceManager.h>
 #include <core/FileSystem.h>
 #include <core/SystemError.h>


using namespace std;
namespace wosh {
 namespace services {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

ServiceStandaloneBundle::ServiceStandaloneBundle() : BundleGeneric()
 {
	BundleGeneric::setName( _ServiceStandalone_NAME );
	//Object::getURI().setKernelLocal();
	//Object::getURI().setName( "ServiceStandalone_STATIC_PATH" );
	Object::getURI().registerGlobal();
	Properties.update( _KEY_Version, _ServiceStandalone_VERSION );

//Log.setLevel(LOG_VERBOSE);

	Log(LOG_VERBOSE, " Setting default properties and permissions.." );

	Log(LOG_VERBOSE, " Registering methods.." );


	setBundleState(Bundle::STATE_CREATED, false);
 }

ServiceStandaloneBundle::~ServiceStandaloneBundle()
 {
	Log(LOG_VERBOSE, " Destroying.." );

	if ( isBundleRunning() ) {
		Log(LOG_WARNING, "~ServiceStandaloneBundle() : Destroying while Bundle is running! Trying to stop.." );
		bundleStop();
	 }

	Log(LOG_VERBOSE, ":~ServiceStandaloneBundle() : Destroyed." );
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL

WRESULT ServiceStandaloneBundle::bundleStart()
 {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STARTING) )
		return WRET_ERR_WRONG_STATE;

	setBundleState( Bundle::STATE_STARTED );
	return ret_loaded + bus_ok;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT ServiceStandaloneBundle::bundleStop()
 {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STOPPING) )
		return WRET_ERR_WRONG_STATE;

	setBundleState( Bundle::STATE_STOPPED );
	return ret_saved;
 }

////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////// BUS EVENTS

void ServiceStandaloneBundle::busMessage( const Message& message, const Bus* source )
 {
	(void)source;
	if ( message.isEmpty() ) return;



	// manage messages for myself
	if ( !MessageFilter::isMessageTarget(message, this) )
		return;

//	BundleGeneric::busMessage(message, source);
	Message* reply = MPC_busMessageRequest( message, Methods, Log );
	if ( reply == NULL )
		return;
	reply->setSource( this );
 }

////////////////////////////////////////////////////////////////////////////////////////////////// BUS EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENT

bool ServiceStandaloneBundle::readingProperty( Property* , const PropertiesProvider* )
 {
	return true;
 }

bool ServiceStandaloneBundle::updatingProperty( Property* property_new, const Property* property_old, const PropertiesProvider* source )
 {
	return BundleGeneric::updatingProperty( property_new, property_old, source );
 }

////////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENT
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS

///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace services
}; // namespace wosh
