/** @file    ServiceWithWorkerBundle.cpp
 *  @author  Alessandro Polo
 *  @version $Id: ServiceWithWorkerBundle.cpp 2142 2010-03-16 12:45:50Z alex $
 *  @brief
 * File containing methods for the wosh::services::ServiceWithWorkerBundle class.
 * The header for this class can be found in ServiceWithWorkerBundle.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "ServiceWithWorkerBundle.h"

 #include "ServiceWithWorkerWorker.h"

 #include <core/MethodsDef.h>
 #include <core/MessageProcessingCommon.h>
 #include <core/Fact.h>
 #include <core/WoshKernel.h>


using namespace std;
namespace wosh {
 namespace services {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

ServiceWithWorkerBundle::ServiceWithWorkerBundle() : BundleGeneric()
 {
	BundleGeneric::setName( _ServiceWithWorker_NAME );
	Properties.update( _KEY_Version, _ServiceWithWorker_VERSION );

//Log.setLevel(LOG_VERBOSE);

	Log(LOG_VERBOSE, " Configuring ServiceWithWorker worker.." );
	this->myWorker = new ServiceWithWorkerWorker(*this);

	Log(LOG_VERBOSE, " Setting default properties and permissions.." );

	Log(LOG_VERBOSE, " Registering methods.." );


	setBundleState(Bundle::STATE_CREATED, false);
 }


ServiceWithWorkerBundle::~ServiceWithWorkerBundle()
 {
	Log(LOG_INFO, " Destroying.." );

	if ( isBundleRunning() ) {
		Log(LOG_WARNING, "~ServiceWithWorkerBundle() : Destroying while Bundle is running! Trying to stop.." );
		bundleStop();
	 }

	delete this->myWorker; this->myWorker = NULL;

	Log(LOG_VERBOSE, ":~ServiceWithWorkerBundle() : Destroyed." );
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL

WRESULT ServiceWithWorkerBundle::bundleStart()
 {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STARTING) )
		return WRET_ERR_WRONG_STATE;

	WRESULT ret = BundleGeneric::start_SynchThread( this->myWorker );

	// BUNDLE-STATE (STARTED) will be updated async by WORKER, through call: thread_event()
	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT ServiceWithWorkerBundle::bundleStop()
 {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STOPPING) )
		return WRET_ERR_WRONG_STATE;

	WRESULT ret = BundleGeneric::stop_SynchThread( this->myWorker );

	// BUNDLE-STATE (STOPPED) will be updated async by WORKER, through call: thread_event()
	return ret;
 }

////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ServiceWithWorkerBundle::busMessage( const Message& message, const Bus* source )
 {
	if ( message.isEmpty() ) return;
	BundleGeneric::busMessage(message, source);

 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENT

bool ServiceWithWorkerBundle::updatingProperty( Property* property_new, const Property* property_old, const PropertiesProvider* source )
 {
	// maybe is a (common) BundleGeneric property, let base implementation process it
	return BundleGeneric::updatingProperty(property_new, property_old, source);
 }

////////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENT
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS

///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace services
}; // namespace wosh
