/**
 * @file    test_DSM.cpp
 * @brief   Test classes: wosh::Property, wosh::PropertiesProvider
 *          Indirectly tests also wosh::Variant
 *
 * @version $Id: test_DSM.cpp 2067 2010-03-09 23:40:30Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/

 #include "test_DSM_globals.h"
  
using namespace wosh;
using namespace std;
namespace wosh {
 namespace testsuite {

 
// meyer singleton
Bus& getBusDSM() {
	static BusRing bus_DSM(_Bus_DSM);
	return bus_DSM;
 }

void test_init() {
	log(LOG_INFO, "test_DSM()", "Initializing Testsuite.." );
	log(LOG_VERBOSE, "test_DSM()", "Creating and Starting DSM Bus.." );
	getBusDSM().startBus(); // synch, no timeout

	log(LOG_VERBOSE, "test_DSM()", "Creating DataSet.." );
	TestSharedObject::getDataSet().push_back( new TestSharedObject("obj1", 123) );
	TestSharedObject::getDataSet().push_back( new TestSharedObject("obj3", 789) );
	TestSharedObject::getDataSet().push_back( new TestSharedObject("obj2", 456) );
 }


void test_free() {
	log(LOG_INFO, "test_free()", "Destroying Testsuite.." );

	// heap error:
	//log(LOG_VERBOSE, "test_free()", "Destroying DSM instances.." );
	//while( SharedMemory::getInstances().size() ) {
	//	SharedMemory* dsm = SharedMemory::getInstances().itemAt(0);
	//	if ( dsm != NULL )
	//		delete dsm; // because list is updated by DSM destructor!
	//	else
	//		SharedMemory::getInstances().erase(SharedMemory::getInstances().begin());
	// }

	log(LOG_VERBOSE, "test_free()", "Stopping DSM Bus.." );
	getBusDSM().stopBus(); // synch, no timeout

	log(LOG_VERBOSE, "test_DSM()", "Destroying DataSet.." );
	TestSharedObject::getDataSet().free();

 }


SharedMemory* create_dsm( bool feed_dataset ) {
	log(LOG_WARNING, " CREATE_AGENT()", "Creating new Instance..[%i]", feed_dataset );
	SharedMemory* dsm = new SharedMemory( _TupleSpace_Name );
	dsm->getURI().setKernelName("woshfake");
	dsm->BusDSM.setBus(&getBusDSM());
	dsm->BusDSM.setMessageHandler(dsm);
	getBusDSM().addBusListener(&dsm->BusDSM);
	if ( feed_dataset )
		dsm->acquireDataSet( TestSharedObject::getDataSet() );
	dsm->init();
	return dsm;
 }

WRESULT delete_dsm( SharedMemory* dsm ) {
	if ( dsm == NULL )
		return WRET_ERR_PARAM;
	log(LOG_WARNING, " DESTROY_AGENT()", "Destroying Instance %ld..", dsm->getID() );
	WRESULT ret = dsm->deinit();
	if ( WFAILED(ret) )
		return ret;
	delete dsm;
	return WRET_OK;
 }
 
SharedMemory* getDsmById( long id ) {
	List22<SharedMemory*>::Iterator it = std::find_if(SharedMemory::getInstances().begin(), SharedMemory::getInstances().end(),
						Functor_Validator<SharedMemory,long>(id, &SharedMemory::getID ) );
	if ( it == SharedMemory::getInstances().end() )
		return NULL;
	return *it;
 }


 }; // namespace testsuite
}; // namespace wosh
