/**
 * @file    test_DSM.h
 * @brief   
 *
 * @version	$Id: test_DSM.h 1499 2009-12-31 16:39:34Z alex $
 * @author	Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/

#ifndef __WOSH_TestSuite_testglobals_DSM_H__
 #define __WOSH_TestSuite_testglobals_DSM_H__
 
 #include <core/Object.h>
 #include <core/Containers.h>
 #include <core/Functional.h>
 #include <core/BusRing.h>
 #include <core/BusConnector.h>
 #include <core/SharedMemory.h>
 #include <core/SharedObject.h>
 
using namespace wosh;
using namespace std;
namespace wosh {
 namespace testsuite {

#define _TupleSpace_Name	"testspace"

// meyer singleton
Bus& getBusDSM();

void test_init();

void test_free();

SharedMemory* create_dsm( bool feed_dataset );

WRESULT delete_dsm( SharedMemory* dsm );

SharedMemory* getDsmById( long id );

//---------------------------------------------------------------------


class TestSharedObject : public wosh::SharedObject
 {
	WOSH_CLASSNAME(TestSharedObject)
	WOSH_CLASS_ABSTRACT(false)
	WOSH_CLONE_CONSTRUCTOR(TestSharedObject)
	WOSH_COMPARE_RTTI(TestSharedObject)

	public:
		TestSharedObject( const std::string& name, long arg1 = 0, double arg2 = 0 )
			: wosh::SharedObject(), objName(name), value1(arg1), value2(arg2) { }

		TestSharedObject( const TestSharedObject& m )
			: wosh::SharedObject(m), objName(m.objName), value1(m.value1), value2(m.value2) { }

		virtual ~TestSharedObject() { }

		std::string objName;
		long value1;
		double value2;

	public:
		TestSharedObject& operator=( const Object& ) { return *this; }
		bool operator==( const TestSharedObject& ) const { return false; }
		bool operator!=(const TestSharedObject& other) const { return !(*this == other);  }
		inline std::string toString() const { return objName; }
		WRESULT store( IEncoderWriter*  ) const { return WRET_ERR_ILLEGAL_USE; }
		WRESULT retrieve( IEncoderReader*  ) { return WRET_ERR_ILLEGAL_USE; }

	public:
		// meyer singleton
		static List22<TestSharedObject*>& getDataSet() {
			static List22<TestSharedObject*> list;
			return list;
		 }


 };







 }; // namespace testsuite
}; // namespace wosh


#endif // __WOSH_TestSuite_testglobals_DSM_H__
