/** @file    testform_DSM.cpp
 *  @author  Alessandro Polo
 *  @version $Id: testform_DSM.cpp 2029 2010-03-04 09:06:49Z alex $
 *  @brief
 * File containing methods for the wosh::testsuite::testform_DSM class.
 * The header for this class can be found in testform_DSM.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include "test_DSM_globals.h"
 #include "testform_DSM.h"
 #include "testform_DSM_helpers.h"
 
 #include <core/Request.h>
 #include <core/Response.h>
 #include <core/NEW_List.h>
 #include <core/WoshKernel.h>
 #include <core/MethodsDef.h>
 #include <framework/gui/LogHandlerQRenderer.h>

 #include <QApplication>
 #include <QtGui>
 
using namespace wosh::testsuite;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
int main( int argc, char *argv[] ){
 //   Q_INIT_RESOURCE(application);
    QApplication app(argc, argv);
    app.setOrganizationName("WOSH");
    app.setApplicationName("DSM Testsuite");

	test_init();

    testform_DSM mainWin;
    mainWin.show();

	int ret = app.exec();

	test_free();

    return ret;
 }
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

namespace wosh {
 namespace testsuite {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

testform_DSM::testform_DSM() : QMainWindow()
 {
	this->m_ui = new Ui::testform_DSM();
	this->m_ui->setupUi(this);

	// LOG SETUP
	this->cHandler = new wosh::gui::LogHandlerQRenderer();
	this->cHandler->setColoured(true);
	this->cHandler->open();
//	connect(this->cHandler, SIGNAL(appendLine(const QString&)), this->m_ui->txtConsole, SLOT(insertPlainText(const QString&)), Qt::QueuedConnection);
	connect(this->cHandler, SIGNAL(appendLine(const QString&)), this->m_ui->txtConsole, SLOT(insertHtml(const QString&)), Qt::QueuedConnection);
	wosh::DefaultSharedLogHandler::getInstance()->addHandler(this->cHandler);


	connect(this->m_ui->menu_WOSH, SIGNAL( triggered(QAction*) ), this, SLOT(menu_triggered(QAction*)));
	connect(this->m_ui->menuDSMs, SIGNAL( triggered(QAction*) ), this, SLOT(menu_triggered(QAction*)));

	getBusDSM().addBusListener(this);

	// DSM List
	this->dsmModel = new DsmTableModel(this);
	this->m_ui->tableViewDSM->setModel( this->dsmModel );
	connect(this->m_ui->tableViewDSM, SIGNAL(clicked(const QModelIndex&)), this, SLOT(tableViewDSM_clicked(const QModelIndex&)));

	// tab Inspector [0]
	this->dsmAgentsModel = new DsmAgentsTableModel(this);
	this->m_ui->tableViewDsmAgents->setModel( this->dsmAgentsModel );
	connect(this->m_ui->cmbDsmMode, SIGNAL(activated(int)), this, SLOT(cmbDsmMode_activated(int)));
	connect(this->m_ui->btnDsmRaiseElection, SIGNAL(clicked()), this, SLOT(btnDsmRaiseElection_clicked()));
	connect(this->m_ui->btnDsmRefreshAgents, SIGNAL(clicked()), this, SLOT(btnDsmRefreshAgents_clicked()));

	// tab Transactions [1]
	this->dsmTransactionsModel = new TransactionTableModel(this);
	this->m_ui->tableViewDsmTransactions->setModel( this->dsmTransactionsModel );
	connect(this->m_ui->btnDsmRefreshTransactions, SIGNAL(clicked()), this, SLOT(btnDsmRefreshTransactions_clicked()));
	this->dsmTransactionsHistoryModel = new TransactionHistoryTableModel(this);
	this->m_ui->tableViewDsmTransactionsHistory->setModel( this->dsmTransactionsHistoryModel );

	// tab TupleSpace [2]
	this->dsmTupleSpaceModel = new TupleSpaceTableModel(this);
	this->m_ui->tableViewTupleSpace->setModel( this->dsmTupleSpaceModel );
	connect(this->m_ui->btnDsmRefreshTupleSpace, SIGNAL(clicked()), this, SLOT(btnDsmRefreshTupleSpace_clicked()));

 }

testform_DSM::~testform_DSM() {
	wosh::DefaultSharedLogHandler::getInstance()->deleteHandler(this->cHandler);
	disconnect(this->cHandler, SIGNAL(appendLine(const QString&)), this->m_ui->txtConsole, SLOT(insertPlainText(const QString&)));
	this->cHandler->close();
	delete this->cHandler; this->cHandler = NULL;

	getBusDSM().removeBusListener(this);
	delete this->m_ui;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void testform_DSM::menu_triggered( QAction* action ) {
	
	if ( action == this->m_ui->actionDSMs_Add ) {
		create_dsm(false);
		this->dsmModel->reset();
	 }
	else if ( action == this->m_ui->actionDSMs_AddwDataset ) {
		create_dsm(true);
		this->dsmModel->reset();
	 }
	else if ( action == this->m_ui->actionDSMs_Remove ) {
		SharedMemory* dsm = SharedMemory::getInstances().itemAt( this->m_ui->tableViewDSM->currentIndex().row() );
		delete_dsm( dsm );
		this->dsmModel->reset();
	 }
	else if ( action == this->m_ui->actionDSMs_ForceRefresh ) {
		this->dsmModel->reset();
	 }
 }

void testform_DSM::tableViewDSM_clicked( const QModelIndex& index ) {
	SharedMemoryInfo dsm_info;
	short primary = -1; int tr_history_size = 0;
	SharedMemory::getInstances().transactionBeginRead();
	SharedMemory* dsm = SharedMemory::getInstances().itemAt(index.row());
	if ( dsm != NULL )  {
		dsm_info = dsm->getInfo();
		primary = dsm->isPrimary();
		tr_history_size = dsm->getTransactionsHistorySize();
	 }
	SharedMemory::getInstances().transactionEnd();
	if ( dsm_info.id == 0 ) return;
	switch( this->m_ui->tabWidget->currentIndex() ) {
		case 0: { // Inspector tab
				this->dsmAgentsModel->reload(dsm_info.id);
				this->m_ui->tableViewDsmAgents->resizeColumnsToContents();
				this->m_ui->chkDsmPrimary->setChecked( (bool)primary );
				this->m_ui->cmbDsmMode->setCurrentIndex( dsm_info.mode );
				this->m_ui->spinTransactionHistorySize->setValue(tr_history_size);
				break;
			 }
		case 1: { // Transaction tab
				this->dsmTransactionsModel->reload(dsm_info.id);
				this->m_ui->tableViewDsmTransactions->resizeColumnsToContents();
				this->dsmTransactionsHistoryModel->reload(dsm_info.id);
				this->m_ui->tableViewDsmTransactionsHistory->resizeColumnsToContents();
				break;
			 }
		case 2: { // TupleSpace tab
				this->dsmTupleSpaceModel->reload(dsm_info.id);
				this->m_ui->tableViewTupleSpace->resizeColumnsToContents();
				break;
			 }
		default: break;
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////// DSM INSPECTOR

void testform_DSM::cmbDsmMode_activated( int index ) {
	SharedMemory::getInstances().transactionBeginRead();
	SharedMemory* dsm = SharedMemory::getInstances().itemAt( this->m_ui->tableViewDSM->currentIndex().row() );
	if ( dsm == NULL ) {
		QMessageBox::warning(this, tr("Failed"), tr("Please select a DSM."));
		SharedMemory::getInstances().transactionEnd();
		return;
	 }
	//dsm->setMode();
	SharedMemory::getInstances().transactionEnd();
 }

void testform_DSM::btnDsmRaiseElection_clicked() {
	SharedMemory::getInstances().transactionBeginRead();
	SharedMemory* dsm = SharedMemory::getInstances().itemAt( this->m_ui->tableViewDSM->currentIndex().row() );
	if ( dsm == NULL ) {
		QMessageBox::warning(this, tr("Failed"), tr("Please select a DSM."));
		SharedMemory::getInstances().transactionEnd();
		return;
	 }
	dsm->doElectionRequest();
	SharedMemory::getInstances().transactionEnd();
 }

void testform_DSM::btnDsmRefreshAgents_clicked() {
	SharedMemory::getInstances().transactionBeginRead();
	SharedMemory* dsm = SharedMemory::getInstances().itemAt( this->m_ui->tableViewDSM->currentIndex().row() );
	if ( dsm == NULL ) {
		QMessageBox::warning(this, tr("Failed"), tr("Please select a DSM."));
		SharedMemory::getInstances().transactionEnd();
		return;
	 }
	long id = dsm->getID();
	SharedMemory::getInstances().transactionEnd();
	this->dsmAgentsModel->reload(id);
	this->m_ui->tableViewDsmAgents->resizeColumnsToContents();
 }

void testform_DSM::btnDsmRefreshTransactions_clicked() {
	SharedMemory::getInstances().transactionBeginRead();
	SharedMemory* dsm = SharedMemory::getInstances().itemAt( this->m_ui->tableViewDSM->currentIndex().row() );
	if ( dsm == NULL ) {
		QMessageBox::warning(this, tr("Failed"), tr("Please select a DSM."));
		SharedMemory::getInstances().transactionEnd();
		return;
	 }
	long id = dsm->getID();
	SharedMemory::getInstances().transactionEnd();
	this->dsmTransactionsModel->reload(id);
	this->dsmTransactionsHistoryModel->reload(id);
	this->m_ui->tableViewDsmTransactions->resizeColumnsToContents();
	this->m_ui->tableViewDsmTransactionsHistory->resizeColumnsToContents();
 }

/////////////////////////////////////////////////////////////////////////////////////////////// DSM INSPECTOR
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////// TUPLES-SPACE INSPECTOR

void testform_DSM::btnDsmRefreshTupleSpace_clicked() {
	SharedMemory::getInstances().transactionBeginRead();
	SharedMemory* dsm = SharedMemory::getInstances().itemAt( this->m_ui->tableViewDSM->currentIndex().row() );
	if ( dsm == NULL ) {
		QMessageBox::warning(this, tr("Failed"), tr("Please select a DSM."));
		SharedMemory::getInstances().transactionEnd();
		return;
	 }
	long id = dsm->getID();
	SharedMemory::getInstances().transactionEnd();
	this->dsmTupleSpaceModel->reload(id);
	this->m_ui->tableViewTupleSpace->resizeColumnsToContents();
 }

////////////////////////////////////////////////////////////////////////////////////// TUPLES-SPACE INSPECTOR
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void testform_DSM::busMessage( const Message& message, const Bus* source ) {
	(void)source;
	if ( message.isEmpty() ) return;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace testsuite
}; // namespace wosh
