/**
 * @class   wosh::testsuite::testform_DSM
 * @brief   
 *          
 *          
 *
 ****************************************************************************
 * @version $Id: testform_DSM.h 2184 2010-03-20 13:19:23Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_TestSuite_testform_helpers_DSM_H__
 #define __WOSH_TestSuite_testform_helpers_DSM_H__

 #include "test_DSM_globals.h"
 #include <QtGui>
 #include <QAbstractTableModel>
 #include <QDateTime>



using namespace wosh;
using namespace std;
namespace wosh {
 namespace testsuite {



class DsmTableModel : public QAbstractTableModel
 {
	Q_OBJECT

	public:
		DsmTableModel(QObject *parent = 0) : QAbstractTableModel(parent) { }

		virtual ~DsmTableModel() { }

		using QAbstractTableModel::reset;

	public:

		virtual int rowCount(const QModelIndex &parent = QModelIndex()) const {
			(void)parent;
			return SharedMemory::getInstances().count();
		 }

		virtual int columnCount(const QModelIndex &parent = QModelIndex()) const {
			(void)parent; return 7;
		 }

		virtual QVariant data(const QModelIndex &index, int role) const {
			if (  !index.isValid() ) return QVariant();
			if ( role == Qt::DisplayRole || role == Qt::EditRole ) {
				MutexLockerRead mL(SharedMemory::getInstances().getMutex());
				SharedMemory* dsm = SharedMemory::getInstances().itemAt(index.row());
				if ( dsm == NULL ) return QVariant();
				switch(index.column()) {
					case 0: return QVariant( dsm->getID() );
					case 1: return QVariant( QDateTime().fromTime_t((long)dsm->getSeniority()) );
					case 2: return QVariant( dsm->isPrimary() );
					case 3: return QVariant( dsm->getMode() );
					case 4: return QVariant( dsm->getClock() );
					case 5: return QVariant( dsm->getDataSet().count() );
					case 6: return QVariant( dsm->getTransactions().count() );
					default: return QVariant();
				 }
			 }
			else if ( role == Qt::BackgroundColorRole ) { }
			else if ( role == Qt::ToolTipRole ) { }
			return QVariant();
		 }

		virtual QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const {
			if (role != Qt::DisplayRole) return QVariant();
			 if (orientation == Qt::Horizontal ) {
				switch(section) {
					case 0: return QVariant("ID");
					case 1: return QVariant("Seniority");
					case 2: return QVariant("Primary");
					case 3: return QVariant("Mode");
					case 4: return QVariant("Clock");
					case 5: return QVariant("#Tuples");
					case 6: return QVariant("#Transactions");
					default: return QVariant();
				 }
			 }
			return QVariant();
		 }

	public: // editable
		 Qt::ItemFlags flags(const QModelIndex &index) const {
			return QAbstractItemModel::flags(index) | Qt::ItemIsEnabled;// | Qt::ItemIsEditable;
		  }


}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class DsmAgentsTableModel : public QAbstractTableModel
 {
	Q_OBJECT

	public:
		DsmAgentsTableModel(QObject *parent = 0) : QAbstractTableModel(parent) { }

		virtual ~DsmAgentsTableModel() { }

		using QAbstractTableModel::reset;

		void setID( long id )			{ this->dsmId = id; }

		void reload( long id ) {
			setID(id);
			reset();
		 }

	public:

		virtual int rowCount(const QModelIndex &parent = QModelIndex()) const {
			(void)parent;
			MutexLockerRead mL(SharedMemory::getInstances().getMutex());
			SharedMemory* dsm = getDsm();
			if ( dsm == NULL ) return 0;
			return dsm->getAgents().count();
		 }

		virtual int columnCount(const QModelIndex &parent = QModelIndex()) const {
			(void)parent; return 4;
		 }

		virtual QVariant data(const QModelIndex &index, int role) const {
			if (  !index.isValid() ) return QVariant();
			if ( role == Qt::DisplayRole || role == Qt::EditRole ) {
				MutexLockerRead mL(SharedMemory::getInstances().getMutex());
				SharedMemory* dsm = getDsm();
				if ( dsm == NULL ) return QVariant();
				dsm->getAgents().transactionBeginRead();
				const SharedMemoryInfo* dsm_info_orig = dsm->getAgents().valueAt(index.row());
				if ( dsm_info_orig == NULL ) { 
					dsm->getAgents().transactionEnd();
					return QVariant();
				}
				const SharedMemoryInfo dsm_info = *dsm_info_orig;
				dsm->getAgents().transactionEnd();
				switch(index.column()) {
					case 0: return QVariant( dsm_info.id );
					case 1: return QVariant( QDateTime().fromTime_t((long)dsm_info.seniority) );
					case 2: return QVariant( dsm_info.mode );
					case 3: return QVariant( dsm_info.clock );
					default: return QVariant();
				 }
			 }
			else if ( role == Qt::BackgroundColorRole ) { }
			else if ( role == Qt::ToolTipRole ) { }
			return QVariant();
		 }

		virtual QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const {
			if (role != Qt::DisplayRole) return QVariant();
			 if (orientation == Qt::Horizontal ) {
				switch(section) {
					case 0: return QVariant("ID");
					case 1: return QVariant("Seniority");
					case 2: return QVariant("Mode");
					case 3: return QVariant("Clock");
					default: return QVariant();
				 }
			 }
			return QVariant();
		 }

	public: // editable
		 Qt::ItemFlags flags(const QModelIndex &index) const {
			return QAbstractItemModel::flags(index) | Qt::ItemIsEnabled;// | Qt::ItemIsEditable;
		  }

	protected:
		inline SharedMemory* getDsm() const	{ return getDsmById(dsmId); }
		long dsmId;

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class TupleSpaceTableModel : public QAbstractTableModel
 {
	Q_OBJECT

	public:
		TupleSpaceTableModel(QObject *parent = 0) : QAbstractTableModel(parent) { }

		virtual ~TupleSpaceTableModel() { }

		using QAbstractTableModel::reset;

		void setID( long id )			{ this->dsmId = id; }
		
		void reload( long id ) {
			setID(id);
			reset();
		 }

	public:

		virtual int rowCount(const QModelIndex &parent = QModelIndex()) const {
			(void)parent;
			MutexLockerRead mL(SharedMemory::getInstances().getMutex());
			SharedMemory* dsm = getDsm();
			if ( dsm == NULL ) return 0;
			return dsm->getDataSet().count();
		 }

		virtual int columnCount(const QModelIndex &parent = QModelIndex()) const {
			(void)parent; return 4;
		 }

		virtual QVariant data(const QModelIndex &index, int role) const {
			if (  !index.isValid() ) return QVariant();
			if ( role == Qt::DisplayRole || role == Qt::EditRole ) {
				MutexLockerRead mL(SharedMemory::getInstances().getMutex());
				SharedMemory* dsm = getDsm();
				if ( dsm == NULL ) return QVariant();
				dsm->getDataSet().transactionBeginRead();
				string tupleName = dsm->getDataSet().keyAt(index.row());
				const TestSharedObject* sobj = dynamic_cast<const TestSharedObject*>(dsm->getDataSet().valueAt(index.row()));
				dsm->getDataSet().transactionEnd();
				switch(index.column()) {
					case 0: return QVariant( tupleName.c_str() );
					case 1: return QVariant( sobj->objName.c_str() );
					case 2: return QVariant( sobj->value1 );
					case 3: return QVariant( sobj->value2 );
					default: return QVariant();
				 }
			 }
			else if ( role == Qt::BackgroundColorRole ) { }
			else if ( role == Qt::ToolTipRole ) { }
			return QVariant();
		 }

		virtual QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const {
			if (role != Qt::DisplayRole) return QVariant();
			 if (orientation == Qt::Horizontal ) {
				switch(section) {
					case 0: return QVariant("TupleName");
					case 1: return QVariant("Object Name");
					case 2: return QVariant("value1");
					case 3: return QVariant("value2");
					default: return QVariant();
				 }
			 }
			return QVariant();
		 }

	public: // editable
		 Qt::ItemFlags flags(const QModelIndex &index) const {
			return QAbstractItemModel::flags(index) | Qt::ItemIsEnabled;// | Qt::ItemIsEditable;
		  }

	protected:
		inline SharedMemory* getDsm() const	{ return getDsmById(dsmId); }
		long dsmId;

}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class TransactionTableModel : public QAbstractTableModel
 {
	Q_OBJECT

	public:
		TransactionTableModel(QObject *parent = 0) : QAbstractTableModel(parent) { }

		virtual ~TransactionTableModel() { }

		using QAbstractTableModel::reset;

		void setID( long id )			{ this->dsmId = id; }

		void reload( long id ) {
			setID(id);
			reset();
		 }

	public:

		virtual int rowCount(const QModelIndex &parent = QModelIndex()) const {
			(void)parent;
			MutexLockerRead mL(SharedMemory::getInstances().getMutex());
			SharedMemory* dsm = getDsm();
			if ( dsm == NULL ) return 0;
			return dsm->getTransactions().count();
		 }

		virtual int columnCount(const QModelIndex &parent = QModelIndex()) const {
			(void)parent; return 8;
		 }

		virtual QVariant data(const QModelIndex &index, int role) const {
			if (  !index.isValid() ) return QVariant();
			if ( role == Qt::DisplayRole || role == Qt::EditRole ) {
				MutexLockerRead mL(SharedMemory::getInstances().getMutex());
				SharedMemory* dsm = getDsm();
				if ( dsm == NULL ) return QVariant();
				dsm->getTransactions().transactionBeginRead();
				const Transaction* transaction = dsm->getTransactions().itemAt(index.row());
				if ( transaction == NULL ) { 
					dsm->getTransactions().transactionEnd();
					return QVariant();
				}
				QVariant v;
				switch(index.column()) {
					case 0: v = QVariant( transaction->getID() ); break;
					case 1: v = QVariant( transaction->getCreatorID() ); break;
					case 2: v = QVariant( transaction->getClassName() ); break;
					case 3: v = QVariant( transaction->getState() ); break;
					case 4: v = QVariant( transaction->getACKsCount() ); break;
					case 5: v = QVariant( transaction->getNACKsCount() ); break;
					case 6: v = QVariant( transaction->getNoReplyCount() ); break;
					case 7: v = QVariant( transaction->getAcksMap().size() ); break;
					default: break;
				 }
				dsm->getTransactions().transactionEnd();
				return v;
			 }
			else if ( role == Qt::BackgroundColorRole ) { }
			else if ( role == Qt::ToolTipRole ) { }
			return QVariant();
		 }

		virtual QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const {
			if (role != Qt::DisplayRole) return QVariant();
			 if (orientation == Qt::Horizontal ) {
				switch(section) {
					case 0: return QVariant("ID");
					case 1: return QVariant("Creator");
					case 2: return QVariant("Type");
					case 3: return QVariant("State");
					case 4: return QVariant("ACKs");
					case 5: return QVariant("NACKs");
					case 6: return QVariant("NoReply");
					case 7: return QVariant("#Agents");
					default: return QVariant();
				 }
			 }
			return QVariant();
		 }

	protected:
		inline SharedMemory* getDsm() const	{ return getDsmById(dsmId); }
		long dsmId;

}; // class def



//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class TransactionHistoryTableModel : public QAbstractTableModel
 {
	Q_OBJECT

	public:
		TransactionHistoryTableModel(QObject *parent = 0) : QAbstractTableModel(parent) { }

		virtual ~TransactionHistoryTableModel() { }

		using QAbstractTableModel::reset;

		void setID( long id )			{ this->dsmId = id; }

		void reload( long id ) {
			setID(id);
			reset();
		 }

	public:

		virtual int rowCount(const QModelIndex &parent = QModelIndex()) const {
			(void)parent;
			MutexLockerRead mL(SharedMemory::getInstances().getMutex());
			SharedMemory* dsm = getDsm();
			if ( dsm == NULL ) return 0;
			return dsm->getTransactionsHistory().count();
		 }

		virtual int columnCount(const QModelIndex &parent = QModelIndex()) const {
			(void)parent; return 8;
		 }

		virtual QVariant data(const QModelIndex &index, int role) const {
			if (  !index.isValid() ) return QVariant();
			if ( role == Qt::DisplayRole || role == Qt::EditRole ) {
				MutexLockerRead mL(SharedMemory::getInstances().getMutex());
				SharedMemory* dsm = getDsm();
				if ( dsm == NULL ) return QVariant();
				dsm->getTransactionsHistory().transactionBeginRead();
				const Transaction* transaction = dsm->getTransactionsHistory().itemAt(index.row());
				if ( transaction == NULL ) { 
					dsm->getTransactionsHistory().transactionEnd();
					return QVariant();
				}
				QVariant v;
				switch(index.column()) {
					case 0: v = QVariant( transaction->getID() ); break;
					case 1: v = QVariant( transaction->getCreatorID() ); break;
					case 2: v = QVariant( transaction->getClassName() ); break;
					case 3: v = QVariant( transaction->getState() ); break;
					case 4: v = QVariant( transaction->getACKsCount() ); break;
					case 5: v = QVariant( transaction->getNACKsCount() ); break;
					case 6: v = QVariant( transaction->getNoReplyCount() ); break;
					case 7: v = QVariant( transaction->getAcksMap().size() ); break;
					default: break;
				 }
				dsm->getTransactionsHistory().transactionEnd();
				return v;
			 }
			else if ( role == Qt::BackgroundColorRole ) { }
			else if ( role == Qt::ToolTipRole ) { }
			return QVariant();
		 }

		virtual QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const {
			if (role != Qt::DisplayRole) return QVariant();
			 if (orientation == Qt::Horizontal ) {
				switch(section) {
					case 0: return QVariant("ID");
					case 1: return QVariant("Creator");
					case 2: return QVariant("Type");
					case 3: return QVariant("State");
					case 4: return QVariant("ACKs");
					case 5: return QVariant("NACKs");
					case 6: return QVariant("NoReply");
					case 7: return QVariant("#Agents");
					default: return QVariant();
				 }
			 }
			return QVariant();
		 }

	protected:
		inline SharedMemory* getDsm() const	{ return getDsmById(dsmId); }
		long dsmId;

}; // class def



 }; // namespace testsuite
}; // namespace wosh


#endif // __WOSH_TestSuite_testform_helpers_DSM_H__
