/**
 * @file    test_Channel.cpp
 * @brief   Test classes: wosh::Property, wosh::PropertiesProvider
 *          Indirectly tests also wosh::Data
 *
 * @version $Id: test_Channel.cpp 2067 2010-03-09 23:40:30Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/

 #include "testsuite.h"
 #include "test_Channel.h"

 #include <core/Channel.h>
 #include <core/LogContext.h>
 #include <framework/network/ChannelUdp.h>
 #include <framework/network/ChannelTcp.h>

 #include <iostream>


class ChannelListenerNotifier : public ChannelListener {
	public:
		virtual void channel_Connected( Channel* channel, std::string host_name ) {
			Log::log( "ChannelListenerNotifier::channel_Connected(%s)", host_name.c_str() );
		 }
		virtual void channel_Disconnected( Channel* channel, std::string host_name ) {
			Log::log( "ChannelListenerNotifier::channel_Disconnected(%s)", host_name.c_str() );
		 }

		virtual void channel_incomingMessage( Channel* channel, Message* message ) {
			Log::log( "ChannelListenerNotifier::channel_incomingMessage():" );
			//message
		 }
		virtual void channel_incomingData( Channel* channel, const char* data, int size ) {
			Log::log( "ChannelListenerNotifier::channel_incomingData(%i) : \n %s", size, data );
		 }

	public:
		ChannelListenerNotifier() : ChannelListener() { }
		virtual ~ChannelListenerNotifier() { }
};



int test_Channel()
 {
	Log::log( "test_Channel() : Starting TEST" );

	//test_ChannelUdp();
	test_ChannelTcp();


	Log::log( "test_Channel() : Finished TEST" );
	return 0;
 }


void test_ChannelUdp()
 {
	Log::log( " test_ChannelUdp() : Starting TEST for class 'ChannelUdp'" );

	ChannelUdp chUdp;
	ChannelListenerNotifier chListener;

	chUdp.setChannelListener(&chListener);

	chUdp.applyingProperty( _Channel_KEY_BindAddress, "192.168.0.1:7676" );
	chUdp.applyingProperty( _Channel_KEY_Priority, 50 );
	chUdp.applyingProperty( _Channel_KEY_AutoBind, "true" );


	chUdp.listen(true);




	Log::log( " test_ChannelUdp() : Finished TEST for class 'ChannelUdp'" );
 }


void test_ChannelTcp()
 {
	Log::log( " test_ChannelTcp() : Starting TEST for class 'ChannelTcp'" );

	ChannelTcp chTcp;
	ChannelListenerNotifier chListener;

	chTcp.setChannelListener(&chListener);

	chTcp.applyingProperty( _Channel_KEY_BindAddress, "192.168.0.1:9595" );
	chTcp.applyingProperty( _Channel_KEY_Priority, 50 );
	chTcp.applyingProperty( _Channel_KEY_AutoBind, "true" );


	bool listen = true;



	if ( listen )
	 {
		qDebug() << "Listening..";
		chTcp.listen(true);

		qDebug() << "Waiting User input..";
		string message;
		std::cin >> message;
	 }
	else
	 {
		qDebug() << "Connecting..";
		chTcp.connect();

		qDebug() << "Waiting User input..";
		string message;
		std::cin >> message;
	 }


	Log::log( " test_ChannelTcp() : Finished TEST for class 'ChannelTcp'" );
 }




