/**
 * @file    test_MessageBus.cpp
 * @brief   Test classes: wosh::Property, wosh::PropertiesProvider
 *          Indirectly tests also wosh::Data
 *
 * @version $Id: test_MessageBus.cpp 2067 2010-03-09 23:40:30Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/

 #include "test_MessageBus.h"

 #include <woshDefs.h>

 #include <core/Bus.h>
 #include <core/BusRing.h>
 #include <core/BusConnector.h>
 #include <core/BusManager.h>


 #include <core/LogContext.h>
 #include <core/Utilities.h>

 #include <QtDebug>


int test_MessageBus()
 {
	Log::log( "test_MessageBus() : Starting TEST" );

	test_BusRing();

	test_BusManager();

	BusManager busman;
	test_BusConnector();


	Log::log( "test_MessageBus() : Finished TEST" );
	return 0;
 }





class prompt_MessageHandler : public virtual MessageHandler  {
	public:
		prompt_MessageHandler() {}
		virtual ~prompt_MessageHandler() {}

	void busMessage( const Message& message, const Bus* source ) {
		Log::log( " prompt_MessageHandler::busMessage() : ID" );
	 }

};






void test_BusRing()
 {
	Log::log( " test_BusRing() : Starting TEST for class 'BusRing'" );

	BusRing bus;
	prompt_MessageHandler listener;

	bus.addMessageHandler(&listener);
	bus.addMessageHandler(&listener);

	bus.postMessage( new Message() );

	bus.flushMessages();
	ASSERT( bus.getQueueSize() == 0 )

	bus.removeMessageHandler(&listener);
	bus.removeMessageHandler(&listener);

//	bus.postMessage( new Message() );
//	ASSERT( bus.getQueueSize() == 1 )

	Log::log( " test_BusRing() : Finished TEST for class 'BusRing'" );
 }


void test_BusManager()
 {
	Log::log( " test_BusManager() : Starting TEST for class 'BusManager'" );

	BusManager busman;

	ASSERT( &busman == BusManager::getInstance() )
	ASSERT( busman.getBusCount() == 0 )

	BusRing bus1( "wosh.Bus.TestBus1" );
	BusRing bus2( "wosh.Bus.TestBus2" );

	ASSERT( busman.getBusCount() == 2 )

	Log::log( " test_BusManager() : Finished TEST for class 'BusManager'" );
 }


void test_BusConnector()
 {
	Log::log( " test_BusConnector() : Starting TEST for class 'BusConnector'" );

	BusRing bus( "wosh.Bus.TestBus" );
	BusConnector conn;

	prompt_MessageHandler listener;


	conn.setMessageHandler(&listener);

	ASSERT( !conn.isConnected() )
	
	conn.connect( "wosh.Bus.TestBus" );

	ASSERT( conn.isConnected() )

	conn.postMessage( new Message() );

	conn.disconnect();

	ASSERT( !conn.isConnected() )

	conn.disconnect();

	conn.connect( "wosh.Bus.TestBus_not_found" );

	ASSERT( !conn.isConnected() )


	Log::log( " test_BusConnector() : Finished TEST for class 'BusConnector'" );
 }








