/**
 * @file    test_NetworkTcp.cpp
 * @brief   Test classes: wosh::Property, wosh::PropertiesProvider
 *          Indirectly tests also wosh::Data
 *
 * @version $Id: test_NetworkTcp.cpp 2067 2010-03-09 23:40:30Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/

 #include "testsuite.h"
 #include "test_NetworkTcp.h"

 #include <core/LogContext.h>
 #include <framework/network/SocketTcp.h>
 #include <framework/network/ServerTcp.h>

 #include <iostream>


class SocketTcpListenerNotifier : public ISocketTcpListener {
	public:

		virtual void tcpDataAvailable( int data_len, SocketTcp* socket )
		 {
			Log::log( "SocketTcpListenerNotifier::tcpDataAvailable(%i)", data_len );
			char* buffer = new char[data_len];
			socket->read( buffer, data_len );
			qDebug() << "DATA: " << buffer;
			delete [] buffer;
		 }

	public:
		SocketTcpListenerNotifier() : ISocketTcpListener() { }
		virtual ~SocketTcpListenerNotifier() { }
};


class ServerTcpListenerNotifier : public IServerTcpListener {
	public:

		virtual void serverTcp_incomingConnection( ServerTcp* source, SocketTcp* connection )
		 {
			(void)source;
			Log::log( "ServerTcpListenerNotifier::serverTcp_incomingConnection(%s)", connection->getAddressPort().c_str() );

			string message;
	//		std::cin >> message;
	//		connection->sendDatagram( message.c_str(), message.size() );
		 }

	public:
		ServerTcpListenerNotifier() : IServerTcpListener() { }
		virtual ~ServerTcpListenerNotifier() { }
};



int test_NetworkTcp()
 {
	Log::log( "test_NetworkTcp() : Starting TEST" );

	qDebug() << "Select socket/server:";
	string type;
	std::cin >> type;
	if ( type == "socket" )
		test_SocketTcp();
	else if ( type == "server" )
		test_ServerTcp();
	else
		qCritical() << " => Unknown selection.";

	Log::log( "test_NetworkTcp() : Finished TEST" );
	return 0;
 }


void test_ServerTcp()
 {
	Log::log( "test_ServerTcp() : Starting TEST" );

	ServerTcp srvTcp;
	ServerTcpListenerNotifier tcpListener;
	srvTcp.setListener(&tcpListener);

	qDebug() << "Type Binding address:port [ex. 192.168.0.1:8787]: ";
	string address_port;
	std::cin >> address_port;

	qDebug() << "Binding..";
	WRESULT ret_conn = srvTcp.bindSocket( address_port );
	if ( ret_conn != WRET_OK ) {
		qCritical() << "FAILED Binding!!";
		return;
	 }

	qDebug() << "Binded!  Waiting Connections..";
	qDebug() << "Waiting User input..";
	string message;
	std::cin >> message;

//	void write();
	//void read( char* buffer, int size);

	qDebug() << "Unbinding..";
	WRESULT ret_disc = srvTcp.unbindSocket();
	if ( ret_disc != WRET_OK ) {
		qCritical() << "FAILED Unbinding!!";
		return;
	 }


	Log::log( "test_ServerTcp() : Finished TEST" );
	return;
 }


void test_SocketTcp()
 {
	Log::log( " test_SocketTcp() : Starting TEST for class 'SocketTcp'" );

	SocketTcp sockTcp;
	SocketTcpListenerNotifier tcpListener;
	sockTcp.setListener(&tcpListener);

	qDebug() << "Type Target address:port [ex. 192.168.0.1:8787]: ";
	string address_port;
	std::cin >> address_port;

	qDebug() << "Connecting..";
	WRESULT ret_conn = sockTcp.connectTo( address_port );
	if ( ret_conn != WRET_OK ) {
		qCritical() << "FAILED Connecting!!";
		return;
	 }

	qDebug() << "Connected!";
	string message = "";
	qDebug() << "Waiting User input..";
	while ( message != "QUIT" ) {
		std::cin >> message;
		
		if ( message == "WRITE" ) {
			std::cin >> message;
			sockTcp.sendDatagram( message.c_str(), message.size() );
			message = "";
		 }
	 }
//	void write();
	//void read( char* buffer, int size);

	qDebug() << "Disconnecting..";
	WRESULT ret_disc = sockTcp.disconnect();
	if ( ret_disc != WRET_OK ) {
		qCritical() << "FAILED Disconnecting!!";
		return;
	 }

	 

	Log::log( " test_SocketTcp() : Finished TEST for class 'SocketTcp'" );
 }




